@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5178 $:
+$Revision: 6151 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-05 14:46:13 +0200 (Tue, 05 Jul 2011) $
+$Date: 2012-07-09 12:31:11 +0200 (Mon, 09 Jul 2012) $
 
 ********************************************************************/
 
@@ -32,6 +32,7 @@ $Date: 2011-07-05 14:46:13 +0200 (Tue, 05 Jul 2011) $
 #include "items/wire.h"
 #include "connectors/connectoritem.h"
 #include "items/moduleidnames.h"
+#include "utils/bezier.h"
 
 int SelectItemCommand::selectItemCommandID = 3;
 int ChangeNoteTextCommand::changeNoteTextCommandID = 5;
@@ -42,6 +43,7 @@ int BaseCommand::nextIndex = 0;
 BaseCommand::BaseCommand(BaseCommand::CrossViewType crossViewType, SketchWidget* sketchWidget, QUndoCommand *parent)
 	: QUndoCommand(parent)
 {
+	m_undoOnly = m_redoOnly = false;
 	m_crossViewType = crossViewType;
 	m_sketchWidget = sketchWidget;
 	m_parentCommand = parent;
@@ -71,6 +73,15 @@ QString BaseCommand::getDebugString() const {
 	return QString("%1 %2").arg(getParamString()).arg(text());
 }
 
+void BaseCommand::setUndoOnly() {
+	m_undoOnly = true;
+}
+
+void BaseCommand::setRedoOnly()
+{
+	m_redoOnly = true;
+}
+
 QString BaseCommand::getParamString() const {
 	return QString("%1 %2")
 		.arg(m_sketchWidget->viewName())
@@ -192,7 +203,12 @@ void AddItemCommand::turnOffFirstRedo() {
 }
 
 QString AddItemCommand::getParamString() const {
-	return "AddItemCommand " + AddDeleteItemCommand::getParamString();
+	return "AddItemCommand " + AddDeleteItemCommand::getParamString() +
+        QString(" loc:%1,%2 pt1:%3,%4 pt2:%5,%6")
+            .arg(m_viewGeometry.loc().x()).arg(m_viewGeometry.loc().y())
+            .arg(m_viewGeometry.line().p1().x()).arg(m_viewGeometry.line().p1().y())
+            .arg(m_viewGeometry.line().p2().x()).arg(m_viewGeometry.line().p2().y())
+        ;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -255,6 +271,38 @@ QString MoveItemCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+SimpleMoveItemCommand::SimpleMoveItemCommand(SketchWidget* sketchWidget, long itemID, QPointF & oldP, QPointF & newP, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_itemID = itemID;
+    m_old = oldP;
+    m_new = newP;
+}
+
+void SimpleMoveItemCommand::undo()
+{
+    m_sketchWidget->simpleMoveItem(m_itemID, m_old);
+}
+
+void SimpleMoveItemCommand::redo()
+{
+    m_sketchWidget->simpleMoveItem(m_itemID, m_new);
+}
+
+QString SimpleMoveItemCommand::getParamString() const {
+	return QString("SimpleMoveItemCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" id:%1 old.x:%2 old.y:%3 new.x:%4 new.y:%5")
+		.arg(m_itemID)
+		.arg(m_old.x())
+		.arg(m_old.y())
+		.arg(m_new.x())
+		.arg(m_new.y())		
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
 MoveItemsCommand::MoveItemsCommand(SketchWidget* sketchWidget, bool updateRatsnest, QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
@@ -306,7 +354,7 @@ QString MoveItemsCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-RotateItemCommand::RotateItemCommand(SketchWidget* sketchWidget, long itemID, qreal degrees, QUndoCommand *parent)
+RotateItemCommand::RotateItemCommand(SketchWidget* sketchWidget, long itemID, double degrees, QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
     m_itemID = itemID;
@@ -409,7 +457,7 @@ QString ChangeConnectionCommand::getParamString() const {
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 ChangeWireCommand::ChangeWireCommand(SketchWidget* sketchWidget, long fromID,
-									 QLineF oldLine, QLineF newLine, QPointF oldPos, QPointF newPos, 
+									 const QLineF & oldLine, const QLineF & newLine, QPointF oldPos, QPointF newPos, 
 									 bool updateConnections, bool updateRatsnest,
 									 QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
@@ -425,12 +473,16 @@ ChangeWireCommand::ChangeWireCommand(SketchWidget* sketchWidget, long fromID,
 
 void ChangeWireCommand::undo()
 {
-    m_sketchWidget->changeWire(m_fromID, m_oldLine, m_oldPos, m_updateConnections, m_updateRatsnest);
+	if (!m_redoOnly) {
+		m_sketchWidget->changeWire(m_fromID, m_oldLine, m_oldPos, m_updateConnections, m_updateRatsnest);
+	}
 }
 
 void ChangeWireCommand::redo()
 {
-    m_sketchWidget->changeWire(m_fromID, m_newLine, m_newPos, m_updateConnections, m_updateRatsnest);
+	if (!m_undoOnly) {
+		m_sketchWidget->changeWire(m_fromID, m_newLine, m_newPos, m_updateConnections, m_updateRatsnest);
+	}
 }
 
 QString ChangeWireCommand::getParamString() const {
@@ -447,41 +499,348 @@ QString ChangeWireCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-ChangeLegCommand::ChangeLegCommand(SketchWidget* sketchWidget, long fromID, const QString & fromConnectorID,
-									 QLineF oldLine, QLineF newLine, QUndoCommand *parent)
+ChangeWireCurveCommand::ChangeWireCurveCommand(SketchWidget* sketchWidget, long fromID,
+									 const Bezier * oldBezier, const Bezier * newBezier, 
+									 QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
+	m_firstTime = false;
     m_fromID = fromID;
-	m_oldLine = oldLine;
-    m_newLine = newLine;
+	m_oldBezier = m_newBezier = NULL;
+	if (oldBezier) {
+		m_oldBezier = new Bezier;
+		m_oldBezier->copy(oldBezier);
+	}
+	if (newBezier) {
+		m_newBezier = new Bezier;
+		m_newBezier->copy(newBezier);
+	}
+}
+
+void ChangeWireCurveCommand::undo()
+{
+	if (!m_redoOnly) {
+		m_sketchWidget->changeWireCurve(m_fromID, m_oldBezier);
+	}
+}
+
+void ChangeWireCurveCommand::redo()
+{
+	if (!m_undoOnly) {
+		if (m_firstTime) {
+			m_firstTime = false;
+		}
+		else {
+			m_sketchWidget->changeWireCurve(m_fromID, m_newBezier);
+		}
+	}
+}
+
+void ChangeWireCurveCommand::setFirstTime() {
+	m_firstTime = true;
+}
+
+QString ChangeWireCurveCommand::getParamString() const {
+	QString oldBezier;
+	QString newBezier;
+	if (m_oldBezier) {
+		oldBezier += QString("(%1,%2)").arg(m_oldBezier->cp0().x()).arg(m_oldBezier->cp0().y());
+		oldBezier += QString("(%1,%2)").arg(m_oldBezier->cp1().x()).arg(m_oldBezier->cp1().y());
+	}
+	if (m_newBezier) {
+		newBezier += QString("(%1,%2)").arg(m_newBezier->cp0().x()).arg(m_newBezier->cp0().y());
+		newBezier += QString("(%1,%2)").arg(m_newBezier->cp1().x()).arg(m_newBezier->cp1().y());
+	}
+
+	return QString("ChangeWireCurveCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 oldp:%2 newp:%3")
+		.arg(m_fromID)
+		.arg(oldBezier)
+		.arg(newBezier)
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+ChangeLegCommand::ChangeLegCommand(SketchWidget* sketchWidget, long fromID, const QString & fromConnectorID,
+									 const QPolygonF & oldLeg, const QPolygonF & newLeg, bool relative, bool active,
+									 const QString & why, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+	m_why = why;
+	m_simple = false;
+    m_fromID = fromID;
+	m_oldLeg = oldLeg;
+    m_newLeg = newLeg;
+	m_relative = relative;
     m_fromConnectorID = fromConnectorID;
+	m_active = active;
 }
 
 void ChangeLegCommand::undo()
 {
-    m_sketchWidget->changeLeg(m_fromID, m_fromConnectorID, m_oldLine);
+	if (!m_redoOnly) {
+		m_sketchWidget->changeLeg(m_fromID, m_fromConnectorID, m_oldLeg, m_relative, m_why);
+	}
+}
+
+void ChangeLegCommand::setSimple()
+{
+	m_simple = true;
 }
 
 void ChangeLegCommand::redo()
 {
-    m_sketchWidget->changeLeg(m_fromID, m_fromConnectorID, m_newLine);
+	if (!m_undoOnly) {
+		if (m_simple) {
+			m_sketchWidget->changeLeg(m_fromID, m_fromConnectorID, m_newLeg, m_relative, m_why);
+		}
+		else {
+			m_sketchWidget->recalcLeg(m_fromID, m_fromConnectorID, m_newLeg, m_relative, m_active, m_why);
+		}
+	}
 }
 
 QString ChangeLegCommand::getParamString() const {
+
+	QString oldLeg;
+	QString newLeg;
+	foreach (QPointF p, m_oldLeg) {
+		oldLeg += QString("(%1,%2)").arg(p.x()).arg(p.y());
+	}
+	foreach (QPointF p, m_newLeg) {
+		newLeg += QString("(%1,%2)").arg(p.x()).arg(p.y());
+	}
 	return QString("ChangeLegCommand ") 
 		+ BaseCommand::getParamString() + 
-		QString(" fromid:%1 fromc:%2 oldr:%3,%4,%5,%6 newr:%7,%8,%9,%10")
+		QString(" fromid:%1 fromc:%2 %3 old:%4 new:%5")
 		.arg(m_fromID)
 		.arg(m_fromConnectorID)
-		.arg(m_oldLine.x1()).arg(m_oldLine.y1()).arg(m_oldLine.x2()).arg(m_oldLine.y2())		
-		.arg(m_newLine.x1()).arg(m_newLine.y1()).arg(m_newLine.x2()).arg(m_newLine.y2())		
+		.arg(m_why)
+		.arg(oldLeg)
+		.arg(newLeg)
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+MoveLegBendpointCommand::MoveLegBendpointCommand(SketchWidget* sketchWidget, long fromID, const QString & fromConnectorID,
+												int index, QPointF oldPos, QPointF newPos, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_fromID = fromID;
+	m_oldPos = oldPos;
+    m_newPos = newPos;
+    m_fromConnectorID = fromConnectorID;
+	m_index = index;
+}
+
+void MoveLegBendpointCommand::undo()
+{
+	if (!m_redoOnly) {
+		m_sketchWidget->moveLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_oldPos);
+	}
+}
+
+void MoveLegBendpointCommand::redo()
+{
+	if (!m_undoOnly) {
+		m_sketchWidget->moveLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_newPos);
+	}
+}
+
+QString MoveLegBendpointCommand::getParamString() const {
+
+	return QString("MoveLegBendpointCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 fromc:%2 ix:%3 old:%4,%5 new:%6,%7")
+		.arg(m_fromID)
+		.arg(m_fromConnectorID)
+		.arg(m_index)
+		.arg(m_oldPos.x())
+		.arg(m_oldPos.y())
+		.arg(m_newPos.x())
+		.arg(m_newPos.y())
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+ChangeLegCurveCommand::ChangeLegCurveCommand(SketchWidget* sketchWidget, long fromID, const QString & connectorID, int index,
+									 const Bezier * oldBezier, const Bezier * newBezier, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+	m_firstTime = false;
+    m_fromID = fromID;
+	m_oldBezier = m_newBezier = NULL;
+	if (oldBezier) {
+		m_oldBezier = new Bezier;
+		m_oldBezier->copy(oldBezier);
+	}
+	if (newBezier) {
+		m_newBezier = new Bezier;
+		m_newBezier->copy(newBezier);
+	}
+
+	m_fromConnectorID = connectorID;
+	m_index = index;
+}
+
+void ChangeLegCurveCommand::undo()
+{
+    m_sketchWidget->changeLegCurve(m_fromID, m_fromConnectorID, m_index,  m_oldBezier);
+}
+
+void ChangeLegCurveCommand::redo()
+{
+	if (m_firstTime) {
+		m_firstTime = false;
+	}
+	else if (!m_undoOnly) {
+		m_sketchWidget->changeLegCurve(m_fromID, m_fromConnectorID, m_index, m_newBezier);
+	}
+}
+
+void ChangeLegCurveCommand::setFirstTime() {
+	m_firstTime = true;
+}
+
+QString ChangeLegCurveCommand::getParamString() const {
+	QString oldBezier;
+	QString newBezier;
+	if (m_oldBezier) {
+		oldBezier += QString("(%1,%2)").arg(m_oldBezier->cp0().x()).arg(m_oldBezier->cp0().y());
+		oldBezier += QString("(%1,%2)").arg(m_oldBezier->cp1().x()).arg(m_oldBezier->cp1().y());
+	}
+	if (m_newBezier) {
+		newBezier += QString("(%1,%2)").arg(m_newBezier->cp0().x()).arg(m_newBezier->cp0().y());
+		newBezier += QString("(%1,%2)").arg(m_newBezier->cp1().x()).arg(m_newBezier->cp1().y());
+	}
+
+	return QString("ChangeLegCurveCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 oldp:%2 newp:%3")
+		.arg(m_fromID)
+		.arg(oldBezier)
+		.arg(newBezier)
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+ChangeLegBendpointCommand::ChangeLegBendpointCommand(SketchWidget* sketchWidget, long fromID, const QString & connectorID, 
+									int oldCount, int newCount, int index, QPointF pos,
+									const Bezier * bezier0, const Bezier * bezier1, const Bezier * bezier2, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+	m_firstTime = false;
+    m_fromID = fromID;
+	m_bezier0 = m_bezier1 = m_bezier2 = NULL;
+	if (bezier0 != NULL) {
+		m_bezier0 = new Bezier;
+		m_bezier0->copy(bezier0);
+	}
+	if (bezier1 != NULL) {
+		m_bezier1 = new Bezier;
+		m_bezier1->copy(bezier1);
+	}
+	if (bezier2 != NULL) {
+		m_bezier2 = new Bezier;
+		m_bezier2->copy(bezier2);
+	}
+
+	m_fromConnectorID = connectorID;
+	m_index = index;
+	m_oldCount = oldCount;
+	m_newCount = newCount;
+	m_pos = pos;
+}
+
+void ChangeLegBendpointCommand::undo()
+{
+		if (m_newCount < m_oldCount) {
+			m_sketchWidget->addLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_pos, m_bezier0, m_bezier1);
+		}
+		else {
+			m_sketchWidget->removeLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_bezier0);
+		}
+}
+
+void ChangeLegBendpointCommand::redo()
+{
+	if (m_firstTime) {
+		m_firstTime = false;
+	}
+	else {
+		if (m_newCount > m_oldCount) {
+			m_sketchWidget->addLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_pos, m_bezier1, m_bezier2);
+		}
+		else {
+			m_sketchWidget->removeLegBendpoint(m_fromID, m_fromConnectorID, m_index, m_bezier2);
+		}
+	}
+}
+
+void ChangeLegBendpointCommand::setFirstTime() {
+	m_firstTime = true;
+}
+
+QString ChangeLegBendpointCommand::getParamString() const {
+	QString bezier;
+	if (m_bezier0) {
+		bezier += QString("(%1,%2)").arg(m_bezier0->cp0().x()).arg(m_bezier0->cp0().y());
+		bezier += QString("(%1,%2)").arg(m_bezier0->cp1().x()).arg(m_bezier0->cp1().y());
+	}
+
+	return QString("ChangeLegBendpointCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 newp:%2")
+		.arg(m_fromID)
+		.arg(bezier)
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+RotateLegCommand::RotateLegCommand(SketchWidget* sketchWidget, long fromID, const QString & fromConnectorID,
+									 const QPolygonF & oldLeg, bool active, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_fromID = fromID;
+	m_oldLeg = oldLeg;
+    m_fromConnectorID = fromConnectorID;
+	m_active = active;
+}
+
+void RotateLegCommand::undo()
+{
+}
+
+void RotateLegCommand::redo()
+{
+	m_sketchWidget->rotateLeg(m_fromID, m_fromConnectorID, m_oldLeg, m_active);
+}
+
+QString RotateLegCommand::getParamString() const {
+
+	QString oldLeg;
+	foreach (QPointF p, m_oldLeg) {
+		oldLeg += QString("(%1,%2)").arg(p.x()).arg(p.y());
+	}
+	return QString("RotateLegCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 fromc:%2 old:%3")
+		.arg(m_fromID)
+		.arg(m_fromConnectorID)
+		.arg(oldLeg)
 		;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 ChangeLayerCommand::ChangeLayerCommand(SketchWidget *sketchWidget, long fromID,
-									qreal oldZ, qreal newZ, 
+									double oldZ, double newZ, 
 									ViewLayer::ViewLayerID oldLayer, ViewLayer::ViewLayerID newLayer,
     								QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
@@ -539,6 +898,13 @@ void SelectItemCommand::copyUndo(SelectItemCommand * sother) {
   	}
 }
 
+void SelectItemCommand::copyRedo(SelectItemCommand * sother) {
+   	this->m_redoIDs.clear();
+   	for (int i = 0; i < sother->m_redoIDs.size(); i++) {
+   		this->m_redoIDs.append(sother->m_redoIDs[i]);
+  	}
+}
+
 void SelectItemCommand::clearRedo() {
 	m_redoIDs.clear();
 }
@@ -638,7 +1004,7 @@ ChangeZCommand::~ChangeZCommand() {
 	m_triplets.clear();
 }
 
-void ChangeZCommand::addTriplet(long id, qreal oldZ, qreal newZ) {
+void ChangeZCommand::addTriplet(long id, double oldZ, double newZ) {
 	m_triplets.insert(id, new RealPair (oldZ, newZ));
 }
 
@@ -652,11 +1018,11 @@ void ChangeZCommand::redo()
    m_sketchWidget->changeZ(m_triplets, second);
 }
 
-qreal ChangeZCommand::first(RealPair * pair) {
+double ChangeZCommand::first(RealPair * pair) {
 	return pair->first;
 }
 
-qreal ChangeZCommand::second(RealPair * pair) {
+double ChangeZCommand::second(RealPair * pair) {
 	return pair->second;
 }
 
@@ -665,7 +1031,7 @@ QString ChangeZCommand::getParamString() const {
 		+ BaseCommand::getParamString();
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////singlev///////////////////////////////////////////////////////////////////////
 
 CheckStickyCommand::CheckStickyCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, long itemID, bool checkCurrent, CheckType checkType, QUndoCommand *parent)
 : BaseCommand(crossViewType, sketchWidget, parent)
@@ -812,19 +1178,19 @@ void CleanUpWiresCommand::addTrace(SketchWidget * sketchWidget, Wire * wire)
 	addSubCommand(new WireWidthChangeCommand(sketchWidget, wire->id(), wire->width(), wire->width(), NULL));
 	
 	foreach (ConnectorItem * toConnectorItem, wire->connector0()->connectedToItems()) {	
-		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::SingleView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
+		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::CrossView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
 								wire->id(), "connector0", 
 								ViewLayer::specFromID(wire->viewLayerID()),
 								false, NULL));
 	}
 	foreach (ConnectorItem * toConnectorItem, wire->connector1()->connectedToItems()) {	
-		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::SingleView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
+		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::CrossView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
 							wire->id(), "connector1", 
 							ViewLayer::specFromID(wire->viewLayerID()),
 							false, NULL));
 	}
 
-	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::SingleView, ModuleIDNames::WireModuleIDName, wire->viewLayerSpec(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
+	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::CrossView, ModuleIDNames::WireModuleIDName, wire->viewLayerSpec(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
 }
 
 bool CleanUpWiresCommand::hasTraces(SketchWidget * sketchWidget) {
@@ -839,7 +1205,7 @@ QString CleanUpWiresCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-WireColorChangeCommand::WireColorChangeCommand(SketchWidget* sketchWidget, long wireId, const QString &oldColor, const QString &newColor, qreal oldOpacity, qreal newOpacity, QUndoCommand *parent)
+WireColorChangeCommand::WireColorChangeCommand(SketchWidget* sketchWidget, long wireId, const QString &oldColor, const QString &newColor, double oldOpacity, double newOpacity, QUndoCommand *parent)
 : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
 	m_wireId = wireId;
@@ -867,7 +1233,7 @@ QString WireColorChangeCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-WireWidthChangeCommand::WireWidthChangeCommand(SketchWidget* sketchWidget, long wireId, qreal oldWidth, qreal newWidth, QUndoCommand *parent)
+WireWidthChangeCommand::WireWidthChangeCommand(SketchWidget* sketchWidget, long wireId, double oldWidth, double newWidth, QUndoCommand *parent)
 : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
 	m_wireId = wireId;
@@ -1005,6 +1371,37 @@ QString MoveLabelCommand::getParamString() const {
 
 ///////////////////////////////////////////////
 
+MoveLockCommand::MoveLockCommand(SketchWidget *sketchWidget, long id, bool oldLock, bool newLock, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_itemID = id;
+    m_oldLock = oldLock;
+    m_newLock = newLock;
+}
+
+void MoveLockCommand::undo()
+{
+    m_sketchWidget->setMoveLock(m_itemID, m_oldLock);
+}
+
+void MoveLockCommand::redo()
+{
+    m_sketchWidget->setMoveLock(m_itemID, m_newLock);
+}
+
+
+QString MoveLockCommand::getParamString() const {
+	return QString("MoveLockCommand ") 
+		+ BaseCommand::getParamString()
+		+ QString(" id:%1 o:%2 n:%3") 
+			.arg(m_itemID)
+			.arg(m_oldLock)
+			.arg(m_newLock);
+
+}
+
+///////////////////////////////////////////////
+
 ChangeLabelTextCommand::ChangeLabelTextCommand(SketchWidget *sketchWidget, long id, 
 											   const QString & oldText, const QString & newText, 
 											   QUndoCommand *parent)
@@ -1016,11 +1413,11 @@ ChangeLabelTextCommand::ChangeLabelTextCommand(SketchWidget *sketchWidget, long
 }
 
 void ChangeLabelTextCommand::undo() {
-	m_sketchWidget->setInstanceTitle(m_itemID, m_oldText, false, true);
+	m_sketchWidget->setInstanceTitle(m_itemID, m_newText, m_oldText, false, true);
 }
 
 void ChangeLabelTextCommand::redo() {
-    m_sketchWidget->setInstanceTitle(m_itemID, m_newText, false, true);
+    m_sketchWidget->setInstanceTitle(m_itemID, m_oldText, m_newText, false, true);
 }
 
 QString ChangeLabelTextCommand::getParamString() const {
@@ -1130,7 +1527,7 @@ QString ChangeNoteTextCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-RotateFlipLabelCommand::RotateFlipLabelCommand(SketchWidget* sketchWidget, long itemID, qreal degrees, Qt::Orientations orientation, QUndoCommand *parent)
+RotateFlipLabelCommand::RotateFlipLabelCommand(SketchWidget* sketchWidget, long itemID, double degrees, Qt::Orientations orientation, QUndoCommand *parent)
     : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
     m_itemID = itemID;
@@ -1187,7 +1584,7 @@ QString ResizeNoteCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-ResizeBoardCommand::ResizeBoardCommand(SketchWidget * sketchWidget, long itemID, qreal oldWidth, qreal oldHeight, qreal newWidth, qreal newHeight, QUndoCommand * parent)
+ResizeBoardCommand::ResizeBoardCommand(SketchWidget * sketchWidget, long itemID, double oldWidth, double oldHeight, double newWidth, double newHeight, QUndoCommand * parent)
 : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
 {
 	m_itemID = itemID;
@@ -1198,11 +1595,15 @@ ResizeBoardCommand::ResizeBoardCommand(SketchWidget * sketchWidget, long itemID,
 }
 
 void ResizeBoardCommand::undo() {
-	m_sketchWidget->resizeBoard(m_itemID, m_oldWidth, m_oldHeight);
+    if (!m_redoOnly) {
+	    m_sketchWidget->resizeBoard(m_itemID, m_oldWidth, m_oldHeight);
+    }
 }
 
 void ResizeBoardCommand::redo() {
-	m_sketchWidget->resizeBoard(m_itemID, m_newWidth, m_newHeight);
+    if (!m_undoOnly) {
+	    m_sketchWidget->resizeBoard(m_itemID, m_newWidth, m_newHeight);
+    }
 }
 
 QString ResizeBoardCommand::getParamString() const {
@@ -1246,35 +1647,6 @@ QString TransformItemCommand::getParamString() const {
 
 /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-SketchBackgroundColorChangeCommand::SketchBackgroundColorChangeCommand(SketchWidget* sketchWidget, const QString &oldColor, const QString &newColor, QUndoCommand *parent)
-: BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
-{
-	m_oldColor = oldColor;
-	m_newColor = newColor;
-}
-
-void SketchBackgroundColorChangeCommand::undo() {
-	QColor color;
-	color.setNamedColor(m_oldColor);
-	m_sketchWidget->setBackground(color);
-}
-
-void SketchBackgroundColorChangeCommand::redo() {
-	QColor color;
-	color.setNamedColor(m_newColor);
-	m_sketchWidget->setBackground(color);
-}
-
-QString SketchBackgroundColorChangeCommand::getParamString() const {
-	return QString("SketchBackgroundColorChangeCommand ") 
-		+ BaseCommand::getParamString()
-		+ QString(" viewid:%1 oldcolor:%2 newcolor:%3" ) 
-			.arg(m_sketchWidget->viewIdentifier()).arg(m_oldColor).arg(m_newColor);
-
-}
-
-/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-
 SetResistanceCommand::SetResistanceCommand(SketchWidget * sketchWidget, long itemID, QString oldResistance, QString newResistance, QString oldPinSpacing, QString newPinSpacing, QUndoCommand * parent)
 : BaseCommand(BaseCommand::CrossView, sketchWidget, parent)
 {
@@ -1423,13 +1795,15 @@ LoadLogoImageCommand::LoadLogoImageCommand(SketchWidget *sketchWidget, long id,
 }
 
 void LoadLogoImageCommand::undo() {
-	m_sketchWidget->loadLogoImage(m_itemID, m_oldSvg, m_oldAspectRatio, m_oldFilename);
+    if (!m_redoOnly) {
+	    m_sketchWidget->loadLogoImage(m_itemID, m_oldSvg, m_oldAspectRatio, m_oldFilename);
+    }
 }
 
 void LoadLogoImageCommand::redo() {
 	if (m_newFilename.isEmpty()) {
 	}
-	else {
+	else if (!m_undoOnly) {
 		m_sketchWidget->loadLogoImage(m_itemID, m_newFilename, m_addName);
 	}
 }
@@ -1499,3 +1873,125 @@ QString SetDropOffsetCommand::getParamString() const {
 			.arg(m_itemID).arg(m_dropOffset.x()).arg(m_dropOffset.y());
 
 }
+
+///////////////////////////////////////////////
+
+RenamePinsCommand::RenamePinsCommand(SketchWidget *sketchWidget, long id, const QStringList & oldOnes, const QStringList & newOnes, bool singleRow, QUndoCommand *parent) :
+	BaseCommand(BaseCommand::CrossView, sketchWidget, parent)
+{
+	m_oldLabels = oldOnes;
+	m_newLabels = newOnes;
+	m_itemID = id;
+	m_singleRow = singleRow;
+}
+
+void RenamePinsCommand::undo() {
+	m_sketchWidget->renamePins(m_itemID, m_oldLabels, m_singleRow);
+}
+
+void RenamePinsCommand::redo() {
+	m_sketchWidget->renamePins(m_itemID, m_newLabels, m_singleRow);
+}
+
+QString RenamePinsCommand::getParamString() const {
+	return QString("RenamePinsCommand ") 
+		+ BaseCommand::getParamString()
+		+ QString(" id:%1 sr:%2") 
+			.arg(m_itemID).arg(m_singleRow);
+
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+GroundFillSeedCommand::GroundFillSeedCommand(SketchWidget* sketchWidget, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+	setText(QObject::tr("Set Ground Fill Seed"));
+}
+
+void GroundFillSeedCommand::undo()
+{
+	foreach(GFSThing gfsThing, m_items) {
+		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, !gfsThing.seed);
+	}
+}
+
+void GroundFillSeedCommand::redo()
+{
+	foreach(GFSThing gfsThing, m_items) {
+		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, gfsThing.seed);
+	}
+}
+
+void GroundFillSeedCommand::addItem(long id, const QString & connectorID, bool seed)
+{
+	GFSThing gfsThing;
+	gfsThing.id = id;
+	gfsThing.connectorID = connectorID;
+	gfsThing.seed = seed;
+	m_items.append(gfsThing);
+}
+
+QString GroundFillSeedCommand::getParamString() const {
+	return QString("GroundFillSeedCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" items:%1")
+		.arg(m_items.count())
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+WireExtrasCommand::WireExtrasCommand(SketchWidget* sketchWidget, long fromID,
+									 const QDomElement & oldExtras, const QDomElement & newExtras,
+									 QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+
+    m_fromID = fromID;
+	m_newExtras = newExtras;
+    m_oldExtras = oldExtras;
+}
+
+void WireExtrasCommand::undo()
+{
+	if (!m_redoOnly) {
+		m_sketchWidget->setWireExtras(m_fromID, m_oldExtras);
+	}
+}
+
+void WireExtrasCommand::redo()
+{
+	if (!m_undoOnly) {
+		m_sketchWidget->setWireExtras(m_fromID, m_newExtras);
+	}
+}
+
+QString WireExtrasCommand::getParamString() const {
+	return QString("WireExtrasCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 ")
+		.arg(m_fromID)
+		;
+}
+
+////////////////////////////////////
+
+
+TemporaryCommand::TemporaryCommand(const QString & text) : QUndoCommand(text) {
+    m_enabled = true;
+}
+
+TemporaryCommand::~TemporaryCommand() {
+}
+
+void TemporaryCommand::setEnabled(bool enabled) {
+    m_enabled = enabled;
+}
+
+ void TemporaryCommand::redo() {
+     if (m_enabled) {
+         QUndoCommand::redo();
+     }
+}
+
