@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify\
 it under the terms of the GNU General Public License as published by
@@ -18,14 +18,12 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5178 $:
+$Revision: 6112 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-05 14:46:13 +0200 (Tue, 05 Jul 2011) $
+$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 ********************************************************************/
 
-
-
 #ifndef COMMANDS_H
 #define COMMANDS_H
 
@@ -39,6 +37,8 @@ $Date: 2011-07-05 14:46:13 +0200 (Tue, 05 Jul 2011) $
 #include "utils/misc.h"
 #include "items/itembase.h"
 
+/////////////////////////////////////////////
+
 class BaseCommand : public QUndoCommand
 {
 public:
@@ -64,6 +64,8 @@ public:
 	void subUndo(int index);
 	void subRedo(int index);
 	int index() const;
+	void setUndoOnly();
+	void setRedoOnly();
 
 protected:
 	virtual QString getParamString() const;
@@ -76,8 +78,12 @@ protected:
 	QList<BaseCommand *> m_commands;
 	QUndoCommand * m_parentCommand;
 	int m_index;
+	bool m_undoOnly;
+	bool m_redoOnly;
 };
 
+/////////////////////////////////////////////
+
 class AddDeleteItemCommand : public BaseCommand
 {
 public:
@@ -99,6 +105,8 @@ protected:
 	ViewLayer::ViewLayerSpec m_viewLayerSpec;
 };
 
+/////////////////////////////////////////////
+
 class AddItemCommand : public AddDeleteItemCommand
 {
 public:
@@ -119,6 +127,8 @@ protected:
 	RestoreIndexesCommand * m_restoreIndexesCommand;
 };
 
+/////////////////////////////////////////////
+
 class DeleteItemCommand : public AddDeleteItemCommand
 {
 public:
@@ -131,6 +141,8 @@ protected:
 
 };
 
+/////////////////////////////////////////////
+
 class MoveItemCommand : public BaseCommand
 {
 public:
@@ -148,6 +160,25 @@ protected:
     ViewGeometry m_new;
 };
 
+/////////////////////////////////////////////
+
+class SimpleMoveItemCommand : public BaseCommand
+{
+public:
+    SimpleMoveItemCommand(class SketchWidget *sketchWidget, long id, QPointF & oldP, QPointF & newP, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_itemID;
+    QPointF m_old;
+    QPointF m_new;
+};
+
+/////////////////////////////////////////////
 
 struct MoveItemThing {
 	long id;
@@ -173,10 +204,12 @@ protected:
 	QList<MoveItemThing> m_items;
 };
 
+/////////////////////////////////////////////
+
 class RotateItemCommand : public BaseCommand
 {
 public:
-    RotateItemCommand(class SketchWidget *sketchWidget, long id, qreal degrees, QUndoCommand *parent);
+    RotateItemCommand(class SketchWidget *sketchWidget, long id, double degrees, QUndoCommand *parent);
     void undo();
     void redo();
 
@@ -185,9 +218,11 @@ protected:
 
 protected:
     long m_itemID;
-    qreal m_degrees;
+    double m_degrees;
 };
 
+/////////////////////////////////////////////
+
 class FlipItemCommand : public BaseCommand
 {
 
@@ -204,6 +239,8 @@ protected:
     Qt::Orientations m_orientation;
 };
 
+/////////////////////////////////////////////
+
 class TransformItemCommand : public BaseCommand
 {
 
@@ -221,6 +258,8 @@ protected:
     QMatrix m_newMatrix;
 };
 
+/////////////////////////////////////////////
+
 class ChangeConnectionCommand : public BaseCommand
 {
 public:
@@ -247,11 +286,13 @@ protected:
 
 };
 
+/////////////////////////////////////////////
+
 class ChangeWireCommand : public BaseCommand
 {
 public:
     ChangeWireCommand(class SketchWidget *sketchWidget, long fromID,
-    					QLineF oldLine, QLineF newLine, QPointF oldPos, QPointF newPos,
+    					const QLineF & oldLine, const QLineF & newLine, QPointF oldPos, QPointF newPos,
     					bool updateConnections, bool updateRatsnest,
     					QUndoCommand *parent);
     void undo();
@@ -270,11 +311,61 @@ protected:
     bool m_updateConnections;
 };
 
+/////////////////////////////////////////////
+
+class ChangeWireCurveCommand : public BaseCommand
+{
+public:
+    ChangeWireCurveCommand(class SketchWidget *sketchWidget, long fromID,
+    					const class Bezier * oldBezier, const class Bezier * newBezier,
+    					QUndoCommand *parent);
+    void undo();
+    void redo();
+	void setFirstTime();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_fromID;
+    class Bezier * m_newBezier;
+    class Bezier * m_oldBezier;
+    bool m_firstTime;
+};
+
+/////////////////////////////////////////////
+
 class ChangeLegCommand : public BaseCommand
 {
 public:
     ChangeLegCommand(class SketchWidget *sketchWidget, long fromID, const QString & fromConnectorID,
-    					QLineF oldLine, QLineF newLine,  QUndoCommand *parent);
+    					const QPolygonF & oldLeg, const QPolygonF & newLeg, bool relative, bool active, const QString & why, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+	void setSimple();
+
+protected:
+	QString getParamString() const;
+
+protected:
+	QString m_fromConnectorID;
+    long m_fromID;
+    QPolygonF m_newLeg;
+    QPolygonF m_oldLeg;
+	bool m_relative;
+	bool m_active;
+	bool m_simple;
+	QString m_why;
+};
+
+/////////////////////////////////////////////
+
+class MoveLegBendpointCommand : public BaseCommand
+{
+public:
+    MoveLegBendpointCommand(class SketchWidget *sketchWidget, long fromID, const QString & fromConnectorID,
+    						 int index, QPointF oldPos, QPointF newPos, QUndoCommand *parent);
     void undo();
     void redo();
 
@@ -284,15 +375,89 @@ protected:
 protected:
 	QString m_fromConnectorID;
     long m_fromID;
-    QLineF m_newLine;
-    QLineF m_oldLine;
+    QPointF m_newPos;
+    QPointF m_oldPos;
 };
 
+/////////////////////////////////////////////
+
+class ChangeLegCurveCommand : public BaseCommand
+{
+public:
+    ChangeLegCurveCommand(class SketchWidget *sketchWidget, long fromID, const QString & fromConnectorID, int index,
+    					const class Bezier * oldBezier, const class Bezier * newBezier,
+    					QUndoCommand *parent);
+    void undo();
+    void redo();
+	void setFirstTime();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_fromID;
+    class Bezier * m_newBezier;
+    class Bezier * m_oldBezier;
+    bool m_firstTime;
+	QString m_fromConnectorID;
+	int m_index;
+};
+
+/////////////////////////////////////////////
+
+class ChangeLegBendpointCommand : public BaseCommand
+{
+public:
+    ChangeLegBendpointCommand(class SketchWidget *sketchWidget, long fromID, const QString & fromConnectorID,
+    					int oldCount, int newCount, int index, QPointF pos, 
+						const class Bezier *, const class Bezier *, const class Bezier *, QUndoCommand *parent);
+    void undo();
+    void redo();
+	void setFirstTime();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_fromID;
+    class Bezier * m_bezier0;
+    class Bezier * m_bezier1;
+    class Bezier * m_bezier2;
+    bool m_firstTime;
+	QString m_fromConnectorID;
+	int m_index;
+	int m_oldCount;
+	int m_newCount;
+	QPointF m_pos;
+};
+
+/////////////////////////////////////////////
+
+class RotateLegCommand : public BaseCommand
+{
+public:
+    RotateLegCommand(class SketchWidget *sketchWidget, long fromID, const QString & fromConnectorID,
+    					const QPolygonF & oldLeg, bool active, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+	QString m_fromConnectorID;
+    long m_fromID;
+    QPolygonF m_oldLeg;
+	bool m_active;
+};
+
+/////////////////////////////////////////////
+
 class ChangeLayerCommand : public BaseCommand
 {
 public:
     ChangeLayerCommand(class SketchWidget *sketchWidget, long fromID,
-					  qreal oldZ, qreal newZ, 
+					  double oldZ, double newZ, 
 					  ViewLayer::ViewLayerID oldLayer, ViewLayer::ViewLayerID newLayer,
     				  QUndoCommand *parent);
     void undo();
@@ -303,12 +468,14 @@ protected:
 
 protected:
     long m_fromID;
-    qreal m_newZ;
-    qreal m_oldZ;
+    double m_newZ;
+    double m_oldZ;
 	ViewLayer::ViewLayerID m_newLayer;
     ViewLayer::ViewLayerID m_oldLayer;
 };
 
+/////////////////////////////////////////////
+
 class SelectItemCommand : public BaseCommand
 {
 public:
@@ -330,6 +497,7 @@ public:
     int id() const;
 	bool mergeWith(const QUndoCommand *other);
 	void copyUndo(SelectItemCommand * sother);
+	void copyRedo(SelectItemCommand * sother);
 	void setSelectItemType(SelectItemType);
 	bool updated();
 	void setUpdated(bool);
@@ -348,6 +516,7 @@ protected:
     static int selectItemCommandID;
 };
 
+/////////////////////////////////////////////
 
 class ChangeZCommand : public BaseCommand
 {
@@ -355,7 +524,7 @@ public:
     ChangeZCommand(class SketchWidget *sketchWidget, QUndoCommand *parent);
 	~ChangeZCommand();
 
-    void addTriplet(long id, qreal oldZ, qreal newZ);
+    void addTriplet(long id, double oldZ, double newZ);
     void undo();
     void redo();
 
@@ -363,8 +532,8 @@ protected:
 	QString getParamString() const;
 
 protected:
-    static qreal first(RealPair *);
-    static qreal second(RealPair *);
+    static double first(RealPair *);
+    static double second(RealPair *);
 
 protected:
     QHash<long, RealPair *> m_triplets;
@@ -377,6 +546,8 @@ struct StickyThing {
 	long toID;
 };
 
+/////////////////////////////////////////////
+
 class CheckStickyCommand : public BaseCommand
 {
 public:
@@ -406,6 +577,7 @@ protected:
 	CheckType m_checkType;
 };
 
+/////////////////////////////////////////////
 
 class WireColorChangeCommand : public BaseCommand
 {
@@ -413,7 +585,7 @@ public:
 	WireColorChangeCommand(
 		SketchWidget* sketchWidget,
 		long wireId, const QString &oldColor, const QString &newColor,
-		qreal oldOpacity, qreal newOpacity,
+		double oldOpacity, double newOpacity,
 		QUndoCommand *parent=0);
     void undo();
     void redo();
@@ -426,17 +598,18 @@ protected:
 	long m_wireId;
 	QString m_oldColor;
 	QString m_newColor;
-	qreal m_oldOpacity;
-	qreal m_newOpacity;
+	double m_oldOpacity;
+	double m_newOpacity;
 };
 
+/////////////////////////////////////////////
 
 class WireWidthChangeCommand : public BaseCommand
 {
 public:
 	WireWidthChangeCommand(
 		SketchWidget* sketchWidget,
-		long wireId, qreal oldWidth, qreal newWidth,
+		long wireId, double oldWidth, double newWidth,
 		QUndoCommand *parent);
     void undo();
     void redo();
@@ -446,10 +619,12 @@ protected:
 
 protected:
 	long m_wireId;
-	qreal m_oldWidth;
-	qreal m_newWidth;
+	double m_oldWidth;
+	double m_newWidth;
 };
 
+/////////////////////////////////////////////
+
 class RoutingStatusCommand : public BaseCommand 
 {
 public:
@@ -465,6 +640,8 @@ protected:
 	RoutingStatus m_newRoutingStatus;
 };
 
+/////////////////////////////////////////////
+
 struct RatsnestConnectThing
 {
 	long id;
@@ -501,6 +678,8 @@ protected:
 	Direction m_direction;
 };
 
+/////////////////////////////////////////////
+
 class RestoreLabelCommand : public BaseCommand
 {
 public:
@@ -516,6 +695,8 @@ protected:
     QDomElement m_element;
 };
 
+/////////////////////////////////////////////
+
 class ShowLabelFirstTimeCommand : public BaseCommand
 {
 public:
@@ -532,6 +713,8 @@ protected:
     bool m_newVis;
 };
 
+/////////////////////////////////////////////
+
 class MoveLabelCommand : public BaseCommand
 {
 public:
@@ -550,6 +733,26 @@ protected:
     QPointF m_newOffset;
 };
 
+/////////////////////////////////////////////
+
+class MoveLockCommand : public BaseCommand
+{
+public:
+    MoveLockCommand(class SketchWidget *sketchWidget, long id, bool oldLock, bool newLock, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_itemID;
+    bool m_oldLock;
+    bool m_newLock;
+};
+
+/////////////////////////////////////////////
+
 class IncLabelTextCommand : public BaseCommand
 {
 public:
@@ -565,6 +768,7 @@ protected:
 	bool m_firstTime;
 };
 
+/////////////////////////////////////////////
 
 class ChangeLabelTextCommand : public BaseCommand
 {
@@ -582,6 +786,8 @@ protected:
     QString m_newText;
 };
 
+/////////////////////////////////////////////
+
 class ChangeNoteTextCommand : public BaseCommand
 {
 public:
@@ -606,10 +812,12 @@ protected:
 	static int changeNoteTextCommandID;
 };
 
+/////////////////////////////////////////////
+
 class RotateFlipLabelCommand : public BaseCommand
 {
 public:
-	RotateFlipLabelCommand(class SketchWidget *sketchWidget, long id, qreal degrees, Qt::Orientations, QUndoCommand *parent);
+	RotateFlipLabelCommand(class SketchWidget *sketchWidget, long id, double degrees, Qt::Orientations, QUndoCommand *parent);
     void undo();
     void redo();
 
@@ -618,10 +826,12 @@ protected:
 
 protected:
     long m_itemID;
-    qreal m_degrees;
+    double m_degrees;
 	Qt::Orientations m_orientation;
 };
 
+/////////////////////////////////////////////
+
 class ResizeNoteCommand : public BaseCommand
 {
 public:
@@ -638,10 +848,12 @@ protected:
 	QSizeF m_newSize;
 };
 
+/////////////////////////////////////////////
+
 class ResizeBoardCommand : public BaseCommand
 {
 public:
-	ResizeBoardCommand(class SketchWidget *, long itemID, qreal oldWidth, qreal oldHeight, qreal newWidth, qreal newHeight, QUndoCommand * parent);
+	ResizeBoardCommand(class SketchWidget *, long itemID, double oldWidth, double oldHeight, double newWidth, double newHeight, QUndoCommand * parent);
 	void undo();
 	void redo();
 
@@ -649,27 +861,14 @@ protected:
 	QString getParamString() const;
 
 protected:
-	qreal m_oldWidth;
-	qreal m_oldHeight;
-	qreal m_newWidth;
-	qreal m_newHeight;
+	double m_oldWidth;
+	double m_oldHeight;
+	double m_newWidth;
+	double m_newHeight;
 	long m_itemID;
 };
 
-class SketchBackgroundColorChangeCommand : public BaseCommand
-{
-public:
-	SketchBackgroundColorChangeCommand(SketchWidget* sketchWidget, const QString &oldColor, const QString &newColor, QUndoCommand *parent=0);
-    void undo();
-    void redo();
-
-protected:
-	QString getParamString() const;
-
-protected:
-	QString m_oldColor;
-	QString m_newColor;
-};
+/////////////////////////////////////////////
 
 class SetResistanceCommand : public BaseCommand
 {
@@ -689,6 +888,8 @@ protected:
 	long m_itemID;
 };
 
+/////////////////////////////////////////////
+
 class SetPropCommand : public BaseCommand
 {
 public:
@@ -707,6 +908,8 @@ protected:
 	long m_itemID;
 };
 
+/////////////////////////////////////////////
+
 class ResizeJumperItemCommand : public BaseCommand
 {
 public:
@@ -727,6 +930,8 @@ protected:
 	long m_itemID;
 };
 
+/////////////////////////////////////////////
+
 class ShowLabelCommand : public BaseCommand
 {
 public:
@@ -744,6 +949,7 @@ protected:
 
 };
 
+/////////////////////////////////////////////
 
 class LoadLogoImageCommand : public BaseCommand
 {
@@ -764,6 +970,7 @@ protected:
 	bool m_addName;
 };
 
+/////////////////////////////////////////////
 
 class ChangeBoardLayersCommand : public BaseCommand
 {
@@ -780,6 +987,7 @@ protected:
     int m_newLayers;
 };
 
+/////////////////////////////////////////////
 
 class SetDropOffsetCommand : public BaseCommand
 {
@@ -796,5 +1004,84 @@ protected:
 	QPointF m_dropOffset;
 };
 
+/////////////////////////////////////////////
+
+class RenamePinsCommand : public BaseCommand
+{
+public:
+    RenamePinsCommand(class SketchWidget *sketchWidget, long id, const QStringList & oldOnes, const QStringList & newOnes, bool singleRow, QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_itemID;
+	QStringList m_oldLabels;
+	QStringList m_newLabels;
+	bool m_singleRow;
+};
+
+/////////////////////////////////////////////
+
+struct GFSThing {
+	long id;
+	QString connectorID;
+	bool seed;
+};
+
+class GroundFillSeedCommand : public BaseCommand
+{
+public:
+    GroundFillSeedCommand(class SketchWidget *sketchWidget, QUndoCommand *parent);
+    void undo();
+    void redo();
+	void addItem(long id, const QString & connectorID, bool seed);
+
+protected:
+	QString getParamString() const;
+
+protected:
+	QList<GFSThing> m_items;
+};
+
+/////////////////////////////////////////////
+
+class WireExtrasCommand : public BaseCommand
+{
+public:
+    WireExtrasCommand(class SketchWidget *sketchWidget, long fromID,
+    					const QDomElement & oldExtras, const QDomElement & newExtras,
+    					QUndoCommand *parent);
+    void undo();
+    void redo();
+
+protected:
+	QString getParamString() const;
+
+protected:
+    long m_fromID;
+	QDomElement m_oldExtras;
+	QDomElement m_newExtras;
+
+};
+
+/////////////////////////////////////////////
+
+class TemporaryCommand : public QUndoCommand
+{
+
+public:
+	TemporaryCommand(const QString & text);
+	~TemporaryCommand();
+
+    void setEnabled(bool);
+
+    void redo();
+
+protected:
+    bool m_enabled;
+};
 
 #endif // COMMANDS_H
