@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5175 $:
+$Revision: 6140 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-04 23:05:58 +0200 (Mon, 04 Jul 2011) $
+$Date: 2012-07-04 15:38:22 +0200 (Wed, 04 Jul 2012) $
 
 ********************************************************************/
 
@@ -28,7 +28,6 @@ $Date: 2011-07-04 23:05:58 +0200 (Mon, 04 Jul 2011) $
 #include <QHBoxLayout>
 #include <QSettings>
 #include <QPalette>
-#include <QFormLayout>
 #include <QFontMetricsF>
 #include <qmath.h>
 
@@ -51,8 +50,6 @@ const int HtmlInfoView::STANDARD_ICON_IMG_WIDTH = 32;
 const int HtmlInfoView::STANDARD_ICON_IMG_HEIGHT = 32;
 const int IconSpace = 0;
 
-QHash<QString, QPixmap *> HtmlInfoView::m_pixmaps;
-
 /////////////////////////////////////
 
 QLabel * addLabel(QHBoxLayout * hboxLayout, QPixmap * pixmap) {
@@ -80,7 +77,7 @@ QSize TagLabel::sizeHint() const
 	if (t.isEmpty()) return hint;
 
 	QFontMetricsF fm(this->font());
-	qreal textWidth = fm.width(t);
+	double textWidth = fm.width(t);
 
 	QWidget * w = this->window();
 	QPoint pos(0,0);
@@ -109,13 +106,15 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_lastTitleItemBase = NULL;
 	m_lastTagsModelPart = NULL;
 	m_lastConnectorItem = NULL;
-	m_lastIconModelPart = NULL;
+	m_lastIconItemBase = NULL;
 	m_lastPropsModelPart = NULL;
 	m_lastPropsItemBase = NULL;
 
 	m_partTitle = NULL;
 	m_partUrl = NULL;
 	m_partVersion = NULL;
+	m_lockCheckbox = NULL;
+	m_stickyCheckbox = NULL;
 	m_connDescr = NULL;
 	m_tagsTextLabel = NULL;
 	m_lastSwappingEnabled = false;
@@ -133,6 +132,8 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 
 	m_titleEdit = new FLineEdit(mainFrame);
 	m_titleEdit->setObjectName("instanceTitleEditor");
+	m_titleEdit->setToolTip(tr("Change the part label here"));
+	m_titleEdit->setAlignment(Qt::AlignLeft);
 
 	connect(m_titleEdit, SIGNAL(editingFinished()), this, SLOT(setInstanceTitle()));
 	connect(m_titleEdit, SIGNAL(mouseEnter()), this, SLOT(instanceTitleEnter()));
@@ -156,32 +157,43 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	hboxLayout->setContentsMargins(0, 0, 0, 0);
 	hboxLayout->addSpacing(IconSpace);
 	m_icon1 = addLabel(hboxLayout, NoIcon);
+	m_icon1->setToolTip(tr("Part breadboard view image"));
 	m_icon2 = addLabel(hboxLayout, NoIcon);
+	m_icon2->setToolTip(tr("Part schematic view image"));
 	m_icon3 = addLabel(hboxLayout, NoIcon);
+	m_icon3->setToolTip(tr("Part pcb view image"));
 
+	QVBoxLayout * versionLayout = new QVBoxLayout();
+
+	QHBoxLayout * subVersionLayout = new QHBoxLayout();
 	m_partVersion = new QLabel();
-	m_partVersion->setObjectName("infoViewPartTitle");
-	hboxLayout->addWidget(m_partVersion);
+	m_partVersion->setObjectName("infoViewPartVersion");
+	m_partVersion->setToolTip(tr("Part version number"));
+    m_partVersion->setOpenExternalLinks(false);
+    m_partVersion->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    connect(m_partVersion, SIGNAL(linkActivated(const QString &)), this, SLOT(clickObsolete(const QString &)));
+	subVersionLayout->addWidget(m_partVersion, 0, Qt::AlignLeft);
+	subVersionLayout->addStretch(1);
+	versionLayout->addLayout(subVersionLayout);
+
+    QHBoxLayout * cbLayout = new QHBoxLayout();
 	
-	m_locationUnits = NULL;
-	m_location = NULL;
+	m_lockCheckbox = new QCheckBox(tr("Locked"));
+	m_lockCheckbox->setObjectName("infoViewLockCheckbox");
+	m_lockCheckbox->setToolTip(tr("Change the locked state of the part in this view. A locked part can't be moved."));
+	connect(m_lockCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeLock(bool)));
+	cbLayout->addWidget(m_lockCheckbox);
 
-	/*
-	hboxLayout->addSpacing(20);
+	m_stickyCheckbox = new QCheckBox(tr("Sticky"));
+	m_stickyCheckbox->setObjectName("infoViewLockCheckbox");
+	m_stickyCheckbox->setToolTip(tr("Change the \"sticky\" state of the part in this view. When a sticky part is moved, objects on top of it also move."));
+	connect(m_stickyCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeSticky(bool)));
+	cbLayout->addWidget(m_stickyCheckbox);
+
+    versionLayout->addLayout(cbLayout);
+
+	hboxLayout->addLayout(versionLayout);
 	
-	m_locationUnits = new QComboBox();
-	m_location = new QLabel();
-
-	m_locationUnits->addItem("in");
-	m_locationUnits->addItem("mm");
-	m_locationUnits->setCurrentIndex(0);
-	m_locationUnits->setVisible(false);
-	hboxLayout->addWidget(m_locationUnits);
-
-	hboxLayout->addWidget(m_location);
-	connect(m_locationUnits, SIGNAL(currentIndexChanged(int)), this, SLOT(updateLocation()));
-	*/
-
 	hboxLayout->addSpacerItem(new QSpacerItem(IconSpace, 1, QSizePolicy::Expanding));
 	iconFrame->setLayout(hboxLayout);
 	vlo->addWidget(iconFrame);
@@ -270,11 +282,6 @@ HtmlInfoView::~HtmlInfoView() {
 		delete propThing;
 	}
 	m_propThings.clear();
-
-	foreach (QPixmap * pixmap, m_pixmaps.values()) {
-		delete pixmap;
-	}
-	m_pixmaps.clear();
 }
 
 void HtmlInfoView::cleanup() {
@@ -311,7 +318,7 @@ void HtmlInfoView::hoverLeaveItem(InfoGraphicsView * infoGraphicsView, QGraphics
 	m_setContentTimer.start();
 }
 
-void HtmlInfoView::viewConnectorItemInfo(ConnectorItem * connectorItem) {
+void HtmlInfoView::viewConnectorItemInfo(QGraphicsSceneHoverEvent *, ConnectorItem * connectorItem) {
 
 	int count = connectorItem ? connectorItem->connectionsCount() : 0;
 	if (m_lastConnectorItem == connectorItem && m_lastConnectorItemCount == count) return;
@@ -320,20 +327,17 @@ void HtmlInfoView::viewConnectorItemInfo(ConnectorItem * connectorItem) {
 	m_lastConnectorItemCount = count;
 
 	Connector * connector = NULL;
-	ConnectorShared * connectorShared = NULL;
 	if (connectorItem) {
 		if (connectorItem->attachedTo() != m_lastItemBase) {
 			return;
 		}
 
 		connector = connectorItem->connector();
-		connectorShared = connector->connectorShared();
-	
 	}
 
 	if (m_connDescr) {
 		m_connDescr->setText(connectorItem ? tr("connected to %n item(s)", "", connectorItem->connectionsCount()) : "");
-		m_connName->setText(connectorShared ? connectorShared->name() : "");
+		m_connName->setText(connector ? connector->connectorSharedName() : "");
 		m_connType->setText(connector ? Connector::connectorNameFromType(connector->connectorType()) : "");
 	}
 
@@ -343,18 +347,18 @@ void HtmlInfoView::hoverEnterConnectorItem(InfoGraphicsView *igv, QGraphicsScene
 	Q_UNUSED(event)
 	Q_UNUSED(swappingEnabled)
 	Q_UNUSED(igv)
-	viewConnectorItemInfo(item);
+	viewConnectorItemInfo(event, item);
 }
 
 void HtmlInfoView::hoverLeaveConnectorItem(InfoGraphicsView *igv, QGraphicsSceneHoverEvent *event, ConnectorItem *connItem) {
 	Q_UNUSED(event);
 	Q_UNUSED(connItem);
 	Q_UNUSED(igv);
-	viewConnectorItemInfo(NULL);
+	viewConnectorItemInfo(event, NULL);
 }
 
 void HtmlInfoView::appendStuff(ItemBase* item, bool swappingEnabled) {
-	Wire *wire = dynamic_cast<Wire*>(item);
+	Wire *wire = qobject_cast<Wire*>(item);
 	if (wire) {
 		appendWireStuff(wire, swappingEnabled);
 	} else {
@@ -373,7 +377,7 @@ void HtmlInfoView::appendWireStuff(Wire* wire, bool swappingEnabled) {
 	QString nameString = tr("Wire");
 	if (swappingEnabled) {
 		if (wire->getRatsnest()) {
-			nameString = tr("Rat's nest wire");
+			nameString = tr("Ratsnest wire");
 		} 
 		else if(wire->getTrace()) {
 			nameString = tr("Trace wire %1").arg(autoroutable);
@@ -382,11 +386,12 @@ void HtmlInfoView::appendWireStuff(Wire* wire, bool swappingEnabled) {
 	else {
 		 nameString = modelPart->description();
 	}
-	partTitle(nameString, modelPart->version(), modelPart->url());
+	partTitle(nameString, modelPart->version(), modelPart->url(), modelPart->isObsolete());
+	m_lockCheckbox->setVisible(false);
+	m_stickyCheckbox->setVisible(false);
 
 	setUpTitle(wire);
-	setUpIcons(wire->modelPart());
-	setUpLocation(swappingEnabled ? wire : NULL);
+	setUpIcons(wire, swappingEnabled);
 
 	displayProps(modelPart, wire, swappingEnabled);
 	addTags(modelPart);
@@ -405,9 +410,7 @@ void HtmlInfoView::appendItemStuff(ItemBase * itemBase, ModelPart * modelPart, b
 	if (modelPart->modelPartShared() == NULL) return;
 
 	setUpTitle(itemBase);
-	setUpIcons(modelPart);
-
-	setUpLocation(swappingEnabled ? itemBase : NULL);
+	setUpIcons(itemBase, swappingEnabled);
 
 	QString nameString;
 	if (swappingEnabled) {
@@ -416,7 +419,16 @@ void HtmlInfoView::appendItemStuff(ItemBase * itemBase, ModelPart * modelPart, b
 	else {
 		nameString = modelPart->description();
 	}
-	partTitle(nameString, modelPart->version(), modelPart->url());
+	partTitle(nameString, modelPart->version(), modelPart->url(), modelPart->isObsolete());
+	m_lockCheckbox->setVisible(true);
+	m_lockCheckbox->setChecked(itemBase->moveLock());
+    if (itemBase->isBaseSticky()) {
+	    m_stickyCheckbox->setVisible(true);
+        m_stickyCheckbox->setChecked(itemBase->isLocalSticky());
+    }
+    else {
+	    m_stickyCheckbox->setVisible(false);
+    }
 
 	displayProps(modelPart, itemBase, swappingEnabled);
 	addTags(modelPart);
@@ -487,12 +499,13 @@ void HtmlInfoView::reloadContent(InfoGraphicsView * infoGraphicsView) {
 void HtmlInfoView::setNullContent()
 {
 	setUpTitle(NULL);
-	partTitle("", "", "");
-	setUpIcons(NULL);
+	partTitle("", "", "", false);
+	m_lockCheckbox->setVisible(false);
+	m_stickyCheckbox->setVisible(false);
+	setUpIcons(NULL, false);
 	displayProps(NULL, NULL, false);
 	addTags(NULL);
-	viewConnectorItemInfo(NULL);
-	setUpLocation(NULL);
+	viewConnectorItemInfo(NULL, NULL);
 	m_connFrame->setVisible(false);
 	m_propFrame->setVisible(false);
 	m_proplabel->setVisible(false);
@@ -501,32 +514,32 @@ void HtmlInfoView::setNullContent()
 }
 
 void HtmlInfoView::setInstanceTitle() {
-	FLineEdit * edit = dynamic_cast<FLineEdit *>(sender());
+	FLineEdit * edit = qobject_cast<FLineEdit *>(sender());
 	if (edit == NULL) return;
 	if (!edit->isEnabled()) return;
 	if (m_infoGraphicsView == NULL) return;
 	if (m_currentItem == NULL) return;
 
 	DebugDialog::debug(QString("set instance title to %1").arg(edit->text()));
-	m_infoGraphicsView->setInstanceTitle(m_currentItem->id(), edit->text(), true, false);
+	m_infoGraphicsView->setInstanceTitle(m_currentItem->id(), m_partTitle->text(), edit->text(), true, false);
 }
 
 void HtmlInfoView::instanceTitleEnter() {
-	FLineEdit * edit = dynamic_cast<FLineEdit *>(sender());
+	FLineEdit * edit = qobject_cast<FLineEdit *>(sender());
 	if (edit->isEnabled()) {
 		setInstanceTitleColors(edit, QColor(0xeb, 0xeb, 0xee), QColor(0x00, 0x00, 0x00)); //c8c8c8, 575757
 	}
 }
 
 void HtmlInfoView::instanceTitleLeave() {
-	FLineEdit * edit = dynamic_cast<FLineEdit *>(sender());
+	FLineEdit * edit = qobject_cast<FLineEdit *>(sender());
 	if (edit->isEnabled()) {
 		setInstanceTitleColors(edit, QColor(0xd2, 0xd2, 0xd7), QColor(0x00, 0x00, 0x00)); //b3b3b3, 575757
 	}
 }
 
 void HtmlInfoView::instanceTitleEditable(bool editable) {
-	FLineEdit * edit = dynamic_cast<FLineEdit *>(sender());
+	FLineEdit * edit = qobject_cast<FLineEdit *>(sender());
 	if (editable) {
 		setInstanceTitleColors(edit, QColor(0xff, 0xff, 0xff), QColor(0x00, 0x00, 0x00)); //fcfcfc, 000000
 	}
@@ -543,7 +556,10 @@ void HtmlInfoView::setInstanceTitleColors(FLineEdit * edit, const QColor & base,
 
 void HtmlInfoView::setUpTitle(ItemBase * itemBase) 
 {
-	if (itemBase == m_lastTitleItemBase) return;
+	if (itemBase == m_lastTitleItemBase) {
+        if (itemBase == NULL) return;
+        if (itemBase->instanceTitle().compare(m_titleEdit->text()) == 0) return;
+    }
 
 	m_lastTitleItemBase = itemBase;
 	if (itemBase) {
@@ -562,52 +578,30 @@ void HtmlInfoView::setUpTitle(ItemBase * itemBase)
 		m_titleEdit->setEnabled(false);
 		m_titleEdit->setText("");
 	}
+	// helps keep it left aligned?
+	m_titleEdit->setCursorPosition(0);
 
 }
 
-void HtmlInfoView::setUpIcons(ModelPart * modelPart) {
-	if (m_lastIconModelPart == modelPart) return;
+void HtmlInfoView::setUpIcons(ItemBase * itemBase, bool swappingEnabled) {
+	if (m_lastIconItemBase == itemBase) return;
 	
-	m_lastIconModelPart = modelPart;
+	m_lastIconItemBase = itemBase;
 
-	QPixmap *pixmap0 = NULL;
 	QPixmap *pixmap1 = NULL;
 	QPixmap *pixmap2 = NULL;
 	QPixmap *pixmap3 = NULL;
 
-	QSize size = NoIcon->size();
+    QSize size = NoIcon->size();
 
-	if (modelPart != NULL) {
-		pixmap0 = getPixmap(modelPart, ViewIdentifierClass::IconView);
-		pixmap1 = getPixmap(modelPart, ViewIdentifierClass::BreadboardView);
-		pixmap2 = getPixmap(modelPart, ViewIdentifierClass::SchematicView);
-		pixmap3 = getPixmap(modelPart, ViewIdentifierClass::PCBView);
-	}
+    if (itemBase) {
+        itemBase->getPixmaps(pixmap1, pixmap2, pixmap3, swappingEnabled, size);
+    }
 
 	QPixmap* use1 = pixmap1;
 	QPixmap* use2 = pixmap2;
 	QPixmap* use3 = pixmap3;
 
-	// use the icon image instead of the breadboard image, unless the item doesn't have a breadboard view
-	if (pixmap1 && pixmap2 && pixmap3) {
-		use1 = pixmap0;
-		if (pixmap2 == pixmap1) {
-			use2 = pixmap0;
-		}
-		if (pixmap3 == pixmap1) {
-			use3 = pixmap0;
-		}
-	}
-	else if (pixmap3) {
-		use3 = pixmap0;
-	}
-	else if (pixmap1) {
-		use1 = pixmap0;
-	}
-	else if (pixmap2) {
-		use2 = pixmap0;
-	}
-
 	if (use1 == NULL) use1 = NoIcon;
 	if (use2 == NULL) use2 = NoIcon;
 	if (use3 == NULL) use3 = NoIcon;
@@ -615,6 +609,10 @@ void HtmlInfoView::setUpIcons(ModelPart * modelPart) {
 	m_icon1->setPixmap(*use1);
 	m_icon2->setPixmap(*use2);
 	m_icon3->setPixmap(*use3);
+
+    if (pixmap1 != NULL) delete pixmap1;
+    if (pixmap2 != NULL) delete pixmap2;
+    if (pixmap3 != NULL) delete pixmap3;
 }
 
 void HtmlInfoView::addTags(ModelPart * modelPart) {
@@ -632,7 +630,7 @@ void HtmlInfoView::addTags(ModelPart * modelPart) {
 	m_tagsTextLabel->setText(modelPart->tags().join(", "));
 }
 
-void HtmlInfoView::partTitle(const QString & title, const QString & version, const QString & url) {
+void HtmlInfoView::partTitle(const QString & title, const QString & version, const QString & url, bool isObsolete) {
 	if (m_partTitle == NULL) return;
 
 	if (m_lastPartTitle == title && m_lastPartVersion == version) return;
@@ -651,7 +649,7 @@ void HtmlInfoView::partTitle(const QString & title, const QString & version, con
 
 	m_partTitle->setText(title);
 	if (!version.isEmpty()) {
-		m_partVersion->setText(tr("v. %1").arg(version));
+		m_partVersion->setText(tr("v. %1 %2").arg(version).arg(isObsolete ? QString("<a href='x'>%1</a>").arg(tr("obsolete")) : ""));
 	}
 	else m_partVersion->setText("");
 }
@@ -660,7 +658,7 @@ void HtmlInfoView::displayProps(ModelPart * modelPart, ItemBase * itemBase, bool
 {
 	bool repeatPossible = (modelPart == m_lastPropsModelPart && itemBase == m_lastPropsItemBase && swappingEnabled == m_lastPropsSwappingEnabled);
 	if (repeatPossible && modelPart == NULL && itemBase == NULL) {
-		DebugDialog::debug("display props bail");
+		//DebugDialog::debug("display props bail");
 		return;
 	}
 
@@ -805,90 +803,13 @@ void HtmlInfoView::clearPropThingPlugin(PropThing * propThing, QWidget * plugin)
 }
 
 
-QPixmap * HtmlInfoView::getPixmap(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier) {
-	if (!modelPart->hasViewFor(viewIdentifier)) return NULL;
-
-	QString baseName = modelPart->hasBaseNameFor(viewIdentifier);
-	if (baseName.isEmpty()) return NULL;
-
-	QString filename = ItemBase::getSvgFilename(modelPart, baseName);
-	if (filename.isEmpty()) {
-		return NULL;
-	}
-
-	QPixmap * cached = m_pixmaps.value(filename, NULL);
-	if (cached) {
-		return cached;
-	}
-
-	QSvgRenderer renderer(filename);
-
-	QSize size = NoIcon->size();
-	QPixmap * pixmap = new QPixmap(size);
-	pixmap->fill(Qt::transparent);
-	QPainter painter(pixmap);
-	// preserve aspect ratio
-	QSize def = renderer.defaultSize();
-	qreal newW = size.width();
-	qreal newH = newW * def.height() / def.width();
-	if (newH > size.height()) {
-		newH = size.height();
-		newW = newH * def.width() / def.height();
-	}
-	QRectF bounds((size.width() - newW) / 2.0, (size.height() - newH) / 2.0, newW, newH);
-	renderer.render(&painter, bounds);
-	painter.end();
-
-	m_pixmaps.insert(filename, pixmap);
-
-	return pixmap;
-}
-
-void HtmlInfoView::setUpLocation(ItemBase * itemBase) {
-	if (m_location == NULL) return;
-	if (m_locationUnits == NULL) return;
-
-	if (itemBase == NULL) {
-		m_location->setText("");
-		m_locationUnits->setVisible(false);
-		return;
-	}
-
-	m_locationUnits->setVisible(true);
-	QPointF p = itemBase->pos();
-	qreal x = p.x() / FSvgRenderer::printerScale();
-	qreal y = p.y() / FSvgRenderer::printerScale();
-
-	qreal units = (m_locationUnits->currentText().compare("in") == 0) ? 1.0 : 25.4;
-
-	if (itemBase->itemType() == ModelPart::Wire) {
-		Wire * wire = qobject_cast<Wire *>(itemBase);
-		QPointF q = p + wire->line().p2();
-		m_location->setText(tr("x:%1 %2\ny:%3 %4")
-			.arg(x * units, 0, 'f', 3)
-			.arg(q.x() * units / FSvgRenderer::printerScale(), 0, 'f', 3)
-			.arg(y * units, 0, 'f', 3)
-			.arg(q.y() * units / FSvgRenderer::printerScale(), 0, 'f', 3)
-		);
-		return;
-	}
-
-	m_location->setText(tr("x:%1\ny:%2")
-		.arg(x * units, 0, 'f', 3)
-		.arg(y * units, 0, 'f', 3)
-	);
-}
-
-void HtmlInfoView::updateLocation() {
-	setUpLocation(m_currentItem);
-}
-
 QHash<QString, QString> HtmlInfoView::getPartProperties(ModelPart * modelPart, ItemBase * itemBase, bool wantDebug, QStringList & keys) 
 {
 	QHash<QString, QString> properties;
 	QString family;
 	QString partNumber;
 	if (modelPart) {
+        // TODO: someday get local props
 		properties = modelPart->properties();
 		family = properties.value("family", "").toLower();
 		if (itemBase) {
@@ -924,11 +845,12 @@ QHash<QString, QString> HtmlInfoView::getPartProperties(ModelPart * modelPart, I
 				keys.insert(insertAt++, "class");
 			}
 			else {
-				FSvgRenderer * renderer = FSvgRenderer::getByModuleID(modelPart->moduleID(), ViewLayer::Icon);
-				if (renderer != NULL) {
-					properties.insert("svg", renderer->filename());
-					keys.insert(insertAt++, "svg");
-				}
+                DebugDialog::debug("htmlinfoview has modelpart without itembase?");
+				//FSvgRenderer * renderer = FSvgRenderer::getByModuleID(modelPart->moduleID(), ViewLayer::Icon);
+				//if (renderer != NULL) {
+				//	properties.insert("svg", renderer->filename());
+				//	keys.insert(insertAt++, "svg");
+				//}
 			}
 			if (modelPart->modelPartShared()) {
 				properties.insert("fzp",  modelPart->path());
@@ -944,3 +866,24 @@ QHash<QString, QString> HtmlInfoView::getPartProperties(ModelPart * modelPart, I
 
 	return properties;
 }
+
+void HtmlInfoView::changeLock(bool lockState)
+{
+	if (m_currentItem == NULL) return;
+	if (m_currentItem->itemType() == ModelPart::Wire) return;
+
+	m_currentItem->setMoveLock(lockState);
+}
+
+
+void HtmlInfoView::changeSticky(bool lockState)
+{
+	if (m_currentItem == NULL) return;
+	if (m_currentItem->itemType() == ModelPart::Wire) return;
+
+	m_currentItem->setLocalSticky(lockState);
+}
+
+void HtmlInfoView::clickObsolete(const QString &) {
+    emit clickObsoleteSignal();
+}
