@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,20 +18,24 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5162 $:
+$Revision: 6269 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-02 22:39:11 +0200 (Sat, 02 Jul 2011) $
+$Date: 2012-08-09 11:22:00 +0200 (Thu, 09 Aug 2012) $
 
 ********************************************************************/
 
 #include "mysterypart.h"
 #include "../utils/graphicsutils.h"
+#include "../utils/familypropertycombobox.h"
 #include "../fsvgrenderer.h"
 #include "../sketch/infographicsview.h"
 #include "../commands.h"
 #include "../utils/textutils.h"
 #include "../layerattributes.h"
 #include "partlabel.h"
+#include "pinheader.h"
+#include "../connectors/connectoritem.h"
+
 
 #include <QDomNodeList>
 #include <QDomDocument>
@@ -46,7 +50,8 @@ static const int MinSipPins = 1;
 static const int MaxSipPins = 64;
 static const int MinDipPins = 4;
 static const int MaxDipPins = 64;
-int MysteryPart::NoExcusePins = 0;
+
+static HoleClassThing TheHoleThing;
 
 // TODO
 //	save into parts bin
@@ -54,19 +59,14 @@ int MysteryPart::NoExcusePins = 0;
 MysteryPart::MysteryPart( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
 	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
 {
-	m_changingSpacing = false;
-	m_chipLabel = modelPart->prop("chip label").toString();
+	m_chipLabel = modelPart->localProp("chip label").toString();
 	if (m_chipLabel.isEmpty()) {
 		m_chipLabel = modelPart->properties().value("chip label", "?");
-		modelPart->setProp("chip label", m_chipLabel);
-	}
-	m_spacing = modelPart->prop("spacing").toString();
-	if (m_spacing.isEmpty()) {
-		m_spacing = modelPart->properties().value("spacing", "300mil");
-		modelPart->setProp("spacing", m_spacing);
+		modelPart->setLocalProp("chip label", m_chipLabel);
 	}
 
-	m_renderer = NULL;
+    setUpHoleSizes("mystery", TheHoleThing);
+
 }
 
 MysteryPart::~MysteryPart() {
@@ -78,106 +78,50 @@ void MysteryPart::setProp(const QString & prop, const QString & value) {
 		return;
 	}
 
-	if (prop.compare("spacing", Qt::CaseInsensitive) == 0) {
-		setSpacing(value, false);
-		return;
-	}
-
 	PaletteItem::setProp(prop, value);
 }
 
-void MysteryPart::setSpacing(QString spacing, bool force) {
-	if (!force && m_spacing.compare(spacing) == 0) return;
-	if (!isDIP()) return;
-
-	switch (this->m_viewIdentifier) {
-		case ViewIdentifierClass::BreadboardView:
-		case ViewIdentifierClass::PCBView:
-			{
-				InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-				if (infoGraphicsView == NULL) break;
-
-				// hack the dom element and call setUpImage
-				FSvgRenderer::removeFromHash(moduleID(), "");
-				QDomElement element = LayerAttributes::getSvgElementLayers(modelPart()->domDocument(), m_viewIdentifier);
-				if (element.isNull()) break;
-
-				QString filename = element.attribute("image");
-				if (filename.isEmpty()) break;
-
-				if (spacing.indexOf(Digits) < 0) break;
-
-				QString newSpacing = Digits.cap(0);		
-				filename.replace(DigitsMil, newSpacing + "mil");
-				element.setAttribute("image", filename);
-
-				m_changingSpacing = true;
-				resetImage(infoGraphicsView);
-				m_changingSpacing = false;
-
-				if (m_viewIdentifier == ViewIdentifierClass::BreadboardView) {
-					if (modelPart()->properties().value("chip label", "").compare(m_chipLabel) != 0) {
-						setChipLabel(m_chipLabel, true);
-					}
-				}
-
-				updateConnections();
-			}
-			break;
-		default:
-			break;
-	}
-
-	m_spacing = spacing;
-	modelPart()->setProp("spacing", spacing);
-
-	updateTooltip();
-    if (m_partLabel) m_partLabel->displayTextsIf();
-
-}
-
 void MysteryPart::setChipLabel(QString chipLabel, bool force) {
 
 	if (!force && m_chipLabel.compare(chipLabel) == 0) return;
 
+	m_chipLabel = chipLabel;
+
 	QString svg;
 	switch (this->m_viewIdentifier) {
 		case ViewIdentifierClass::BreadboardView:
+			svg = makeSvg(chipLabel, true);
+			break;
 		case ViewIdentifierClass::SchematicView:
-			svg = makeSvg(chipLabel);
+			svg = makeSvg(chipLabel, false);
+			svg = retrieveSchematicSvg(svg);
 			break;
 		default:
 			break;
 	}
 
-	if (!svg.isEmpty()) {
-		if (m_renderer == NULL) {
-			m_renderer = new FSvgRenderer(this);
-		}
-		//DebugDialog::debug(svg);
+    if (!svg.isEmpty()) {
+	    resetRenderer(svg);
+    }
 
-		bool result = m_renderer->fastLoad(svg.toUtf8());
-		if (result) {
-			setSharedRendererEx(m_renderer);
-		}
-	}
+	modelPart()->setLocalProp("chip label", chipLabel);
 
-	m_chipLabel = chipLabel;
-	modelPart()->setProp("chip label", chipLabel);
-
-	updateTooltip();
     if (m_partLabel) m_partLabel->displayTextsIf();
 
 }
 
-QString MysteryPart::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, qreal dpi) 
+QString MysteryPart::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
 {
 	QString svg = PaletteItem::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
 	switch (viewLayerID) {
 		case ViewLayer::Breadboard:
-		case ViewLayer::Schematic:
 		case ViewLayer::Icon:
 			return TextUtils::replaceTextElement(svg, "label", m_chipLabel);
+
+		case ViewLayer::Schematic:
+			svg = retrieveSchematicSvg(svg);
+			return TextUtils::removeSVGHeader(svg);
+
 		default:
 			break;
 	}
@@ -185,13 +129,28 @@ QString MysteryPart::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStri
 	return svg; 
 }
 
-QString MysteryPart::makeSvg(const QString & chipLabel) {
+QString MysteryPart::retrieveSchematicSvg(QString & svg) {
+
+	bool hasLocal = false;
+	QStringList labels = getPinLabels(hasLocal);
+		
+	if (hasLocal) {
+		svg = makeSchematicSvg(labels, false);
+	}
+
+	return TextUtils::replaceTextElement(svg, "label", m_chipLabel);
+}
+
+
+QString MysteryPart::makeSvg(const QString & chipLabel, bool replace) {
 	QString path = filename();
 	QFile file(filename());
 	QString svg;
 	if (file.open(QFile::ReadOnly)) {
 		svg = file.readAll();
 		file.close();
+		if (!replace) return svg;
+
 		return TextUtils::replaceTextElement(svg, "label", chipLabel);
 	}
 
@@ -199,18 +158,28 @@ QString MysteryPart::makeSvg(const QString & chipLabel) {
 }
 
 QStringList MysteryPart::collectValues(const QString & family, const QString & prop, QString & value) {
-	if (prop.compare("spacing", Qt::CaseInsensitive) == 0) {
+	if (prop.compare("layout", Qt::CaseInsensitive) == 0) {
+        // TODO: translate these
+        QStringList values;
+        values << "Single Row" << "Double Row";
+        value = values.at(moduleID().contains("sip", Qt::CaseInsensitive) ? 0 : 1);
+        return values;
+    }
+
+	if (prop.compare("pin spacing", Qt::CaseInsensitive) == 0) {
 		QStringList values;
+        QString spacing;
+        TextUtils::getPinsAndSpacing(moduleID(), spacing);
 		if (isDIP()) {
 			foreach (QString f, spacings()) {
 				values.append(f);
 			}
 		}
 		else {
-			values.append(m_spacing);
+			values.append(spacing);
 		}
 
-		value = m_spacing;
+		value = spacing;
 		return values;
 	}
 
@@ -254,6 +223,10 @@ bool MysteryPart::collectExtraInfo(QWidget * parent, const QString & family, con
 		return true;
 	}
 
+    if (prop.compare("hole size", Qt::CaseInsensitive) == 0) {
+        return collectHoleSizeInfo(TheHoleThing.holeSizeValue, parent, swappingEnabled, returnProp, returnValue, returnWidget);
+	}
+
 	return PaletteItem::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
 }
 
@@ -262,10 +235,6 @@ QString MysteryPart::getProperty(const QString & key) {
 		return m_chipLabel;
 	}
 
-	if (key.compare("spacing", Qt::CaseInsensitive) == 0) {
-		return m_spacing;
-	}
-
 	return PaletteItem::getProperty(key);
 }
 
@@ -277,10 +246,9 @@ void MysteryPart::addedToScene(bool temporary)
 {
 	if (this->scene()) {
 		setChipLabel(m_chipLabel, true);
-		setSpacing(m_spacing, true);
 	}
 
-    return PaletteItem::addedToScene(temporary);
+    PaletteItem::addedToScene(temporary);
 }
 
 const QString & MysteryPart::title() {
@@ -300,7 +268,7 @@ bool MysteryPart::hasCustomSVG() {
 }
 
 void MysteryPart::chipLabelEntry() {
-	QLineEdit * edit = dynamic_cast<QLineEdit *>(sender());
+	QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
 	if (edit == NULL) return;
 
 	if (edit->text().compare(this->chipLabel()) == 0) return;
@@ -311,36 +279,6 @@ void MysteryPart::chipLabelEntry() {
 	}
 }
 
-ConnectorItem* MysteryPart::newConnectorItem(Connector *connector) {
-	if (m_changingSpacing) {
-		return connector->connectorItemByViewLayerID(viewIdentifier(), viewLayerID());
-	}
-
-	return PaletteItem::newConnectorItem(connector);
-}
-
-ConnectorItem* MysteryPart::newConnectorItem(ItemBase * layerKin, Connector *connector) {
-	if (m_changingSpacing) {
-		return connector->connectorItemByViewLayerID(viewIdentifier(), layerKin->viewLayerID());
-	}
-
-	return PaletteItem::newConnectorItem(layerKin, connector);
-}
-
-const QString & MysteryPart::spacing() {
-	return m_spacing;
-}
-
-bool MysteryPart::onlySpacingChanges(QMap<QString, QString> & propsMap) {
-	if (propsMap.value("spacing", "").compare(m_spacing) == 0) return false;
-
-	if (modelPart()->properties().value("pins", "").compare(propsMap.value("pins", "")) != 0) return false;
-
-	if (otherPropsChange(propsMap)) return false;
-
-	return true;
-}
-
 bool MysteryPart::isDIP() {
 	QString layout = modelPart()->properties().value("layout", "");
 	return (layout.indexOf("double", 0, Qt::CaseInsensitive) >= 0);
@@ -364,26 +302,47 @@ ItemBase::PluralType MysteryPart::isPlural() {
 
 QString MysteryPart::genSipFZP(const QString & moduleid)
 {
-	return PaletteItem::genFZP(moduleid, "mystery_part_sipFzpTemplate", MinSipPins, MaxSipPins, 1);
+    return genxFZP(moduleid, "mystery_part_sipFzpTemplate", MinSipPins, MaxSipPins, 1);
 }
 
 QString MysteryPart::genDipFZP(const QString & moduleid)
 {
-	return PaletteItem::genFZP(moduleid, "mystery_part_dipFzpTemplate", MinDipPins, MaxDipPins, 2);
+    return genxFZP(moduleid, "mystery_part_dipFzpTemplate", MinDipPins, MaxDipPins, 2);
+
+}
+
+QString MysteryPart::genxFZP(const QString & moduleid, const QString & templateName, int minPins, int maxPins, int step) {
+    QString spacingString;
+    TextUtils::getPinsAndSpacing(moduleid, spacingString);
+    QString result = PaletteItem::genFZP(moduleid, templateName, minPins, maxPins, step, false);
+   	result.replace(".percent.", "%");
+	result = result.arg(spacingString);
+	return hackFzpHoleSize(result, moduleid);
+}
+
+QString MysteryPart::hackFzpHoleSize(const QString & fzp, const QString & moduleid) {
+    int hsix = moduleid.lastIndexOf(HoleSizePrefix);
+    if (hsix >= 0) {
+        return PaletteItem::hackFzpHoleSize(fzp, moduleid, hsix);
+    }
+
+    return fzp;
 }
 
 QString MysteryPart::genModuleID(QMap<QString, QString> & currPropsMap)
 {
 	QString value = currPropsMap.value("layout");
+    bool single = value.contains("single", Qt::CaseInsensitive);
 	QString pins = currPropsMap.value("pins");
-	if (value.contains("single", Qt::CaseInsensitive)) {
-		return QString("mystery_part_%1").arg(pins);
+    QString spacing = currPropsMap.value("pin spacing", "300mil");
+	if (single) {
+		return QString("mystery_part_sip_%1_100mil").arg(pins);
 	}
 	else {
 		int p = pins.toInt();
 		if (p < 4) p = 4;
 		if (p % 2 == 1) p--;
-		return QString("mystery_part_%1_dip_300mil").arg(p);
+		return QString("mystery_part_dip_%1_%2").arg(p).arg(spacing);
 	}
 }
 
@@ -392,34 +351,55 @@ QString MysteryPart::makeSchematicSvg(const QString & expectedFileName)
 	bool sip = expectedFileName.contains("sip", Qt::CaseInsensitive);
 
 	QStringList pieces = expectedFileName.split("_");
-	if (sip) {
-		if (pieces.count() != 5) return "";
-	}
-	else {
-		if (pieces.count() != 4) return "";
-	}
 
 	int pins = pieces.at(2).toInt();
-	int increment = 300;
-	qreal totalHeight = (pins * increment) + 330;
+	QStringList labels;
+	for (int i = 0; i < pins; i++) {
+		labels << QString::number(i + 1);
+	}
+
+	return makeSchematicSvg(labels, sip);
+}
+
+QString MysteryPart::makeSchematicSvg(const QStringList & labels, bool sip) 
+{	
+	int increment = GraphicsUtils::StandardSchematicSeparationMils;   // 7.5mm;
 	int border = 30;
+	double totalHeight = (labels.count() * increment) + increment + border;
 	int textOffset = 50;
 	int repeatTextOffset = 50;
 	int fontSize = 255;
+	int labelFontSize = 130;
+	int defaultLabelWidth = 30;
 	QString labelText = "?";
+	double textMax = defaultLabelWidth;
+	QFont font("Droid Sans", labelFontSize * 72 / GraphicsUtils::StandardFritzingDPI, QFont::Normal);
+	QFontMetricsF fm(font);
+	for (int i = 0; i < labels.count(); i++) {
+		double w = fm.width(labels.at(i));
+		if (w > textMax) textMax = w;
+	}
+	textMax = textMax * GraphicsUtils::StandardFritzingDPI / 72;
+
+	int totalWidth = (5 * increment) + border;
+	int innerWidth = 4 * increment;
+	if (textMax > defaultLabelWidth) {
+		totalWidth += (textMax - defaultLabelWidth);
+		innerWidth += (textMax - defaultLabelWidth);
+	}
 
 	QString header("<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n"
 					"<svg xmlns:svg='http://www.w3.org/2000/svg' \n"
 					"xmlns='http://www.w3.org/2000/svg' \n"
 					"version='1.2' baseProfile='tiny' \n"
-					"width='1.53in' height='%1in' viewBox='0 0 1530 %2'>\n"
+					"width='%7in' height='%1in' viewBox='0 0 %8 %2'>\n"
 					"<g id='schematic'>\n"
-					"<rect x='315' y='15' fill='none' width='1200' height='%3' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' />\n"
-					"<text id='label' x='915' y='%4' fill='#000000' stroke='none' font-family='DroidSans' text-anchor='middle' font-size='%5' >%6</text>\n");
+					"<rect x='315' y='15' fill='none' width='%9' height='%3' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' />\n"
+					"<text id='label' x='%11' y='%4' fill='#000000' stroke='none' font-family='DroidSans' text-anchor='middle' font-size='%5' >%6</text>\n");
 
 	if (!sip) {
-		header +=	"<circle fill='#000000' cx='1330' cy='200' r='150' stroke-width='0' />\n"
-					"<text x='1330' fill='#FFFFFF' y='305' font-family='DroidSans' text-anchor='middle' font-weight='bold' stroke-width='0' font-size='275' >?</text>\n";
+		header +=	"<circle fill='#000000' cx='%10' cy='200' r='150' stroke-width='0' />\n"
+					"<text x='%10' fill='#FFFFFF' y='305' font-family='DroidSans' text-anchor='middle' font-weight='bold' stroke-width='0' font-size='275' >?</text>\n";
 	}
 	else {
 		labelText = "IC";
@@ -428,26 +408,33 @@ QString MysteryPart::makeSchematicSvg(const QString & expectedFileName)
 	}
 
 	QString svg = header
-		.arg(totalHeight / 1000)
+		.arg(totalHeight / GraphicsUtils::StandardFritzingDPI)
 		.arg(totalHeight)
 		.arg(totalHeight - border)
 		.arg((totalHeight / 2) + textOffset)
 		.arg(fontSize)
-		.arg(labelText);
+		.arg(labelText)
+		.arg(totalWidth / 1000.0)
+		.arg(totalWidth)
+		.arg(innerWidth)
+		.arg(totalWidth - 200)
+		.arg(increment + textMax + ((totalWidth - increment - textMax) / 2.0))
+		;
 
 
 	QString repeat("<line fill='none' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' x1='15' y1='%1' x2='300' y2='%1'  />\n"
 					"<rect x='0' y='%2' fill='none' width='300' height='30' id='connector%3pin' stroke-width='0' />\n"
 					"<rect x='0' y='%2' fill='none' width='30' height='30' id='connector%3terminal' stroke-width='0' />\n"
-					"<text id='label%3' x='390' y='%4' font-family='DroidSans' stroke='none' fill='#000000' text-anchor='start' font-size='130' >%5</text>\n");
+					"<text id='label%3' x='390' y='%4' font-family='DroidSans' stroke='none' fill='#000000' text-anchor='start' font-size='%6' >%5</text>\n");
   
-	for (int i = 0; i < pins; i++) {
+	for (int i = 0; i < labels.count(); i++) {
 		svg += repeat
-			.arg(315 + (i * increment))
-			.arg(300 + (i * increment))
+			.arg(increment + (border / 2) + (i * increment))
+			.arg(increment + (i * increment))
 			.arg(i)
-			.arg(300 + repeatTextOffset + (i * increment))
-			.arg(i + 1);
+			.arg(increment + repeatTextOffset + (i * increment))
+			.arg(labels.at(i))
+			.arg(labelFontSize);
 	}
 
 	svg += "</g>\n";
@@ -465,12 +452,9 @@ QString MysteryPart::makeBreadboardSvg(const QString & expectedFileName)
 
 QString MysteryPart::makeBreadboardDipSvg(const QString & expectedFileName) 
 {
-	QStringList pieces = expectedFileName.split("_");
-	if (pieces.count() != 6) return "";
-
-	int pins = pieces.at(2).toInt();
-	qreal spacing = TextUtils::convertToInches(pieces.at(4)) * 100;
-
+    QString spacingString;
+    int pins = TextUtils::getPinsAndSpacing(expectedFileName, spacingString);
+	double spacing = TextUtils::convertToInches(spacingString) * 100;
 
 	int increment = 10;
 
@@ -495,7 +479,7 @@ QString MysteryPart::makeBreadboardDipSvg(const QString & expectedFileName)
 					"</svg>\n");
 
 
-	header = TextUtils::incrementTemplateString(header, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	header = TextUtils::incrementTemplateString(header, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 	header = header.arg(TextUtils::getViewBoxCoord(header, 3) / 100.0);
 	if (spacing == 10) {
 		header.replace("{{6.0}}", "8.0");
@@ -511,15 +495,17 @@ QString MysteryPart::makeBreadboardDipSvg(const QString & expectedFileName)
 	header.replace("{", "[");
 	header.replace("}", "]");
 
-	QString svg = TextUtils::incrementTemplateString(header, 1, increment * ((pins - 4) / 2), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	QString svg = TextUtils::incrementTemplateString(header, 1, increment * ((pins - 4) / 2), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 
-	repeatB = TextUtils::incrementTemplateString(repeatB, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	repeatB = TextUtils::incrementTemplateString(repeatB, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 	repeatB.replace("{", "[");
 	repeatB.replace("}", "]");
 
-	NoExcusePins = pins;
-	QString repeatTs = TextUtils::incrementTemplateString(repeatT, pins / 2, increment, TextUtils::standardMultiplyPinFunction, negCopyPinFunction);
-	QString repeatBs = TextUtils::incrementTemplateString(repeatB, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction);
+	int userData[2];
+	userData[0] = pins;
+	userData[1] = 1;
+	QString repeatTs = TextUtils::incrementTemplateString(repeatT, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::negIncCopyPinFunction, userData);
+	QString repeatBs = TextUtils::incrementTemplateString(repeatB, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction, NULL);
 
 	return svg.arg(TextUtils::getViewBoxCoord(svg, 2) / 100).arg(repeatTs).arg(repeatBs);
 }
@@ -545,23 +531,108 @@ QString MysteryPart::makeBreadboardSipSvg(const QString & expectedFileName)
 					"</g>\n"
 					"</svg>\n");
 
-	QString svg = TextUtils::incrementTemplateString(header, 1, increment * (pins - 1), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	QString svg = TextUtils::incrementTemplateString(header, 1, increment * (pins - 1), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 
 	QString repeat("<rect id='connector%1terminal' stroke='none' stroke-width='0' x='[1.87]' y='25.586' fill='#8C8C8C' width='2.3' height='2.0'/>\n"
 					"<rect id='connector%1pin' stroke='none' stroke-width='0' x='[1.87]' y='23.336' fill='#8C8C8C' width='2.3' height='4.25'/>\n");
 
-	QString repeats = TextUtils::incrementTemplateString(repeat, pins, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction);
+	QString repeats = TextUtils::incrementTemplateString(repeat, pins, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction, NULL);
 
 	return svg.arg(TextUtils::getViewBoxCoord(svg, 2) / 100).arg(repeats);
 }
 
-QString MysteryPart::negCopyPinFunction(int pin, const QString & argString) 
-{ 
-	// TODO: pass an argument to these copyPin functions
-	return argString.arg(NoExcusePins - (pin + 1)); 
+bool MysteryPart::changePinLabels(bool singleRow, bool sip) {
+	Q_UNUSED(singleRow);
+
+	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return true;
+
+	bool hasLocal = false;
+	QStringList labels = getPinLabels(hasLocal);
+	if (labels.count() == 0) return true;
+
+	QString svg = MysteryPart::makeSchematicSvg(labels, sip);
+	resetRenderer(svg);
+
+	return true;
 }
 
-QString MysteryPart::incCopyPinFunction(int pin, const QString & argString) 
-{ 
-	return argString.arg(pin + 1); 
+void MysteryPart::swapEntry(const QString & text) {
+
+    FamilyPropertyComboBox * comboBox = qobject_cast<FamilyPropertyComboBox *>(sender());
+    if (comboBox == NULL) return;
+
+    QString layout = m_propsMap.value("layout");
+
+    if (comboBox->prop().contains("layout", Qt::CaseInsensitive)) {
+        layout = text;
+    }
+
+    if (layout.isEmpty()) {
+        if (moduleID().contains("sip", Qt::CaseInsensitive)) {
+            layout = "single";
+        }
+    }
+
+    if (layout.contains("single", Qt::CaseInsensitive)) {
+        generateSwap(text, genModuleID, genSipFZP, makeBreadboardSvg, makeSchematicSvg, PinHeader::makePcbSvg);
+    }
+    else {
+        generateSwap(text, genModuleID, genDipFZP, makeBreadboardSvg, makeSchematicSvg, makePcbDipSvg);
+    }
+    PaletteItem::swapEntry(text);
+}
+
+QString MysteryPart::makePcbDipSvg(const QString & expectedFileName) 
+{
+    QString spacingString;
+	int pins = TextUtils::getPinsAndSpacing(expectedFileName, spacingString);
+    if (pins == 0) return "";  
+
+	QString header("<?xml version='1.0' encoding='UTF-8'?>\n"
+				    "<svg baseProfile='tiny' version='1.2' width='%1in' height='%2in' viewBox='0 0 %3 %4' xmlns='http://www.w3.org/2000/svg'>\n"
+				    "<desc>Fritzing footprint SVG</desc>\n"
+					"<g id='silkscreen'>\n"
+					"<line stroke='white' stroke-width='10' x1='10' x2='10' y1='10' y2='%5'/>\n"
+					"<line stroke='white' stroke-width='10' x1='10' x2='%6' y1='%5' y2='%5'/>\n"
+					"<line stroke='white' stroke-width='10' x1='%6' x2='%6' y1='%5' y2='10'/>\n"
+					"<line stroke='white' stroke-width='10' x1='10' x2='%7' y1='10' y2='10'/>\n"
+					"<line stroke='white' stroke-width='10' x1='%8' x2='%6' y1='10' y2='10'/>\n"
+					"</g>\n"
+					"<g id='copper1'><g id='copper0'>\n"
+					"<rect id='square' fill='none' height='55' stroke='rgb(255, 191, 0)' stroke-width='20' width='55' x='32.5' y='32.5'/>\n");
+
+	double outerBorder = 10;
+	double silkSplitTop = 100;
+	double offsetX = 60;
+	double offsetY = 60;
+	double spacing = TextUtils::convertToInches(spacingString) * GraphicsUtils::StandardFritzingDPI; 
+	double totalWidth = 120 + spacing;
+	double totalHeight = (100 * pins / 2) + (outerBorder * 2);
+	double center = totalWidth / 2;
+
+	QString svg = header.arg(totalWidth / GraphicsUtils::StandardFritzingDPI).arg(totalHeight / GraphicsUtils::StandardFritzingDPI).arg(totalWidth).arg(totalHeight)
+							.arg(totalHeight - outerBorder).arg(totalWidth - outerBorder)
+							.arg(center - (silkSplitTop / 2)).arg(center + (silkSplitTop / 2));
+
+	QString circle("<circle cx='%1' cy='%2' fill='none' id='connector%3pin' r='27.5' stroke='rgb(255, 191, 0)' stroke-width='20'/>\n");
+
+	int y = offsetY;
+	for (int i = 0; i < pins / 2; i++) {
+		svg += circle.arg(offsetX).arg(y).arg(i);
+		svg += circle.arg(totalWidth - offsetX).arg(y).arg(pins - 1 - i);
+		y += 100;
+	}
+
+	svg += "</g></g>\n";
+	svg += "</svg>\n";
+
+    int hsix = expectedFileName.indexOf(HoleSizePrefix);
+    if (hsix >= 0) {
+        return hackSvgHoleSizeAux(svg, expectedFileName);
+    }
+
+	return svg;
+}
+
+void MysteryPart::makeLocalMods(QByteArray &, const QString &) {
 }
