@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,21 +18,23 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6218 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 
 ********************************************************************/
 
 #include "note.h"
-#include "../items/graphicssvglineitem.h"
 #include "../debugdialog.h"
 #include "../sketch/infographicsview.h"
 #include "../model/modelpart.h"
 #include "../utils/resizehandle.h"
 #include "../utils/textutils.h"
+#include "../utils/graphicsutils.h"
+#include "../fsvgrenderer.h"
 
 #include <QTextFrame>
+#include <QTextLayout>
 #include <QTextFrameFormat>
 #include <QApplication>
 #include <QTextDocumentFragment>
@@ -74,11 +76,11 @@ const int Note::initialMinWidth = 140;
 const int Note::initialMinHeight = 45;
 const int borderWidth = 3;
 
-const qreal InactiveOpacity = 0.5;
+const double InactiveOpacity = 0.5;
 
 QString Note::initialTextString;
 
-QRegExp urlTag("<a.*href=[\"']([^\"]+[.\\s]*)[\"'].*>");    
+QRegExp UrlTag("<a.*href=[\"']([^\"]+[.\\s]*)[\"'].*>");    
 
 ///////////////////////////////////////
 
@@ -97,6 +99,17 @@ void findA(QDomElement element, QList<QDomElement> & aElements)
 	}
 }
 
+
+QString addText(const QString & text, bool inUrl) 
+{
+	if (text.isEmpty()) return "";
+
+	return QString("<tspan fill='%1' >%2</tspan>\n")
+				.arg(inUrl ? "#0000ff" : "#000000")
+				.arg(text)
+				;
+}
+
 ///////////////////////////////////////
 
 class NoteGraphicsTextItem : public QGraphicsTextItem
@@ -209,7 +222,12 @@ Note::Note( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdent
 
     setFlag(QGraphicsItem::ItemIsSelectable, true);
 
-	m_rect.setRect(0, 0, viewGeometry.rect().width(), viewGeometry.rect().height());
+	if (viewGeometry.rect().width() == 0 || viewGeometry.rect().height() == 0) {
+		m_rect.setRect(0, 0, Note::initialMinWidth, Note::initialMinHeight);
+	}
+	else {
+		m_rect.setRect(0, 0, viewGeometry.rect().width(), viewGeometry.rect().height());
+	}
 	m_pen.setWidth(borderWidth);
 	m_pen.setCosmetic(true);
 	m_pen.setBrush(QColor(0xff, 0xd5, 0x0e));
@@ -222,7 +240,7 @@ Note::Note( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdent
 	QPixmap pixmap(":/resources/images/icons/noteResizeGrip.png");
 	m_resizeGrip = new ResizeHandle(pixmap, Qt::SizeFDiagCursor, this);
 	connect(m_resizeGrip, SIGNAL(mousePressSignal(QGraphicsSceneMouseEvent *, ResizeHandle *)), this, SLOT(handleMousePressSlot(QGraphicsSceneMouseEvent *, ResizeHandle *)));
-	connect(m_resizeGrip, SIGNAL(zoomChangedSignal(qreal)), this, SLOT(handleZoomChangedSlot(qreal)));
+	connect(m_resizeGrip, SIGNAL(zoomChangedSignal(double)), this, SLOT(handleZoomChangedSlot(double)));
 
 	m_graphicsTextItem = new NoteGraphicsTextItem();
 	QFont font("Droid Sans", 9, QFont::Normal);
@@ -286,7 +304,7 @@ void Note::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWid
     painter->drawRect(m_rect);
 
 	if (option->state & QStyle::State_Selected) {
-		GraphicsSvgLineItem::qt_graphicsItem_highlightSelected(this, painter, option, boundingRect(), QPainterPath(), NULL);
+		GraphicsUtils::qt_graphicsItem_highlightSelected(painter, option, boundingRect(), QPainterPath());
     }
 
 	if (m_inactive) {
@@ -309,7 +327,7 @@ QPainterPath Note::shape() const
 void Note::positionGrip() {
 	QSizeF gripSize = m_resizeGrip->boundingRect().size();
 	QSizeF sz = this->boundingRect().size(); 
-	qreal scale = m_resizeGrip->currentScale();
+	double scale = m_resizeGrip->currentScale();
 	QPointF offset((gripSize.width() + borderWidth - 1) / scale, (gripSize.height() + borderWidth - 1) / scale);
 	QPointF p(sz.width(), sz.height());
 	m_resizeGrip->setPos(p - offset);
@@ -339,8 +357,8 @@ void Note::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 	}
 
 	if (m_inResize) {
-		qreal minWidth = emptyMinWidth;
-		qreal minHeight = emptyMinHeight;
+		double minWidth = emptyMinWidth;
+		double minHeight = emptyMinHeight;
 		QSizeF gripSize = m_resizeGrip->boundingRect().size();
 		QSizeF minSize = m_graphicsTextItem->document()->size() + gripSize + gripSize;
 		if (minSize.height() > minHeight) minHeight = minSize.height();
@@ -348,10 +366,10 @@ void Note::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {
 		QRectF rect = boundingRect();
 		rect.moveTopLeft(this->pos());
 
-		qreal oldX1 = rect.x();
-		qreal oldY1 = rect.y();
-		qreal newX = event->scenePos().x() + m_inResize->resizeOffset().x();
-		qreal newY = event->scenePos().y() + m_inResize->resizeOffset().y();
+		double oldX1 = rect.x();
+		double oldY1 = rect.y();
+		double newX = event->scenePos().x() + m_inResize->resizeOffset().x();
+		double newY = event->scenePos().y() + m_inResize->resizeOffset().y();
 		QRectF newR;
 
 		if (newX - oldX1 < minWidth) {
@@ -505,7 +523,6 @@ void Note::setText(const QString & text, bool check) {
 	}
 }
 
-
 QString Note::text() {
 	return m_graphicsTextItem->document()->toHtml();
 }
@@ -585,14 +602,14 @@ void Note::linkDialog() {
 		if (!atStart) {
 			textCursor.setPosition(wasAnchor - 1, QTextCursor::KeepAnchor);
 			QString html = textCursor.selection().toHtml();
-			if (urlTag.indexIn(html) >= 0) {
+			if (UrlTag.indexIn(html) >= 0) {
 				gotUrl = true;
 			}
 		}
 		if (!gotUrl && !atEnd) {
 			textCursor.setPosition(wasAnchor + 1, QTextCursor::KeepAnchor);
 			QString html = textCursor.selection().toHtml();
-			if (urlTag.indexIn(html) >= 0) {
+			if (UrlTag.indexIn(html) >= 0) {
 				gotUrl = true;
 			}
 		}
@@ -601,7 +618,7 @@ void Note::linkDialog() {
 	else {
 		QString html = textCursor.selection().toHtml();
 		DebugDialog::debug(html);
-		if (urlTag.indexIn(html) >= 0) {
+		if (UrlTag.indexIn(html) >= 0) {
 			gotUrl = true;
 		}
 	}
@@ -610,7 +627,7 @@ void Note::linkDialog() {
 	QString originalText;
 	QString originalUrl;
 	if (gotUrl) {
-		originalUrl = urlTag.cap(1);
+		originalUrl = UrlTag.cap(1);
 		ld.setUrl(originalUrl);
 		QString html = m_graphicsTextItem->toHtml();
 
@@ -680,7 +697,7 @@ void Note::linkDialog() {
 	}
 }
 
-void Note::handleZoomChangedSlot(qreal scale) {
+void Note::handleZoomChangedSlot(double scale) {
 	Q_UNUSED(scale);
 	positionGrip();
 }
@@ -712,7 +729,6 @@ bool Note::hasPartNumberProperty()
 	return false;
 }
 
-
 bool Note::rotationAllowed() {
 	return false;
 }
@@ -720,3 +736,103 @@ bool Note::rotationAllowed() {
 bool Note::rotation45Allowed() {
 	return false;
 }
+
+QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
+{
+	Q_UNUSED(svgHash);
+
+	switch (viewLayerID) {
+		case ViewLayer::BreadboardNote:
+			if (viewIdentifier() != ViewIdentifierClass::BreadboardView) return ___emptyString___;
+			break;
+		case ViewLayer::PcbNote:
+			if (viewIdentifier() != ViewIdentifierClass::PCBView) return ___emptyString___;
+			break;
+		case ViewLayer::SchematicNote:
+			if (viewIdentifier() != ViewIdentifierClass::SchematicView) return ___emptyString___;
+			break;
+		default:
+			return ___emptyString___;
+	}
+
+	QString svg = "<g>";
+
+	QString penColor = blackOnly ? "#000000" : m_pen.color().name();
+	double penWidth = m_pen.widthF() * dpi / GraphicsUtils::SVGDPI;
+	QString brushColor = blackOnly ? "none" : m_brush.color().name();
+	svg += QString("<rect x='%1' y='%2' width='%3' height='%4' fill='%5' stroke='%6' stroke-width='%7' />")
+		.arg(penWidth / 2)
+		.arg(penWidth / 2)
+		.arg((m_rect.width() * dpi / GraphicsUtils::SVGDPI) - penWidth)
+		.arg((m_rect.height() * dpi / GraphicsUtils::SVGDPI) - penWidth)
+		.arg(brushColor)
+		.arg(penColor)
+		.arg(penWidth)
+		;
+
+	QTextCursor textCursor = m_graphicsTextItem->textCursor();
+	QSizeF gripSize = m_resizeGrip->boundingRect().size();
+	double docLeft = gripSize.width();
+	double docTop = gripSize.height();
+	for (QTextBlock block = m_graphicsTextItem->document()->begin(); block.isValid(); block = block.next()) {
+		QTextLayout * layout = block.layout();
+		double left = block.blockFormat().leftMargin() + docLeft + layout->position().x();
+		double top = block.blockFormat().topMargin() + docTop + layout->position().y();
+		for (int i = 0; i < layout->lineCount(); i++) {
+			QTextLine line = layout->lineAt(i);
+			QRectF r = line.rect();
+			int start = line.textStart();
+			int count = line.textLength();
+
+			QString soFar;
+
+			svg += QString("<text  x='%1' y='%2' font-family='%3' stroke='none' fill='#000000' text-anchor='left' font-size='%4' >\n")
+				.arg((left + r.left()) * dpi / GraphicsUtils::SVGDPI)
+				.arg((top + r.top() + line.ascent()) * dpi / GraphicsUtils::SVGDPI)
+				.arg("Droid Sans")
+				.arg(line.ascent() * dpi / GraphicsUtils::SVGDPI) 
+				;
+
+
+			bool inUrl = false;
+			for (int i = 0; i < count; i++) {
+				textCursor.setPosition(i + start + block.position(), QTextCursor::MoveAnchor);
+				textCursor.setPosition(i + start + block.position() + 1, QTextCursor::KeepAnchor);
+				QString html = textCursor.selection().toHtml();
+				if (UrlTag.indexIn(html) >= 0) {
+					if (inUrl) {
+						soFar += block.text().mid(start + i, 1);
+						continue;
+					}
+
+					svg += addText(soFar, false);
+					soFar = block.text().mid(start + i, 1);
+					inUrl = true;
+				}
+				else {
+					if (!inUrl) {
+						soFar += block.text().mid(start + i, 1);
+						continue;
+					}
+
+					svg += addText(soFar, !blackOnly);
+					soFar = block.text().mid(start + i, 1);
+					inUrl = false;
+				}
+			}
+			svg += addText(soFar, inUrl && !blackOnly);
+			svg += "</text>";
+		}
+	}	
+
+
+	svg += "</g>";
+
+
+	return svg;	
+}
+
+ViewIdentifierClass::ViewIdentifier Note::useViewIdentifierForPixmap(ViewIdentifierClass::ViewIdentifier, bool) 
+{
+    return ViewIdentifierClass::IconView;
+}
