@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5209 $:
+$Revision: 6112 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-09 17:47:19 +0200 (Sat, 09 Jul 2011) $
+$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 ********************************************************************/
 
@@ -37,6 +37,8 @@ $Date: 2011-07-09 17:47:19 +0200 (Sat, 09 Jul 2011) $
 #include "partlabel.h"
 #include "../utils/focusoutcombobox.h"
 #include "../utils/textutils.h"
+#include "../utils/graphicsutils.h"
+#include "../utils/cursormaster.h"
 
 #include <QBrush>
 #include <QPen>
@@ -44,64 +46,87 @@ $Date: 2011-07-09 17:47:19 +0200 (Sat, 09 Jul 2011) $
 #include <QDir>
 #include <QMessageBox>
 #include <QLineEdit>
+#include <QApplication>
+#include <QEvent>
+#include <qmath.h>
+
+static QPointF RotationCenter;
+static QPointF RotationAxis;
+static QTransform OriginalTransform;
 
 
 PaletteItemBase::PaletteItemBase(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu ) :
 	ItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
 {
-	m_syncSelected = false;
+	m_inRotation = m_syncSelected = false;
 	m_offset.setX(0);
 	m_offset.setY(0);
  	m_blockItemSelectedChange = false;
 	this->setPos(viewGeometry.loc());
     setFlag(QGraphicsItem::ItemIsSelectable, true);
-#if QT_VERSION >= 0x040600
 	setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
-#endif
 	setAcceptHoverEvents(true);
 
 	if (hasPartNumberProperty()) {
-		QString savedValue = modelPart->prop(ModelPartShared::PartNumberPropertyName).toString();
+		QString savedValue = modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString();
 		if (savedValue.isEmpty()) {
 			savedValue = modelPart->properties().value(ModelPartShared::PartNumberPropertyName, "");
 			if (!savedValue.isEmpty()) {
-				modelPart->setProp(ModelPartShared::PartNumberPropertyName, savedValue);
+				modelPart->setLocalProp(ModelPartShared::PartNumberPropertyName, savedValue);
 			}
 		}
 	}
 }
 
+QRectF PaletteItemBase::boundingRectWithoutLegs() const
+{
+	return QRectF(0, 0, m_size.width(), m_size.height());
+}
+
 QRectF PaletteItemBase::boundingRect() const
 {
-    return QRectF(0, 0, m_size.width(), m_size.height());
+	if (!hasRubberBandLeg()) {
+		return QRectF(0, 0, m_size.width(), m_size.height());
+	}
+
+	//debugInfo(QString("bounding rect %1 %2 %3 %4").arg(r.left()).arg(r.top()).arg(r.width()).arg(r.height()));
+	return shape().controlPointRect();
+}
+
+QPainterPath PaletteItemBase::hoverShape() const
+{
+	// TODO: figure out real shape of svg
+    QPainterPath path;
+    path.addRect(0, 0, m_size.width(), m_size.height());
+
+	if (!hasRubberBandLeg()) return path;
+
+	foreach (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
+		if (connectorItem->hasRubberBandLeg()) {
+			path.addPath(connectorItem->mapToParent(connectorItem->hoverShape()));
+		}
+	}
+
+	path.setFillRule(Qt::WindingFill);
+	return path;
 }
 
 QPainterPath PaletteItemBase::shape() const
 {
-	if (!m_shape.isEmpty()) {
-		return ItemBase::shape();
-	}
-
 	// TODO: figure out real shape of svg
     QPainterPath path;
+    path.addRect(0, 0, m_size.width(), m_size.height());
 
-	// returns a custom shape for the now-obsolete arduino board, which allows some kind of click-thru
-	// need to clean this up...
+	if (!hasRubberBandLeg()) return path;
 
-	if ((m_viewLayerID == ViewLayer::Copper0) && 
-		((itemType() == ModelPart::Board) || (itemType() == ModelPart::ResizableBoard)) && 
-		(m_modelPart->moduleID().compare("1234ABDE24_ST") == 0)) 
-	{
-		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! hack alert !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! eventually this should be replaced by someting in the actual fzp file
-		// hack for testing click through on the arduino
-		path.addRect(m_size.width() * 0.25, 0, 3 * m_size.width() / 4, m_size.height() / 10);
-		path.addRect(m_size.width() * 0.5, 9 * m_size.height() / 10, m_size.width() * 0.5, m_size.height() / 10);
-		return path;
+	foreach (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
+		if (connectorItem->hasRubberBandLeg()) {
+			path.addPath(connectorItem->mapToParent(connectorItem->shape()));
+		}
 	}
 
-    path.addRect(0, 0, m_size.width(), m_size.height());
-    return path;
+	path.setFillRule(Qt::WindingFill);
+	return path;
 }
 
 void PaletteItemBase::saveGeometry() {
@@ -155,6 +180,33 @@ void PaletteItemBase::paint(QPainter *painter, const QStyleOptionGraphicsItem *o
 	ItemBase::paint(painter, option, widget);
 }
 
+void PaletteItemBase::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+{
+	if (m_hidden) return;
+
+	if (!hasRubberBandLeg()) {
+		ItemBase::paintHover(painter, option, widget);
+		return;
+	}
+
+	QPainterPath path;
+	path.addRect(0, 0, m_size.width(), m_size.height());
+	ItemBase::paintHover(painter, option, widget, path);
+}
+
+void PaletteItemBase::paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+{
+	if (m_hidden) return;
+
+	if (!hasRubberBandLeg()) {
+		ItemBase::paintSelected(painter, option, widget);
+		return;
+	}
+
+	QRectF r(0, 0, m_size.width(), m_size.height());
+	GraphicsUtils::qt_graphicsItem_highlightSelected(painter, option, r, QPainterPath());
+}
+
 void PaletteItemBase::mousePressConnectorEvent(ConnectorItem * connectorItem, QGraphicsSceneMouseEvent * event) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
@@ -175,17 +227,105 @@ void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 {
 	Q_UNUSED(originalItem);
 
+	m_inRotation = false;
+
+	QPointF corner;
+	if (freeRotationAllowed(event->modifiers()) && inRotationLocation(event->scenePos(), event->modifiers(), corner)) {
+		this->saveGeometry();
+		m_inRotation = true;
+		RotationCenter = mapToScene(this->boundingRectWithoutLegs().center());
+		RotationAxis = event->scenePos(); 
+		OriginalTransform = this->transform();
+		/*
+		DebugDialog::debug(QString("%11:in rotation:%1,%2 a:%3,%4 t:%5,%6,%7,%8 %9,%10")
+			.arg(RotationCenter.x()).arg(RotationCenter.y())
+			.arg(RotationAxis.x()).arg(RotationAxis.y())
+			.arg(OriginalTransform.m11()).arg(OriginalTransform.m12()).arg(OriginalTransform.m21()).arg(OriginalTransform.m22())
+			.arg(OriginalTransform.dx()).arg(OriginalTransform.dy())
+			.arg((long) this, 0, 16)
+			);
+			*/
+		this->debugInfo("in rotation");
+		InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+		if (infoGraphicsView) infoGraphicsView->setAnyInRotation();
+		return;
+	}
+
 	ItemBase::mousePressEvent(event);
 	if (canFindConnectorsUnder()) {
-		foreach (QGraphicsItem * childItem, childItems()) {
-			ConnectorItem * connectorItem = dynamic_cast<ConnectorItem *>(childItem);
-			if (connectorItem == NULL) continue;
-
+		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
 			connectorItem->setOverConnectorItem(NULL);
 		}
 	}
 }
 
+void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+{
+	if (!m_inRotation) {
+		ItemBase::mouseMoveEvent(event);
+		return;
+	}
+
+	// TODO: doesn't account for scaling
+	// see http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
+	double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
+	double a1 = atan2(RotationAxis.y() - RotationCenter.y(), RotationAxis.x() - RotationCenter.x());
+	double a2 = atan2(event->scenePos().y() - RotationCenter.y(), event->scenePos().x() - RotationCenter.x());
+
+	double deltaAngle = (a2 - a1) * 180 / M_PI;
+	//DebugDialog::debug(QString("original:%1 delta:%2").arg(originalAngle).arg(deltaAngle));
+	switch (m_viewIdentifier) {
+		case ViewIdentifierClass::BreadboardView:
+		case ViewIdentifierClass::PCBView:
+			{
+				double nearest = qRound((originalAngle + deltaAngle) / 45) * 45;
+				if (qAbs(originalAngle + deltaAngle - nearest) < 6) {
+					deltaAngle = nearest - originalAngle;
+					//DebugDialog::debug(QString("\tdelta angle %1").arg(deltaAngle));
+				}
+			}
+			break;
+		case ViewIdentifierClass::SchematicView:
+			{
+				double nearest = qRound((originalAngle + deltaAngle) / 90) * 90;
+				deltaAngle = nearest - originalAngle;
+			}
+			break;
+		default:
+			return;
+	}
+
+	ItemBase * chief = layerKinChief();
+	// restore viewGeometry to original angle
+	chief->getViewGeometry().setTransform(OriginalTransform);
+	foreach (ItemBase * itemBase, chief->layerKin()) {
+		itemBase->getViewGeometry().setTransform(OriginalTransform);
+	}
+
+	//DebugDialog::debug(QString("rotating item %1 da:%2 oa:%3 %4").arg(QTime::currentTime().toString("HH:mm:ss.zzz")).arg(deltaAngle).arg(originalAngle).arg((long) this, 0, 16));
+	chief->rotateItem(deltaAngle);
+}
+
+void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
+{
+	if (!m_inRotation) {
+		ItemBase::mouseReleaseEvent(event);
+		return;
+	}
+
+	m_inRotation = false;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView) {
+		// TODO: doesn't account for scaling
+		// see: http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
+		double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
+		double currentAngle = atan2(transform().m12(), transform().m11()) * 180 / M_PI;
+		rotateItem(originalAngle - currentAngle);		// put it back; undo command will redo it
+		saveGeometry();
+		infoGraphicsView->triggerRotate(this->layerKinChief(), currentAngle - originalAngle);
+	}
+}
+
 bool PaletteItemBase::canFindConnectorsUnder() {
 	return true;
 }
@@ -193,29 +333,22 @@ bool PaletteItemBase::canFindConnectorsUnder() {
 void PaletteItemBase::findConnectorsUnder() {
 	if (!canFindConnectorsUnder()) return;
 
-	for (int i = 0; i < childItems().count(); i++) {
-		ConnectorItem * connectorItem = dynamic_cast<ConnectorItem *>(childItems()[i]);
-		if (connectorItem == NULL) continue;
-
-		if (connectorItem->connector()->connectorType() == Connector::Female) {
-			// also helps speed things up
-			continue;
+	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+		switch (connectorItem->connector()->connectorType()) {
+			case Connector::Female:
+			case Connector::Pad:
+				continue;
+			default:
+				break;
 		}
 
 		connectorItem->findConnectorUnder(true, false, ConnectorItem::emptyConnectorItemList, false, NULL);
-
 	}
 }
 
-void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) {
-	ItemBase::mouseReleaseEvent(event);
-}
-
 bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 	bool hasMale = false;
-	foreach (QGraphicsItem * childItem, childItems()) {
-		ConnectorItem * item = dynamic_cast<ConnectorItem *>(childItem);
-		if (item == NULL) continue;
+	foreach (ConnectorItem * item, cachedConnectorItems()) {
 		if (item->connectorType() == Connector::Male) {
 			hasMale = true;
 			continue;
@@ -235,24 +368,25 @@ bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 }
 
 void PaletteItemBase::collectWireConnectees(QSet<Wire *> & wires) {
-	foreach (QGraphicsItem * childItem, childItems()) {
-		ConnectorItem * item = dynamic_cast<ConnectorItem *>(childItem);
-		if (item == NULL) continue;
-
+	foreach (ConnectorItem * item, cachedConnectorItems()) {
 		foreach (ConnectorItem * toConnectorItem, item->connectedToItems()) {
 			if (toConnectorItem->attachedToItemType() == ModelPart::Wire) {
 				if (toConnectorItem->attachedTo()->isVisible()) {
-					wires.insert(dynamic_cast<Wire *>(toConnectorItem->attachedTo()));
+					wires.insert(qobject_cast<Wire *>(toConnectorItem->attachedTo()));
 				}
 			}
 		}
 	}
 }
-
-bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, QString & error)
+bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
 {
-	LayerAttributes layerAttributes;
-	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, viewIdentifier, viewLayerID, viewLayerSpec, layerAttributes, error);
+    return setUpImage(modelPart, NULL, viewIdentifier, viewLayers, viewLayerID, viewLayerSpec, doConnectors, layerAttributes, error);
+}
+
+
+bool PaletteItemBase::setUpImage(ModelPart * modelPart, QDomDocument * domDocument, ViewIdentifierClass::ViewIdentifier viewIdentifier, const LayerHash & viewLayers, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, bool doConnectors, LayerAttributes & layerAttributes, QString & error)
+{
+	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, domDocument, viewIdentifier, viewLayerID, viewLayerSpec, layerAttributes, error);
 	if (renderer == NULL) {
 		return false;
 	}
@@ -261,7 +395,7 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::Vie
 	m_canFlipHorizontal = layerAttributes.canFlipHorizontal();
 	m_filename = layerAttributes.filename();
 	//DebugDialog::debug(QString("filename %1").arg(m_filename) );
-	setSticky(layerAttributes.sticky());
+	setSticky(layerAttributes.isSticky());
 	QString elementID = layerAttributes.layerName();
 	setViewLayerID(elementID, viewLayers);
 
@@ -273,7 +407,6 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::Vie
 
 	if (doConnectors) {
 		setUpConnectors(renderer, modelPart->ignoreTerminalPoints());
-		setConnectorTooltips();
 	}
 
 	if (!m_viewGeometry.transform().isIdentity()) {
@@ -284,12 +417,9 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::Vie
 	return true;
 }
 
-void PaletteItemBase::setSharedRendererEx(FSvgRenderer * renderer) {
-	setSharedRenderer(renderer);
-	m_size = renderer->defaultSizeF();
-}
-
 void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTerminalPoints) {
+	clearConnectorItemCache();
+
 	if (m_viewIdentifier == ViewIdentifierClass::PCBView && ViewLayer::isNonCopperLayer(m_viewLayerID)) {
 		//DebugDialog::debug(QString("skip connectors: %1 vid:%2 vlid:%3")
 		//				   .arg(this->title())
@@ -312,13 +442,13 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 
 		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, m_viewLayerID);
 		if (svgIdLayer == NULL) {
-			DebugDialog::debug("svgidlayer fail");
+			DebugDialog::debug(QString("svgidlayer fail %1 vid:%2 vlid:%3").arg(connector->connectorSharedID(), m_viewIdentifier, m_viewLayerID));
 			continue;
 		}
 
 		bool result = renderer->setUpConnector(svgIdLayer, ignoreTerminalPoints);
 		if (!result) {
-			DebugDialog::debug("setup connector fail");
+			DebugDialog::debug(QString("setup connector fail %1 vid:%2 vlid:%3").arg(connector->connectorSharedID(), m_viewIdentifier, m_viewLayerID));
 			continue;
 		}
 
@@ -329,10 +459,9 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 		connectorItem->setRect(svgIdLayer->m_rect);
 		connectorItem->setTerminalPoint(svgIdLayer->m_point);
 		connectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
-		if (svgIdLayer->m_bendableLeg) {
-			// do setBendable after setRect and setTerminalPoint
-			connectorItem->setBendableLeg(QColor(svgIdLayer->m_bendColor), 
-										TextUtils::convertToInches(svgIdLayer->m_bendStrokeWidth) * FSvgRenderer::printerScale());
+		if (!svgIdLayer->m_legId.isEmpty()) {
+			m_hasRubberBandLeg = true;
+			connectorItem->setRubberBandLeg(QColor(svgIdLayer->m_legColor), svgIdLayer->m_legStrokeWidth, svgIdLayer->m_legLine);
 		}
 
 		//DebugDialog::debug(QString("terminal point %1 %2").arg(terminalPoint.x()).arg(terminalPoint.y()) );
@@ -358,22 +487,25 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 
 void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
 	if (from->connectorType() != Connector::Female) return;
-	if (m_viewIdentifier != ViewIdentifierClass::BreadboardView) return;
 
 	// female connectors really only operate in breadboard view
+	if (m_viewIdentifier != ViewIdentifierClass::BreadboardView) return;
 
-
-	if (to->hasBendableLeg() || from->hasBendableLeg()) {
-		// bendable legs true up their own connectors, no need to position the entire part
-		// TODO: if a part has some bendable legs and some not, bailing out here might not work
-		return;
+	QTransform t = this->transform();
+	if (t.isRotating()) {
+		// map a square and see if it's a diamond
+		QRectF test(0, 0, 10, 10);
+		QRectF result = t.mapRect(test);
+		if (result.width() != test.width() || result.height() != test.height()) {
+			return;
+		}
 	}
 
-	// female connectors are equivalent to sticky
-
 	QPointF fromTerminalPoint = from->sceneAdjustedTerminalPoint(to);
 	QPointF toTerminalPoint = to->sceneAdjustedTerminalPoint(from);
 
+	if (fromTerminalPoint == toTerminalPoint) return;
+
 	this->setPos(this->pos() + fromTerminalPoint - toTerminalPoint);
 	updateConnections();
 }
@@ -386,8 +518,55 @@ void PaletteItemBase::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 	}
 
 	ItemBase::hoverEnterEvent(event);
+	if (event->isAccepted()) {
+		if (hasRubberBandLeg()) {
+			//DebugDialog::debug("---pab set override cursor");
+			CursorMaster::instance()->addCursor(this, cursor());
+
+			bool connected = false;
+			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+				if (connectorItem->connectionsCount() > 0) {
+					connected = true;
+					break;
+				}
+			}
+		}
+
+		checkFreeRotation(event->modifiers(), event->scenePos());
+	}
+
 }
 
+void PaletteItemBase::hoverMoveEvent ( QGraphicsSceneHoverEvent * event ) {
+	checkFreeRotation(event->modifiers(), event->scenePos());
+}
+
+void PaletteItemBase::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
+	if (hasRubberBandLeg()) {
+		//DebugDialog::debug("------pab restore override cursor");
+	}
+
+	CursorMaster::instance()->removeCursor(this);
+
+	ItemBase::hoverLeaveEvent(event);
+}
+
+void PaletteItemBase::cursorKeyEvent(Qt::KeyboardModifiers modifiers)
+{
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
+	checkFreeRotation(modifiers, infoGraphicsView->mapToScene(p));
+	if (hasRubberBandLeg()) {
+		QCursor cursor;
+		if (modifiers & altOrMetaModifier()) {
+			cursor = *CursorMaster::RubberbandCursor;
+		}
+		else {
+			cursor = *CursorMaster::MoveCursor;
+		}
+		CursorMaster::instance()->addCursor(this, cursor);
+	}
+}
 
 void PaletteItemBase::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
 	if (lowerConnectorLayerVisible(this)) {
@@ -414,7 +593,7 @@ LayerKinPaletteItem *PaletteItemBase::newLayerKinPaletteItem(PaletteItemBase * c
 	return lk;
 }
 
-QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, qreal dpi) 
+QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
 {
 	QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
 	QString path = filename();
@@ -443,7 +622,15 @@ QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<Q
 	if (!result) {
 		return "";
 	}
-		
+
+	if (hasRubberBandLeg()) {
+		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+			if (!connectorItem->hasRubberBandLeg()) continue;
+
+			splitter.gReplace(connectorItem->legID(m_viewIdentifier, m_viewLayerID));
+		}
+	}
+
 	result = splitter.normalize(dpi, xmlName, blackOnly);
 	if (!result) {
 		return "";
@@ -466,7 +653,7 @@ bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family,
 
 		QLineEdit * lineEdit = new QLineEdit();
 		lineEdit->setEnabled(swappingEnabled);
-		QString current = m_modelPart->prop(ModelPartShared::PartNumberPropertyName).toString();
+		QString current = m_modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString();
 		lineEdit->setText(current);
 		connect(lineEdit, SIGNAL(editingFinished()), this, SLOT(partPropertyEntry()));	
 		lineEdit->setObjectName("infoViewLineEdit");		
@@ -481,7 +668,7 @@ bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family,
 void PaletteItemBase::setProp(const QString & prop, const QString & value) 
 {	
 	if (prop.compare(ModelPartShared::PartNumberPropertyName) == 0) {
-		modelPart()->setProp(ModelPartShared::PartNumberPropertyName, value);
+		modelPart()->setLocalProp(ModelPartShared::PartNumberPropertyName, value);
 		if (m_partLabel) m_partLabel->displayTextsIf();
 		return;
 	}
@@ -489,18 +676,83 @@ void PaletteItemBase::setProp(const QString & prop, const QString & value)
 	ItemBase::setProp(prop, value);
 }
 
-
 void PaletteItemBase::partPropertyEntry() {
 	QLineEdit * lineEdit = qobject_cast<QLineEdit *>(sender());
 	if (lineEdit == NULL) return;
 
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
-		infoGraphicsView->setProp(this, ModelPartShared::PartNumberPropertyName, "", m_modelPart->prop(ModelPartShared::PartNumberPropertyName).toString(), lineEdit->text(), true);
+		infoGraphicsView->setProp(this, ModelPartShared::PartNumberPropertyName, "", m_modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString(), lineEdit->text(), true);
 	}
 }
 
 
+const QCursor * PaletteItemBase::getCursor(Qt::KeyboardModifiers modifiers)
+{
+	if (hasRubberBandLeg()) {
+		if ((modifiers & altOrMetaModifier())) {
+			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+				if (connectorItem->connectionsCount() > 0) {
+					return CursorMaster::RubberbandCursor;
+				}
+			}
+		}
+	}
+
+	//DebugDialog::debug("returning move cursor");
+	return CursorMaster::MoveCursor;
+}
+
+bool PaletteItemBase::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
+	Q_UNUSED(modifiers);
+	//if ((modifiers & altOrMetaModifier()) == 0) return false;
+	if (!isSelected()) return false;
+	if (this->moveLock()) return false;
+
+	return rotation45Allowed();
+}
+
+bool PaletteItemBase::inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers modifiers, QPointF & returnPoint)
+{
+	if (!freeRotationAllowed(modifiers)) return false;
+	if (m_viewIdentifier == ViewIdentifierClass::SchematicView) return false;
+
+	QRectF r = this->boundingRectWithoutLegs();
+	QPolygonF polygon;
+	polygon.append(mapToScene(r.topLeft()));
+	polygon.append(mapToScene(r.topRight()));
+	polygon.append(mapToScene(r.bottomRight()));
+	polygon.append(mapToScene(r.bottomLeft()));
+	foreach (QPointF p, polygon) {
+		double dsqd = GraphicsUtils::distanceSqd(p, scenePos);
+		if (dsqd < 9) {
+			returnPoint = p;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool PaletteItemBase::inRotation() {
+	return m_inRotation;
+}
+
+void PaletteItemBase::checkFreeRotation(Qt::KeyboardModifiers modifiers, QPointF scenePos)
+{
+	if (!freeRotationAllowed(modifiers)) return;
+
+	QPointF returnPoint;
+	bool inCorner = inRotationLocation(scenePos, modifiers, returnPoint);
+
+	if (inCorner) {
+		CursorMaster::instance()->addCursor(this, *CursorMaster::RotateCursor);
+	}
+	else {
+		CursorMaster::instance()->addCursor(this, cursor());
+	}
+}
+
 /*
 
 void PaletteItemBase::setPos(const QPointF & pos) {
@@ -508,9 +760,10 @@ void PaletteItemBase::setPos(const QPointF & pos) {
 	DebugDialog::debug(QString("set pos %1 %2, %3").arg(this->id()).arg(pos.x()).arg(pos.y()) );
 }
 
-void PaletteItemBase::setPos(qreal x, qreal y) {
+void PaletteItemBase::setPos(double x, double y) {
 	ItemBase::setPos(x, y);
 }
 
 
 */
+
