@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6218 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-07-26 10:32:57 +0200 (Thu, 26 Jul 2012) $
 
 ********************************************************************/
 
@@ -33,18 +33,15 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include "../utils/graphicsutils.h"
 #include "../utils/textutils.h"
 #include "../installedfonts.h"
+#include "../fsvgrenderer.h"
 
 #include <QGraphicsScene>
-#include <QTextDocument>
-#include <QTextFrameFormat>
-#include <QTextFrame>
-#include <QStyle>
 #include <QMenu>
 #include <QApplication>
 #include <QInputDialog>
-#include <QFontMetricsF>
 #include <QStringList>
 #include <QFont>
+#include <QSvgRenderer>
 
 // TODO:
 //		** selection: coordinate with part selection: it's a layerkin
@@ -107,15 +104,17 @@ enum PartLabelAction {
 
 static QMultiHash<long, PartLabel *> AllPartLabels;
 static const QString LabelTextKey = "";
-static const qreal InactiveOpacity = 0.4;
+static const double InactiveOpacity = 0.4;
 
 ///////////////////////////////////////////
 
 PartLabel::PartLabel(ItemBase * owner, QGraphicsItem * parent)
-	: QGraphicsSimpleTextItem(parent)
+	: QGraphicsSvgItem(parent)
 {
+	m_renderer = NULL;
 	m_owner = owner;
 	m_spaceBarWasPressed = false;
+	m_text = m_displayText = "";
 
 	m_inactive = m_hidden = m_initialized = false;
 	m_displayKeys.append(LabelTextKey);
@@ -156,20 +155,24 @@ void PartLabel::showLabel(bool showIt, ViewLayer * viewLayer) {
 			}
 		}
 
-		m_owner->scene()->addItem(this);
 		this->setZValue(viewLayer->nextZ());
 		m_viewLayerID = viewLayer->viewLayerID();
+		m_owner->scene()->addItem(this);
+		setUpText();
+		setPlainText(m_owner->instanceTitle());
+		m_initialized = true;
+
 		QRectF obr = m_owner->boundingRect();
-		QRectF tbr = QGraphicsSimpleTextItem::boundingRect();
+		QRectF tbr = QGraphicsSvgItem::boundingRect();
 		QPointF initial = (flipped) 
 			? m_owner->pos() + QPointF(-tbr.width(), -tbr.height())
 			: m_owner->pos() + QPointF(obr.width(), -tbr.height());
 		this->setPos(initial);
 		m_offset = initial - m_owner->pos();
-		m_initialized = true;
 		if (flipped) {
 			transformLabel(QTransform().scale(-1,1));
 		}
+
 	}
 
 	setVisible(showIt);
@@ -228,7 +231,7 @@ void PartLabel::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 		return;
 	}
 
-	QGraphicsSimpleTextItem::mouseMoveEvent(event);
+	QGraphicsSvgItem::mouseMoveEvent(event);
 }
 
 void PartLabel::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
@@ -248,7 +251,7 @@ void PartLabel::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 		m_owner->partLabelMoved(m_initialPosition, m_initialOffset, pos(), m_offset);
 	}
 
-	QGraphicsSimpleTextItem::mouseReleaseEvent(event);
+	QGraphicsSvgItem::mouseReleaseEvent(event);
 }
 
 
@@ -286,7 +289,8 @@ void PartLabel::displayTexts() {
 		text = "?";					// make sure there's something visible
 	}
 
-	QGraphicsSimpleTextItem::setText(text);
+	m_displayText = text;
+	resetSvg();
 }
 
 bool PartLabel::initialized() {
@@ -342,8 +346,8 @@ void PartLabel::saveInstance(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeAttribute("z", QString::number(zValue()));
 	streamWriter.writeAttribute("xOffset", QString::number(m_offset.x()));
 	streamWriter.writeAttribute("yOffset", QString::number(m_offset.y()));
-	streamWriter.writeAttribute("textColor", brush().color().name());
-	streamWriter.writeAttribute("fontSize", QString::number(font().pointSizeF()));
+	streamWriter.writeAttribute("textColor", m_color.name());
+	streamWriter.writeAttribute("fontSize", QString::number(m_font.pointSizeF()));
 	GraphicsUtils::saveTransform(streamWriter, transform());
 	foreach (QString key, m_displayKeys) {
 		streamWriter.writeStartElement("displayKey");
@@ -361,16 +365,16 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 	setVisible(labelGeometry.attribute("visible").compare("true") == 0);
 	QPointF p = pos();
 	bool ok = false;
-	qreal x = labelGeometry.attribute("x").toDouble(&ok);
+	double x = labelGeometry.attribute("x").toDouble(&ok);
 	if (ok) p.setX(x);
-	qreal y = labelGeometry.attribute("y").toDouble(&ok);
+	double y = labelGeometry.attribute("y").toDouble(&ok);
 	if (ok) p.setY(y);
 	setPos(p);
 	x = labelGeometry.attribute("xOffset").toDouble(&ok);
 	if (ok) m_offset.setX(x);
 	y = labelGeometry.attribute("yOffset").toDouble(&ok);
 	if (ok) m_offset.setY(y);
-	qreal z = labelGeometry.attribute("z").toDouble(&ok);
+	double z = labelGeometry.attribute("z").toDouble(&ok);
 	if (ok) this->setZValue(z);
 
 	//ignore the textColor attribute so the labels are always set from standard colors
@@ -378,18 +382,18 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 	//c.setNamedColor(labelGeometry.attribute("textColor"));
 	//setBrush(QBrush(c));
 
-	qreal fs = labelGeometry.attribute("fontSize").toDouble(&ok);
+	setUpText();
+	m_initialized = true;
+	double fs = labelGeometry.attribute("fontSize").toDouble(&ok);
 	if (!ok) {
 		InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
 		if (infographics != NULL) {
-                        fs = infographics->getLabelFontSizeMedium();
+            fs = infographics->getLabelFontSizeMedium();
 			ok = true;
 		}
 	}
 	if (ok) {
-		QFont font = this->font();
-		font.setPointSizeF(fs);
-		this->setFont(font);
+		m_font.setPointSizeF(fs);
 	}
 
 	m_displayKeys.clear();
@@ -404,6 +408,8 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 		if (m_owner->hasPartNumberProperty()) m_displayKeys.append(ModelPartShared::PartNumberPropertyName);
 	}
 
+	displayTexts();
+
 	QTransform t;
 	if (GraphicsUtils::loadTransform(labelGeometry.firstChildElement("transform"), t)) {
 		setTransform(t);
@@ -528,7 +534,7 @@ void PartLabel::initMenu()
 	}
 }
 
-void PartLabel::rotateFlipLabel(qreal degrees, Qt::Orientations orientation) {
+void PartLabel::rotateFlipLabel(double degrees, Qt::Orientations orientation) {
 	if (degrees != 0) {
 		transformLabel(QTransform().rotate(degrees));
 	}
@@ -543,6 +549,7 @@ void PartLabel::rotateFlipLabel(qreal degrees, Qt::Orientations orientation) {
 			yScale = 1;
 		}
 		else return;
+
 		transformLabel(QTransform().scale(xScale,yScale));
 	}
 }
@@ -550,8 +557,8 @@ void PartLabel::rotateFlipLabel(qreal degrees, Qt::Orientations orientation) {
 void PartLabel::transformLabel(QTransform currTransf) 
 {
 	QRectF rect = this->boundingRect();
-	qreal x = rect.width() / 2.0;
-	qreal y = rect.height() / 2.0;
+	double x = rect.width() / 2.0;
+	double y = rect.height() / 2.0;
 	QTransform transf = transform() * QTransform().translate(-x, -y) * currTransf * QTransform().translate(x, y);
 	setTransform(transf);
 }
@@ -559,11 +566,7 @@ void PartLabel::transformLabel(QTransform currTransf)
 void PartLabel::setUpText() {
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infographics != NULL) {
-		QFont font;
-		QColor color;
-		infographics->getLabelFont(font, color, m_owner->viewLayerSpec());
-		setBrush(QBrush(color));
-		setFont(font);		
+		infographics->getLabelFont(m_font, m_color, m_owner->viewLayerSpec());
 	}
 }
 
@@ -572,16 +575,13 @@ QVariant PartLabel::itemChange(QGraphicsItem::GraphicsItemChange change, const Q
 	switch (change) {
 		case QGraphicsItem::ItemSceneHasChanged:
 			if (this->scene()) {
-				setUpText();
-				setPlainText(m_owner->instanceTitle());
-
 			}
 			break;
 		default:
 			break;
 	}
 
-	return QGraphicsSimpleTextItem::itemChange(change, value);
+	return QGraphicsSvgItem::itemChange(change, value);
 }
 
 void PartLabel::ownerSelected(bool selected) 
@@ -611,20 +611,19 @@ void PartLabel::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 		displayAct->setChecked(m_displayKeys.contains(data));
 	}
 
-        m_tinyAct->setChecked(false);
+    m_tinyAct->setChecked(false);
 	m_smallAct->setChecked(false);
 	m_mediumAct->setChecked(false);
 	m_largeAct->setChecked(false);
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infographics != NULL) {
-		QFont font = this->font();
-		int fs = font.pointSize();
-                if (fs == infographics->getLabelFontSizeTiny()) {
-                        m_tinyAct->setChecked(true);
+		int fs = m_font.pointSize();
+		if (fs == infographics->getLabelFontSizeTiny()) {
+			m_tinyAct->setChecked(true);
 		}
-                else if (fs == infographics->getLabelFontSizeSmall()) {
-                        m_smallAct->setChecked(true);
-                }
+		else if (fs == infographics->getLabelFontSizeSmall()) {
+            m_smallAct->setChecked(true);
+        }
 		else if (fs == infographics->getLabelFontSizeMedium()) {
 			m_mediumAct->setChecked(true);
 		}
@@ -660,6 +659,7 @@ void PartLabel::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 		case PartLabelFontSizeMedium:
 		case PartLabelFontSizeLarge:
 			setFontSize(action);
+			resetSvg();
 			break;
 		case PartLabelDisplayLabelText:
 			setLabelDisplay(LabelTextKey);
@@ -671,7 +671,7 @@ void PartLabel::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 }
 
 void PartLabel::rotateFlip(int action) {
-	qreal degrees = 0;
+	double degrees = 0;
 	Qt::Orientations orientation = 0;
 	switch (action) {
 		case PartLabelRotate45CW:
@@ -750,10 +750,10 @@ void PartLabel::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 	}
 
 	if (m_owner->isSelected()) {
-		GraphicsSvgLineItem::qt_graphicsItem_highlightSelected(this, painter, option, boundingRect(), shape(), NULL);
+		GraphicsUtils::qt_graphicsItem_highlightSelected(painter, option, boundingRect(), shape());
     }
 
-    QGraphicsSimpleTextItem::paint(painter, option, widget);
+    QGraphicsSvgItem::paint(painter, option, widget);
 
 	if (m_inactive) {
 		painter->restore();
@@ -764,11 +764,11 @@ void PartLabel::setFontSize(int action) {
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infographics == NULL) return;
 
-	qreal fs = 0;
+	double fs = 0;
 	switch (action) {
-                case PartLabelFontSizeTiny:
-                        fs = infographics->getLabelFontSizeTiny();
-                        break;
+        case PartLabelFontSizeTiny:
+            fs = infographics->getLabelFontSizeTiny();
+            break;
 		case PartLabelFontSizeSmall:
 			fs = infographics->getLabelFontSizeSmall();
 			break;
@@ -781,9 +781,8 @@ void PartLabel::setFontSize(int action) {
 		default:
 			return;
 	}
-	QFont font = this->font();
-	font.setPointSize(fs);
-	setFont(font);
+	m_font.setPointSize(fs);
+
 }
 
 void PartLabel::setLabelDisplay(const QString & key) {
@@ -796,19 +795,27 @@ void PartLabel::setLabelDisplay(const QString & key) {
 	displayTexts();
 }
 
-int mapToSVGWeight(int w) {
-	int v = 400;
+QString mapToSVGWeight(int w) {
+	int percent = 50;
 	switch (w) {
-		case QFont::Light: v = 25; break;
-		case QFont::Normal:	v = 50; break;
-		case QFont::DemiBold: v = 63; break;
-		case QFont::Bold: v = 75; break;
-		case QFont::Black: v = 87; break;
+		case QFont::Light: 
+			percent = 25; 
+			break;
+		case QFont::Normal:	
+			return "normal";    // 50
+		case QFont::DemiBold: 
+			percent = 63;
+			break;
+		case QFont::Bold: 
+			return "bold";		// 75
+		case QFont::Black: 
+			percent = 87; 
+			break;
 		default:
-			return v;
+			return "normal";
 	}
 
-	return (qRound(8 * v / 100.0) * 100) + 100;
+	return QString::number((qRound(8 * percent / 100.0) * 100) + 100);   // 	normal | bold | bolder | lighter | 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900 | inherit
 }
 
 QString mapToSVGStyle(QFont::Style style) {
@@ -820,31 +827,83 @@ QString mapToSVGStyle(QFont::Style style) {
 	}
 }
 
-QString PartLabel::makeSvg(bool blackOnly, qreal dpi, qreal printerScale) {
-	if (this->text().isEmpty()) return "";
+QString PartLabel::makeSvg(bool blackOnly, double dpi, double printerScale, bool includeTransform) {
+	double w, h;
+	QString svg = makeSvgAux(blackOnly, dpi, printerScale, w, h);
+	if (includeTransform) {
+		QTransform t = transform();
+		if (!t.isIdentity()) {
+			svg = TextUtils::svgTransform(svg, t, false, QString());
+		}
+	}
+	return svg;
+}
 
-	QFont f = font();
-	QFontMetricsF fm(f);
-	qreal y = fm.ascent();
+
+QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, double & w, double & h) 
+{
+	if (m_displayText.isEmpty()) return "";
+
+	double pixels = m_font.pointSizeF() * printerScale / 72;
+	double y = pixels * 0.75;
+	//DebugDialog::debug(QString("initial y:%1").arg(y));
 	
 	QString svg = QString("<g font-size='%1' font-style='%2' font-weight='%3' fill='%4' font-family=\"'%5'\" id='%6' fill-opacity='1' stroke='none' >")
-		.arg(f.pointSizeF() * dpi / 72)
-		.arg(mapToSVGStyle(f.style()))
-		.arg(mapToSVGWeight(f.weight()))
-		.arg(blackOnly ? "#000000" : brush().color().name())
-		.arg(InstalledFonts::InstalledFontsNameMapper.value(f.family()))
+		.arg(m_font.pointSizeF() * dpi / 72)
+		.arg(mapToSVGStyle(m_font.style()))
+		.arg(mapToSVGWeight(m_font.weight()))
+		.arg(blackOnly ? "#000000" : m_color.name())
+		.arg(InstalledFonts::InstalledFontsNameMapper.value(m_font.family()))
 		.arg(ViewLayer::viewLayerXmlNameFromID(m_viewLayerID)
 		);
 
-	QStringList texts = text().split("\n");
+	w = 0;
+	QStringList texts = m_displayText.split("\n");
 	foreach (QString t, texts) {
+		QString t1 = TextUtils::convertExtendedChars(TextUtils::escapeAnd(t));
 		svg += QString("<text x='0' y='%1'>%2</text>")
 			.arg(y * dpi / printerScale)
-			.arg(TextUtils::stripNonValidXMLCharacters(TextUtils::escapeAnd(t)));
-		y += fm.height();
+			.arg(t1);
+		y += pixels;
+		w = qMax(w, t.length() * pixels * 0.75);
+		//DebugDialog::debug(QString("\t%1, %2").arg(w).arg(y));
 	}
 
 	svg += "</g>";
-    QTransform t = transform();
-    return TextUtils::svgTransform(svg, t, false, QString());
+
+	h = y - (pixels / 2);
+
+	//QFontInfo fontInfo(m_font);
+	//DebugDialog::debug(QString("%1 match:%2 ps:%3 sty:%4 w:%5")
+	//	.arg(fontInfo.family())
+	//	.arg(fontInfo.exactMatch())
+	//	.arg(fontInfo.pointSizeF())
+	//	.arg(fontInfo.style())
+	//	.arg(fontInfo.weight())
+	//	);
+
+	return svg;
+
+	//DebugDialog::debug(QString("final:%1 %2 %3").arg(w).arg(h).arg(m_font.toString()));
+    
+}
+
+void PartLabel::resetSvg()
+{
+
+	double w, h;
+	QString innerSvg = makeSvgAux(false, GraphicsUtils::StandardFritzingDPI, GraphicsUtils::SVGDPI, w, h);
+	if (innerSvg.isEmpty()) return;
+
+	QString svg = TextUtils::makeSVGHeader(GraphicsUtils::SVGDPI, GraphicsUtils::StandardFritzingDPI, w, h) + innerSvg + "\n</svg>";
+
+	if (m_renderer == NULL) {
+		m_renderer = new QSvgRenderer(this);
+	}
+
+	// using renderer()->load() doesn't seem to work, so keep a separate shared renderer as a workaround
+	bool loaded = m_renderer->load(svg.toUtf8());
+	if (loaded) {
+		setSharedRenderer(m_renderer);
+	}
 }
