@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,15 +18,16 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5200 $:
+$Revision: 6268 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 08:55:33 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 
 ********************************************************************/
 
 #include "stripboard.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/textutils.h"
+#include "../svg/gerbergenerator.h"
 #include "../fsvgrenderer.h"
 #include "../sketch/infographicsview.h"
 #include "moduleidnames.h"
@@ -62,12 +63,14 @@ Stripbit::Stripbit(const QPainterPath & path, ConnectorItem * connectorItem, int
 	
 	if (SpotFaceCutterCursor == NULL) {
 		QBitmap bitmap(":resources/images/cursor/spot_face_cutter.bmp");
-		SpotFaceCutterCursor = new QCursor(bitmap, bitmap, 0, 31);
+		QBitmap bitmapm(":resources/images/cursor/spot_face_cutter_mask.bmp");
+		SpotFaceCutterCursor = new QCursor(bitmap, bitmapm, 0, 0);
 	}
 
 	if (MagicWandCursor == NULL) {
 		QBitmap bitmap(":resources/images/cursor/magic_wand.bmp");
-		MagicWandCursor = new QCursor(bitmap, bitmap, 7, 24);
+		QBitmap bitmapm(":resources/images/cursor/magic_wand_mask.bmp");
+		MagicWandCursor = new QCursor(bitmap, bitmapm, 0, 0);
 	}
 
 	setZValue(-999);			// beneath connectorItems
@@ -94,7 +97,7 @@ Stripbit::~Stripbit() {
 }
 
 void Stripbit::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
-	qreal newOpacity = 1;
+	double newOpacity = 1;
 	if (m_removed) {
 		if (m_inHover) newOpacity = 0.50;
 		else newOpacity = 0.00;
@@ -104,7 +107,7 @@ void Stripbit::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 		else newOpacity = 1.00;
 	}
 
-	qreal opacity = painter->opacity();
+	double opacity = painter->opacity();
 	painter->setOpacity(newOpacity);
 	QGraphicsPathItem::paint(painter, option, widget);
 	painter->setOpacity(opacity);
@@ -282,7 +285,7 @@ Stripboard::Stripboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifi
 	foreach (Connector * connector, modelPart->connectors().values()) {
 		ConnectorShared * connectorShared = connector->connectorShared();
 		int cx, cy;
-		getXY(cx, cy, connectorShared->name());
+		getXY(cx, cy, connector->connectorSharedName());
 		BusShared * busShared = m_buses.at(cy);
 		busShared->addConnectorShared(connectorShared);
 	}
@@ -293,11 +296,11 @@ Stripboard::Stripboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifi
 Stripboard::~Stripboard() {
 }
 
-QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, qreal dpi) 
+QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi) 
 {
 	QString svg = Perfboard::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
 	if (svg.isEmpty()) return svg;
-	if (!svg.contains("boardoutline")) return svg;
+	if (!svg.contains(GerberGenerator::MagicBoardOutlineID)) return svg;
 
 	/*
 	QFile file(filename());
@@ -321,11 +324,11 @@ QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStrin
 
 		stripSvg += QString("<path stroke='none' stroke-width='0' fill='%6' " 
 						"d='m%1,%2 %3,0 0,%4 -%3,0z m0,%4a%5,%5  0,1,0 0,-%4z  m%3,-%4a%5,%5 0,1,0 0,%4z'/>\n")
-					.arg(p.x() * dpi / FSvgRenderer::printerScale())
-					.arg(p.y() * dpi / FSvgRenderer::printerScale())
-					.arg(r.width() * dpi / FSvgRenderer::printerScale() )
-					.arg(r.height() * dpi / FSvgRenderer::printerScale())
-					.arg(r.height() * dpi * .5 / FSvgRenderer::printerScale()) 
+					.arg(p.x() * dpi / GraphicsUtils::SVGDPI)
+					.arg(p.y() * dpi / GraphicsUtils::SVGDPI)
+					.arg(r.width() * dpi / GraphicsUtils::SVGDPI )
+					.arg(r.height() * dpi / GraphicsUtils::SVGDPI)
+					.arg(r.height() * dpi * .5 / GraphicsUtils::SVGDPI) 
 					.arg(blackOnly ? "black" : "#c49c59")
 				;
 	}
@@ -361,10 +364,7 @@ void Stripboard::addedToScene(bool temporary)
 
 	ConnectorItem * ciFirst = NULL;
 	ConnectorItem * ciNext = NULL;
-	foreach (QGraphicsItem * item, items) {
-		ConnectorItem * ci = dynamic_cast<ConnectorItem *>(item);
-		if (ci == NULL) continue;
-
+	foreach (ConnectorItem * ci, cachedConnectorItems()) {
 		int cx, cy;
 		getXY(cx, cy, ci->connectorSharedName());
 		if (cy == 0 && cx == 0) {
@@ -372,10 +372,7 @@ void Stripboard::addedToScene(bool temporary)
 			break;
 		}
 	}
-	foreach (QGraphicsItem * item, items) {
-		ConnectorItem * ci = dynamic_cast<ConnectorItem *>(item);
-		if (ci == NULL) continue;
-
+	foreach (ConnectorItem * ci, cachedConnectorItems()) {
 		int cx, cy;
 		getXY(cx, cy, ci->connectorSharedName());
 		if (cy == 0 && cx == 1) {
@@ -390,8 +387,8 @@ void Stripboard::addedToScene(bool temporary)
 	QRectF r1 = ciFirst->rect();
 	QRectF r2 = ciNext->rect();
 
-	qreal h = r1.height();
-	qreal w = r2.center().x() - r1.center().x();
+	double h = r1.height();
+	double w = r2.center().x() - r1.center().x();
 
 	r1.moveTo(-(r1.width() / 2), 0);
 	r2.moveTo(w - (r2.width() / 2), 0);
@@ -408,10 +405,7 @@ void Stripboard::addedToScene(bool temporary)
 
 	QHash<int, Stripbit *> stripbits;
 
-	foreach (QGraphicsItem * item, items) {
-		ConnectorItem * ci = dynamic_cast<ConnectorItem *>(item);
-		if (ci == NULL) continue;
-
+	foreach (ConnectorItem * ci, cachedConnectorItems()) {
 		int cx, cy;
 		getXY(cx, cy, ci->connectorSharedName());
 		if (cx >= x - 1) {
@@ -440,7 +434,7 @@ void Stripboard::addedToScene(bool temporary)
 		}
 	}
 
-	QString config = modelPart()->prop("buses").toString();
+	QString config = prop("buses");
 	if (config.isEmpty()) return;
 
 	QStringList removed = config.split(" ", QString::SkipEmptyParts);
@@ -460,11 +454,6 @@ QString Stripboard::genModuleID(QMap<QString, QString> & currPropsMap)
 	return size + ModuleIDNames::StripboardModuleIDName;
 }
 
-QString Stripboard::makeBreadboardSvg(const QString & size) 
-{
-	return Perfboard::makeBreadboardSvg(size);
-}
-
 void Stripboard::initCutting(Stripbit *) 
 {
 	m_beforeCut.clear();
@@ -566,13 +555,11 @@ void Stripboard::reinitBuses(bool triggerUndo)
 
 	modelPart()->clearBuses();
 	modelPart()->initBuses();
-	modelPart()->setProp("buses",  busPropertyString);
+	modelPart()->setLocalProp("buses",  busPropertyString);
 
 	
 	QList<ConnectorItem *> visited;
-	foreach (QGraphicsItem * item, childItems()) {
-		ConnectorItem * connectorItem = dynamic_cast<ConnectorItem *>(item);
-		if (connectorItem == NULL) continue;
+	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
 		if (visited.contains(connectorItem)) continue;
 
 		connectorItem->restoreColor(true, 0, true);
@@ -616,7 +603,7 @@ void Stripboard::setProp(const QString & prop, const QString & value)
 	reinitBuses(false);
 }
 
-void Stripboard::getConnectedColor(ConnectorItem * ci, QBrush * &brush, QPen * &pen, qreal & opacity, qreal & negativePenWidth, bool & negativeOffsetRect) {
+void Stripboard::getConnectedColor(ConnectorItem * ci, QBrush * &brush, QPen * &pen, double & opacity, double & negativePenWidth, bool & negativeOffsetRect) {
 	Perfboard::getConnectedColor(ci, brush, pen, opacity, negativePenWidth, negativeOffsetRect);
 	opacity *= .66667;
 }
