@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5201 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 10:02:11 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
@@ -29,6 +29,8 @@ $Date: 2011-07-08 10:02:11 +0200 (Fri, 08 Jul 2011) $
 
 #include <QUndoView>
 #include <QUndoGroup>
+#include <QRadioButton>
+#include <QLineEdit>
 #include <QToolButton>
 #include <QPushButton>
 #include <QStackedWidget>
@@ -41,6 +43,7 @@ $Date: 2011-07-08 10:02:11 +0200 (Fri, 08 Jul 2011) $
 #include "viewlayer.h"
 #include "sketchtoolbutton.h"
 #include "program/programwindow.h"
+#include "svg/svg2gerber.h"
 #include "viewswitcher/viewswitcher.h"
 
 QT_BEGIN_NAMESPACE
@@ -77,16 +80,41 @@ protected:
 	QPointer <ItemBase> m_itemBase;
 };
 
+struct GridSizeThing
+{
+	QLineEdit * lineEdit;
+	QDoubleValidator * validator;
+	QRadioButton * mmButton;
+	QRadioButton * inButton;
+	double defaultGridSize;
+    QString gridSizeText;
+	QString viewName;
+	QString shortName;
+    
+    GridSizeThing(const QString & viewName, const QString & shortName, double defaultSize, const QString & gridSizeText);
+};
+
+class GridSizeDialog : public QDialog {
+    Q_OBJECT
+
+public:
+    GridSizeDialog(GridSizeThing *);
+    GridSizeThing * gridSizeThing();
+
+protected:
+    GridSizeThing * m_gridSizeThing;
+};
+
 class MainWindow : public FritzingWindow
 {
     Q_OBJECT
 public:
-    MainWindow(class PaletteModel *, ReferenceModel *refModel);
+    MainWindow(class PaletteModel *, ReferenceModel *refModel, QWidget * parent);
     MainWindow(QFile & fileToLoad);
 	~MainWindow();
 
-    void load(const QString & fileName, bool setAsLastOpened, bool addToRecent, const QString & displayName);
-	bool loadWhich(const QString & fileName, bool setAsLastOpened, bool addToRecent);
+    void mainLoad(const QString & fileName, const QString & displayName);
+	bool loadWhich(const QString & fileName, bool setAsLastOpened, bool addToRecent, const QString & displayName);
 	void notClosableForAWhile();
 	QAction *raiseWindowAction();
 	QSizeGrip *sizeGrip();
@@ -97,7 +125,7 @@ public:
 	class PartsEditorMainWindow* getPartsEditor(ModelPart *modelPart, long id, ItemBase * fromItem, class PartsBinPaletteWidget* requester);
 	ModelPart *loadPartFromFile(const QString& newPartPath, bool connectorsChanged=false);
 	void addDefaultParts();
-	void init();
+    void init(class PaletteModel *, ReferenceModel *refModel, bool lockFiles);
 	void showFileProgressDialog(const QString & path);
 	void setFileProgressPath(const QString & path);
 	void clearFileProgressDialog();
@@ -113,17 +141,32 @@ public:
 	// if we consider a part as the smallest ("atomic") entity inside
 	// fritzing, then this functions may help with the bundle tasks
 	// on the complex entities: sketches, bins, modules (?)
-	void saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave);
-	void loadBundledNonAtomicEntity(const QString &filename, Bundler *bundler, bool addToBin);
+	void saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave, bool askForFilename, const QString & destFolderPath, bool saveModel, bool deleteLeftovers);
+	void loadBundledNonAtomicEntity(const QString &filename, Bundler *bundler, bool addToBin, bool dontAsk);
+	void saveAsShareable(const QString & path, bool saveModel);
+
 	
-	void exportToGerber(const QString & exportDir, ItemBase * board, bool displayMessageBoxes);
-	void setCurrentFile(const QString &fileName, bool addToRecent, bool recovered, const QString & backupName);
-	void setRecovered(bool);
+	void setCurrentFile(const QString &fileName, bool addToRecent, bool setAsLastOpened);
 	void setReportMissingModules(bool);
+	QList<SketchWidget *> sketchWidgets();
+	void setCloseSilently(bool);
+	void exportToGerber(const QString & outputDir);
+	class PCBSketchWidget * pcbView();
+	void noBackup();
+	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID);
+    void swapLayers(ItemBase * itemBase, int layers, const QString & msg, bool flip, int delay);
+	bool saveAsAux(const QString & fileName);
+	void swapObsolete(bool displayFeedback);
+	void selectAllObsolete(bool displayFeedback);
+	void hideTempPartsBin();
+	const QString & fritzingVersion();
+	void removeGroundFill(ViewLayer::ViewLayerID, QUndoCommand * parentCommand);
+	void groundFill(ViewLayer::ViewLayerID);
+	void copperFill(ViewLayer::ViewLayerID);
 
 public:
 	static void initNames();
-	static MainWindow * newMainWindow(PaletteModel * paletteModel, ReferenceModel *refModel, const QString & displayPath, bool showProgress);
+	static MainWindow * newMainWindow(PaletteModel * paletteModel, ReferenceModel *refModel, const QString & displayPath, bool showProgress, bool lockFiles);
 	static void setAutosavePeriod(int);
 	static void setAutosaveEnabled(bool);
 
@@ -137,15 +180,29 @@ signals:
 
 public slots:
 	void ensureClosable();
-	ModelPart* loadBundledPart(const QString &fileName, bool addToBin=true);
+	ModelPart* loadBundledPart(const QString &fileName, bool addToBin);
 	void partsEditorClosed(long id);
 	void importFilesFromPrevInstall();
 	void acceptAlienFiles();
 	void statusMessage(QString message, int timeout);
 	void warnSMD(const QString & moduleID);
+    void showPCBView();
+	void groundFill();
+	void removeGroundFill();
+	void copperFill();
+	void setOneGroundFillSeed();
+	void setGroundFillSeeds();	
+	void clearGroundFillSeeds();
+	void changeBoardLayers(int layers, bool doEmit);
+	void selectAllObsolete();
+	void swapObsolete();
+    void swapBoardImageSlot(SketchWidget * sketchWidget, ItemBase * itemBase, const QString & filename, const QString & moduleID, bool addName);
+	void updateTraceMenu();
+	void updateFileMenu();
 
 protected slots:
-	void load();
+	void mainLoad();
+	void revert();
 	void openRecentOrExampleFile();
     void print();
     void doExport();
@@ -166,21 +223,19 @@ protected slots:
     void actualSize();
 	void hundredPercentSize();
     void alignToGrid();
-	void alignToGridSettings();
-	void setBackgroundColor();
+    void showGrid();
+    void setGridSize();
+    void setBackgroundColor();
     void showBreadboardView();
     void showSchematicView();
-    void showPCBView();
 	void showPartsBinIconView();
 	void showPartsBinListView();
     void updateEditMenu();
     void updateLayerMenu(bool resetLayout = false);
     void updatePartMenu();
     void updateWireMenu();
-	void updateFileMenu();
     void updateTransformationActions();
 	void updateRecentFileActions();
-	void updateTraceMenu();
     void tabWidget_currentChanged(int index);
     void createNewPart();
     void createNewSketch();
@@ -207,20 +262,27 @@ protected slots:
 	void rotate180();
 	void rotate45ccw();
 	void rotate45cw();
+	void rotateIncCCW();
+	void rotateIncCW();
+	void rotateIncCCWRubberBand();
+	void rotateIncCWRubberBand();
 	void flipHorizontal();
 	void flipVertical();
 	void showAllLayers();
 	void hideAllLayers();
 	void showInViewHelp();
 	void addBendpoint();
+	void convertToVia();
+	void convertToBendpoint();
+	void flattenCurve();
 	void disconnectAll();
 
 	void openInPartsEditor();
 	void openPartsEditor(PaletteItem *);
 
-	void updateZoomSlider(qreal zoom);
-	void updateZoomOptionsNoMatterWhat(qreal zoom);
-	void updateViewZoom(qreal newZoom);
+	void updateZoomSlider(double zoom);
+	void updateZoomOptionsNoMatterWhat(double zoom);
+	void updateViewZoom(double newZoom);
 
 	void loadPart(const QString &newPartPath, long partsEditorId, bool connectorsChanged);
 
@@ -232,20 +294,20 @@ protected slots:
 	void activeLayerTop();
 	void activeLayerBottom();
 	void activeLayerBoth();
+	void toggleActiveLayer();
 	void createTrace();
 	void excludeFromAutoroute();
 	void selectAllTraces();
+	void selectAllCopperFill();
 	void updateRoutingStatus();
 	void selectAllExcludedTraces();
+	void selectAllIncludedTraces();
 	void selectAllJumperItems();
 	void selectAllVias();
 
-	void saveBundledSketch();
 	void shareOnline();
 	void saveBundledPart(const QString &moduleId=___emptyString___);
-	void saveBundledAux(ModelPart *mp, const QDir &destFolder);
-	void loadBundledSketch(const QString &fileName);
-	void loadBundledPart();
+	QStringList saveBundledAux(ModelPart *mp, const QDir &destFolder);
 
 	void binSaved(bool hasAlienParts);
 	void routingStatusSlot(SketchWidget *, const RoutingStatus &);
@@ -261,8 +323,6 @@ protected slots:
 	void reportBug();
 	void enableDebug();
 	void tidyWires();
-	void groundFill();
-	void removeGroundFill();
 	void changeWireColor(bool checked);
 
 	void startSaveInstancesSlot(const QString & fileName, ModelPart *, QXmlStreamWriter &);
@@ -271,9 +331,6 @@ protected slots:
 	void exportNormalizedSVG();
 	void exportNormalizedFlattenedSVG();
 
-	void selectAllObsolete();
-	void swapObsolete();
-
 	void launchExternalProcess();
 	bool externalProcess(QString & name, QString & path, QStringList & args);
 	void processError(QProcess::ProcessError processError);
@@ -287,12 +344,11 @@ protected slots:
 	void openProgramWindow();
 	void linkToProgramFile(const QString & filename, const QString & language, const QString & programmer, bool addLink, bool strong);
 	void designRulesCheck();
-	void subSwapSlot(SketchWidget *, ItemBase *, ViewLayer::ViewLayerSpec, QUndoCommand * parentCommand);
+	void subSwapSlot(SketchWidget *, ItemBase *, const QString & newModuleID, ViewLayer::ViewLayerSpec, long & newID, QUndoCommand * parentCommand);
 	void updateLayerMenuSlot();
 	bool save();
 	bool saveAs();
-	void changeBoardLayers(int layers, bool doEmit);
-    void backupSketch();
+	void backupSketch();
 	void undoStackCleanChanged(bool isClean);
 	void autosaveNeeded(int index = 0);
 	void firstTimeHelpHidden();
@@ -302,17 +358,23 @@ protected slots:
 	void updateNet();
 	void selectMoveLock();
 	void moveLock();
-	void searchPartsBin();
+	void setSticky();
 	void showNavigator();
 	void autorouterSettings();
 	void warnSMDReally();
 	void boardDeletedSlot();
-	void cursorLocationSlot(qreal, qreal);
+	void cursorLocationSlot(double, double);
 	void locationLabelClicked();
+	void swapSelectedMap(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
 	void swapSelectedDelay(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
 	void swapSelectedTimeout();
 	void filenameIfSlot(QString & filename);
-
+	void openURL();
+	void setActiveWire(Wire *);
+	void setActiveConnectorItem(ConnectorItem *);
+	void gridUnits(bool);
+	void restoreDefaultGrid();
+    void checkLoadedTraces();
 
 protected:
 	void initSketchWidget(SketchWidget *);
@@ -322,12 +384,7 @@ protected:
     void createExportActions();
     void createOrderFabAct();
     void createOpenExampleMenu();
-    void populateMenuFromXMLFile(
-    		QMenu *parentMenu, QStringList &actionsTracker,
-    		const QString &folderPath, const QString &indexFileName/*,
-    		const QString &rootNode, const QString &indexNode,
-    		const QString &submenuNode*/
-    );
+    void populateMenuFromXMLFile(QMenu *parentMenu, QStringList &actionsTracker, const QString &folderPath, const QString &indexFileName);
     QHash<QString, struct SketchDescriptor *> indexAvailableElements(QDomElement &domElem, const QString &srcPrefix, QStringList & actionsTracker);
     void populateMenuWithIndex(const QHash<QString, struct SketchDescriptor *> &, QMenu * parentMenu, QDomElement &domElem);
     void populateMenuFromFolderContent(QMenu *parentMenu, const QString &path);
@@ -343,7 +400,6 @@ protected:
 	void connectPairs();
 	void connectPair(SketchWidget * signaller, SketchWidget * slotter);
 	void closeEvent(QCloseEvent * event);
-	bool saveAsAux(const QString & fileName);
 	void saveAsAuxAux(const QString & fileName);
 	void printAux(QPrinter &printer, bool removeBackground, bool paginate);
 	void exportAux(QString fileName, QImage::Format format, bool removeBackground);
@@ -354,9 +410,9 @@ protected:
 	void exportToGerber();
     void exportBOM();
     void exportNetlist();
-    void exportSvg(qreal res, bool selectedItems, bool flatten);
-	void exportSvgWatermark(QString & svg, qreal res);
-	void exportEtchable(bool wantPDF, bool wantSVG, bool flip);
+    void exportSvg(double res, bool selectedItems, bool flatten);
+	void exportSvgWatermark(QString & svg, double res);
+	void exportEtchable(bool wantPDF, bool wantSVG);
 
 	QList<QWidget*> getButtonsForView(ViewIdentifierClass::ViewIdentifier viewId);
 	const QString untitledFileName();
@@ -365,17 +421,14 @@ protected:
 	QString getExtensionString();
 	QStringList getExtensions();
 	const QString defaultSaveFolder();
-	const QString & fileName();
 	bool undoStackIsEmpty();
 
 	void createTraceMenuActions();
 	void hideShowTraceMenu();
 
-	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin=true);
-	bool loadBundledAux(QDir &unzipDir, QList<ModelPart*> mps);
-	bool preloadBundledAux(QDir &unzipDir);
-	void copyToSvgFolder(const QFileInfo& file, const QString &destFolder = "contrib");
-	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToBin=true, const QString &destFolder="contrib");
+	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	void copyToSvgFolder(const QFileInfo& file, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
 
 	void closeIfEmptySketch(MainWindow* mw);
 	bool whatToDoWithAlienFiles();
@@ -414,38 +467,39 @@ protected:
 	QString genIcon(SketchWidget *, LayerList &  partViewLayerIDs, LayerList & wireViewLayerIDs);
 
 	bool alreadyOpen(const QString & fileName);
-	bool loadCustomBoardShape();
 	void svgMissingLayer(const QString & layername, const QString & path);
-	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID);
 	long swapSelectedAuxAux(ItemBase * itemBase, const QString & moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, QUndoCommand * parentCommand);
-	bool swapSpecial(QMap<QString, QString> & currPropsMap);
+	bool swapSpecial(const QString & prop, QMap<QString, QString> & currPropsMap);
 
 	void enableAddBendpointAct(QGraphicsItem *);
 	class FileProgressDialog * exportProgress();
 	QString constructFileName(const QString & differentiator, const QString & extension);
 	bool isGroundFill(ItemBase * itemBase);
 
-	QString getBoardSilkscreenSvg(ItemBase * board, int res, QSizeF & imageSize);
-	QString mergeBoardSvg(QString & svg, ItemBase * board, int res, QSizeF & imageSize, bool flip);
+	QString getBoardSvg(ItemBase * board, int res, LayerList &);
+	QString mergeBoardSvg(QString & svg, ItemBase * board, int res, bool flip, LayerList &);
 
 	bool wannaRestart();
 
-	QString clipToBoard(QString svgString, ItemBase * board, const QString & layerName);
-	int doSilk(LayerList silkLayerIDs, const QString & silkName, const QString & gerberSuffix, ItemBase * board, const QString & exportDir, bool displayMessageBoxes);
-	int doMask(LayerList maskLayerIDs, const QString & maskName, const QString & gerberSuffix, ItemBase * board, const QString & exportDir, bool displayMessageBoxes);
-	int doCopper(ItemBase * board, LayerList & viewLayerIDs, const QString & copperName, const QString & copperSuffix, const QString & exportDir, bool displayMessageBoxes);
-	int doDrill(ItemBase * board, const QString & exportDir, bool displayMessageBoxes);
-	void displayMessage(const QString & message, bool displayMessageBoxes);
 	void updateActiveLayerButtons();
+	int activeLayerIndex();
 	bool hasLinkedProgramFiles(const QString & filename, QStringList & linkedProgramFiles);
 	void pasteAux(bool pasteInPlace);
 
 	void routingStatusLabelMouse(QMouseEvent*, bool show);
 	class Wire * retrieveWire();
-	void updatePartsBinMenu(QMenu * partsBinMenu, QMenu * binMenu, int skip);
+	class ConnectorItem * retrieveConnectorItem();
 	QString getBomProps(ItemBase *);
-	void swapSelectedMap(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
-
+	ModelPart * findReplacedby(ModelPart * originalModelPart);
+	void groundFillAux(bool fillGroundTraces, ViewLayer::ViewLayerID viewLayerID);
+	void connectStartSave(bool connect);
+	void loadBundledSketch(const QString &fileName, bool addToRecent, bool setAsLastOpened);
+    void dropEvent(QDropEvent *event);
+    void dragEnterEvent(QDragEnterEvent *event);
+    void mainLoadAux(const QString & fileName);
+    bool updateExportActions();
+	QWidget * createGridSizeForm(GridSizeThing *);
+    void massageOutput(QString & svg, bool doMask, bool doSilk, bool doPaste, QString & maskTop, QString & maskBottom, const QString & fileName, ItemBase * board, int dpi, const LayerList &);
 
 protected:
 	static void removeActionsStartingAt(QMenu *menu, int start=0);
@@ -506,15 +560,14 @@ protected:
 	QMenu *m_fileMenu;
 	QAction *m_newAct;
 	QAction *m_openAct;
+	QAction *m_revertAct;
 	QMenu *m_openRecentFileMenu;
     QAction *m_openRecentFileActs[MaxRecentFiles];
 	QMenu *m_openExampleMenu;
-	QMenu * m_partsBinFileMenu;
 	QAction *m_saveAct;
 	QAction *m_saveAsAct;
 	QAction *m_pageSetupAct;
 	QAction *m_printAct;
-	QAction *m_saveAsBundledAct;
 	QAction *m_shareOnlineAct;
 
 	QAction * m_launchExternalProcessAct;
@@ -540,8 +593,6 @@ protected:
 	QAction *m_exportGerberAct;
 	QAction *m_exportEtchablePdfAct;
 	QAction *m_exportEtchableSvgAct;
-	QAction *m_exportEtchablePdfFlipAct;
-	QAction *m_exportEtchableSvgFlipAct;
 	QAction *m_exportBomAct;
 	QAction *m_exportNetlistAct;
 	QAction *m_exportSvgAct;
@@ -569,8 +620,6 @@ protected:
 	QAction *m_exportNormalizedFlattenedSvgAction;
     QAction *m_openInPartsEditorAct;
     QMenu *m_addToBinMenu;
-	QMenu * m_partsBinPartMenu;
-	QAction *m_partsBinSearchAct;
 
 
 	QMenu *m_rotateMenu;
@@ -581,11 +630,11 @@ protected:
 	QAction *m_rotate45cwAct;
 
 	QAction *m_moveLockAct;
+	QAction *m_stickyAct;
 	QAction *m_selectMoveLockAct;
 	QAction *m_flipHorizontalAct;
 	QAction *m_flipVerticalAct;
 	QAction *m_showPartLabelAct;
-	QAction *m_loadBundledPart;
 	QAction *m_saveBundledPart;
 	QAction *m_disconnectAllAct;
 	QAction *m_selectAllObsoleteAct;
@@ -593,7 +642,10 @@ protected:
 	QAction * m_openProgramWindowAct;
 
 	QAction *m_addBendpointAct;
-
+	QAction *m_convertToViaAct;
+	QAction *m_convertToBendpointAct;
+    QAction * m_convertToBendpointSeparator;
+	QAction *m_flattenCurveAct;
     QAction *m_showAllLayersAct;
 	QAction *m_hideAllLayersAct;
 
@@ -605,7 +657,8 @@ protected:
     QAction *m_actualSizeAct;
     QAction *m_100PercentSizeAct;
     QAction *m_alignToGridAct;
-    QAction *m_alignToGridSettingsAct;
+    QAction *m_showGridAct;
+    QAction *m_setGridSizeAct;
     QAction *m_setBackgroundColorAct;
     QAction *m_showBreadboardAct;
     QAction *m_showSchematicAct;
@@ -628,28 +681,36 @@ protected:
 	// Trace Menu
 	QMenu *m_pcbTraceMenu;
 	QMenu *m_schematicTraceMenu;
+	QMenu *m_breadboardTraceMenu;
 	QAction *m_autorouteAct;
 	QAction *m_orderFabAct;
 	QAction *m_activeLayerTopAct;
 	QAction *m_activeLayerBottomAct;
 	QAction *m_activeLayerBothAct;
-	QAction *m_createTraceAct;
 	class WireAction *m_createTraceWireAct;
-	class WireAction *m_updateNetAct;			// when we're confident, we can remove this
+	class WireAction *m_createWireWireAct;
 	QAction *m_createJumperAct;
 	QAction *m_changeTraceLayerAct;
 	QAction *m_excludeFromAutorouteAct;
 	class WireAction *m_excludeFromAutorouteWireAct;
 	QAction *m_selectAllTracesAct;
+	QAction *m_selectAllWiresAct;
+	QAction *m_selectAllCopperFillAct;
 	QAction *m_updateRoutingStatusAct;
 	QAction *m_selectAllExcludedTracesAct;
+	QAction *m_selectAllIncludedTracesAct;
 	QAction *m_selectAllJumperItemsAct;
 	QAction *m_selectAllViasAct;
 	QAction *m_groundFillAct;
 	QAction *m_removeGroundFillAct;
+	QAction *m_copperFillAct;
+	class ConnectorItemAction *m_setOneGroundFillSeedAct;
+	QAction *m_setGroundFillSeedsAct;
+	QAction *m_clearGroundFillSeedsAct;
 	QAction *m_designRulesCheckAct;
 	QAction *m_autorouterSettingsAct;
 	QAction *m_tidyWiresAct;
+	QAction *m_checkLoadedTracesAct;
 
 	// Help Menu
     QMenu *m_helpMenu;
@@ -701,7 +762,7 @@ protected:
 	QPointer<class FileProgressDialog> m_fileProgressDialog;
 	QPointer<class ZoomSlider> m_zoomSlider;
 	QPointer<QLabel> m_locationLabel;
-	bool m_locationLabelInches;
+	QString m_locationLabelUnits;
 
 	QByteArray m_externalProcessOutput;
 
@@ -712,12 +773,16 @@ protected:
 	QTimer m_autosaveTimer;
 	bool m_autosaveNeeded;
 	bool m_backingUp;
-	bool m_recovered;
 	QString m_bundledSketchName;
 	RoutingStatus m_routingStatus;
 	bool m_smdOneSideWarningGiven;
 	bool m_orderFabEnabled;		
+	bool m_closeSilently;
+	QString m_fzzFolder;
+	QHash<QString, struct LockedFile *> m_fzzFiles;
 	SwapTimer m_swapTimer;
+	QPointer<Wire> m_activeWire;
+	QPointer<ConnectorItem> m_activeConnectorItem;
 
 public:
 	static int RestartNeeded;
@@ -725,6 +790,7 @@ public:
 	static bool AutosaveEnabled;
 	static QString BackupFolder;
 
+
 protected:
 	static const QString UntitledSketchName;
 	static int UntitledSketchIndex;
