@@ -1,7 +1,7 @@
 /*******************************************************************
 skw
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
@@ -41,7 +41,6 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include "items/via.h"
 #include "fsvgrenderer.h"
 #include "items/note.h"
-#include "svg/svg2gerber.h"
 #include "eagle/fritzing2eagle.h"
 #include "sketch/breadboardsketchwidget.h"
 #include "sketch/schematicsketchwidget.h"
@@ -63,10 +62,10 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include "items/moduleidnames.h"
 #include "navigator/miniviewcontainer.h"
 #include "utils/zoomslider.h"
-#include "dialogs/alignsettingsdialog.h"
 #include "layerpalette.h"
 #include "program/programwindow.h"
 #include "utils/autoclosemessagebox.h"
+#include "svg/gerbergenerator.h"
 #include "processeventblocker.h"
 
 static QString eagleActionType = ".eagle";
@@ -84,11 +83,36 @@ static QHash<QString, QImage::Format> fileExportFormats;
 static QHash<QString, QString> fileExtFormats;
 
 static QRegExp AaCc("[aAcC]");
+static QRegExp LabelNumber("([^\\d]+)(.*)");
+
+static const double InchesPerMeter = 39.3700787;
 
 ////////////////////////////////////////////////////////
 
-bool sortPartList(ItemBase * b1, ItemBase * b2){
-    return b1->instanceTitle().toLower() < b2->instanceTitle().toLower();
+bool sortPartList(ItemBase * b1, ItemBase * b2) {
+    bool result = b1->instanceTitle().toLower() < b2->instanceTitle().toLower();
+
+    int ix1 = LabelNumber.indexIn(b1->instanceTitle());
+    if (ix1 < 0) return result;
+
+    QString label1 = LabelNumber.cap(1);
+    QString number1 = LabelNumber.cap(2);
+
+    int ix2 = LabelNumber.indexIn(b2->instanceTitle());
+    if (ix2 < 0) return result;
+
+    QString label2 = LabelNumber.cap(1);
+    QString number2 = LabelNumber.cap(2);
+    if (label2.compare(label1, Qt::CaseInsensitive) != 0) return result;
+
+    bool ok;
+    double d1 = number1.toDouble(&ok);
+    if (!ok) return result;
+
+    double d2 = number2.toDouble(&ok);
+    if (!ok) return result;
+
+    return d1 < d2;
 }
 
 /////////////////////////////////////////////////////////
@@ -133,13 +157,26 @@ void MainWindow::print() {
 void MainWindow::exportEtchable() {
 	if (sender() == NULL) return;
 
-	bool wantSvg = sender()->property("svg").toBool();
-	exportEtchable(!wantSvg, wantSvg, sender()->property("flip").toBool());
+    bool wantSvg = sender()->property("svg").toBool();
+	exportEtchable(!wantSvg, wantSvg);
 }
 
 
-void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
+void MainWindow::exportEtchable(bool wantPDF, bool wantSVG)
 {
+    int boardCount;
+    ItemBase * board = m_pcbGraphicsView->findSelectedBoard(boardCount);
+    if (boardCount == 0) {
+        QMessageBox::critical(this, tr("Fritzing"),
+                   tr("Your sketch does not have a board yet! Please add a PCB in order to export etchable."));
+        return;
+    }
+    if (board == NULL) {
+        QMessageBox::critical(this, tr("Fritzing"),
+                   tr("Etchable export can only handle one board at a time--please select the board you want to export."));
+        return;
+    }
+
 	RoutingStatus routingStatus;
 	m_pcbGraphicsView->updateRoutingStatus(NULL, routingStatus, true);
 	if (routingStatus.m_connectorsLeftToRoute > 0) {
@@ -155,99 +192,133 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 		if (ret != QMessageBox::Yes) return;
 	}
 
-	QStringList fileNames;
 	QString path = defaultSaveFolder();
-	QString suffix = (wantPDF) ? pdfActionType : svgActionType;
 	QString extFmt = (wantPDF) ? fileExtFormats.value(pdfActionType) : fileExtFormats.value(svgActionType);
-	QString fileExt;
+	QString fileExt = extFmt;
 
-	if (m_pcbGraphicsView->boardLayers() == 1) {
-		QString fileName = FolderUtils::getSaveFileName(this,
-			tr("Export PDF or SVG for etchable PCB production..."),
-			path+"/"+constructFileName("etch", suffix),
-			extFmt,
-			&fileExt
-		);
+	QString suffix = (wantPDF) ? pdfActionType : svgActionType;
+    QString prefix = "";
+    if (boardCount > 1) {
+        prefix = QString("%1_%2_").arg(board->instanceTitle()).arg(board->id());
+    }
 
-		if (fileName.isEmpty()) {
-			return; //Cancel pressed
-		}
+	QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
+												defaultSaveFolder(),
+												QFileDialog::ShowDirsOnly
+												| QFileDialog::DontResolveSymlinks);
+	if (exportDir.isEmpty()) return;
 
-		if(!alreadyHasExtension(fileName, suffix)) {
-			fileName += suffix;
-		}
-
-		fileNames.append(fileName);
-	}
-	else {
-		QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
-												 defaultSaveFolder(),
-												 QFileDialog::ShowDirsOnly
-												 | QFileDialog::DontResolveSymlinks);
-		if (exportDir.isEmpty()) return;
-
-		FolderUtils::setOpenSaveFolder(exportDir);
-		fileNames.append(exportDir + "/" + constructFileName("etch_bottom", suffix));
-		fileNames.append(exportDir + "/" + constructFileName("etch_top", suffix));
-		fileExt = extFmt;
-	}
-
-	
+	FolderUtils::setOpenSaveFolder(exportDir);
 	FileProgressDialog * fileProgressDialog = exportProgress();
 
-    ItemBase * board = m_pcbGraphicsView->findBoard();
+    QRectF r = board->sceneBoundingRect();
+    QSizeF boardImageSize(r.width(), r.height());
+  
+	QStringList fileNames;
 
+	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_copper_bottom%1", suffix));
+	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_mask_bottom%1", suffix));
+	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_paste_mask_bottom%1", suffix));
+	if (m_pcbGraphicsView->boardLayers() > 1) {
+		fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_copper_top%1", suffix));
+		fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_mask_top%1", suffix));
+		fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_paste_mask_top%1", suffix));
+	}
+	fileNames.append(exportDir + "/" + constructFileName(prefix + "etch_silk_top%1", suffix));
+
+	QString maskTop, maskBottom;
+	QList<ItemBase *> copperLogoItems, holes;
 	for (int ix = 0; ix < fileNames.count(); ix++) {
+		bool doMask = false;
+		bool doSilk = false;
+        bool doPaste = false;
 		QString fileName = fileNames[ix];
 		LayerList viewLayerIDs;
-		if (ix == 0) {
+		if (fileName.contains("copper_bottom")) {
 			viewLayerIDs << ViewLayer::GroundPlane0 << ViewLayer::Copper0 << ViewLayer::Copper0Trace;
 		}
-		else {
+		else if (fileName.contains("mask_bottom")) {
+			doMask = true;
+			viewLayerIDs << ViewLayer::Copper0;
+            doPaste = fileName.contains("paste");
+		}
+		else if (fileName.contains("copper_top")) {
 			viewLayerIDs << ViewLayer::GroundPlane1 << ViewLayer::Copper1 << ViewLayer::Copper1Trace;
 		}
+		else if (fileName.contains("mask_top")) {
+			viewLayerIDs << ViewLayer::Copper1;
+			doMask = true;
+            doPaste = fileName.contains("paste");
+		}
+		else if (fileName.contains("silk_top")) {
+			viewLayerIDs << ViewLayer::Silkscreen1 << ViewLayer::Silkscreen1Label;
+			doSilk = true;
+		}
+
+		if (doMask) {
+			m_pcbGraphicsView->hideCopperLogoItems(copperLogoItems);
+		}
+		if (doPaste) {
+			m_pcbGraphicsView->hideHoles(holes);
+		}
 
-		QSizeF imageSize;
 		if (wantSVG) {
 			bool empty;
-			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, GraphicsUtils::IllustratorDPI, false, false, false, empty);
-			svg = mergeBoardSvg(svg, board, GraphicsUtils::IllustratorDPI, imageSize, flip);
-			
-			QFile file(fileName);
-			file.open(QIODevice::WriteOnly);
-			QTextStream out(&file);
-			out.setCodec("UTF-8");
-			out << svg;
-			file.close();
+		    QSizeF imageSize;
+			QString svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageSize, board, GraphicsUtils::IllustratorDPI, false, false, empty);
+			massageOutput(svg, doMask, doSilk, doPaste, maskTop, maskBottom, fileName, board, GraphicsUtils::IllustratorDPI, viewLayerIDs);		
+			QString merged = mergeBoardSvg(svg, board, GraphicsUtils::IllustratorDPI, false, viewLayerIDs);
+            TextUtils::writeUtf8(fileName.arg(""), merged);
+			merged = mergeBoardSvg(svg, board, GraphicsUtils::IllustratorDPI, true, viewLayerIDs);
+            TextUtils::writeUtf8(fileName.arg("_mirror"), merged);
 		}
 		else {
-			QPrinter printer(QPrinter::HighResolution);
-			printer.setOutputFormat(filePrintFormats[fileExt]);
-			printer.setOutputFileName(fileName);
-			int res = printer.resolution();
-			bool empty;
-			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, res, false, false, false, empty);
-			svg = mergeBoardSvg(svg, board, res, imageSize, flip);
+            QString svg;
+            QList<bool> flips;
+            flips << false << true;
+            foreach (bool flip, flips) {
+                QString mirror = flip ? "_mirror" : "";
+			    QPrinter printer(QPrinter::HighResolution);
+			    printer.setOutputFormat(filePrintFormats[fileExt]);
+			    printer.setOutputFileName(fileName.arg(mirror));
+			    int res = printer.resolution();
+
+                if (svg.isEmpty()) {
+			        bool empty;
+                    QSizeF imageSize;
+			        svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageSize, board, res, false, false, empty);
+			        massageOutput(svg, doMask, doSilk, doPaste, maskTop, maskBottom, fileName, board, res, viewLayerIDs);
+                }
 			
-			// now convert to pdf
-			QSvgRenderer svgRenderer;
-			svgRenderer.load(svg.toLatin1());
-			qreal trueWidth = imageSize.width() / FSvgRenderer::printerScale();
-			qreal trueHeight = imageSize.height() / FSvgRenderer::printerScale();
-			QRectF target(0, 0, trueWidth * res, trueHeight * res);
+                QString merged = mergeBoardSvg(svg, board, res, flip, viewLayerIDs);
+			
+			    // now convert to pdf
+			    QSvgRenderer svgRenderer;
+			    svgRenderer.load(merged.toLatin1());
+			    double trueWidth = boardImageSize.width() / GraphicsUtils::SVGDPI;
+			    double trueHeight = boardImageSize.height() / GraphicsUtils::SVGDPI;
+			    QRectF target(0, 0, trueWidth * res, trueHeight * res);
 
-			QSizeF psize((target.width() + printer.paperRect().width() - printer.width()) / res, 
-						 (target.height() + printer.paperRect().height() - printer.height()) / res);
-			printer.setPaperSize(psize, QPrinter::Inch);
+			    QSizeF psize((target.width() + printer.paperRect().width() - printer.width()) / res, 
+						     (target.height() + printer.paperRect().height() - printer.height()) / res);
+			    printer.setPaperSize(psize, QPrinter::Inch);
 
-			QPainter painter;
-			if (painter.begin(&printer))
-			{
-				svgRenderer.render(&painter, target);
-			}
+			    QPainter painter;
+			    if (painter.begin(&printer))
+			    {
+				    svgRenderer.render(&painter, target);
+			    }
 
-			painter.end();
+			    painter.end();
+            }
 		}
+		if (doMask) {
+			m_pcbGraphicsView->restoreCopperLogoItems(copperLogoItems);
+		}
+		if (doPaste) {
+			m_pcbGraphicsView->restoreCopperLogoItems(holes);
+		}
+
 	}
 
 	m_statusBar->showMessage(tr("Sketch exported"), 2000);
@@ -321,21 +392,47 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 
 }
 
-QString MainWindow::mergeBoardSvg(QString & svg, ItemBase * board, int res, QSizeF & imageSize, bool flip) {
-	QSizeF boardImageSize;
-	QString boardSvg = getBoardSilkscreenSvg(board, res, boardImageSize);
-	if (boardSvg.isEmpty()) return svg;
+QString MainWindow::mergeBoardSvg(QString & svg, ItemBase * board, int res, bool flip, LayerList & viewLayerIDs) {
+	QString boardSvg = getBoardSvg(board, res, viewLayerIDs);
 
-	//QByteArray byteArray;
-	//SvgFileSplitter::changeStrokeWidth(boardSvg, res / 360.0, true, byteArray);
-	imageSize = boardImageSize;
+    LayerList outlineLayerIDs = ViewLayer::outlineLayers();
+	QSizeF imageSize;
+	bool empty;
 
-	return TextUtils::mergeSvg(boardSvg /* QString(byteArray) */, svg, "", flip);
+	QString outlineSvg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, outlineLayerIDs, true, imageSize, board, res, false, false, empty);
+	outlineSvg = GerberGenerator::cleanOutline(outlineSvg);
+    outlineSvg = TextUtils::slamStrokeAndFill(outlineSvg, "black", "0.5", "none");
+
+    if (!boardSvg.isEmpty() && !outlineSvg.isEmpty()) {
+        boardSvg = TextUtils::mergeSvg(boardSvg, outlineSvg, "", false);
+    }
+    else if (boardSvg.isEmpty()) {
+        boardSvg = outlineSvg;
+    }
+	
+	return TextUtils::convertExtendedChars(TextUtils::mergeSvg(boardSvg, svg, "", flip));
 }
 
-QString MainWindow::getBoardSilkscreenSvg(ItemBase * board, int res, QSizeF & imageSize) {
+QString MainWindow::getBoardSvg(ItemBase * board, int res,  LayerList & viewLayerIDs) {
 	if (board == NULL) return ___emptyString___;
 
+    board = board->layerKinChief();
+    QList<ItemBase *> boardLayers;
+    boardLayers << board;
+    foreach (ItemBase * lk, board->layerKin()) {
+        boardLayers << lk;
+    }
+
+    bool gotOne = false;
+    foreach (ItemBase * boardLayer, boardLayers) {
+        if (viewLayerIDs.contains(boardLayer->viewLayerID())) {
+            gotOne = true;
+            break;
+        }
+    }
+
+    if (!gotOne) return "";
+
 	m_pcbGraphicsView->setIgnoreSelectionChangeEvents(true);
 
 	QList<QGraphicsItem *> items = m_pcbGraphicsView->scene()->selectedItems();
@@ -343,10 +440,10 @@ QString MainWindow::getBoardSilkscreenSvg(ItemBase * board, int res, QSizeF & im
 		item->setSelected(false);
 	}
 	board->setSelected(true);
-	LayerList viewLayerIDs;
-	viewLayerIDs << ViewLayer::Silkscreen1;
+
 	bool empty;
-	QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, res, true, false, false, empty);
+    QSizeF imageSize;
+	QString svg = m_pcbGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, true, imageSize, board, res, true, false, empty);
 	board->setSelected(false);
 	foreach (QGraphicsItem * item, items) {
 		item->setSelected(true);
@@ -429,6 +526,8 @@ void MainWindow::doExport() {
 
 void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeBackground) 
 {
+    double resMultiplier = 3;
+
 	QRectF source = m_currentGraphicsView->scene()->itemsBoundingRect();
 	QGraphicsItem * watermark = m_currentGraphicsView->addWatermark(":resources/images/watermark_fritzing_outline.svg");
 	if (watermark) {
@@ -450,10 +549,10 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 	}
 	*/
 
-	QSize imgSize(width, height);
+	QSize imgSize(width * resMultiplier, height * resMultiplier);
 	QImage image(imgSize,format);
-	image.setDotsPerMeterX(1200*254);
-	image.setDotsPerMeterY(1200*254);
+	image.setDotsPerMeterX(InchesPerMeter * GraphicsUtils::SVGDPI * resMultiplier);
+	image.setDotsPerMeterY(InchesPerMeter * GraphicsUtils::SVGDPI * resMultiplier);
 	QPainter painter;
 	QColor color;
 	if (removeBackground) {
@@ -463,7 +562,7 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 
 	painter.begin(&image);
 	//m_currentGraphicsView->render(&painter);
-	QRectF target(0, 0, width, height);
+	QRectF target(0, 0, imgSize.width(), imgSize.height());
 	m_currentGraphicsView->scene()->render(&painter, target, source, Qt::KeepAspectRatio);
 	painter.end();
 
@@ -479,8 +578,8 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 	if (imageWriter.supportsOption(QImageIOHandler::Description)) {
 		imageWriter.setText("", TextUtils::CreatedWithFritzingString);
 	}
+    imageWriter.setQuality(100);
 	bool result = imageWriter.write(image);
-	//bool result = image.save(fileName);
 	if (!result) {
 		QMessageBox::warning(this, tr("Fritzing"), tr("Unable to save %1").arg(fileName) );
 	}
@@ -488,7 +587,7 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 
 void MainWindow::printAux(QPrinter &printer, bool removeBackground, bool paginate) {
 	int res = printer.resolution();
-	qreal scale2 = res / FSvgRenderer::printerScale();
+	double scale2 = res / GraphicsUtils::SVGDPI;
 	DebugDialog::debug(QString("p.w:%1 p.h:%2 pager.w:%3 pager.h:%4 paperr.w:%5 paperr.h:%6 source.w:%7 source.h:%8")
 		.arg(printer.width())
 		.arg(printer.height())
@@ -628,11 +727,11 @@ bool MainWindow::saveAsAux(const QString & fileName) {
 	undoStackCleanChanged(true);
 
 	m_statusBar->showMessage(tr("Saved '%1'").arg(fileName), 2000);
-    setCurrentFile(fileName, true, false, "");
+    setCurrentFile(fileName, true, true);
 
-	if(m_restarting && m_fileName != ___emptyString___) {
+	if(m_restarting && !m_fwFilename.isEmpty()) {
 		QSettings settings;
-		settings.setValue("lastOpenSketch",m_fileName);
+		settings.setValue("lastOpenSketch",m_fwFilename);
 	}
 
    // mark the stack clean so we update the window dirty flag
@@ -648,18 +747,129 @@ bool MainWindow::saveAsAux(const QString & fileName) {
 
 void MainWindow::saveAsAuxAux(const QString & fileName) {
     QApplication::setOverrideCursor(Qt::WaitCursor);
-	connect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
-		this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)), Qt::DirectConnection);
 
-	m_sketchModel->save(fileName, false);
+	connectStartSave(true);
 
-	disconnect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
-			   this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)));
+	QDir dir(this->m_fzzFolder);
+	QStringList nameFilters("*" + FritzingSketchExtension);
+	QFileInfoList fileList = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+	foreach (QFileInfo fileInfo, fileList) {
+		QFile file(fileInfo.absoluteFilePath());
+		file.remove();
+	}
+
+	QString fzName = dir.absoluteFilePath(QFileInfo(fileName).completeBaseName() + FritzingSketchExtension); 
+	m_sketchModel->save(fzName, false);
+
+	saveAsShareable(fileName, false);
+
+	connectStartSave(false);
 
 	QApplication::restoreOverrideCursor();
 }
 
 
+void MainWindow::saveAsShareable(const QString & path, bool saveModel)
+{
+	QString filename = path;
+	QHash<QString, ModelPart *> saveParts;
+	foreach (QGraphicsItem * item, m_pcbGraphicsView->scene()->items()) {
+		ItemBase * itemBase = dynamic_cast<ItemBase *>(item);
+		if (itemBase == NULL) continue;
+		if (itemBase->modelPart()->isCore()) continue;
+	
+		saveParts.insert(itemBase->moduleID(), itemBase->modelPart());
+	}
+	saveBundledNonAtomicEntity(filename, FritzingBundleExtension, this, saveParts.values(), false, m_fzzFolder, saveModel, true);
+
+}
+
+void MainWindow::saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave, bool askForFilename, const QString & destFolderPath, bool saveModel, bool deleteLeftovers) {
+	QStringList names;
+
+	QString fileExt;
+	QString path = defaultSaveFolder() + "/" + QFileInfo(filename).fileName()+"z";
+	QString bundledFileName = askForFilename 
+		? FolderUtils::getSaveFileName(this, tr("Specify a file name"), path, tr("Fritzing (*%1)").arg(extension), &fileExt)
+		: filename;
+
+	if (bundledFileName.isEmpty()) return; // Cancel pressed
+
+    FileProgressDialog progress("Saving...", 0, this);
+
+	if(!alreadyHasExtension(bundledFileName, extension)) {
+		bundledFileName += extension;
+	}
+
+	ProcessEventBlocker::processEvents();
+
+	QDir destFolder;
+	QString dirToRemove;
+	if (destFolderPath.isEmpty()) {
+		destFolder = QDir::temp();
+		FolderUtils::createFolderAnCdIntoIt(destFolder, FolderUtils::getRandText());
+		dirToRemove = destFolder.path();
+	}
+	else {
+		destFolder = QDir(destFolderPath);
+	}
+
+	QString aux = QFileInfo(bundledFileName).fileName();
+	QString destSketchPath = // remove the last "z" from the extension
+							 destFolder.path()+"/"+aux.left(aux.size()-1);
+	DebugDialog::debug("saving entity temporarily to "+destSketchPath);
+
+    QStringList skipSuffixes;
+
+	if (extension.compare(FritzingBundleExtension) == 0) {
+		for (int i = 0; i < m_linkedProgramFiles.count(); i++) {
+			LinkedFile * linkedFile = m_linkedProgramFiles.at(i);
+			QFileInfo fileInfo(linkedFile->linkedFilename);
+			QFile file(linkedFile->linkedFilename);
+			file.copy(destFolder.absoluteFilePath(fileInfo.fileName()));
+		}
+        skipSuffixes << FritzingBinExtension << FritzingBundleExtension;
+	}
+
+	if (saveModel) {
+		QString prevFileName = filename;
+		ProcessEventBlocker::processEvents();
+		bundler->saveAsAux(destSketchPath);
+		filename = prevFileName;
+	}
+
+	foreach(ModelPart* mp, partsToSave) {
+		names.append(saveBundledAux(mp, destFolder));
+	}
+
+	if (deleteLeftovers) {
+		QStringList nameFilters;
+		nameFilters << ("*" + FritzingPartExtension) << "*.svg";   
+		QDir dir(destFolder);
+		QStringList fileList = dir.entryList(nameFilters, QDir::Files | QDir::NoSymLinks);
+		foreach (QString fileName, fileList) {
+			if (!names.contains(fileName)) {
+				QFile::remove(dir.absoluteFilePath(fileName));
+			}
+		}
+	}
+
+	QApplication::processEvents();
+
+	if(!FolderUtils::createZipAndSaveTo(destFolder, bundledFileName, skipSuffixes)) {
+		QMessageBox::warning(
+			this,
+			tr("Fritzing"),
+			tr("Unable to export %1 as shareable").arg(bundledFileName)
+		);
+	}
+
+	if (!dirToRemove.isEmpty()) {
+		FolderUtils::rmdir(dirToRemove);
+	}
+}
+
+
 void MainWindow::createExportActions() {
 
 	m_saveAct = new QAction(tr("&Save"), this);
@@ -672,11 +882,6 @@ void MainWindow::createExportActions() {
 	m_saveAsAct->setStatusTip(tr("Save the current sketch"));
 	connect(m_saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
 
-	m_saveAsBundledAct = new QAction(tr("Save As Shareable..."), this);
-	m_saveAsBundledAct->setShortcut(tr("Alt+Ctrl+S"));
-	m_saveAsBundledAct->setStatusTip(tr("Export current sketch and its non-core parts"));
-	connect(m_saveAsBundledAct, SIGNAL(triggered()), this, SLOT(saveBundledSketch()));
-
 	m_shareOnlineAct = new QAction(tr("Share online..."), this);
 	m_shareOnlineAct->setStatusTip(tr("Post a project to the Fritzing website"));
 	connect(m_shareOnlineAct, SIGNAL(triggered()), this, SLOT(shareOnline()));
@@ -721,35 +926,21 @@ void MainWindow::createExportActions() {
 	m_exportEagleAct->setStatusTip(tr("Export the current sketch to Eagle CAD"));
 	connect(m_exportEagleAct, SIGNAL(triggered()), this, SLOT(doExport()));
 
-        m_exportGerberAct = new QAction(tr("Extended Gerber (RS-274X)..."), this);
+    m_exportGerberAct = new QAction(tr("Extended Gerber (RS-274X)..."), this);
 	m_exportGerberAct->setData(gerberActionType);
-        m_exportGerberAct->setStatusTip(tr("Export the current sketch to Extended Gerber format (RS-274X) for professional PCB production"));
+    m_exportGerberAct->setStatusTip(tr("Export the current sketch to Extended Gerber format (RS-274X) for professional PCB production"));
 	connect(m_exportGerberAct, SIGNAL(triggered()), this, SLOT(doExport()));
 
 	m_exportEtchablePdfAct = new QAction(tr("Etchable (PDF)..."), this);
 	m_exportEtchablePdfAct->setStatusTip(tr("Export the current sketch to PDF for DIY PCB production (photoresist)"));
 	m_exportEtchablePdfAct->setProperty("svg", false);
-	m_exportEtchablePdfAct->setProperty("flip", false);
 	connect(m_exportEtchablePdfAct, SIGNAL(triggered()), this, SLOT(exportEtchable()));
 
-	m_exportEtchablePdfFlipAct = new QAction(tr("Etchable mirrored (PDF)..."), this);
-	m_exportEtchablePdfFlipAct->setStatusTip(tr("Export the current sketch to PDF for DIY PCB production (tone transfer)"));
-	m_exportEtchablePdfFlipAct->setProperty("svg", false);
-	m_exportEtchablePdfFlipAct->setProperty("flip", true);
-	connect(m_exportEtchablePdfFlipAct, SIGNAL(triggered()), this, SLOT(exportEtchable()));
-
 	m_exportEtchableSvgAct = new QAction(tr("Etchable (SVG)..."), this);
 	m_exportEtchableSvgAct->setStatusTip(tr("Export the current sketch to SVG for DIY PCB production (photoresist)"));
 	m_exportEtchableSvgAct->setProperty("svg", true);
-	m_exportEtchableSvgAct->setProperty("flip", false);
 	connect(m_exportEtchableSvgAct, SIGNAL(triggered()), this, SLOT(exportEtchable()));
 
-	m_exportEtchableSvgFlipAct = new QAction(tr("Etchable mirrored (SVG)..."), this);
-	m_exportEtchableSvgFlipAct->setStatusTip(tr("Export the current sketch to SVG for DIY PCB production (tone transfer)"));
-	m_exportEtchableSvgFlipAct->setProperty("svg", true);
-	m_exportEtchableSvgFlipAct->setProperty("flip", true);
-	connect(m_exportEtchableSvgFlipAct, SIGNAL(triggered()), this, SLOT(exportEtchable()));
-
 	/*m_pageSetupAct = new QAction(tr("&Page Setup..."), this);
 	m_pageSetupAct->setShortcut(tr("Shift+Ctrl+P"));
 	m_pageSetupAct->setStatusTip(tr("Setup the current sketch page"));
@@ -825,7 +1016,7 @@ void MainWindow::exportToEagle() {
 	*/
 }
 
-void MainWindow::exportSvg(qreal res, bool selectedItems, bool flatten) {
+void MainWindow::exportSvg(double res, bool selectedItems, bool flatten) {
 	QString path = defaultSaveFolder();
 	QString fileExt;
 	QString fileName = FolderUtils::getSaveFileName(this,
@@ -848,7 +1039,7 @@ void MainWindow::exportSvg(qreal res, bool selectedItems, bool flatten) {
 
 	QSizeF imageSize;
 	bool empty;
-	QString svg = m_currentGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, false, imageSize, NULL, res, selectedItems, flatten, false, empty);
+	QString svg = m_currentGraphicsView->renderToSVG(GraphicsUtils::SVGDPI, viewLayerIDs, false, imageSize, NULL, res, selectedItems, false, empty);
 	if (svg.isEmpty()) {
 		// tell the user something reasonable
 		return;
@@ -858,17 +1049,12 @@ void MainWindow::exportSvg(qreal res, bool selectedItems, bool flatten) {
 		exportSvgWatermark(svg, res);
 	}
 
-	QFile file(fileName);
-	file.open(QIODevice::WriteOnly);
-	QTextStream out(&file);
-	out.setCodec("UTF-8");
-	out << svg;
-	file.close();
+    TextUtils::writeUtf8(fileName, TextUtils::convertExtendedChars(svg));
 	delete fileProgressDialog;
 
 }
 
-void MainWindow::exportSvgWatermark(QString & svg, qreal res)
+void MainWindow::exportSvgWatermark(QString & svg, double res)
 {
 	QFile file(":/resources/images/watermark_fritzing_outline.svg");
 	if (!file.open(QFile::ReadOnly)) return;
@@ -878,11 +1064,8 @@ void MainWindow::exportSvgWatermark(QString & svg, qreal res)
 
 	if (!watermarkSvg.contains("<svg")) return;
 
-	QXmlStreamReader wStreamReader(watermarkSvg);
-	QSizeF watermarkSize = FSvgRenderer::parseForWidthAndHeight(wStreamReader);
-
-	QXmlStreamReader streamReader(svg);
-	QSizeF svgSize = FSvgRenderer::parseForWidthAndHeight(streamReader);
+	QSizeF watermarkSize = TextUtils::parseForWidthAndHeight(watermarkSvg);
+	QSizeF svgSize = TextUtils::parseForWidthAndHeight(svg);
 
 	SvgFileSplitter splitter;
 	bool result = splitter.splitString(watermarkSvg, "watermark");
@@ -935,8 +1118,7 @@ void MainWindow::exportBOM() {
 	foreach (ItemBase * itemBase, partList) {
 		if (itemBase->itemType() != ModelPart::Part) continue;
 
-        QString label = itemBase->instanceTitle();
-        QString desc = itemBase->title();
+        QString desc = getBomProps(itemBase);
 
         if(!shoppingList.contains(desc)) {
             shoppingList.insert(desc, 1);
@@ -961,13 +1143,14 @@ void MainWindow::exportBOM() {
         it.next();
 		ItemBase * itemBase = descrs.value(it.key());
 
-		shoppingListString += bomRowTemplate.arg(it.value()).arg(it.key()).arg(getBomProps(itemBase));
+		// it.value() = count; it.key() = bomProps
+		shoppingListString += bomRowTemplate.arg(it.value()).arg(itemBase->title()).arg(it.key());
     }
 
 	QString bom = bomTemplate
 		.arg("Fritzing Bill of Materials")
-		.arg(QFileInfo(m_fileName).fileName())
-		.arg(m_fileName)
+		.arg(QFileInfo(m_fwFilename).fileName())
+		.arg(m_fwFilename)
 		.arg(QDateTime::currentDateTime().toString("dddd, MMMM d yyyy, hh:mm:ss"))
 		.arg(assemblyString)
 		.arg(shoppingListString)
@@ -998,18 +1181,12 @@ void MainWindow::exportBOM() {
 		fileName += bomActionType;
     }
 
-    QFile fp(fileName);
-   	if (fp.open(QIODevice::WriteOnly)) {
-   		QTextStream out(&fp);
-		out.setCodec("UTF-8");
-   		out << bom;
-		fp.close();
-	}
-	else {
+   	if (!TextUtils::writeUtf8(fileName, bom)) {
 		QMessageBox::warning(this, tr("Fritzing"), tr("Unable to save BOM file, but the text is on the clipboard."));
 	}
 
-	if (fp.exists()) {
+    QFileInfo info(fileName);
+	if (info.exists()) {
 		QDesktopServices::openUrl(QString("file:///%1").arg(fileName));
 	}
 
@@ -1029,7 +1206,7 @@ void MainWindow::exportNetlist() {
 	doc.setContent(QString("<?xml version='1.0' encoding='UTF-8'?>\n") + TextUtils::CreatedWithFritzingXmlComment);
 	QDomElement netlist = doc.createElement("netlist");
 	doc.appendChild(netlist);
-	netlist.setAttribute("sketch", QFileInfo(m_fileName).fileName());
+	netlist.setAttribute("sketch", QFileInfo(m_fwFilename).fileName());
     netlist.setAttribute("date", QDateTime::currentDateTime().toString());
 
 	// TODO: filter out 'ignore' connectors
@@ -1128,14 +1305,10 @@ void MainWindow::exportNetlist() {
 
 }
 
-
-
 FileProgressDialog * MainWindow::exportProgress() {
-	return (new FileProgressDialog("Exporting...", 0, this));
+    return (new FileProgressDialog("Exporting...", 0, this));
 }
 
-
-
 void MainWindow::exportNormalizedSVG() {
 	exportSvg(GraphicsUtils::StandardFritzingDPI, true, false);
 }
@@ -1170,3 +1343,86 @@ QString MainWindow::getBomProps(ItemBase * itemBase)
 	return pString;
 }
 
+void MainWindow::exportToGerber() {
+
+    //NOTE: this assumes just one board per sketch
+
+    int boardCount;
+	ItemBase * board = m_pcbGraphicsView->findSelectedBoard(boardCount);
+
+    // barf an error if there's no board
+    if (boardCount == 0) {
+        QMessageBox::critical(this, tr("Fritzing"),
+                   tr("Your sketch does not have a board yet!  Please add a PCB in order to export to Gerber."));
+        return;
+    }
+    if (board == NULL) {
+        QMessageBox::critical(this, tr("Fritzing"),
+                   tr("Gerber export can only handle one board at a time--please select the board you want to export."));
+        return;
+    }
+
+    QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
+                                             defaultSaveFolder(),
+                                             QFileDialog::ShowDirsOnly
+                                             | QFileDialog::DontResolveSymlinks);
+
+	if (exportDir.isEmpty()) return;
+
+	FileProgressDialog * fileProgressDialog = exportProgress();
+
+	FolderUtils::setOpenSaveFolder(exportDir);
+	m_pcbGraphicsView->saveLayerVisibility();
+	m_pcbGraphicsView->setAllLayersVisible(true);
+
+    QFileInfo info(m_fwFilename);
+    QString prefix = info.completeBaseName();
+    if (boardCount > 1) {
+        prefix += QString("_%1_%2").arg(board->instanceTitle()).arg(board->id());
+    }
+	GerberGenerator::exportToGerber(prefix, exportDir, board, m_pcbGraphicsView, true);
+	m_pcbGraphicsView->restoreLayerVisibility();
+	m_statusBar->showMessage(tr("Sketch exported to Gerber"), 2000);
+
+	delete fileProgressDialog;
+}
+
+void MainWindow::exportToGerber(const QString & exportDir) {
+	GerberGenerator::exportToGerber(m_fwFilename, exportDir, NULL, m_pcbGraphicsView, false);
+}
+
+void MainWindow::connectStartSave(bool doConnect) {
+
+	if (doConnect) {
+		connect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
+				this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)), Qt::DirectConnection);
+	}
+	else {
+		disconnect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
+				this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)));
+	}
+}
+
+QString MainWindow::constructFileName(const QString & differentiator, const QString & suffix) {
+	QString fn = QFileInfo(m_fwFilename).completeBaseName();
+	fn += "_" + (differentiator.isEmpty() ? m_currentGraphicsView->getShortName() : differentiator);
+	return fn + suffix;
+}
+
+void MainWindow::massageOutput(QString & svg, bool doMask, bool doSilk, bool doPaste, QString & maskTop, QString & maskBottom, const QString & fileName, ItemBase * board, int dpi, const LayerList & viewLayerIDs)
+{
+	if (doPaste) {
+        // must test doPaste first, since doMask will also be true
+        svg = pcbView()->makePasteMask(svg, board, dpi, viewLayerIDs);
+    }
+	else if (doSilk) {
+		QString use = (fileName.contains("bottom")) ? maskBottom : maskTop;
+		use = TextUtils::expandAndFill(use, "white", GerberGenerator::MaskClearanceMils * 2 * dpi / 1000);
+		svg = TextUtils::mergeSvg(svg, use, "", false);
+	}
+    else if (doMask) {
+		if (fileName.contains("bottom")) maskBottom = svg; else maskTop = svg;
+		svg = TextUtils::expandAndFill(svg, "black", GerberGenerator::MaskClearanceMils * 2 * dpi / 1000);
+	}
+}
+
