@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5218 $:
+$Revision: 6268 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-11 10:48:15 +0200 (Mon, 11 Jul 2011) $
+$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 
 ********************************************************************/
 
@@ -41,29 +41,12 @@ long ModelPart::m_nextIndex = 0;
 const int ModelPart::indexMultiplier = 10;
 QStringList ModelPart::m_possibleFolders;
 
-static QHash<QString, QList<ModelPart *>* > InstanceTitleIncrements;
+typedef QHash<QString, ModelPartList*> InstanceTitleIncrementHash;
+static QHash<QObject *, InstanceTitleIncrementHash *> AllInstanceTitleIncrements; 
+InstanceTitleIncrementHash NullInstanceTitleIncrements;
+
 static const QRegExp InstanceTitleRegExp("^(.*[^\\d])(\\d+)$");
 
-QList<ModelPart *> * ensureInstanceTitleIncrements(const QString & prefix) {
-	QList<ModelPart *> * modelParts = InstanceTitleIncrements.value(prefix, NULL);
-	if (modelParts == NULL) {
-		modelParts =  new QList<ModelPart *>;
-		InstanceTitleIncrements.insert(prefix, modelParts);
-	}
-	return modelParts;
-}
-
-void clearOldInstanceTitle(ModelPart * modelPart, const QString & title) 
-{
-	int ix = InstanceTitleRegExp.indexIn(title);
-	if (ix >= 0) {
-		QString prefix = InstanceTitleRegExp.cap(1);
-		QList<ModelPart *> * modelParts = InstanceTitleIncrements.value(prefix, NULL);
-		if (modelParts) {
-			modelParts->removeOne(modelPart);
-		}
-	}
-}
 
 ////////////////////////////////////////
 
@@ -86,15 +69,23 @@ ModelPart::ModelPart(QDomDocument * domDocument, const QString & path, ItemType
 
 void ModelPart::commonInit(ItemType type) {
 	m_type = type;
-	m_core = false;
-	m_alien = false;
+	m_locationFlags = 0;
 	m_indexSynched = false;
 }
 
 ModelPart::~ModelPart() {
 	//DebugDialog::debug(QString("deleting modelpart %1 %2").arg((long) this, 0, 16).arg(m_index));
 
-	clearOldInstanceTitle(this, m_instanceTitle);
+	clearOldInstanceTitle(m_instanceTitle);
+
+	InstanceTitleIncrementHash * itih = AllInstanceTitleIncrements.value(this);
+	if (itih) {
+		AllInstanceTitleIncrements.remove(this);
+		foreach (ModelPartList * list, itih->values()) {
+			delete list;
+		}
+		delete itih;
+	}
 
 	if (m_originalModelPartShared) {		// TODO: make this a QSharedPointer
 		if (m_modelPartShared) {
@@ -114,7 +105,7 @@ ModelPart::~ModelPart() {
 	clearBuses();
 }
 
-const QString & ModelPart::moduleID() {
+const QString & ModelPart::moduleID() const {
 	if (m_modelPartShared != NULL) return m_modelPartShared->moduleID();
 
 	return ___emptyString___;
@@ -155,9 +146,7 @@ void ModelPart::copy(ModelPart * modelPart) {
 
 	m_type = modelPart->itemType();
 	m_modelPartShared = modelPart->modelPartShared();
-	m_core = modelPart->isCore();
-	m_contrib = modelPart->isContrib();
-	m_alien = modelPart->isAlien();
+	m_locationFlags = modelPart->m_locationFlags;
 }
 
 void ModelPart::copyNew(ModelPart * modelPart) {
@@ -176,6 +165,10 @@ ModelPartShared * ModelPart::modelPartShared() {
 	return m_modelPartShared;
 }
 
+ModelPartSharedRoot * ModelPart::modelPartSharedRoot() {
+	return qobject_cast<ModelPartSharedRoot *>(m_modelPartShared);
+}
+
 void ModelPart::setModelPartShared(ModelPartShared * modelPartShared) {
 	m_modelPartShared = modelPartShared;
 }
@@ -196,11 +189,28 @@ ItemBase * ModelPart::viewItem(QGraphicsScene * scene) {
 	return NULL;
 }
 
+ItemBase * ModelPart::viewItem(ViewIdentifierClass::ViewIdentifier viewIdentifier) {
+	foreach (ItemBase * itemBase, m_viewItems) {
+		if (itemBase->viewIdentifier() == viewIdentifier) return itemBase;
+	}
+
+	return NULL;
+}
+
 void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & streamWriter, bool startDocument) {
 	if (startDocument) {
 		streamWriter.writeStartDocument();
     	streamWriter.writeStartElement("module");
 		streamWriter.writeAttribute("fritzingVersion", Version::versionString());
+		ModelPartSharedRoot * root = modelPartSharedRoot();
+		if (root) {
+			if (!root->icon().isEmpty()) {
+				streamWriter.writeAttribute("icon", root->icon());
+			}
+			if (!root->searchTerm().isEmpty()) {
+				streamWriter.writeAttribute("search", root->searchTerm());
+			}
+		}
 		QString title = this->title();
 		if(!title.isNull() && !title.isEmpty()) {
 			streamWriter.writeTextElement("title",title);
@@ -251,6 +261,27 @@ void ModelPart::saveInstance(QXmlStreamWriter & streamWriter)
 		}
 	}
 
+	bool writeLocal = false;
+	foreach (Connector * connector, this->connectors()) {
+		if (!connector->connectorLocalName().isEmpty()) {
+			writeLocal = true;
+			break;
+		}
+	}
+
+	if (writeLocal) {
+		streamWriter.writeStartElement("localConnectors");
+		foreach (Connector * connector, this->connectors()) {
+			if (!connector->connectorLocalName().isEmpty()) {
+				streamWriter.writeStartElement("localConnector");
+				streamWriter.writeAttribute("id", connector->connectorSharedID());
+				streamWriter.writeAttribute("name", connector->connectorLocalName());
+				streamWriter.writeEndElement();
+			}
+		}
+		streamWriter.writeEndElement();
+	}
+
 	foreach (QByteArray byteArray, dynamicPropertyNames()) {
 		streamWriter.writeStartElement("property");
 		streamWriter.writeAttribute("name",  byteArray.data());
@@ -459,9 +490,15 @@ const QDomElement & ModelPart::instanceDomElement() {
 }
 
 const QString & ModelPart::title() {
+	if (!m_localTitle.isEmpty()) return m_localTitle;
+
 	if (m_modelPartShared != NULL) return m_modelPartShared->title();
 
-	return ___emptyString___;
+	return m_localTitle;
+}
+
+void ModelPart::setLocalTitle(const QString & localTitle) {
+	m_localTitle = localTitle;
 }
 
 const QString & ModelPart::version() {
@@ -519,27 +556,44 @@ bool ModelPart::ignoreTerminalPoints() {
 }
 
 bool ModelPart::isCore() {
-	return m_core;
+	return (m_locationFlags & CoreFlag) != 0;
 }
 
 void ModelPart::setCore(bool core) {
-	m_core = core;
+	setLocationFlag(core, CoreFlag);
 }
 
 bool ModelPart::isContrib() {
-	return m_contrib;
+	return (m_locationFlags & ContribFlag) != 0;
 }
 
 void ModelPart::setContrib(bool contrib) {
-	m_contrib = contrib;
+	setLocationFlag(contrib, ContribFlag);
 }
 
 bool ModelPart::isAlien() {
-	return m_alien;
+	return (m_locationFlags & AlienFlag) != 0;;
 }
 
 void ModelPart::setAlien(bool alien) {
-	m_alien = alien;
+	setLocationFlag(alien, AlienFlag);
+}
+
+bool ModelPart::isFzz() {
+	return (m_locationFlags & FzzFlag) != 0;;
+}
+
+void ModelPart::setFzz(bool fzz) {
+	setLocationFlag(fzz, FzzFlag);
+}
+
+void ModelPart::setLocationFlag(bool setting, LocationFlag flag) {
+	if (setting) {
+		m_locationFlags |= flag;
+	}
+	else {
+		m_locationFlags &= ~flag;
+	}
 }
 
 bool ModelPart::hasViewIdentifier(ViewIdentifierClass::ViewIdentifier viewIdentifier) {
@@ -580,44 +634,6 @@ QList<ModelPart*> ModelPart::getAllNonCoreParts() {
 	return retval;
 }
 
-QList<SvgAndPartFilePath> ModelPart::getAvailableViewFiles() {
-	QDomElement viewsElems = modelPartShared()->domDocument()->documentElement().firstChildElement("views");
-	QHash<ViewIdentifierClass::ViewIdentifier, SvgAndPartFilePath> viewImages;
-
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::IconView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::BreadboardView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::SchematicView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::PCBView);
-
-	return viewImages.values();
-}
-
-void ModelPart::grabImagePath(QHash<ViewIdentifierClass::ViewIdentifier, SvgAndPartFilePath> &viewImages, QDomElement &viewsElems, ViewIdentifierClass::ViewIdentifier viewId) {
-	QDomElement viewElem = viewsElems.firstChildElement(ViewIdentifierClass::viewIdentifierXmlName(viewId));
-	if(!viewElem.isNull()) {
-		QString partspath = FolderUtils::getUserDataStorePath("parts")+"/svg";
-		QDomElement layerElem = viewElem.firstChildElement("layers");
-		if (!layerElem.isNull()) {
-			QString imagepath = layerElem.attribute("image");
-			QString folderinparts = inWhichFolder(partspath, imagepath);
-			if(folderinparts != ___emptyString___) {
-				SvgAndPartFilePath st(partspath,folderinparts,imagepath);
-				viewImages[viewId] = st;
-			}
-		}
-	}
-}
-
-QString ModelPart::inWhichFolder(const QString &partspath, const QString &imagepath) {
-	QStringList pf = possibleFolders();
-	for(int i=0; i < pf.size(); i++) {
-		if (QFileInfo( partspath+"/"+pf[i]+"/"+imagepath ).exists()) {
-			return pf[i];
-		}
-	}
-	return ___emptyString___;
-}
-
 bool ModelPart::hasViewID(long id) {
 	foreach (ItemBase * item, m_viewItems) {
 		if (item->id() == id) return true;
@@ -626,7 +642,7 @@ bool ModelPart::hasViewID(long id) {
 	return false;
 }
 
-const QString & ModelPart::instanceTitle() {
+const QString & ModelPart::instanceTitle() const {
 	return m_instanceTitle;
 }
 
@@ -638,10 +654,57 @@ void ModelPart::setInstanceText(QString text) {
 	m_instanceText = text;
 }
 
+void ModelPart::clearOldInstanceTitle(const QString & title) 
+{
+	InstanceTitleIncrementHash * itih = NULL;
+	if (parent() == NULL) {
+		itih = &NullInstanceTitleIncrements;
+	}
+	else {
+		itih = AllInstanceTitleIncrements.value(parent(), NULL);
+	}
+
+	if (itih == NULL) return;
+
+	//DebugDialog::debug(QString("clearing title:%1 ix:%2").arg(title).arg(modelPart->modelIndex()));
+	QString prefix = title;
+	int ix = InstanceTitleRegExp.indexIn(title);
+	if (ix >= 0) {
+		prefix = InstanceTitleRegExp.cap(1);
+	}
+	ModelPartList * modelParts = itih->value(prefix, NULL);
+	if (modelParts) {
+		modelParts->removeOne(this);
+		//DebugDialog::debug(QString("\tc:%1").arg(modelParts->count()));
+	}
+}
+
+ModelPartList * ModelPart::ensureInstanceTitleIncrements(const QString & prefix) 
+{
+	InstanceTitleIncrementHash * itih = NULL;
+	if (parent() == NULL) {
+		itih = &NullInstanceTitleIncrements;
+	}
+	else {
+		itih = AllInstanceTitleIncrements.value(parent(), NULL);
+		if (itih == NULL) {
+			itih = new InstanceTitleIncrementHash;
+			AllInstanceTitleIncrements.insert(parent(), itih);
+		}
+	}
+
+	ModelPartList * modelParts = itih->value(prefix, NULL);
+	if (modelParts == NULL) {
+		modelParts =  new ModelPartList;
+		itih->insert(prefix, modelParts);
+	}
+	return modelParts;
+}
+
 void ModelPart::setInstanceTitle(QString title) {
 	if (title.compare(m_instanceTitle) == 0) return;
 
-	clearOldInstanceTitle(this, m_instanceTitle);
+	clearOldInstanceTitle(m_instanceTitle);
 
 	m_instanceTitle = title;
 
@@ -649,9 +712,10 @@ void ModelPart::setInstanceTitle(QString title) {
 	int ix = InstanceTitleRegExp.indexIn(title);
 	if (ix >= 0) {
 		prefix = InstanceTitleRegExp.cap(1);
+		ModelPartList * modelParts = ensureInstanceTitleIncrements(prefix);
+		modelParts->append(this);
 	}
-	QList<ModelPart *> * modelParts = ensureInstanceTitleIncrements(prefix);
-	modelParts->append(this);
+	//DebugDialog::debug(QString("adding title:%1 ix:%2 c:%3").arg(title).arg(modelIndex()).arg(modelParts->count()));
 }
 
 QString ModelPart::getNextTitle(const QString & title) {
@@ -660,10 +724,29 @@ QString ModelPart::getNextTitle(const QString & title) {
 	if (ix >= 0) {
 		prefix = InstanceTitleRegExp.cap(1);
 	}
+	else {
+		bool allDigits = true;
+		foreach (QChar c, title) {
+			if (!c.isDigit()) {
+				allDigits = false;
+				break;
+			}
+		}
+		if (allDigits) {
+			return title;
+		}
+	}
+
 	// TODO: if this were a sorted list, 
-	QList<ModelPart *> * modelParts = ensureInstanceTitleIncrements(prefix);
+	ModelPartList * modelParts = ensureInstanceTitleIncrements(prefix);
 	int highestSoFar = 0;
+	bool gotNull = false;
 	foreach (ModelPart * modelPart, *modelParts) {
+		if (modelPart == NULL) {
+			gotNull = true;
+			continue;
+		}
+
 		QString title = modelPart->instanceTitle();
 		title.remove(0, prefix.length());
 		int count = title.toInt();			// returns zero on failure
@@ -672,6 +755,10 @@ QString ModelPart::getNextTitle(const QString & title) {
 		}
 	}
 
+	if (gotNull) {
+		modelParts->removeAll(NULL);
+	}
+
 	//DebugDialog::debug(QString("returning increment %1, %2").arg(prefix).arg(highestSoFar + 1));
 	return QString("%1%2").arg(prefix).arg(highestSoFar + 1);
 }
@@ -680,22 +767,23 @@ void ModelPart::setOrderedChildren(QList<QObject*> children) {
 	m_orderedChildren = children;
 }
 
-void ModelPart::setProp(const char * name, const QVariant & value) {
+void ModelPart::setLocalProp(const char * name, const QVariant & value) {
+    //DebugDialog::debug(QString("mp set prop %1 %2").arg(name).arg(value.toString()));
 	setProperty(name, value);
 }
 
-QVariant ModelPart::prop(const char * name) const {
+QVariant ModelPart::localProp(const char * name) const {
 	return property(name);
 }
 
-void ModelPart::setProp(const QString & name, const QVariant & value) {
+void ModelPart::setLocalProp(const QString & name, const QVariant & value) {
 	QByteArray b = name.toLatin1();
-	setProp(b.data(), value);
+	setLocalProp(b.data(), value);
 }
 
-QVariant ModelPart::prop(const QString & name) const {
+QVariant ModelPart::localProp(const QString & name) const {
 	QByteArray b = name.toLatin1();
-	return prop(b.data());
+	return localProp(b.data());
 }
 
 const QStringList & ModelPart::possibleFolders() {
@@ -773,9 +861,21 @@ ModelPart::ItemType ModelPart::itemType() const
 	return m_type; 
 };
 
-bool ModelPart::hasBendableLeg()
+void ModelPart::setConnectorLocalName(const QString & id, const QString & name)
 {
-	if (m_modelPartShared == NULL) return false;
+	if (id.isEmpty()) return;
+	Connector * connector = m_connectorHash.value(id, NULL);
+	if (connector) {
+		connector->setConnectorLocalName(name);
+	}
+}
 
-	return (m_modelPartShared->hasBendableLeg());
+QString ModelPart::family(){
+	if (m_modelPartShared) return m_modelPartShared->family();
+
+	return "";
+}
+
+bool ModelPart::hasViewItems() {
+	return (m_viewItems.count() > 0);
 }
