@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5193 $:
+$Revision: 6289 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-06 23:50:51 +0200 (Wed, 06 Jul 2011) $
+$Date: 2012-08-14 04:06:50 +0200 (Tue, 14 Aug 2012) $
 
 ********************************************************************/
 
@@ -43,14 +43,32 @@ void copyPinAttributes(QDomElement & from, QDomElement & to)
 	if (!hybrid.isEmpty()) {
 		to.setAttribute("hybrid", hybrid);
 	}
-	QString bendable = from.attribute("bendableLeg");
-	if (!bendable.isEmpty()) {
-		to.setAttribute("bendableLeg", bendable);
+	QString legId = from.attribute("legId");
+	if (!legId.isEmpty()) {
+		to.setAttribute("legId", legId);
 	}
 }
 
 ///////////////////////////////////////////////
 
+const QString & ModelPartSharedRoot::icon() {
+	return m_icon;
+}
+
+void ModelPartSharedRoot::setIcon(const QString & filename) {
+	m_icon = filename;
+}
+
+const QString & ModelPartSharedRoot::searchTerm() {
+	return m_searchTerm;
+}
+
+void ModelPartSharedRoot::setSearchTerm(const QString & searchTerm) {
+	m_searchTerm = searchTerm;
+}
+
+///////////////////////////////////////////////
+
 const QString ModelPartShared::PartNumberPropertyName = "part number";
 
 ModelPartShared::ModelPartShared() {
@@ -122,7 +140,7 @@ ModelPartShared::ModelPartShared(QDomDocument * domDocument, const QString & pat
 
 void ModelPartShared::commonInit() {
 	m_moduleID = "";
-	m_hasBendableLeg = m_flippedSMD = m_connectorsInitialized = m_ignoreTerminalPoints = m_partlyLoaded = m_needsCopper1 = false;
+	m_flippedSMD = m_connectorsInitialized = m_ignoreTerminalPoints = m_partlyLoaded = m_needsCopper1 = false;
 }
 
 ModelPartShared::~ModelPartShared() {
@@ -285,6 +303,7 @@ void ModelPartShared::setFamily(const QString &family) {
 QHash<QString,QString> & ModelPartShared::properties() {
 	return m_properties;
 }
+
 void ModelPartShared::setProperties(const QHash<QString,QString> &properties) {
 	m_properties = properties;
 	ensurePartNumberProperty();
@@ -363,7 +382,6 @@ void ModelPartShared::initConnectors() {
 	QDomElement connector = connectors.firstChildElement("connector");
 	while (!connector.isNull()) {
 		ConnectorShared * connectorShared = new ConnectorShared(connector);
-		if (connectorShared->hasBendableLeg()) m_hasBendableLeg = true;
 		m_connectorSharedHash.insert(connectorShared->id(), connectorShared);
 
 		connector = connector.nextSiblingElement("connector");
@@ -441,10 +459,17 @@ bool ModelPartShared::needsCopper1() {
 	return m_needsCopper1;
 }
 
-void ModelPartShared::connectorIDs(ViewIdentifierClass::ViewIdentifier viewIdentifier, ViewLayer::ViewLayerID viewLayerID, QStringList & connectorIDs, QStringList & terminalIDs) {
-	foreach (ConnectorShared * connector, m_connectorSharedHash.values()) {
-		connectorIDs.append(connector->pin(viewIdentifier, viewLayerID));
-		terminalIDs.append(connector->terminal(viewIdentifier, viewLayerID));
+void ModelPartShared::connectorIDs(ViewIdentifierClass::ViewIdentifier viewIdentifier, ViewLayer::ViewLayerID viewLayerID, QStringList & connectorIDs, QStringList & terminalIDs, QStringList & legIDs) {
+	foreach (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
+		SvgIdLayer * svgIdLayer = connectorShared->fullPinInfo(viewIdentifier, viewLayerID);
+		if (svgIdLayer == NULL) {
+			continue;
+		}
+		else {
+			connectorIDs.append(svgIdLayer->m_svgId);
+			terminalIDs.append(svgIdLayer->m_terminalId);
+			legIDs.append(svgIdLayer->m_legId);
+		}
 	}
 }
 
@@ -600,7 +625,7 @@ bool ModelPartShared::checkNeedsCopper1(QDomElement & copper0, QDomElement & cop
 	if (!m_replacedby.isEmpty()) return false;
 
 	bool ok;
-	qreal versionNumber = m_version.toDouble(&ok);
+	double versionNumber = m_version.toDouble(&ok);
 	if (ok) {
 		if (versionNumber >= 4.0) return false;
 	}
@@ -663,7 +688,3 @@ void ModelPartShared::ensurePartNumberProperty() {
 	}
 }
 
-bool ModelPartShared::hasBendableLeg()
-{
-	return m_hasBendableLeg;
-}
