@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5204 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
@@ -31,16 +31,16 @@ $Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QSettings>
-#include <QInputDialog>
 #include <QWidgetAction>
+#include <QColorDialog>
+#include <QBuffer>
+#include <QSvgGenerator>
 
 #include "partsbinpalettewidget.h"
-#include "binmanager/stacktabwidget.h"
 #include "partsbincommands.h"
 #include "partsbiniconview.h"
 #include "partsbinlistview.h"
 #include "simpleeditablelabelwidget.h"
-#include "binmanager/binmanager.h"
 #include "searchlineedit.h"
 #include "../utils/misc.h"
 #include "../debugdialog.h"
@@ -48,19 +48,34 @@ $Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
 #include "../infoview/htmlinfoview.h"
 #include "../utils/fileprogressdialog.h"
 #include "../utils/folderutils.h"
+#include "../utils/textutils.h"
+
+
+static QString CustomIconName = "Custom1.png";
+static QString CustomIconTitle = "Fritzing Custom Icon";
+
+inline bool isCustomSvg(const QString & string) {
+	return string.startsWith("<?xml") && string.contains(CustomIconTitle);
+}
+
+static QHash<QString, PaletteModel *> PaletteBinModels;
+
+//////////////////////////////////////////////
 
 PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoView *infoView, WaitPushUndoStack *undoStack, BinManager* manager) :
 	QFrame(manager)
 {
+	m_icon = NULL;
 	m_searchLineEdit = NULL;
 	m_saveQuietly = false;
+	m_fastLoaded = false;
+	m_model = NULL;
 
 	m_loadingProgressDialog = NULL;
 
 	setAcceptDrops(true);
 	setAllowsChanges(true);
 
-	m_tabWidget = NULL;
 	m_manager = manager;
 
 	m_refModel = refModel;
@@ -69,18 +84,36 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 
 	Q_UNUSED(undoStack);
 
-	m_undoStack = new WaitPushUndoStack(this);PartsBinPaletteWidget::
+	m_undoStack = new WaitPushUndoStack(this);
 	connect(m_undoStack, SIGNAL(cleanChanged(bool)), this, SLOT(undoStackCleanChanged(bool)) );
 
-	setupButtons();
-	setupFooter();
+	setupHeader();
 
-	m_iconView = new PartsBinIconView(m_refModel, this, m_binContextMenu, m_partContextMenu);
+	m_iconView = new PartsBinIconView(m_refModel, this);
 	m_iconView->setInfoView(infoView);
 
-	m_listView = new PartsBinListView(m_refModel, this, m_binContextMenu, m_partContextMenu);
+	m_listView = new PartsBinListView(m_refModel, this);
 	m_listView->setInfoView(infoView);
 
+	m_stackedWidget = new QStackedWidget(this);
+	m_stackedWidget->addWidget(m_iconView);
+	m_stackedWidget->addWidget(m_listView);
+
+	QVBoxLayout * vbl = new QVBoxLayout(this);
+    vbl->setMargin(3);
+    vbl->setSpacing(0);
+	vbl->addWidget(m_header);
+
+	QFrame * separator = new QFrame();
+	separator->setMaximumHeight(1);
+	separator->setObjectName("partsBinHeaderSeparator");
+    separator->setFrameShape(QFrame::HLine);
+    separator->setFrameShadow(QFrame::Plain);
+	vbl->addWidget(separator);
+
+	vbl->addWidget(m_stackedWidget);
+	this->setLayout(vbl);
+
 	setObjectName("partsBinContainer");
 	toIconView();
 
@@ -90,17 +123,16 @@ PartsBinPaletteWidget::PartsBinPaletteWidget(ReferenceModel *refModel, HtmlInfoV
 	connect(m_listView, SIGNAL(currentRowChanged(int)), m_iconView, SLOT(setSelected(int)));
 	connect(m_iconView, SIGNAL(selectionChanged(int)), m_listView, SLOT(setSelected(int)));
 
-	connect(m_listView, SIGNAL(currentRowChanged(int)), this, SLOT(updateBinPartsMenu()));
-	connect(m_iconView, SIGNAL(selectionChanged(int)), this, SLOT(updateBinPartsMenu()));
-
-	//connect(m_listView, SIGNAL(clicked()), this, SLOT(updateButtonStates()));
-	//connect(m_iconView, SIGNAL(clicked()(int)), this, SLOT(updateButtonStates()));
+	connect(m_listView, SIGNAL(currentRowChanged(int)), m_manager, SLOT(updateBinCombinedMenuCurrent()));
+	connect(m_iconView, SIGNAL(selectionChanged(int)), m_manager, SLOT(updateBinCombinedMenuCurrent()));
 
 	connect(m_listView, SIGNAL(informItemMoved(int,int)), m_iconView, SLOT(itemMoved(int,int)));
 	connect(m_iconView, SIGNAL(informItemMoved(int,int)), m_listView, SLOT(itemMoved(int,int)));
 	connect(m_listView, SIGNAL(informItemMoved(int,int)), this, SLOT(itemMoved()));
 	connect(m_iconView, SIGNAL(informItemMoved(int,int)), this, SLOT(itemMoved()));
 
+	m_binLabel->setText(m_title);
+
 	m_addPartToMeAction = new QAction(m_title,this);
 	connect(m_addPartToMeAction, SIGNAL(triggered()),this, SLOT(addSketchPartToMe()));
 
@@ -112,8 +144,18 @@ PartsBinPaletteWidget::~PartsBinPaletteWidget() {
 		delete m_model;
 		m_model = NULL;
 	}
+   
+    if (m_icon) delete m_icon;
 }
 
+void PartsBinPaletteWidget::cleanup() {
+    foreach (PaletteModel * paletteModel, PaletteBinModels) {
+        delete paletteModel;
+    }
+    PaletteBinModels.clear();
+}
+
+
 QSize PartsBinPaletteWidget::sizeHint() const {
 	return QSize(DockManager::DockDefaultWidth, DockManager::PartsBinDefaultHeight);
 }
@@ -125,72 +167,52 @@ QString PartsBinPaletteWidget::title() const {
 void PartsBinPaletteWidget::setTitle(const QString &title) {
 	if(m_title != title) {
 		m_title = title;
-		m_addPartToMeAction->setText(title);
-		m_manager->updateTitle(this, title);
+		m_binLabel->setText(title);
 	}
 }
 
-void PartsBinPaletteWidget::setTabWidget(StackTabWidget *tabWidget) {
-	m_tabWidget = tabWidget;
-}
+void PartsBinPaletteWidget::setupHeader()
+{
+	m_combinedBinMenuButton = newToolButton("partsBinCombinedMenuButton");
+	m_combinedBinMenuButton->setMenu(m_manager->combinedMenu());
 
-void PartsBinPaletteWidget::setupFooter() {
-	m_footer = new QFrame(this);
-	m_footer->setObjectName("partsBinFooter");
-
-	QFrame *leftButtons = new QFrame(m_footer);
-	QHBoxLayout *leftLayout = new QHBoxLayout(leftButtons);
-	leftLayout->setMargin(0);
-    leftLayout->setSpacing(0);
-	leftLayout->addWidget(m_showIconViewButton);
-	leftLayout->addWidget(m_showListViewButton);
-
-	QFrame *rightButtons = new QFrame(m_footer);
-	QHBoxLayout *rightLayout = new QHBoxLayout(rightButtons);
-	rightLayout->setDirection(QBoxLayout::RightToLeft);
-	rightLayout->setMargin(0);
-    rightLayout->setSpacing(0);
-	rightLayout->addWidget(m_binMenuButton);
-	rightLayout->addWidget(m_partMenuButton);
-
-	QHBoxLayout *footerLayout = new QHBoxLayout(m_footer);
-	footerLayout->setSpacing(0);
-    footerLayout->setMargin(0);
-	footerLayout->addWidget(leftButtons);
-
-    footerLayout->addSpacing(8);
-	m_searchLineEdit = new SearchLineEdit(m_footer);
+	m_binLabel = new QLabel(this);
+	m_binLabel->setObjectName("partsBinLabel");
+	m_binLabel->setWordWrap(false);
 
+	m_searchLineEdit = new SearchLineEdit(this);
     m_searchLineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 	connect(m_searchLineEdit, SIGNAL(returnPressed()), this, SLOT(search()));
-    connect(m_searchLineEdit, SIGNAL(clicked()), this, SLOT(clickedSearch()));
-	footerLayout->addWidget(m_searchLineEdit);
 
-    footerLayout->addSpacing(2);
+	m_searchStackedWidget = new QStackedWidget(this);
+	m_searchStackedWidget->setObjectName("searchStackedWidget");
 
-	footerLayout->addWidget(rightButtons);
+	m_searchStackedWidget->addWidget(m_binLabel);
+	m_searchStackedWidget->addWidget(m_searchLineEdit);
+
+	m_header = new QFrame(this);
+	m_header->setObjectName("partsBinHeader");
+	QHBoxLayout * hbl = new QHBoxLayout();
+    hbl->setSpacing(0);
+    hbl->setMargin(0);
+
+	hbl->addWidget(m_searchStackedWidget);
+	hbl->addWidget(m_combinedBinMenuButton);
+
+	m_header->setLayout(hbl);
 }
 
 void PartsBinPaletteWidget::setView(PartsBinView *view) {
 	m_currentView = view;
 	if(m_currentView == m_iconView) {
-		m_iconView->show();
-		m_showIconViewButton->setEnabledIcon();
-		m_listView->hide();
-		m_showListViewButton->setDisabledIcon();
-	} else if(m_currentView == m_listView) {
-		m_listView->show();
-		m_showListViewButton->setEnabledIcon();
-		m_iconView->hide();
-		m_showIconViewButton->setDisabledIcon();
+		m_stackedWidget->setCurrentIndex(0);
+		m_manager->updateViewChecks(true);
+	} 
+	else {
+		m_stackedWidget->setCurrentIndex(1);
+		m_manager->updateViewChecks(false);
 	}
 
-	delete layout();
-	QVBoxLayout *lo = new QVBoxLayout(this);
-	lo->setMargin(3);
-	lo->setSpacing(0);
-	lo->addWidget(dynamic_cast<QWidget*>(m_currentView));
-	lo->addWidget(m_footer);
 }
 
 void PartsBinPaletteWidget::toIconView() {
@@ -204,13 +226,14 @@ void PartsBinPaletteWidget::toListView() {
 }
 
 bool PartsBinPaletteWidget::saveAsAux(const QString &filename) {
-	FileProgressDialog progress("Saving...", 0, this);
+    FileProgressDialog progress("Saving...", 0, this);
 
 	QString oldFilename = m_fileName;
 	setFilename(filename);
 	QString title = this->title();
 	if(!title.isNull() && !title.isEmpty()) {
-		m_model->root()->modelPartShared()->setTitle(title);
+		ModelPartSharedRoot * root = m_model->rootModelPartShared();
+		if (root) root->setTitle(title);
 	}
 
 	if(m_orderHasChanged) {
@@ -224,7 +247,8 @@ bool PartsBinPaletteWidget::saveAsAux(const QString &filename) {
 
 	m_undoStack->setClean();
 
-	saveAsLastBin();
+	m_location = BinLocation::findLocation(filename);
+
 	if(oldFilename != m_fileName) {
 		emit fileNameUpdated(this,m_fileName,oldFilename);
 	}
@@ -255,11 +279,59 @@ void PartsBinPaletteWidget::afterModelSetted(PaletteModel *model) {
 }
 
 void PartsBinPaletteWidget::grabTitle(PaletteModel *model) {
-	m_title = model->root()->modelPartShared()->title();
+	ModelPartSharedRoot * root = model->rootModelPartShared();
+	if (!root) return;
+
+	QString iconFilename = root->icon();
+	grabTitle(root->title(), iconFilename);
+	root->setIcon(iconFilename);
+
+	m_searchLineEdit->setText(root->searchTerm());
+}
+
+void PartsBinPaletteWidget::grabTitle(const QString & title, QString & iconFilename)
+{
+	m_title = title;
 	m_addPartToMeAction->setText(m_title);
+	m_binLabel->setText(m_title);
+
+	QString temp = BinManager::StandardBinIcons.value(m_fileName, "");
+	if (!temp.isEmpty()) {
+		iconFilename = temp;
+	}
+	else if (iconFilename.isEmpty()) {
+		iconFilename = CustomIconName;
+	}
+	
+	if (isCustomSvg(iconFilename)) {
+		// convert to image
+		int w = TextUtils::getViewBoxCoord(iconFilename, 2);
+		int h = TextUtils::getViewBoxCoord(iconFilename, 3);
+		QImage image(w, h, QImage::Format_ARGB32);
+		image.fill(0);
+		QRectF target(0, 0, w, h);
+		QSvgRenderer renderer(iconFilename.toUtf8());
+		QPainter painter;
+		painter.begin(&image);
+		renderer.render(&painter, target);
+		painter.end();	
+		//image.save(FolderUtils::getUserDataStorePath("") + "/test icon.png");
+		m_icon = new QIcon(QPixmap::fromImage(image));
+	}
+	else {
+		QString path = ":resources/bins/icons/" + iconFilename;
+		QFile file(path);
+		if (file.exists()) {
+			m_icon = new QIcon(path);
+		}
+	}
 }
 
 void PartsBinPaletteWidget::addPart(ModelPart *modelPart, int position) {
+    if (m_model == NULL) {
+        return;
+    }
+
 	ModelPart *mp = m_model->addModelPart(m_model->root(),modelPart);
 
 	m_iconView->addPart(mp, position);
@@ -270,211 +342,23 @@ void PartsBinPaletteWidget::addPart(ModelPart *modelPart, int position) {
 	}
 }
 
-
-void PartsBinPaletteWidget::setupButtons() {
-	m_showIconViewButton = new ImageButton("IconView",this);
-	m_showIconViewButton->setToolTip(tr("Show as icons"));
-	connect(m_showIconViewButton,SIGNAL(clicked()),this,SLOT(toIconView()));
-
-	m_showListViewButton = new ImageButton("ListView",this);
-	m_showListViewButton->setToolTip(tr("Show as list"));
-	connect(m_showListViewButton,SIGNAL(clicked()),this,SLOT(toListView()));
-
-	// TODO: these could probably be static or moved up to the binManager...
-	createBinMenu();
-	createPartMenu();
-	createContextMenus();
-}
-
 QToolButton* PartsBinPaletteWidget::newToolButton(const QString& btnObjName, const QString& imgPath, const QString &text) {
 	QToolButton *toolBtn = new QToolButton(this);
 	toolBtn->setObjectName(btnObjName);
-	if(text != ___emptyString___) {
+	toolBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
+	toolBtn->setArrowType(Qt::NoArrow);
+	if (!text.isEmpty()) {
 		toolBtn->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
 		toolBtn->setText(text);
 	}
 	toolBtn->setPopupMode(QToolButton::InstantPopup);
-	if(imgPath != ___emptyString___) {
+	if (!imgPath.isEmpty()) {
 		toolBtn->setIcon(QIcon(imgPath));
 	}
 	toolBtn->setArrowType(Qt::NoArrow);
 	return toolBtn;
 }
 
-QAction* PartsBinPaletteWidget::newTitleAction(const QString &text) {
-	QWidgetAction *act = new QWidgetAction(this);
-	QWidget* w = new QLabel(text,this);
-	w->setObjectName("titleAction");
-	act->setDefaultWidget(w);
-	act->setDisabled(true);
-	return act;
-}
-
-void PartsBinPaletteWidget::createBinMenu() {
-	m_binMenuButton = newToolButton("partsBinBinMenu");
-	createOpenBinMenu();
-	m_newBinAction = new QAction(tr("New..."), this);
-	m_closeBinAction = new QAction(tr("Close"),this);
-	m_saveAction = new QAction(tr("Save"),this);
-	m_saveAsAction = new QAction(tr("Save As..."),this);
-	m_saveAsBundledAction = new QAction(tr("Export Bin..."),this);
-	m_renameAction = new QAction(tr("Rename..."),this);
-
-	connect(m_newBinAction, SIGNAL(triggered()),this, SLOT(newBin()));
-	connect(m_closeBinAction, SIGNAL(triggered()),this, SLOT(closeBin()));
-	connect(m_saveAction, SIGNAL(triggered()),this, SLOT(save()));
-	connect(m_saveAsAction, SIGNAL(triggered()),this, SLOT(saveAs()));
-	connect(m_saveAsBundledAction, SIGNAL(triggered()),this, SLOT(saveBundledBin()));
-	connect(m_renameAction, SIGNAL(triggered()),this, SLOT(rename()));
-
-	m_fileMenu = new QMenu(tr("Parts Bin Actions"), this);
-	m_fileMenu->addAction(newTitleAction(tr("Bin")));
-	m_fileMenu->addAction(m_newBinAction);
-	m_fileMenu->addMenu(m_openBinMenu);
-	m_fileMenu->addSeparator();
-	m_fileMenu->addAction(m_closeBinAction);
-	m_fileMenu->addAction(m_saveAction);
-	m_fileMenu->addAction(m_saveAsAction);
-	m_fileMenu->addAction(m_saveAsBundledAction);
-	m_fileMenu->addAction(m_renameAction);
-	m_binMenuButton->setMenu(m_fileMenu);
-
-	connect(m_fileMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinFileMenu()));
-
-}
-
-void PartsBinPaletteWidget::createOpenBinMenu() {
-	m_openBinMenu = new QMenu(tr("Open"),this);
-
-	m_openBinAction = new QAction(tr("From file..."),this);
-	m_openCoreBinAction = new QAction(tr("Core"),this);
-	m_openContribBinAction = new QAction(tr("Contributed Parts"),this);
-
-	connect(m_openBinAction, SIGNAL(triggered()),this, SLOT(openNewBin()));
-	connect(m_openCoreBinAction, SIGNAL(triggered()),this, SLOT(openCoreBin()));
-	connect(m_openContribBinAction, SIGNAL(triggered()),this, SLOT(openContribBin()));
-
-	m_openBinMenu->addAction(m_openCoreBinAction);
-	m_openBinMenu->addSeparator();
-
-	QDir userBinsDir(FolderUtils::getUserDataStorePath("bins"));
-	collectBins(userBinsDir, m_openBinMenu);
-
-	//QMenu * moreMenu = m_openBinMenu->addMenu(tr("More bins"));
-	m_openBinMenu->addSeparator();
-
-	QDir dir(FolderUtils::getApplicationSubFolderPath("bins"));
-	dir.cd("more");
-	collectBins(dir, m_openBinMenu);
-
-	m_openBinMenu->addSeparator();
-	m_openBinMenu->addAction(m_openBinAction);
-
-}
-
-void PartsBinPaletteWidget::collectBins(QDir & dir, QMenu * menu) {
-	QHash<QString,QString> binsInfo;
-
-	QStringList filters;
-	filters << "*"+FritzingBinExtension;
-	QFileInfoList files = dir.entryInfoList(filters);
-	foreach(QFileInfo info, files) {
-		QString binName = getBinName(info);
-		binsInfo[info.filePath()] = binName;
-	}
-
-	foreach(QString binFile, binsInfo.keys()) {
-		//if (binFile.compare(BinManager::ContribPartsBinLocation) == 0) continue;   // if we want to place this on some other menu
-		if (binFile.endsWith("nonCoreParts.fzb")) continue;			// was the "all my parts bin"
-
-		QAction *action = new QAction(binsInfo[binFile],this);
-		action->setData(binFile);
-		connect(action, SIGNAL(triggered()),this, SLOT(openUserBin()));
-		menu->addAction(action);
-	}
-
-}
-
-QString PartsBinPaletteWidget::getBinName(const QFileInfo &info) {
-	QString binTitle = "";
-
-	// TODO: use xmlStreamReader instead of reading the whole file
-
-	QFile binFile(info.filePath());
-	if (binFile.open(QFile::ReadOnly | QFile::Text)) {
-		QString content(binFile.readAll());
-		QRegExp regexp("<title>(.+)</title>");
-
-		if (regexp.indexIn(content) >= 0) {
-			binTitle = regexp.cap(1);
-		}
-	}
-
-	if(binTitle != ___emptyString___) {
-		return binTitle;
-	} else {
-		return info.fileName();
-	}
-}
-
-void PartsBinPaletteWidget::openUserBin() {
-	QAction *action = qobject_cast<QAction *>(sender());
-	if (action) {
-		openNewBin(action->data().toString());
-	}
-}
-
-void PartsBinPaletteWidget::createPartMenu() {
-	m_partMenuButton = newToolButton("partsBinPartMenu");
-
-	m_newPartAction = new QAction(tr("New..."), this);
-	m_importPartAction = new QAction(tr("Import Part..."),this);
-	m_editPartAction = new QAction(tr("Edit..."),this);
-	m_exportPartAction = new QAction(tr("Export Part..."),this);
-	m_removePartAction = new QAction(tr("Remove"),this);
-
-	connect(m_newPartAction, SIGNAL(triggered()),this, SLOT(newPart()));
-	connect(m_importPartAction, SIGNAL(triggered()),this, SLOT(importPart()));
-	connect(m_editPartAction, SIGNAL(triggered()),this, SLOT(editSelected()));
-	connect(m_exportPartAction, SIGNAL(triggered()),this, SLOT(exportSelected()));
-	connect(m_removePartAction, SIGNAL(triggered()),this, SLOT(removeSelected()));
-
-	m_partMenu = new QMenu(tr("Parts Actions"), this);
-	connect(m_partMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinPartsMenu()));
-	m_partMenu->addAction(newTitleAction(tr("Part")));
-	m_partMenu->addAction(m_newPartAction);
-	m_partMenu->addAction(m_importPartAction);
-	m_partMenu->addSeparator();
-	m_partMenu->addAction(m_editPartAction);
-	m_partMenu->addAction(m_exportPartAction);
-	m_partMenu->addAction(m_removePartAction);
-	m_partMenuButton->setMenu(m_partMenu);
-}
-
-void PartsBinPaletteWidget::createContextMenus() {
-	m_binContextMenu = new QMenu(this);
-	m_binContextMenu->addAction(m_closeBinAction);
-	m_binContextMenu->addAction(m_saveAction);
-	m_binContextMenu->addAction(m_saveAsAction);
-	m_binContextMenu->addAction(m_saveAsBundledAction);
-	m_binContextMenu->addAction(m_renameAction);
-	connect(m_binContextMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinFileMenu()));
-
-	m_partContextMenu = new QMenu(this);
-	connect(m_partContextMenu, SIGNAL(aboutToShow()), this, SLOT(updateBinPartsMenu()));
-	m_partContextMenu->addAction(m_editPartAction);
-	m_partContextMenu->addAction(m_exportPartAction);
-	m_partContextMenu->addAction(m_removePartAction);
-}
-
-bool PartsBinPaletteWidget::removeSelected() {
-	if(selected()) {
-		QString modId = selected()->moduleID();
-		removePartCommand(modId);
-		return true;
-	} else return false;
-}
-
 bool PartsBinPaletteWidget::save() {
 	bool result = true;
 	if (FolderUtils::isEmptyFileName(m_fileName,m_untitledFileName) || currentBinIsCore()) {
@@ -482,7 +366,6 @@ bool PartsBinPaletteWidget::save() {
 	} else {
 		saveAsAux(m_fileName);
 	}
-	if(result) m_manager->setDirtyTab(this,false);
 	return result;
 }
 
@@ -511,17 +394,16 @@ void PartsBinPaletteWidget::saveBundledBin() {
 	bool wasModified = m_isDirty;
 	m_manager->mainWindow()->saveBundledNonAtomicEntity(
 		m_fileName, FritzingBundledBinExtension, this,
-		m_model->root()->getAllNonCoreParts()
+		m_model->root()->getAllNonCoreParts(), true, "", true, false
 	);
 	setDirty(wasModified);
-	saveAsLastBin();
 }
 
 bool PartsBinPaletteWidget::loadBundledAux(QDir &unzipDir, QList<ModelPart*> mps) {
 	QStringList namefilters;
 	namefilters << "*"+FritzingBinExtension;
 
-	this->load(unzipDir.entryInfoList(namefilters)[0].filePath(), this);
+	this->load(unzipDir.entryInfoList(namefilters)[0].filePath(), this, false);
 	foreach(ModelPart* mp, mps) {
 		if(mp->isAlien()) { // double check
 			m_alienParts << mp->moduleID();
@@ -533,7 +415,7 @@ bool PartsBinPaletteWidget::loadBundledAux(QDir &unzipDir, QList<ModelPart*> mps
 }
 
 
-bool PartsBinPaletteWidget::open(QString fileName, QObject * progressTarget) {
+bool PartsBinPaletteWidget::open(QString fileName, QWidget * progressTarget, bool fastLoad) {
 	QFile file(fileName);
 	if (!file.exists()) {
        QMessageBox::warning(NULL, tr("Fritzing"),
@@ -553,59 +435,92 @@ bool PartsBinPaletteWidget::open(QString fileName, QObject * progressTarget) {
     file.close();
 
     if(fileName.endsWith(FritzingBinExtension)) {
-    	load(fileName, progressTarget);
-    	saveAsLastBin();
+    	load(fileName, progressTarget, fastLoad);
     	m_isDirty = false;
     } else if(fileName.endsWith(FritzingBundledBinExtension)) {
-    	m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false);
+    	m_manager->mainWindow()->loadBundledNonAtomicEntity(fileName,this,false, false);
     }
 
     return true;
 }
 
-
-void PartsBinPaletteWidget::load(const QString &filename, QObject * progressTarget) {
+void PartsBinPaletteWidget::load(const QString &filename, QWidget * progressTarget, bool fastLoad) {
 	// TODO deleting this local palette reference model deletes modelPartShared held by the palette bin modelParts
 	//PaletteModel * paletteReferenceModel = new PaletteModel(true, true);
 
-	PaletteModel * oldModel = (m_canDeleteModel) ? m_model : NULL;
-	PaletteModel * paletteBinModel = new PaletteModel(true, false, false);
+	m_location = BinLocation::findLocation(filename);
 
-	if (progressTarget == this) {
-		progressTarget = m_loadingProgressDialog = new FileProgressDialog(tr("Loading..."), 200, this);
-		m_loadingProgressDialog->setBinLoadingChunk(200);
-		m_loadingProgressDialog->setBinLoadingCount(1);
-		m_loadingProgressDialog->setMessage(tr("loading bin %1").arg(QFileInfo(filename).baseName()));
-		m_loadingProgressDialog->show();
-	}
-	
-	if (progressTarget) {
-		connect(paletteBinModel, SIGNAL(loadingInstances(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstancesSlot(ModelBase *, QDomElement &)));
-		connect(paletteBinModel, SIGNAL(loadingInstance(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstanceSlot(ModelBase *, QDomElement &)));
-		connect(m_iconView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
-		connect(m_listView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
-	}
-	bool result = paletteBinModel->load(filename, m_refModel);
+	//DebugDialog::debug("loading bin");
 
-	if (!result) {
-		QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("Friting cannot load the parts bin"));
+	if (fastLoad) {
+		QString binName, iconName;
+		if (BinManager::getBinTitle(filename, binName, iconName)) {
+			m_fileName = filename;
+			grabTitle(binName, iconName);
+			m_fastLoaded = true;
+		}
 		return;
 	}
-	setPaletteModel(paletteBinModel,true);
-	m_canDeleteModel = true;					// since we just created this model, we can delete it later
-	if (oldModel) {
-		delete oldModel;
-	}
 
-	if (progressTarget) {
-		disconnect(paletteBinModel, SIGNAL(loadingInstances(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstancesSlot(ModelBase *, QDomElement &)));
-		disconnect(paletteBinModel, SIGNAL(loadingInstance(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstanceSlot(ModelBase *, QDomElement &)));
-		disconnect(m_iconView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
-		disconnect(m_listView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
-		delete m_loadingProgressDialog;
-		m_loadingProgressDialog = NULL;
-	}
+	m_fastLoaded = false;
+    PaletteModel * paletteBinModel = PaletteBinModels.value(filename);
+    if (paletteBinModel == NULL) {
+	    paletteBinModel = new PaletteModel(true, false, false);
+	    DebugDialog::debug("after palette model");
 
+	    QString name = m_title;
+	    if (name.isEmpty()) name = QFileInfo(filename).baseName();
+
+	    bool deleteWhenDone = false;
+        if (progressTarget != NULL) {
+            DebugDialog::debug("open progress " + filename);
+		    deleteWhenDone = true;
+            progressTarget = m_loadingProgressDialog = new FileProgressDialog(tr("Loading..."), 200, progressTarget);
+		    m_loadingProgressDialog->setBinLoadingChunk(200);
+		    m_loadingProgressDialog->setBinLoadingCount(1);
+		    m_loadingProgressDialog->setMessage(tr("loading bin '%1'").arg(name));
+		    m_loadingProgressDialog->show();
+	    }
+	
+	    if (progressTarget) {
+		    connect(paletteBinModel, SIGNAL(loadingInstances(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstancesSlot(ModelBase *, QDomElement &)));
+		    connect(paletteBinModel, SIGNAL(loadingInstance(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstanceSlot(ModelBase *, QDomElement &)));
+		    connect(m_iconView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
+		    connect(m_listView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
+	    }
+	    DebugDialog::debug(QString("loading bin '%1'").arg(name));
+	    bool result = paletteBinModel->load(filename, m_refModel);
+	    DebugDialog::debug(QString("done loading bin '%1'").arg(name));
+
+	    if (!result) {
+		    QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("Friting cannot load the parts bin"));
+	    }
+	    else {
+		    m_fileName = filename;
+		    setPaletteModel(paletteBinModel,true);
+            PaletteBinModels.insert(filename, paletteBinModel);
+	    }
+
+	    if (progressTarget) {
+            DebugDialog::debug("close progress " + filename);
+		    disconnect(paletteBinModel, SIGNAL(loadingInstances(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstancesSlot(ModelBase *, QDomElement &)));
+		    disconnect(paletteBinModel, SIGNAL(loadingInstance(ModelBase *, QDomElement &)), progressTarget, SLOT(loadingInstanceSlot(ModelBase *, QDomElement &)));
+		    disconnect(m_iconView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
+		    disconnect(m_listView, SIGNAL(settingItem()), progressTarget, SLOT(settingItemSlot()));
+		    if (deleteWhenDone) {
+			    m_loadingProgressDialog->close();
+			    delete m_loadingProgressDialog;
+		    }
+		    m_loadingProgressDialog = NULL;
+	    }
+    }
+    else {
+		m_fileName = filename;
+		setPaletteModel(paletteBinModel,true);
+    }
+
+
+	//DebugDialog::debug("done loading bin");
 	//delete paletteReferenceModel;
 }
 
@@ -621,7 +536,6 @@ bool PartsBinPaletteWidget::currentBinIsCore() {
     return m_fileName == BinManager::CorePartsBinLocation;
 }
 
-
 bool PartsBinPaletteWidget::beforeClosing() {
 	bool retval;
 	if (this->isWindowModified()) {
@@ -659,13 +573,7 @@ bool PartsBinPaletteWidget::beforeClosing() {
 	return retval;
 }
 
-void PartsBinPaletteWidget::saveAsLastBin() {
-	QSettings settings;
-	settings.setValue("lastBin",m_fileName);
-}
-
 void PartsBinPaletteWidget::closeEvent(QCloseEvent* event) {
-	saveAsLastBin();
 	QFrame::closeEvent(event);
 }
 
@@ -727,7 +635,7 @@ void PartsBinPaletteWidget::addPartCommand(const QString& moduleID) {
 	/*bool updating = alreadyIn(moduleID);
 
 	QString partTitle = m_refModel->partTitle(moduleID);
-	if(partTitle == ___emptyString___) partTitle = moduleID;
+	if(partTitle.isEmpty()) partTitle = moduleID;
 
 	QString undoStackMsg;
 
@@ -757,100 +665,13 @@ void PartsBinPaletteWidget::addPartCommand(const QString& moduleID) {
 	}
 }
 
-void PartsBinPaletteWidget::removePartCommand(const QString& moduleID) {
-	/*QString partTitle = m_refModel->partTitle(moduleID);
-	if(partTitle == ___emptyString___) partTitle = moduleID;
-
-	QUndoCommand *parentCmd = new QUndoCommand(tr("\"%1\" removed from the bin").arg(partTitle));
-
-	int index = m_listView->position(moduleID);
-	new PartsBinRemoveCommand(this, moduleID, index, parentCmd);
-	m_undoStack->push(parentCmd);*/
-
-	QMessageBox::StandardButton answer = QMessageBox::question(
-		this,
-		tr("Remove from bin"),
-		tr("Do you really want to remove the selected part from the bin?"),
-		QMessageBox::Yes | QMessageBox::No,
-		QMessageBox::Yes
-	);
-	// TODO: make button texts translatable
-	if(answer == QMessageBox::Yes) {
-		m_undoStack->push(new QUndoCommand("Parts bin: part removed"));
-		removePart(moduleID);
-	}
-
-	setDirty();
-}
-
-void PartsBinPaletteWidget::newBin() {
-	m_manager->newBinIn(m_tabWidget);
-}
-
-void PartsBinPaletteWidget::openNewBin(const QString &filename) {
-	m_manager->openBinIn(m_tabWidget,filename);
-}
-
-void PartsBinPaletteWidget::openCoreBin() {
-	m_manager->openCoreBinIn(m_tabWidget);
-}
-
-void PartsBinPaletteWidget::openContribBin() {
-	openNewBin(BinManager::ContribPartsBinLocation);
-}
-
-void PartsBinPaletteWidget::closeBin() {
-	m_manager->closeBinIn(m_tabWidget);
-}
-
-
-void PartsBinPaletteWidget::rename() {
-	if (!m_allowsChanges) {
-		// TODO: disable menu item instead
-		QMessageBox::warning(NULL, tr("Read-only bin"), tr("This bin cannot be renamed."));
-		return;
-	}
-
-	bool ok;
-	QString newTitle = QInputDialog::getText(
-		this,
-		tr("Rename bin"),
-		tr("Please choose a name for the bin:"),
-		QLineEdit::Normal,
-		m_title,
-		&ok
-	);
-	if(ok) {
-		setTitle(newTitle);
-	}
-}
-
-void PartsBinPaletteWidget::newPart() {
-	m_manager->newPartTo(this);
-}
-
-void PartsBinPaletteWidget::importPart() {
-	m_manager->importPartTo(this);
-}
-
-void PartsBinPaletteWidget::editSelected() {
-	m_manager->editSelectedPartFrom(this);
-}
-
-void PartsBinPaletteWidget::exportSelected() {
-	ModelPart *mp = selected();
-	if(mp) {
-		emit savePartAsBundled(mp->moduleID());
-	}
-}
-
 void PartsBinPaletteWidget::itemMoved() {
 	m_orderHasChanged = true;
 	m_manager->setDirtyTab(this);
 }
 
 void PartsBinPaletteWidget::setDirty(bool dirty) {
-	m_manager->setDirtyTab(this,dirty);
+	m_manager->setDirtyTab(this, dirty);
 	m_isDirty = dirty;
 }
 
@@ -875,14 +696,6 @@ PartsBinView *PartsBinPaletteWidget::currentView() {
 	return m_currentView;
 }
 
-bool PartsBinPaletteWidget::isOverFooter(QDropEvent* event) {
-	QRect mappedFooterRect(
-		m_footer->pos(),
-		m_footer->rect().size()
-	);
-	return mappedFooterRect.contains(event->pos());
-}
-
 void PartsBinPaletteWidget::dragEnterEvent(QDragEnterEvent *event) {
 	QFrame::dragEnterEvent(event);
 }
@@ -905,9 +718,10 @@ QAction *PartsBinPaletteWidget::addPartToMeAction() {
 }
 
 void PartsBinPaletteWidget::addSketchPartToMe() {
+    m_manager->openBinIn(this->m_fileName, false);
 	QString moduleID = m_manager->getSelectedModuleIDFromSketch();
 	bool wasAlreadyIn = contains(moduleID);
-	addPart(moduleID);
+	addPart(moduleID, -1);
 	if(!wasAlreadyIn) {
 		setDirty();
 	}
@@ -915,6 +729,9 @@ void PartsBinPaletteWidget::addSketchPartToMe() {
 
 void PartsBinPaletteWidget::setFilename(const QString &filename) {
 	m_fileName = filename;
+	if (m_fileName.compare(BinManager::SearchBinLocation) == 0) {
+		m_searchStackedWidget->setCurrentIndex(1);
+	}
 	bool acceptIt = !currentBinIsCore();
 	setAcceptDrops(acceptIt);
 	m_iconView->setAcceptDrops(acceptIt);
@@ -928,21 +745,30 @@ void PartsBinPaletteWidget::search() {
 	QString searchText = edit->text();
 	if (searchText.isEmpty()) return;
 
-    m_manager->search(searchText);
-}
+	ModelPartSharedRoot * root = m_model->rootModelPartShared();
+	if (root) {
+		root->setSearchTerm(searchText);
+	}
 
-void PartsBinPaletteWidget::clickedSearch() {
-    m_manager->clickedSearch(this);
+    m_manager->search(searchText);
 }
 
 bool PartsBinPaletteWidget::allowsChanges() {
 	return m_allowsChanges;
 }
 
+bool PartsBinPaletteWidget::readOnly() {
+	return !allowsChanges();
+}
+
 void PartsBinPaletteWidget::setAllowsChanges(bool allowsChanges) {
 	m_allowsChanges = allowsChanges;
 }
 
+void PartsBinPaletteWidget::setReadOnly(bool readOnly) {
+	setAllowsChanges(!readOnly);
+}
+
 void PartsBinPaletteWidget::focusSearch() {
 	if (m_searchLineEdit) {
 		QTimer::singleShot(20, this, SLOT(focusSearchAfter()));
@@ -967,25 +793,125 @@ bool PartsBinPaletteWidget::currentViewIsIconView() {
 	return (m_currentView == m_iconView);
 }
 
-QMenu * PartsBinPaletteWidget::getFileMenu() {
-	return m_fileMenu;
+QIcon PartsBinPaletteWidget::icon() {
+	static QIcon emptyIcon;
+	if (m_icon) return *m_icon;
+
+	return emptyIcon;
 }
 
-QMenu * PartsBinPaletteWidget::getPartMenu() {
-	return m_partMenu;
+QMenu * PartsBinPaletteWidget::combinedMenu()
+{
+	return m_manager->combinedMenu(this);
 }
 
-void PartsBinPaletteWidget::updateBinFileMenu() {
-	m_saveAction->setEnabled(m_allowsChanges);
-	m_renameAction->setEnabled(m_allowsChanges);
-	m_closeBinAction->setEnabled(m_tabWidget->count() > 1);
+QMenu * PartsBinPaletteWidget::partContextMenu()
+{
+	return m_manager->partContextMenu(this);
 }
 
-void PartsBinPaletteWidget::updateBinPartsMenu() {
-	ModelPart *mp = selected();
-	bool enabled = (mp != NULL);
-	m_editPartAction->setEnabled(enabled);
-	m_exportPartAction->setEnabled(enabled && !mp->isCore());
-	m_removePartAction->setEnabled(enabled && allowsChanges());
-	m_importPartAction->setEnabled(true);
+QMenu * PartsBinPaletteWidget::binContextMenu()
+{
+	QMenu * menu = m_manager->binContextMenu(this);
+	if (menu == NULL) return NULL;
+
+	QMenu * newMenu = new QMenu();
+	foreach (QAction * action, menu->actions()) {
+		newMenu->addAction(action);
+	}
+
+	// TODO: need to enable/disable actions based on this bin
+
+	ModelPartSharedRoot * root = m_model->rootModelPartShared();
+	if (root) {
+		QString iconFilename = root->icon();
+		if (iconFilename.compare(CustomIconName) == 0 || isCustomSvg(iconFilename)) {
+			newMenu->addSeparator();
+			QAction * action = new QAction(tr("Change icon color..."), newMenu);
+			action->setToolTip(tr("Change the color of the icon for this bin."));
+			connect(action, SIGNAL(triggered()), this, SLOT(changeIconColor()));
+			newMenu->addAction(action);
+		}
+	}
+	return newMenu;
+}
+
+void PartsBinPaletteWidget::changeIconColor() {
+	QImage image(":resources/bins/icons/" + CustomIconName);
+	QColor initial(image.pixel(image.width() / 2, image.height() / 2));
+	QColor color = QColorDialog::getColor(initial, this, tr("Select a color for this icon"), 0 );
+	if (!color.isValid()) return;
+
+	QRgb match = initial.rgba();
+	for (int y = 0; y < image.height(); y++) {
+		for (int x = 0; x < image.width(); x++) {
+			QRgb rgb = image.pixel(x, y);
+			if (qRed(rgb) == qRed(match) && qBlue(rgb) == qBlue(match) && qGreen(rgb) == qGreen(match)) {
+				image.setPixel(x, y, (color.rgb() & 0xffffff) | (qAlpha(rgb) << 24));
+			}
+		}
+	}
+
+	delete m_icon;
+	m_icon = new QIcon(QPixmap::fromImage(image));
+	m_manager->setTabIcon(this, m_icon);
+	setDirty();
+
+	ModelPartSharedRoot * root = m_model->rootModelPartShared();
+
+	if (root) {
+		QSvgGenerator svgGenerator;
+		svgGenerator.setResolution(90);
+		svgGenerator.setTitle(CustomIconTitle);
+		QBuffer buffer;
+		svgGenerator.setOutputDevice(&buffer);
+		QSize sz = image.size();
+		svgGenerator.setSize(sz);
+		svgGenerator.setViewBox(QRect(0, 0, sz.width(), sz.height()));
+		QPainter svgPainter(&svgGenerator);
+		svgPainter.drawImage(QPoint(0,0), image);
+		svgPainter.end();
+		QString svg(buffer.buffer());
+		root->setIcon(svg);
+	}
+}
+
+bool PartsBinPaletteWidget::fastLoaded() {
+	return m_fastLoaded;
+}
+
+BinLocation::Location PartsBinPaletteWidget::location() {
+	return m_location;
+}
+
+bool PartsBinPaletteWidget::canClose() {
+	switch (m_location) {
+	case BinLocation::User:
+		if (m_fileName.compare(BinManager::SearchBinLocation) == 0) return false;
+		if (m_fileName.compare(BinManager::ContribPartsBinLocation) == 0) return false;
+		if (m_fileName.compare(BinManager::MyPartsBinLocation) == 0) return false;	
+		if (m_manager->isTempPartsBin(this)) return false;	
+		return true;
+	case BinLocation::More:
+		return false;
+	case BinLocation::App:
+		return false;
+	case BinLocation::Outside:
+	default:
+		return true;
+	}
+}
+
+void PartsBinPaletteWidget::copyFilesToContrib(ModelPart * mp, QWidget * originator) {
+	m_manager->copyFilesToContrib(mp, originator);
+}
+
+ModelPart * PartsBinPaletteWidget::root() {
+    if (m_model == NULL) return NULL;
+
+    return m_model->root();
+}
+
+bool PartsBinPaletteWidget::isTempPartsBin() {
+    return m_manager->isTempPartsBin(this);
 }
