@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
@@ -59,8 +59,6 @@ QString PartsEditorMainWindow::FooterText;
 QString PartsEditorMainWindow::UntitledPartName;
 QString PartsEditorMainWindow::___partsEditorName___;
 
-const QString PartsEditorMainWindow::templatePath = "/docs/templates/";
-
 int PartsEditorMainWindow::UntitledPartIndex = 1;
 QPointer<PartsEditorMainWindow> PartsEditorMainWindow::m_lastOpened = NULL;
 int PartsEditorMainWindow::m_closedBeforeCount = 0;
@@ -69,7 +67,7 @@ bool PartsEditorMainWindow::m_closeAfterSaving = true;
 #ifdef QT_NO_DEBUG
 	#define CORE_EDITION_ENABLED false
 #else
-	#define CORE_EDITION_ENABLED true
+	#define CORE_EDITION_ENABLED false
 #endif
 
 
@@ -111,6 +109,7 @@ void PartsEditorMainWindow::initText() {
 
 void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTemplate, ItemBase * fromItem)
 {
+	ModelPart * originalModelPart = NULL;
     QFile styleSheet(":/resources/styles/partseditor.qss");
     m_mainFrame = new QFrame(this);
     m_mainFrame->setObjectName("partsEditor");
@@ -134,14 +133,6 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 	m_paletteModel = new PaletteModel(false, false, false);
 
 	if(modelPart == NULL){
-		if (fromTemplate && !createTemplate()){
-			QMessageBox::critical(this, tr("Parts Editor"),
-	                           tr("Error! Cannot create part file.\n"),
-	                           QMessageBox::Close);
-	        // this seems hacky but maybe it's ok
-	        QTimer::singleShot(60, this, SLOT(close()));
-			return;
-		}
 		m_lastOpened = this;
 		m_updateEnabled = true;
 	} else {
@@ -149,7 +140,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 		m_updateEnabled = modelPart->isCore()?
 							CORE_EDITION_ENABLED:
 							(modelPart->isContrib() || modelPart->isAlien()? false: true);
-		m_fileName = modelPart->path();
+		m_fwFilename = modelPart->path();
 		setTitle();
 		UntitledPartIndex--; // TODO Mariano: not good enough
 	}
@@ -157,7 +148,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 	if(!fromTemplate) {
 		m_sketchModel = new SketchModel(true);
 	} else {
-		ModelPart * mp = m_paletteModel->loadPart(m_fileName, false, false);
+		ModelPart * mp = m_paletteModel->loadPart(m_fwFilename, false, false);
 	    // this seems hacky but maybe it's ok
 		if (mp == NULL || mp->modelPartShared() == NULL) {
 			QMessageBox::critical(this, tr("Parts Editor"),
@@ -169,7 +160,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 
 		QHash<QString,QString> properties = mp->modelPartShared()->properties();
 		foreach (QString key, properties.keys()) {
-			QVariant prop = modelPart->prop(key);
+			QVariant prop = modelPart->localProp(key);
 			if (!prop.isNull() && prop.isValid()) {
 				QString p = prop.toString();
 				if (!p.isEmpty()) {
@@ -181,6 +172,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 			}
 		}
 
+		originalModelPart = modelPart;
 		modelPart = mp;
 		m_sketchModel = new SketchModel(modelPart);
 	}
@@ -191,6 +183,17 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 	createCenter(mp, fromItem);
 	createFooter();
 
+	// copy connector local names here if any
+	if (originalModelPart) {
+		foreach (Connector * fromConnector, originalModelPart->connectors()) {
+			foreach (Connector * toConnector, mp->connectors()) {
+				if (toConnector->connectorSharedID().compare(fromConnector->connectorSharedID()) == 0) {
+					toConnector->setConnectorLocalName(fromConnector->connectorLocalName());
+				}
+			}
+		}
+	}
+
 	layout()->setMargin(0);
 	layout()->setSpacing(0);
 
@@ -461,25 +464,6 @@ QGraphicsProxyWidget *PartsEditorMainWindow::emptyViewItem(QString iconFile, QSt
 	return item;
 }
 
-// creates a temp directory and copies all template .fz and .svg files
-// returns false if directory can't be created
-bool PartsEditorMainWindow::createTemplate(){
-	QDir srcDir = QDir(":/resources/part-template");
-	DebugDialog::debug("template source: " + srcDir.path());
-
-	QDir randDir = createTempFolderIfNecessary();
-
-	FolderUtils::replicateDir(srcDir,randDir);
-
-	QFile tempFile(QCoreApplication::applicationDirPath() + templatePath);
-	tempFile.copy(randDir.path() + "/core/template" + FritzingPartExtension);
-
-	m_fileName = randDir.path() + "/core/template" + FritzingPartExtension;
-	DebugDialog::debug("created temp part: " + m_fileName);
-
-	return true;
-}
-
 const QDir& PartsEditorMainWindow::createTempFolderIfNecessary() {
 	if(m_tempDir.path() == ".") {
 		QString randext = FolderUtils::getRandText();
@@ -503,7 +487,7 @@ const QDir& PartsEditorMainWindow::createTempFolderIfNecessary() {
 
 bool PartsEditorMainWindow::save() {
 	if(validateMinRequirements() && wannaSaveAfterWarning(false)) {
-		bool result = FritzingWindow::save();
+		bool result = (m_saveButton->isEnabled()) ? FritzingWindow::save() : saveAs();
 		if(result) {
 			m_cancelCloseButton->setText(tr("close"));
 			if(m_closeAfterSaving) close();
@@ -515,71 +499,70 @@ bool PartsEditorMainWindow::save() {
 }
 
 bool PartsEditorMainWindow::saveAs() {
-	if(validateMinRequirements()) {
-		QString fileNameBU = m_fileName;
-		QString moduleIdBU = m_moduleId;
+	if(!validateMinRequirements()) return false;
 
-		m_moduleId = ___emptyString___;
-		QString title = m_title->text();
-		m_fileName = title != ___emptyString___ ? title+FritzingPartExtension : m_fileName;
+	QString fileNameBU = m_fwFilename;
+	QString moduleIdBU = m_moduleId;
 
-		// TODO Mariano: This folder should be defined in the preferences... some day
-		QString partsFolderPath = FolderUtils::getUserDataStorePath("parts")+"/";
-		QString userPartsFolderPath = partsFolderPath+"user/";
+	m_moduleId = ___emptyString___;
+	QString title = m_title->text();
+	m_fwFilename = !title.isEmpty() ? title+FritzingPartExtension : QFileInfo(m_fwFilename).baseName();
 
-		bool firstTime = true; // Perhaps the user wants to use the default file name, confirm first
-		while(m_fileName.isEmpty()
-			  || QFileInfo(userPartsFolderPath+m_fileName).exists()
-			  || (FolderUtils::isEmptyFileName(m_fileName,untitledFileName()) && firstTime) ) 
-		{
-			bool ok;
-			m_fileName = QInputDialog::getText(
-				this,
-				tr("Save as new part"),
-				tr("There's already a file with this name.\nPlease, specify a new filename"),
-				QLineEdit::Normal,
-				m_fileName,
-				&ok
-			);
-			firstTime = false;
-			if (!ok) {
-				m_moduleId = moduleIdBU;
-				m_fileName = fileNameBU;
-				return false;
-			}
+	// TODO Mariano: This folder should be defined in the preferences... some day
+	QString partsFolderPath = FolderUtils::getUserDataStorePath("parts")+"/";
+	QString userPartsFolderPath = partsFolderPath+"user/";
+
+	bool firstTime = true; // Perhaps the user wants to use the default file name, confirm first
+	while(m_fwFilename.isEmpty()
+			|| QFileInfo(userPartsFolderPath+m_fwFilename).exists()
+			|| (FolderUtils::isEmptyFileName(m_fwFilename,untitledFileName()) && firstTime) ) 
+	{
+		bool ok;
+		m_fwFilename = QInputDialog::getText(
+			this,
+			tr("Save as new part"),
+			tr("There's already a file with this name.\nPlease, specify a new filename"),
+			QLineEdit::Normal,
+			m_fwFilename,
+			&ok
+		);
+		firstTime = false;
+		if (!ok) {
+			m_moduleId = moduleIdBU;
+			m_fwFilename = fileNameBU;
+			return false;
 		}
+	}
 
-		m_fileName.replace(QRegExp("[^A-Za-z0-9\\.]"), "_");
+	m_fwFilename.replace(QRegExp("[^A-Za-z0-9\\.]"), "_");
 
-		Qt::CaseSensitivity cs = Qt::CaseSensitive;
-	#ifdef Q_WS_WIN
-		// seems to be necessary for Windows: getUserDataStorePath() returns a string starting with "c:"
-		// but the file dialog returns a string beginning with "C:"
-		cs = Qt::CaseInsensitive;
-	#endif
+	Qt::CaseSensitivity cs = Qt::CaseSensitive;
+#ifdef Q_WS_WIN
+	// seems to be necessary for Windows: getUserDataStorePath() returns a string starting with "c:"
+	// but the file dialog returns a string beginning with "C:"
+	cs = Qt::CaseInsensitive;
+#endif
 
-		QString filename = !m_fileName.startsWith(userPartsFolderPath, cs)
-						&& !m_fileName.startsWith(partsFolderPath, cs)
-				? userPartsFolderPath+m_fileName
-				: m_fileName;
-		QString guid = "__"+FolderUtils::getRandText()+FritzingPartExtension;
-		if(!alreadyHasExtension(filename, FritzingPartExtension)) {
-			filename += guid;
-		} else {
-			filename.replace(FritzingPartExtension,guid);
-		}
+	QString filename = !m_fwFilename.startsWith(userPartsFolderPath, cs)
+					&& !m_fwFilename.startsWith(partsFolderPath, cs)
+			? userPartsFolderPath+m_fwFilename
+			: m_fwFilename;
+	QString guid = "__"+FolderUtils::getRandText()+FritzingPartExtension;
+	if(!alreadyHasExtension(filename, FritzingPartExtension)) {
+		filename += guid;
+	} else {
+		filename.replace(FritzingPartExtension,guid);
+	}
 
-		makeNonCore();
-		if (!saveAsAux(filename)) return false;
-
-		if(wannaSaveAfterWarning(true)) {
-			m_savedAsNewPart = true;
-			m_updateEnabled = true;
-			updateButtons();
-			if(m_closeAfterSaving) close();
-			return true;
-		}
+	makeNonCore();
+	if (!saveAsAux(filename)) return false;
 
+	if(wannaSaveAfterWarning(true)) {
+		m_savedAsNewPart = true;
+		m_updateEnabled = true;
+		updateButtons();
+		if(m_closeAfterSaving) close();
+		return true;
 	}
 
 	return false;
@@ -620,7 +603,7 @@ bool PartsEditorMainWindow::saveAsAux(const QString & fileName) {
 
     m_partUpdated = true;
 
-    m_fileName = fileName;
+    setFileName(fileName);
     //setCurrentFile(fileName);
 
    // mark the stack clean so we update the window dirty flag
@@ -680,7 +663,7 @@ ModelPartShared* PartsEditorMainWindow::createModelPartShared() {
 }
 
 void PartsEditorMainWindow::cleanUp() {
-	bool decUntitled = m_fileName.startsWith(untitledFileName());
+	bool decUntitled = m_fwFilename.startsWith(untitledFileName());
 	if(decUntitled) {
 		if(m_lastOpened == this) {
 			UntitledPartIndex -= m_closedBeforeCount;
@@ -707,7 +690,7 @@ void PartsEditorMainWindow::closeEvent(QCloseEvent *event) {
 		cleanUp();
 		QMainWindow::closeEvent(event);
 		if(m_partUpdated) {
-			emit partUpdated(m_fileName, m_id, !m_savedAsNewPart &&
+			emit partUpdated(m_fwFilename, m_id, !m_savedAsNewPart &&
 				(m_connsInfo->connectorsCountChanged() || m_views->connectorsPosOrSizeChanged())
 			);
 		}
@@ -789,7 +772,7 @@ bool PartsEditorMainWindow::eventFilter(QObject *object, QEvent *event) {
 			event->ignore();
 			ProcessEventBlocker::processEvents();
 #ifdef Q_WS_MAC
-			FritzingWindow *parent = dynamic_cast<FritzingWindow*>(parentWidget());
+			FritzingWindow *parent = qobject_cast<FritzingWindow*>(parentWidget());
 			if(parent) {
 				parent->notClosableForAWhile();
 			}
