@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5204 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
@@ -50,6 +50,7 @@ $Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
+#include "../utils/ratsnestcolors.h"
 #include "../svg/gedaelement2svg.h"
 #include "../svg/kicadmodule2svg.h"
 #include "../svg/kicadschematic2svg.h"
@@ -66,6 +67,7 @@ PartsEditorView::PartsEditorView(
 		ItemBase * fromItem)
 	: SketchWidget(viewId, parent, size, size)
 {
+    m_alignToGrid = m_showGrid = false;
 	m_viewItem = NULL;
 	m_item = NULL;
 	m_connsLayerID = ViewLayer::UnknownLayer;
@@ -186,7 +188,7 @@ ItemBase * PartsEditorView::addItemAux(ModelPart * modelPart, ViewLayer::ViewLay
 	if(paletteItem) {
 		modelPart->initConnectors();    // is a no-op if connectors already in place
 		QString layerFileName = getLayerFileName(modelPart);
-		if(layerFileName != ___emptyString___) {
+		if(!layerFileName.isEmpty()) {
 			if(paletteItem->createSvgPath(modelPart->path(), layerFileName)) {
 				paletteItem->createSvgFile(paletteItem->svgFilePath()->absolutePath());
 				ViewLayer::ViewLayerID viewLayerID =
@@ -205,7 +207,7 @@ ItemBase * PartsEditorView::addItemAux(ModelPart * modelPart, ViewLayer::ViewLay
 					}
 					if (!svg.isEmpty()) {
 						QSizeF size = m_viewItem->size();
-						svg = TextUtils::makeSVGHeader(FSvgRenderer::printerScale(), GraphicsUtils::StandardFritzingDPI, size.width(), size.height()) + svg + "</svg>";
+						svg = TextUtils::makeSVGHeader(GraphicsUtils::SVGDPI, GraphicsUtils::StandardFritzingDPI, size.width(), size.height()) + svg + "</svg>";
 						paletteItem->setItemSVG(svg);
 					}
 				}
@@ -233,8 +235,8 @@ void PartsEditorView::fitCenterAndDeselect() {
 		QRectF viewRect = rect();
 		QRectF itemsRect = scene()->itemsBoundingRect();
 
-		qreal wRelation = viewRect.width()  / itemsRect.width();
-		qreal hRelation = viewRect.height() / itemsRect.height();
+		double wRelation = viewRect.width()  / itemsRect.width();
+		double hRelation = viewRect.height() / itemsRect.height();
 
 		if(wRelation < hRelation) {
 			m_scaleValue = (wRelation * 100);
@@ -248,8 +250,8 @@ void PartsEditorView::fitCenterAndDeselect() {
 
 void PartsEditorView::setDefaultBackground() {
 	QString bgColor = " PartsEditorView {background-color: rgb(%1,%2,%3);} ";
-	if(m_bgcolors.contains(m_viewIdentifier)) {
-		QColor c = m_bgcolors[m_viewIdentifier];
+	QColor c = standardBackground();
+	if (c.isValid()) {
 		setStyleSheet(styleSheet()+bgColor.arg(c.red()).arg(c.green()).arg(c.blue()));
 	}
 }
@@ -282,7 +284,7 @@ ModelPart *PartsEditorView::createFakeModelPart(SvgAndPartFilePath *svgpath) {
 	const QHash<QString,ConnectorTerminalSvgIdPair> connIds = getConnectorsSvgIds(svgpath->absolutePath());
 	const QStringList layers = getLayers(svgpath->absolutePath());
 
-	QString path = svgpath->relativePath() == ___emptyString___ ? svgpath->absolutePath() : svgpath->relativePath();
+	QString path = svgpath->relativePath().isEmpty() ? svgpath->absolutePath() : svgpath->relativePath();
 	ModelPart * mp = createFakeModelPart(connIds, layers, path);
 
 	return mp;
@@ -522,7 +524,7 @@ void PartsEditorView::findConnectorsLayerIdsAux(QStringList &result, QDomElement
 }
 
 QString PartsEditorView::findConnectorsLayerId(QDomDocument *svgDom) {
-	QString result = ___emptyString___;
+	QString result;
 	QStringList layers;
 	QDomElement docElem = svgDom->documentElement();
 	if(findConnectorsLayerIdAux(result, docElem, layers)) {
@@ -605,12 +607,12 @@ void PartsEditorView::loadFile() {
 		extras[1] = "*.mod";
 	}
 	else {
-		imageFiles = tr("Image Files (%1 %2 %3);;SVG Files (%1);;JPEG Files (%2);;PNG Files (%3)");
+		imageFiles = tr("Image Files (%1 %2 %3);;SVG Files (%1);;JPEG Files (%2);;PNG Files (%3)%4%5");
 	}
 
 	if (m_viewIdentifier == ViewIdentifierClass::SchematicView) {
 		extras[0] = "*.lib";
-		imageFiles = tr("Image & Footprint Files (%1 %2 %3 %4);;SVG Files (%1);;JPEG Files (%2);;PNG Files (%3);;Kicad Schematic Files (%4)");   // 
+		imageFiles = tr("Image & Footprint Files (%1 %2 %3 %4);;SVG Files (%1);;JPEG Files (%2);;PNG Files (%3);;Kicad Schematic Files (%4)%5");   // 
 	}
 
 	QString origPath = FolderUtils::getOpenFileName(this,
@@ -636,7 +638,7 @@ void PartsEditorView::loadFile() {
 			return;
 		}
 	}
-	if(origPath != ___emptyString___) {
+	if(!origPath.isEmpty()) {
 		if(m_startItem) {
 			m_fixedToCenterItems.removeAll(m_startItem);
 			delete m_startItem;
@@ -707,7 +709,7 @@ void PartsEditorView::beforeSVGLoading(const QString &filename, bool &canceled)
 
 	if(fileHasChanged) {
 		file.close();
-		if(!file.open(QIODevice::WriteOnly )) {
+		if(!TextUtils::writeUtf8(filename, fileContent)) {
 			QMessageBox::warning(
 				NULL,
 				tr("Couldn't write into file"),
@@ -720,12 +722,7 @@ void PartsEditorView::beforeSVGLoading(const QString &filename, bool &canceled)
 				"More information at http://fritzing.org/using-svg-images-new-parts/"
 				)
 			);
-		} else {
-			QTextStream out(&file);
-			out.setCodec("UTF-8");
-			out << fileContent;
-			file.close();
-		}
+		} 
 	}
 
 }
@@ -1165,7 +1162,7 @@ void PartsEditorView::setItemProperties() {
 		//m_item->setFlag(QGraphicsItem::ItemClipsChildrenToShape, true);
 		myItem()->highlightConnectors(m_lastSelectedConnId);
 
-		qreal size = 500; // just make sure the user get enough space to play
+		double size = 500; // just make sure the user get enough space to play
 		setSceneRect(0,0,size,size);
 
 
@@ -1205,7 +1202,8 @@ void PartsEditorView::setMismatching(ViewIdentifierClass::ViewIdentifier viewId,
 void PartsEditorView::aboutToSave(bool fakeDefaultIfNone) {
 	if(m_item) {
 		FSvgRenderer renderer;
-		if(renderer.loadSvg(m_item->flatSvgFilePath())) {
+		QByteArray bytes = renderer.loadSvg(m_item->flatSvgFilePath());
+		if (!bytes.isEmpty()) {
 			QRectF svgViewBox = renderer.viewBoxF();
 			QSizeF sceneViewBox = renderer.defaultSizeF();
 			QDomDocument *svgDom = m_item->svgDom();
@@ -1231,22 +1229,13 @@ void PartsEditorView::aboutToSave(bool fakeDefaultIfNone) {
 
 				ensureFilePath(tempFile);
 
-				QFile file(tempFile);
-				if(!file.open(QFile::WriteOnly)) {
+				if(!TextUtils::writeUtf8(tempFile, TextUtils::removeXMLEntities(svgDom->toString()))) {
 					/*QMessageBox::information(NULL,"",
 						QString("Couldn't open file for update, after drawing connectors: '%1'")
 							.arg(tempFile)
 					);*/
+                    DebugDialog::debug(QString("Couldn't open file for update, after drawing connectors: '%1'").arg(tempFile));
 				}
-				else {
-					QTextStream out(&file);
-					out.setCodec("UTF-8");
-					out << TextUtils::removeXMLEntities(svgDom->toString());
-
-					file.close();
-					updateModelPart(tempFile);
-				}
-
 			}
 		} else {
 			DebugDialog::debug("updating part view svg file: could not load file "+m_item->flatSvgFilePath());
@@ -1467,7 +1456,7 @@ void PartsEditorView::addNewTerminalPoints(
 			QRectF tpointRect(tp->mappedPoint(), QPointF(0,0));
 			QRectF svgTpRect = mapFromSceneToSvg(tpointRect,sceneViewBox,svgViewBox);
 
-			qreal halfTPSize = 0.001; // a tiny rectangle
+			double halfTPSize = 0.001; // a tiny rectangle
 			svgTpRect.setSize(QSizeF(halfTPSize*2,halfTPSize*2));
 
 			addRectToSvg(svgDom,connId+"terminal",svgTpRect, connectorsLayerId);
@@ -1501,13 +1490,13 @@ bool PartsEditorView::removeConnectorsIfNeeded(QDomElement &docElem) {
 }
 
 QRectF PartsEditorView::mapFromSceneToSvg(const QRectF &itemRect, const QSizeF &sceneViewBox, const QRectF &svgViewBox) {
-	qreal relationW = svgViewBox.width() / sceneViewBox.width();
-	qreal relationH = svgViewBox.height() / sceneViewBox.height();
+	double relationW = svgViewBox.width() / sceneViewBox.width();
+	double relationH = svgViewBox.height() / sceneViewBox.height();
 
-	qreal x = itemRect.x() * relationW;
-	qreal y = itemRect.y() * relationH;
-	qreal width = itemRect.width() * relationW;
-	qreal height = itemRect.height() * relationH;
+	double x = itemRect.x() * relationW;
+	double y = itemRect.y() * relationH;
+	double width = itemRect.width() * relationW;
+	double height = itemRect.height() * relationH;
 
 	return QRectF(x,y,width,height);
 }
@@ -1521,7 +1510,7 @@ bool PartsEditorView::addRectToSvg(QDomDocument* svgDom, const QString &id, cons
 	connElem.setAttribute("height",rect.height());
 	connElem.setAttribute("fill","none");
 
-	if(connectorsLayerId == ___emptyString___) {
+	if(connectorsLayerId.isEmpty()) {
 		svgDom->firstChildElement("svg").appendChild(connElem);
 		return true;
 	} else {
@@ -1622,22 +1611,22 @@ void PartsEditorView::drawBackground(QPainter *painter, const QRectF &rect) {
 
 	QRectF itemRect = m_item->mapToScene(m_item->boundingRect()).boundingRect();
 	painter->drawRect(itemRect);
-	qreal itemTop = itemRect.top();
-	qreal itemLeft = itemRect.left();
+	double itemTop = itemRect.top();
+	double itemLeft = itemRect.left();
 
 	QVarLengthArray<QLineF, 100> lines;
 
-	for (qreal x = itemLeft; x < rect.right(); x += gridSize) {
+	for (double x = itemLeft; x < rect.right(); x += gridSize) {
 		lines.append(QLineF(x, rect.top(), x, rect.bottom()));
 	}
-	for (qreal x = itemLeft-gridSize; x > rect.left(); x -= gridSize) {
+	for (double x = itemLeft-gridSize; x > rect.left(); x -= gridSize) {
 		lines.append(QLineF(x, rect.top(), x, rect.bottom()));
 	}
 
-	for (qreal y = itemTop; y < rect.bottom(); y += gridSize) {
+	for (double y = itemTop; y < rect.bottom(); y += gridSize) {
 		lines.append(QLineF(rect.left(), y, rect.right(), y));
 	}
-	for (qreal y = itemTop-gridSize; y > rect.top(); y -= gridSize) {
+	for (double y = itemTop-gridSize; y > rect.top(); y -= gridSize) {
 		lines.append(QLineF(rect.left(), y, rect.right(), y));
 	}
 
@@ -1731,7 +1720,7 @@ void PartsEditorView::updatePinsInfo(QList< QPointer<ConnectorShared> > connsSha
 		if (pinInfo == NULL) {
 			notFound << cs;
 		}
-		else if(m_svgIds[connId].connectorId != ___emptyString___) {
+		else if(!m_svgIds[connId].connectorId.isEmpty()) {
 			pinInfo->m_svgId = m_svgIds[connId].connectorId;
 			// terminal points are already updated (by the function updateTerminalPoints)
 				// pinInfo->m_terminalId = m_svgIds[connId].terminalId;
@@ -1750,7 +1739,7 @@ void PartsEditorView::updatePinsInfo(QList< QPointer<ConnectorShared> > connsSha
 		foreach(ConnectorShared* cs, notFound) {
 			QString connId = cs->id();
 			SvgIdLayer* pinInfo = cs->fullPinInfo(m_viewIdentifier, vlid);
-			if (pinInfo != NULL && m_svgIds[connId].connectorId != ___emptyString___) {
+			if (pinInfo != NULL && !m_svgIds[connId].connectorId.isEmpty()) {
 				pinInfo->m_svgId = m_svgIds[connId].connectorId;
 				pinInfo->m_svgViewLayerID = layerID;
 				found << cs;
@@ -1764,15 +1753,9 @@ void PartsEditorView::updatePinsInfo(QList< QPointer<ConnectorShared> > connsSha
 }
 
 QString PartsEditorView::saveSvg(const QString & svg, const QString & newFilePath) {
-	QFile file(newFilePath);
-	if (!file.open(QFile::WriteOnly)) {
-		throw tr("unable to open temp file %1").arg(newFilePath);
-	}
-
-	QTextStream stream(&file);
-	stream.setCodec("UTF-8");
-	stream << svg;
-	file.close();
+    if (!TextUtils::writeUtf8(newFilePath, svg)) {
+        throw tr("unable to open temp file %1").arg(newFilePath);
+    }
 	return newFilePath;
 }
 
@@ -1959,8 +1942,8 @@ void PartsEditorView::ensureFixedToCenterItems() {
 }
 
 void PartsEditorView::ensureFixedToCenter(QGraphicsItem* item) {
-	qreal x = (width()-fixedItemWidth(item))/2;
-	qreal y = (height()-fixedItemHeight(item))/2;
+	double x = (width()-fixedItemWidth(item))/2;
+	double y = (height()-fixedItemHeight(item))/2;
 
 	QPointF pos = mapToScene(x,y);
 
@@ -1978,7 +1961,7 @@ void PartsEditorView::removeIfFixedPos(QGraphicsItem *item) {
 }
 
 
-qreal PartsEditorView::fixedItemWidth(QGraphicsItem* item) {
+double PartsEditorView::fixedItemWidth(QGraphicsItem* item) {
 	QGraphicsProxyWidget* gWidget = dynamic_cast<QGraphicsProxyWidget*>(item);
 	if(gWidget) {
 		return gWidget->widget()->width();
@@ -1987,7 +1970,7 @@ qreal PartsEditorView::fixedItemWidth(QGraphicsItem* item) {
 	}
 }
 
-qreal PartsEditorView::fixedItemHeight(QGraphicsItem* item) {
+double PartsEditorView::fixedItemHeight(QGraphicsItem* item) {
 	QGraphicsProxyWidget* gWidget = dynamic_cast<QGraphicsProxyWidget*>(item);
 	if(gWidget) {
 		return gWidget->widget()->height();
