@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5204 $:
+$Revision: 6112 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 ********************************************************************/
 
@@ -118,7 +118,7 @@ ProgramTab::ProgramTab(QString & filename, QWidget *parent) : QFrame(parent)
 {
 	m_tabWidget = NULL;
 	while (parent != NULL) {
-		QTabWidget * tabWidget = dynamic_cast<QTabWidget *>(parent);
+		QTabWidget * tabWidget = qobject_cast<QTabWidget *>(parent);
 		if (tabWidget) {
 			m_tabWidget = tabWidget;
 			break;
@@ -526,6 +526,11 @@ void ProgramTab::setClean() {
 	textChanged();
 }
 
+void ProgramTab::setDirty() {
+	m_textEdit->document()->setModified(true);
+	textChanged();
+}
+
 void ProgramTab::save() {
 	emit wantToSave(m_tabWidget->currentIndex());
 }
@@ -808,3 +813,8 @@ void ProgramTab::enableProgramButton() {
 
 	m_programButton->setEnabled(enabled);
 }
+
+void ProgramTab::appendToConsole(const QString & text)
+{
+	m_console->append(text);
+}
