@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6112 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-06-28 00:18:10 +0200 (Thu, 28 Jun 2012) $
 
 ********************************************************************/
 
@@ -161,7 +161,7 @@ void ProgramWindow::initLanguages() {
 		}
 		else {
 			QString name = Syntaxer::parseForName(fileInfo.absoluteFilePath());
-			if (!name.isEmpty()  && !name.contains("arduino", Qt::CaseInsensitive)) {		// Arduino is not really available
+			if (!name.isEmpty()) {		// Arduino is not really available      && !name.contains("arduino", Qt::CaseInsensitive)
 				m_languages.insert(name, fileInfo.absoluteFilePath());
 			}
 		}
@@ -370,8 +370,16 @@ void ProgramWindow::setup(const QList<LinkedFile *> & linkedFiles, const QString
                 programTab = addTab();
             }
             QDir dir(alternativePath);
-            QFileInfo fileInfo(linkedFile->filename);
-            programTab->loadProgramFile(linkedFile->filename, dir.absoluteFilePath(fileInfo.fileName()), false);
+            QFileInfo fileInfo(linkedFile->linkedFilename);
+            programTab->loadProgramFile(linkedFile->linkedFilename, dir.absoluteFilePath(fileInfo.fileName()), false);
+			if ((linkedFile->fileFlags & LinkedFile::InBundleFlag) && ((linkedFile->fileFlags & LinkedFile::ReadOnlyFlag) == 0)) { 
+				if (linkedFile->fileFlags & LinkedFile::SameMachineFlag) {
+					programTab->appendToConsole(tr("File '%1' was restored from the .fzz file; the local copy was not found.").arg(fileInfo.fileName()));
+				}
+				else {
+					programTab->appendToConsole(tr("File '%1' was restored from the .fzz file; save a local copy to work with an external editor.").arg(fileInfo.fileName()));
+				}
+			}
 			if (!m_languages.value(linkedFile->language, "").isEmpty()) {
 				programTab->setLanguage(linkedFile->language, false);
 			}
@@ -1002,11 +1010,11 @@ void ProgramWindow::sendProgram() {
 }
 
 ProgramTab * ProgramWindow::currentWidget() {
-	return dynamic_cast<ProgramTab *>(m_tabWidget->currentWidget());
+	return qobject_cast<ProgramTab *>(m_tabWidget->currentWidget());
 }
 
 ProgramTab * ProgramWindow::indexWidget(int index) {
-	return dynamic_cast<ProgramTab *>(m_tabWidget->widget(index));
+	return qobject_cast<ProgramTab *>(m_tabWidget->widget(index));
 }
 
 bool ProgramWindow::alreadyHasProgram(const QString & filename) {
