@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,14 +18,13 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5217 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-11 03:10:34 +0200 (Mon, 11 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
 
-
 #ifndef SKETCHWIDGET_H
 #define SKETCHWIDGET_H
 
@@ -47,11 +46,6 @@ $Date: 2011-07-11 03:10:34 +0200 (Mon, 11 Jul 2011) $
 #include "../utils/misc.h"
 #include "../commands.h"
 
-QT_BEGIN_NAMESPACE
-class QDragEnterEvent;
-class QDropEvent;
-QT_END_NAMESPACE
-
 struct ItemCount {
 	int selCount;
 	int hasLabelCount;
@@ -66,6 +60,26 @@ struct ItemCount {
 	int wireCount;
 };
 
+struct SwapThing {
+    bool firstTime;
+    long newID;
+    ItemBase * itemBase;
+    long newModelIndex;
+    QString newModuleID;
+    ViewLayer::ViewLayerSpec viewLayerSpec;
+    QList<Wire *> wiresToDelete;
+    QUndoCommand * parentCommand;
+};
+
+class SizeItem : public QObject, public QGraphicsLineItem
+{
+	Q_OBJECT
+	
+public:
+	SizeItem();
+	~SizeItem();
+};
+
 class SketchWidget : public InfoGraphicsView
 {
 	Q_OBJECT
@@ -80,17 +94,20 @@ public:
 	ItemBase * addItem(const QString & moduleID, ViewLayer::ViewLayerSpec, BaseCommand::CrossViewType, const ViewGeometry &, long id, long modelIndex, AddDeleteItemCommand * originatingCommand);
     void deleteItem(long id, bool deleteModelPart, bool doEmit, bool later);
     virtual void deleteItem(ItemBase *, bool deleteModelPart, bool doEmit, bool later);
+    void simpleMoveItem(long id, QPointF);
     void moveItem(long id, ViewGeometry &, bool updateRatsnest);
 	void moveItem(long id, const QPointF & p, bool updateRatsnest);
 	void updateWire(long id, const QString & connectorID, bool updateRatsnest);
 
-    void rotateItem(long id, qreal degrees);
+    void rotateItem(long id, double degrees);
     void transformItem(long id, const QMatrix &);
     void flipItem(long id, Qt::Orientations orientation);
     void selectItem(long id, bool state, bool updateInfoView, bool doEmit);
     void selectDeselectAllCommand(bool state);
     void changeWire(long fromID, QLineF line, QPointF pos, bool updateConnections, bool updateRatsnest);   
-    void changeLeg(long fromID, const QString & connectorID, QLineF line);   
+    void changeLeg(long fromID, const QString & connectorID, const QPolygonF &, bool relative, const QString & why);   
+    void recalcLeg(long fromID, const QString & connectorID, const QPolygonF &, bool relative, bool active, const QString & why);   
+    void rotateLeg(long fromID, const QString & connectorID, const QPolygonF &, bool active);   
     void cut();
     void copy();
     void setPaletteModel(PaletteModel *);
@@ -100,23 +117,24 @@ public:
     void clearSelection();
 	virtual void loadFromModelParts(QList<ModelPart *> & modelParts, BaseCommand::CrossViewType, QUndoCommand * parentCommand, 
 									bool offsetPaste, const QRectF * boundingRect, bool seekOutsideConnections, QList<long> & newIDs);
-    void changeZ(QHash<long, RealPair * >, qreal (*pairAccessor)(RealPair *) );
+    void changeZ(QHash<long, RealPair * >, double (*pairAccessor)(RealPair *) );
 	void sendToBack();
 	void sendBackward();
 	void bringForward();
 	void bringToFront();
-	qreal fitInWindow();
-	void rotateX(qreal degrees);
-	void flip(Qt::Orientations orientation);
+	double fitInWindow();
+	void rotateX(double degrees, bool rubberBandLegEnabled);
+	void flipX(Qt::Orientations orientation, bool rubberBandLegEnabled);
 	void addBendpoint(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
+	void flattenCurve(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
 
-	virtual void deleteSelected(Wire *);
+	void deleteSelected(Wire *);
 	PaletteItem *getSelectedPart();
 
     void addViewLayer(ViewLayer *);
     void setAllLayersVisible(bool visible);
-    void setLayerVisible(ViewLayer * viewLayer, bool visible);
-	void setLayerVisible(ViewLayer::ViewLayerID viewLayerID, bool visible);
+    void setLayerVisible(ViewLayer * viewLayer, bool visible, bool doChildLayers);
+	void setLayerVisible(ViewLayer::ViewLayerID viewLayerID, bool visible, bool doChildLayers);
     void setLayerActive(ViewLayer * viewLayer, bool active);
 	void setLayerActive(ViewLayer::ViewLayerID viewLayerID, bool active);
     bool layerIsVisible(ViewLayer::ViewLayerID);
@@ -125,8 +143,9 @@ public:
 	void sortAnyByZ(const QList<QGraphicsItem *> & items, QList<ItemBase *> & bases);
  	void mousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
  	void setBackground(QColor);
+    void setBackgroundColor(QColor, bool setPref);
  	const QColor& background();
-	const QColor& standardBackground();
+	QColor standardBackground();
  	void setItemMenu(QMenu*);
  	void setWireMenu(QMenu*);
 	virtual void changeConnection(long fromID,
@@ -164,14 +183,15 @@ public:
 	void addSchematicViewLayers();
 	void addBreadboardViewLayers();
 
-	void changeWireColor(long wireId, const QString& color, qreal opacity);
-	void changeWireWidth(long wireId, qreal width);
+	void changeWireColor(long wireId, const QString& color, double opacity);
+	void changeWireWidth(long wireId, double width);
 	void changeWireFlags(long wireId, ViewGeometry::WireFlags wireFlags);
 	void setIgnoreSelectionChangeEvents(bool);
 	void hideConnectors(bool hide);
 	void saveLayerVisibility();
 	void restoreLayerVisibility();
-	virtual void updateRoutingStatus(CleanUpWiresCommand*, RoutingStatus &, bool manual);
+	void updateRoutingStatus(CleanUpWiresCommand*, RoutingStatus &, bool manual);
+	void updateRoutingStatus(RoutingStatus &, bool manual);
 	virtual bool hasAnyNets();
 	void ensureLayerVisible(ViewLayer::ViewLayerID);
 
@@ -189,33 +209,35 @@ public:
 	void updateInfoView();
 	virtual void setCurrent(bool current);
 	void partLabelMoved(ItemBase *, QPointF oldPos, QPointF oldOffset, QPointF newPos, QPointF newOffset);
-	void rotateFlipPartLabel(ItemBase *, qreal degrees, Qt::Orientations flipDirection);
-	void rotateFlipPartLabel(long itemID, qreal degrees, Qt::Orientations flipDirection);
+	void rotateFlipPartLabel(ItemBase *, double degrees, Qt::Orientations flipDirection);
+	void rotateFlipPartLabel(long itemID, double degrees, Qt::Orientations flipDirection);
 	void showPartLabels(bool show);
 	void hidePartLabel(ItemBase * item);
 	void noteSizeChanged(ItemBase * itemBase, const QSizeF & oldSize, const QSizeF & newSize);
 	void resizeNote(long itemID, const QSizeF & );
 	class SelectItemCommand* stackSelectionState(bool pushIt, QUndoCommand * parentCommand);
-	QString renderToSVG(qreal printerScale, const LayerList & partLayers, const LayerList & wireLayers, 
-						bool blackOnly, QSizeF & imageSize, ItemBase * offsetPart, qreal dpi, 
-						bool selectedItems, bool flatten, bool fillHoles, bool & empty);
-	QString renderToSVG(qreal printerScale, const LayerList & partLayers, const LayerList & wireLayers, 
-								  bool blackOnly, QSizeF & imageSize, ItemBase * offsetPart, qreal dpi, 
-								  bool flatten, bool fillHoles, 
-								  QList<ItemBase *> & itemBases, QRectF itemsBoundingRect, bool & empty);
+	QString renderToSVG(double printerScale, const LayerList &, 
+						bool blackOnly, QSizeF & imageSize, ItemBase * board, double dpi, 
+						bool selectedItems, bool renderBlocker, bool & empty);
+	QString renderToSVG(double printerScale, const LayerList &, 
+						bool blackOnly, QSizeF & imageSize, ItemBase * board, QRectF & offsetRect, double dpi, 
+						bool selectedItems, bool renderBlocker, bool & empty);
+	QString renderToSVG(double printerScale, bool blackOnly, QSizeF & imageSize, QRectF & offsetRect, double dpi, 
+								  bool renderBlocker, 
+								  QList<QGraphicsItem *> & itemsAndLabels, QRectF itemsBoundingRect, bool & empty);
 
 	bool spaceBarIsPressed();
-	virtual long setUpSwap(ItemBase *, long newModelIndex, const QString & newModuleID, ViewLayer::ViewLayerSpec, bool doEmit, bool noFinalChangeWiresCommand, QUndoCommand * parentCommand);
+	virtual long setUpSwap(SwapThing &, bool master);
 	ConnectorItem * lastHoverEnterConnectorItem();
 	ItemBase * lastHoverEnterItem();
 	LayerHash & viewLayers();
 	virtual void createTrace(Wire*);
 	virtual void updateNet(Wire*);
-	void selectAllWires(ViewGeometry::WireFlag);
+	virtual void selectAllWires(ViewGeometry::WireFlag);
 	virtual void tidyWires();
 	const QString & getShortName();
 	virtual void setClipEnds(class ClipableWire *, bool);
-	void getBendpointWidths(class Wire *, qreal w, qreal & w1, qreal & w2, bool & negativeOffsetRect);
+	void getBendpointWidths(class Wire *, double w, double & w1, double & w2, bool & negativeOffsetRect);
 	virtual bool includeSymbols();
 	void disconnectAll();
 	virtual bool canDisconnectAll();
@@ -225,18 +247,22 @@ public:
 	long createWire(ConnectorItem * from, ConnectorItem * to, ViewGeometry::WireFlags, bool dontUpdate, BaseCommand::CrossViewType, QUndoCommand * parentCommand);
 	int selectAllObsolete();
 	int selectAllMoveLock();
-	int selectAllItemType(ModelPart::ItemType);
+	void setMoveLock(long id, bool lock);
 	bool partLabelsVisible();
 	void restorePartLabel(long itemID, QDomElement & element);
-	void loadLogoImage(long itemID, const QString & oldSvg, const QSizeF oldAspectRatio, const QString & oldFilename, const QString & newFilename, bool addName);
+	void loadLogoImage(ItemBase *, const QString & oldSvg, const QSizeF oldAspectRatio, const QString & oldFilename, const QString & newFilename, bool addName);
 	void loadLogoImage(long itemID, const QString & oldSvg, const QSizeF oldAspectRatio, const QString & oldFilename);
 	void loadLogoImage(long itemID, const QString & newFilename, bool addName);
 	void setNoteFocus(QGraphicsItem *, bool inFocus);
 
 	void alignToGrid(bool);
 	bool alignedToGrid();
-	void saveZoom(qreal);
-	qreal retrieveZoom();
+	void showGrid(bool);
+	bool showingGrid();
+	void setGridSize(const QString &);
+    QString gridSizeText();
+	void saveZoom(double);
+	double retrieveZoom();
 	void initGrid();
 	virtual double defaultGridSizeInches();
 	void clearPasteOffset();
@@ -244,20 +270,47 @@ public:
 	void addFixedToCenterItem2(class SketchMainHelp *item);
 	void collectAllNets(QHash<class ConnectorItem *, int> & indexer, QList< QList<class ConnectorItem *>* > & allPartConnectorItems, bool includeSingletons, bool bothSides);
 	virtual bool routeBothSides();
-	virtual void changeLayer(long id, qreal z, ViewLayer::ViewLayerID viewLayerID);
+	virtual void changeLayer(long id, double z, ViewLayer::ViewLayerID viewLayerID);
 	void ratsnestConnect(ConnectorItem * connectorItem, bool connect);
 	void ratsnestConnect(ItemBase *, bool connect);
+	void ratsnestConnect(ConnectorItem * c1, ConnectorItem * c2, bool connect);
 	virtual void addDefaultParts();
 	float getTopZ();
 	QGraphicsItem * addWatermark(const QString & filename);
 	void copyHeart(QList<ItemBase *> & bases, bool saveBoundingRects, QByteArray & itemData, QList<long> & modelIndexes);
 	void pasteHeart(QByteArray & itemData, bool seekOutsideConnections);
-	virtual ViewGeometry::WireFlag getTraceFlag();
+	ViewGeometry::WireFlag getTraceFlag();
 	void changeBus(ItemBase *, bool connec, const QString & oldBus, const QString & newBus, QList<ConnectorItem *> &, const QString & message);
 	const QString & filenameIf();
 	void setItemDropOffset(long id, QPointF offset);
-	void prepLegChange(ConnectorItem * from,  QLineF oldLine, QLineF newLine, ConnectorItem * to);
-
+	void prepLegBendpointMove(ConnectorItem * from, int index, QPointF oldPos, QPointF newPos, ConnectorItem * to, bool changeConnections);
+	void prepLegCurveChange(ConnectorItem * from, int index, const class Bezier * oldB, const class Bezier * newB, bool triggerFirstTime);
+	void prepLegBendpointChange(ConnectorItem * from, int oldCount, int newCount, int index, QPointF pos, const class Bezier *, const class Bezier *, const class Bezier *, bool triggerFirstTime);
+	void prepLegSelection(ItemBase *);
+	void changeWireCurve(long id, const Bezier *);
+	void changeLegCurve(long id, const QString & connectorID, int index, const Bezier *);
+	void addLegBendpoint(long id, const QString & connectorID, int index, QPointF, const class Bezier *, const class Bezier *);
+	void removeLegBendpoint(long id, const QString & connectorID, int index, const class Bezier *);
+	void moveLegBendpoint(long id, const QString & connectorID, int index, QPointF);
+	bool curvyWires();
+	void setCurvyWires(bool);
+	bool curvyWiresIndicated(Qt::KeyboardModifiers);
+	void triggerRotate(ItemBase *, double degrees);
+	void makeWiresChangeConnectionCommands(const QList<Wire *> & wires, QUndoCommand * parentCommand);
+	void renamePins(ItemBase *, const QStringList & oldLabels, const QStringList & newLabels, bool singleRow);
+	void renamePins(long itemID, const QStringList & labels, bool singleRow);
+	void getRatsnestColor(QColor &);
+	VirtualWire * makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor color);
+	virtual double getRatsnestOpacity();
+	virtual double getRatsnestWidth();
+	void setAnyInRotation();
+	ConnectorItem * findConnectorItem(ConnectorItem * foreignConnectorItem);
+	void setGroundFillSeed(long id, const QString & connectorID, bool seed);
+	void setWireExtras(long id, const QDomElement &);
+    void resolveTemporary(bool, ItemBase *);
+	virtual bool sameElectricalLayer2(ViewLayer::ViewLayerID, ViewLayer::ViewLayerID);
+	void deleteMiddle(QSet<ItemBase *> & deletedItems, QUndoCommand * parentCommand);
+    void setPasting(bool);
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
@@ -297,13 +350,12 @@ protected:
 
 	void cutDeleteAux(QString undoStackMessage);
 	void deleteAux(QSet<ItemBase *> & deletedItems, QUndoCommand * parentCommand, bool doPush);
-	void deleteMiddle(QHash<ItemBase *, SketchWidget *> & otherDeletedItems, QUndoCommand * parentCommand);
 
-	void extendChangeConnectionCommand(BaseCommand::CrossViewType, long fromID, const QString & fromConnectorID,
+	ChangeConnectionCommand * extendChangeConnectionCommand(BaseCommand::CrossViewType, long fromID, const QString & fromConnectorID,
 									   long toID, const QString & toConnectorID,
 									   ViewLayer::ViewLayerSpec,
 									   bool connect, QUndoCommand * parent);
-	void extendChangeConnectionCommand(BaseCommand::CrossViewType, ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem,
+	ChangeConnectionCommand * extendChangeConnectionCommand(BaseCommand::CrossViewType, ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem,
 										ViewLayer::ViewLayerSpec,
 										bool connect, QUndoCommand * parentCommand);
 
@@ -311,7 +363,8 @@ protected:
 	void keyPressEvent(QKeyEvent *);
 	void keyReleaseEvent(QKeyEvent *);
 	void clearTemporaries();
-	virtual void dragWireChanged(class Wire* wire, ConnectorItem * from, ConnectorItem * to);
+	void dragWireChanged(class Wire* wire, ConnectorItem * from, ConnectorItem * to);
+	void dragRatsnestChanged();
 	void killDroppingItem();
 	ViewLayer::ViewLayerID getViewLayerID(ModelPart *, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerSpec);
 	ItemBase * overSticky(ItemBase *);
@@ -324,45 +377,42 @@ protected:
 	void resizeEvent(QResizeEvent *);
 
 	void addViewLayersAux(const LayerList &layers, float startZ = 1.5);
-	void tempConnectWire(Wire * wire, ConnectorItem * from, ConnectorItem * to);
-	void rotateFlip(qreal degrees, Qt::Orientations orientation);
-	virtual bool disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash &, bool doCommand, QUndoCommand * parentCommand);
+	virtual bool disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash &, bool doCommand, bool rubberBandLegEnabled, QUndoCommand * parentCommand);
 	void clearDragWireTempCommand();
 	bool draggingWireEnd();
-	void moveItems(QPoint globalPos, bool checkAutoScroll);
+	void moveItems(QPoint globalPos, bool checkAutoScroll, bool rubberBandLegEnabled);
 	virtual ViewLayer::ViewLayerID multiLayerGetViewLayerID(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerSpec, QDomElement & layers, QString & layerName);
 	virtual BaseCommand::CrossViewType wireSplitCrossView();
 	virtual bool canChainMultiple();
 	virtual bool canChainWire(Wire *);
 	virtual bool canDragWire(Wire *);
 	virtual bool canCreateWire(Wire * dragWire, ConnectorItem * from, ConnectorItem * to);
-	virtual bool modifyNewWireConnections(Wire * dragWire, ConnectorItem * fromOnWire, ConnectorItem * from, ConnectorItem * to, QUndoCommand * parentCommand);
+	//virtual bool modifyNewWireConnections(Wire * dragWire, ConnectorItem * fromOnWire, ConnectorItem * from, ConnectorItem * to, QUndoCommand * parentCommand);
+	virtual void setUpColor(ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, Wire * wire, QUndoCommand * parentCommand);
 	void setupAutoscroll(bool moving);
 	void turnOffAutoscroll();
 	bool checkAutoscroll(QPoint globalPos);
 	virtual void setWireVisible(Wire *);
-	virtual void chainVisible(ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, bool connect);
 	bool matchesLayer(ModelPart * modelPart);
 
 	QByteArray removeOutsideConnections(const QByteArray & itemData, QList<long> & modelIndexes);
 	void addWireExtras(long newID, QDomElement & view, QUndoCommand * parentCommand);
-	virtual bool doRatsnestOnCopy();
 	virtual const QString & hoverEnterWireConnectorMessage(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	virtual const QString & hoverEnterPartConnectorMessage(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	void partLabelChangedAux(ItemBase * pitem,const QString & oldText, const QString &newText);
 	void drawBackground( QPainter * painter, const QRectF & rect );
 	void handleConnect(QDomElement & connect, ModelPart *, const QString & fromConnectorID, ViewLayer::ViewLayerID, QStringList & alreadyConnected, 
 						QHash<long, ItemBase *> & newItems, QUndoCommand * parentCommand, bool seekOutsideConnections);
-	void setUpSwapReconnect(ItemBase* itemBase, long newID, const QString & newModuleID, bool master, QUndoCommand * parentCommand);
+	void setUpSwapReconnect(SwapThing &, ItemBase * itemBase, long newID, bool master);
 	bool swappedGender(ConnectorItem * originalConnectorItem, Connector * newConnector);
 	void setLastPaletteItemSelected(PaletteItem * paletteItem);
 	void setLastPaletteItemSelectedIf(ItemBase * itemBase);
-	void prepDragBendpoint(Wire *, QPoint eventPos);
+	void prepDragBendpoint(Wire *, QPoint eventPos, bool dragCurve);
 	void prepDragWire(Wire *);
 	void clickBackground(QMouseEvent *);
-	void categorizeDragWires(QSet<Wire *> & wires);
-	void categorizeDragLegs();
-	void prepMove(ItemBase * originatingItem);
+	void categorizeDragWires(QSet<Wire *> & wires, QList<ItemBase *> & freeWires);
+	void categorizeDragLegs(bool rubberBandLegEnabled);
+	void prepMove(ItemBase * originatingItem, bool rubberBandLegEnabled);
 	void initBackgroundColor();
 	QPointF calcNewLoc(ItemBase * moveBase, ItemBase * detachFrom);
 	long findPartOrWire(long itemID);
@@ -373,15 +423,17 @@ protected:
 	bool moveByArrow(int dx, int dy, QKeyEvent * );
 	double gridSizeInches();
 	virtual bool canAlignToTopLeft(ItemBase *);
+	virtual bool canAlignToCenter(ItemBase *);
 	virtual void findAlignmentAnchor(ItemBase * originatingItem, QHash<long, ItemBase *> & savedItems, QHash<Wire *, ConnectorItem *> & savedWires);
 	void alignLoc(QPointF & loc, const QPointF startPoint, const QPointF newLoc, const QPointF originalLoc);
 	void copyAux(QList<ItemBase *> & bases, bool saveBoundingRects);
 	void copyDrop();
 	void dropItemEvent(QDropEvent *event);
-	QString makeWireSVG(Wire * wire, QPointF offset, qreal dpi, qreal printerscale, bool blackOnly);
-	QString makeLineSVG(QPointF p1, QPointF p2, qreal width, QString colorString, qreal dpi, qreal printerScale, bool blackOnly);
-	QString makeRectSVG(QRectF r, QPointF offset, qreal dpi, qreal printerscale);
-	QString makeMoveSVG(qreal printerScale, qreal dpi, QPointF & offset); 
+    virtual QString checkDroppedModuleID(const QString & moduleID);
+	QString makeWireSVG(Wire * wire, QPointF offset, double dpi, double printerscale, bool blackOnly);
+	QString makeWireSVGAux(Wire * wire, double width, const QString & color, QPointF offset, double dpi, double printerScale, bool blackOnly);
+
+	QString makeMoveSVG(double printerScale, double dpi, QPointF & offset); 
 	void prepDeleteProps(ItemBase * itemBase, long id, const QString & newModuleID, QUndoCommand * parentCommand);
 	void prepDeleteOtherProps(ItemBase * itemBase, long id, const QString & newModuleID, QUndoCommand * parentCommand);
 	ViewLayer::ViewLayerSpec getViewLayerSpec(ModelPart * modelPart, QDomElement & instance, QDomElement & view, ViewGeometry &);
@@ -391,16 +443,37 @@ protected:
 	virtual bool resizingJumperItemRelease();
 	virtual bool resizingBoardPress(QGraphicsItem * item);
 	virtual bool resizingBoardRelease();
-	bool connectedDirectlyTo(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & byBus);
-	bool connectedDirectlyTo(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & byBus, QList<ConnectorItem *> & visited);
 	virtual QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	virtual bool acceptsTrace(const ViewGeometry &);
 	virtual ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
 	void alignOneToGrid(ItemBase * itemBase);
 	void showPartLabelsAux(bool show, QList<ItemBase *> & itemBases);
-	virtual void changeTrace(Wire * wire, ConnectorItem * from, ConnectorItem * to, QUndoCommand * parentCommand);
-	virtual void extraRenderSvgStep(ItemBase *, QPointF offset, qreal dpi, qreal printerScale, QString & outputSvg);
+	virtual void extraRenderSvgStep(ItemBase *, QPointF offset, double dpi, double printerScale, QString & outputSvg);
 	virtual ViewLayer::ViewLayerSpec createWireViewLayerSpec(ConnectorItem * from, ConnectorItem * to);
+	virtual Wire * createTempWireForDragging(Wire * fromWire, ModelPart * wireModel, ConnectorItem * connectorItem, ViewGeometry & viewGeometry, ViewLayer::ViewLayerSpec);
+	virtual void prereleaseTempWireForDragging(Wire*);
+	void checkFit(ModelPart * newModelPart, ItemBase * itemBase, long newID,
+								QHash<ConnectorItem *, Connector *> & found, QList<ConnectorItem *> & notFound,
+								QHash<ConnectorItem *, ConnectorItem *> & m2f, QHash<ConnectorItem *, Connector *> & byWire,
+								QHash<QString, QPolygonF> & legs, 	QHash<QString, ConnectorItem *> & formerLegs, QUndoCommand * parentCommand);
+	void checkFitAux(ItemBase * tempItemBase, ItemBase * itemBase, long newID,
+								QHash<ConnectorItem *, Connector *> & found, QList<ConnectorItem *> & notFound,
+								QHash<ConnectorItem *, ConnectorItem *> & m2f, QHash<ConnectorItem *, Connector *> & byWire,
+								QHash<QString, QPolygonF> & legs, QHash<QString, ConnectorItem *> & formerLegs, QUndoCommand * parentCommand);
+	void changeLegAux(long fromID, const QString & fromConnectorID, const QPolygonF &, bool reset, bool relative, bool active, const QString & why);
+	void moveLegBendpoints(bool undoOnly, QUndoCommand * parentCommand);
+	void moveLegBendpointsAux(ConnectorItem * connectorItem, bool undoOnly, QUndoCommand * parentCommand);
+	virtual void rotatePartLabels(double degrees, QTransform &, QPointF center, QUndoCommand * parentCommand);
+	bool checkUpdateRatsnest(QList<ConnectorItem *> & connectorItems);
+	void makeRatsnestViewGeometry(ViewGeometry & viewGeometry, ConnectorItem * source, ConnectorItem * dest); 
+	virtual double getTraceWidth();
+	virtual const QString & traceColor(ViewLayer::ViewLayerSpec);
+	void createTrace(Wire * fromWire, const QString & commandString, ViewGeometry::WireFlag);
+	bool createOneTrace(Wire * wire, ViewGeometry::WireFlag flag, bool allowAny, QList<Wire *> & done, QUndoCommand * parentCommand);
+	void removeWire(Wire * w, QList<ConnectorItem *> & ends, QList<Wire *> & done, QUndoCommand * parentCommand);
+    void selectAllWiresFrom(ViewGeometry::WireFlag flag, QList<QGraphicsItem *> & items);
+    bool canConnect(ItemBase * from, ItemBase * to);
+    virtual bool canConnect(Wire * from, ItemBase * to);
 
 protected:
 	static bool lessThan(int a, int b);
@@ -422,7 +495,7 @@ signals:
 	void selectionChangedSignal();
 
 	void resizeSignal();
-	void dropSignal(const QPoint &pos);
+	void dropSignal(const QPoint &pos, ModelPart *);
 	void routingStatusSignal(SketchWidget *, const RoutingStatus &);
 	void movingSignal(SketchWidget *, QUndoCommand * parentCommand);
 	void selectAllItemsSignal(bool state, bool doEmit);
@@ -430,40 +503,46 @@ signals:
 	void disconnectAllSignal(QList<ConnectorItem *>, QHash<ItemBase *, SketchWidget *> & itemsToDelete, QUndoCommand * parentCommand);
 	void setResistanceSignal(long itemID, QString resistance, QString pinSpacing, bool doEmit);
 	void setPropSignal(long itemID, const QString & prop, const QString & value, bool doRedraw, bool doEmit);
-	void setInstanceTitleSignal(long id, const QString & title, bool isUndoable, bool doEmit);
+	void setInstanceTitleSignal(long id, const QString & oldTitle, const QString & newTitle, bool isUndoable, bool doEmit);
 	void statusMessageSignal(QString, int timeout);
 	void showLabelFirstTimeSignal(long itemID, bool show, bool doEmit);
 	void dropPasteSignal(SketchWidget *);
 	void changeBoardLayersSignal(int, bool doEmit);
 	void firstTimeHelpHidden();
-	void disconnectWireSignal(QSet<ItemBase *> &, QList<long> &, QUndoCommand * parentCommand);
 	void deleteTracesSignal(QSet<ItemBase *> & deletedItems, QHash<ItemBase *, SketchWidget *> & otherDeletedItems, QList<long> & deletedIDs, bool isForeign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandPrepSignal(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandFinalSignal(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void warnSMDSignal(const QString &);
-	void cursorLocationSignal(qreal xinches, qreal yinches);
+	void cursorLocationSignal(double xinches, double yinches);
 	void ratsnestConnectSignal(long id, const QString & connectorID, bool connect, bool doEmit);
 	void updatePartLabelInstanceTitleSignal(long itemID);
 	void filenameIfSignal(QString & filename);
+	void collectRatsnestSignal(QList<SketchWidget *> & foreignSketchWidgets);
+	void removeRatsnestSignal(QList<struct ConnectorEdge *> & cutSet, QUndoCommand * parentCommand); 
+	void updateLayerMenuSignal();
+    void swapBoardImageSignal(SketchWidget * sketchWidget, ItemBase * itemBase, const QString & filename, const QString & moduleID, bool addName);
+    void canConnectSignal(Wire * from, ItemBase * to, bool & connect);
+    void swapStartSignal(SwapThing & swapThing, bool master);
 
 protected slots:
-	void sketchWidget_itemAdded(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, SketchWidget * dropOrigin);
-	void sketchWidget_itemDeleted(long id);
-	void sketchWidget_clearSelection();
-	void sketchWidget_itemSelected(long id, bool state);
-	void scene_selectionChanged();
-	void wire_wireChanged(class Wire*, QLineF oldLine, QLineF newLine, QPointF oldPos, QPointF newPos, ConnectorItem * from, ConnectorItem * to);
-	virtual void wire_wireSplit(class Wire*, QPointF newPos, QPointF oldPos, QLineF oldLine);
-	void wire_wireJoin(class Wire*, ConnectorItem * clickedConnectorItem);
+	void itemAddedSlot(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, SketchWidget * dropOrigin);
+	void itemDeletedSlot(long id);
+	void clearSelectionSlot();
+	void itemSelectedSlot(long id, bool state);
+	void selectionChangedSlot();
+	void wireChangedSlot(class Wire*, const QLineF & oldLine, const QLineF & newLine, QPointF oldPos, QPointF newPos, ConnectorItem * from, ConnectorItem * to);
+	void wireChangedCurveSlot(class Wire*, const Bezier * oldB, const Bezier * newB, bool triggerFirstTime);
+	virtual void wireSplitSlot(class Wire*, QPointF newPos, QPointF oldPos, const QLineF & oldLine);
+	void wireJoinSlot(class Wire*, ConnectorItem * clickedConnectorItem);
 	void toggleLayerVisibility();
-	void sketchWidget_wireConnected(long fromID, QString fromConnectorID, long toID, QString toConnectorID);
-	void sketchWidget_wireDisconnected(long fromID, QString fromConnectorID);
-	void sketchWidget_changeConnection(long fromID, QString fromConnectorID, long toID, QString toConnectorID, ViewLayer::ViewLayerSpec, bool connect, bool updateConnections);
+	void wireConnectedSlot(long fromID, QString fromConnectorID, long toID, QString toConnectorID);
+	void wireDisconnectedSlot(long fromID, QString fromConnectorID);
+	void changeConnectionSlot(long fromID, QString fromConnectorID, long toID, QString toConnectorID, ViewLayer::ViewLayerSpec, bool connect, bool updateConnections);
 	void navigatorScrollChange(double x, double y);
 	void restartPasteCount();
 	void dragIsDoneSlot(class ItemDrag *);
 	void statusMessage(QString message, int timeout = 0);
-	void sketchWidget_cleanUpWires(CleanUpWiresCommand *);
+	void cleanUpWiresSlot(CleanUpWiresCommand *);
 	void updateInfoViewSlot();
 	void spaceBarIsPressedSlot(bool);
 	void autoScrollTimeout();
@@ -472,34 +551,40 @@ protected slots:
 	void rememberSticky(long id, QUndoCommand * parentCommand);
 	void rememberSticky(ItemBase *, QUndoCommand * parentCommand);
 	void copyBoundingRectsSlot(QHash<QString, QRectF> &);
-	void disconnectWireSlot(QSet<ItemBase *> &, QList<long> & deletedIDs, QUndoCommand * parentCommand);
+	void deleteRatsnest(Wire *, QUndoCommand * parentCommand);
 	void deleteTracesSlot(QSet<ItemBase *> & deletedItems, QHash<ItemBase *, SketchWidget *> & otherDeletedItems, QList<long> & deletedIDs, bool isForeign, QUndoCommand * parentCommand);
 	void vScrollToZero();
 	void arrowTimerTimeout();
 	void makeDeleteItemCommandPrepSlot(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandFinalSlot(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void updatePartLabelInstanceTitleSlot(long itemID);
+	void changePinLabelsSlot(ItemBase * itemBase, bool singleRow);
+	void changePinLabels(ItemBase *, bool singleRow);
+	void collectRatsnestSlot(QList<SketchWidget *> & foreignSketchWidgets);
+	void removeRatsnestSlot(QList<struct ConnectorEdge *> & cutSet, QUndoCommand * parentCommand);
+    void deleteTemporary();
+    void canConnect(Wire * from, ItemBase * to, bool & connect);
+    long swapStart(SwapThing & swapThing, bool master);
 
 public slots:
 	void changeWireColor(const QString newColor);
 	void changeWireWidthMils(const QString newWidth);
  	void selectAllItems(bool state, bool doEmit);
 	void setNoteText(long itemID, const QString & newText);
-	void setInstanceTitle(long id, const QString & title, bool isUndoable, bool doEmit);
+	void setInstanceTitle(long id, const QString & oldTitle, const QString & newTitle, bool isUndoable, bool doEmit);
 	void incInstanceTitle(long id);
 	void showPartLabel(long id, bool showIt);
 	void checkSticky(long id, bool doEmit, bool checkCurrent, CheckStickyCommand *);
-	void resizeBoard(long id, qreal w, qreal h);
+	void resizeBoard(long id, double w, double h);
 	void resizeJumperItem(long id, QPointF pos, QPointF c0, QPointF c1);
 	void disconnectAllSlot(QList<ConnectorItem *>, QHash<ItemBase *, SketchWidget *> & itemsToDelete, QUndoCommand * parentCommand);
 	void setResistance(long itemID, QString resistance, QString pinSpacing, bool doEmit);
 	void setResistance(QString resistance, QString pinSpacing);
 	void setProp(long itemID, const QString & prop, const QString & value, bool redraw, bool doEmit);
 	void setProp(ItemBase *, const QString & propName, const QString & translatedPropName, const QString & oldValue, const QString & newValue, bool redraw);
-	void setSpacing(const QString & spacing);
-	void setForm(const QString & form);
+	void setHoleSize(ItemBase *, const QString & propName, const QString & translatedPropName, const QString & oldValue, const QString & newValue, QRectF & oldRect, QRectF & newRect, bool redraw);
 	virtual void showLabelFirstTime(long itemID, bool show, bool doEmit);
-	void resizeBoard(qreal w, qreal h, bool doEmit);
+	void resizeBoard(double w, double h, bool doEmit);
 	virtual void changeBoardLayers(int layers, bool doEmit);
 	void updateConnectors();
 	void ratsnestConnect(long id, const QString & connectorID, bool connect, bool doEmit);
@@ -575,8 +660,8 @@ protected:
 	QPointer<ItemBase> m_lastHoverEnterItem;
 	QString m_shortName;
 	QPointer<Wire> m_dragBendpointWire;
+	bool m_dragCurve;
 	QPoint m_dragBendpointPos;
-	QColor m_standardBackgroundColor;
 	StatusConnectStatus m_statusConnectState;
 	QList<QGraphicsItem *> m_inFocus;
 	QString m_viewName;
@@ -585,12 +670,14 @@ protected:
 	int m_arrowTotalY;
 	bool m_movingByMouse;
 	bool m_alignToGrid;
+	bool m_showGrid;
 	double m_gridSizeInches;
+    QString m_gridSizeText;
 	QPointer<ItemBase> m_alignmentItem;
 	QPointF m_alignmentStartPoint;
-	qreal m_zoom;
+	double m_zoom;
 	bool m_draggingBendpoint;
-	QPointer<GraphicsSvgLineItem> m_sizeItem;
+	QPointer<SizeItem> m_sizeItem;
 	int m_autoScrollThreshold;
 	bool m_clearSceneRect;
 	QPointer<ItemBase> m_moveReferenceItem;
@@ -606,12 +693,17 @@ protected:
 	QTimer m_arrowTimer;
 	bool m_middleMouseIsPressed;
 	QMultiHash<ItemBase *, ConnectorItem *> m_stretchingLegs;
+	bool m_curvyWires;
+	bool m_rubberBandLegWasEnabled;
+	RoutingStatus m_routingStatus;
+	bool m_anyInRotation;
+    bool m_pasting;
 
 public:
 	static ViewLayer::ViewLayerID defaultConnectorLayer(ViewIdentifierClass::ViewIdentifier viewId);
+	static const int PropChangeDelay;
 
 protected:
-	static QHash<ViewIdentifierClass::ViewIdentifier,QColor> m_bgcolors;
 	static const int MoveAutoScrollThreshold;
 	static const int DragAutoScrollThreshold;
 };
