@@ -1,11 +1,11 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
+the Free Software Foundation, either version 3 of the License, orF
 (at your option) any later version.
 
 Fritzing is distributed in the hope that it will be useful,
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5219 $:
+$Revision: 6244 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-11 14:13:40 +0200 (Mon, 11 Jul 2011) $
+$Date: 2012-08-04 01:13:27 +0200 (Sat, 04 Aug 2012) $
 
 ********************************************************************/
 
@@ -34,6 +34,7 @@ $Date: 2011-07-11 14:13:40 +0200 (Mon, 11 Jul 2011) $
 #include "../utils/textutils.h"
 #include "../items/wire.h"
 
+#include <QBitArray>
 #include <QPainter>
 #include <QSvgRenderer>
 #include <QDate>
@@ -41,8 +42,9 @@ $Date: 2011-07-11 14:13:40 +0200 (Mon, 11 Jul 2011) $
 #include <qmath.h>
 #include <limits>
 
-static const int MILS = 5;			// operate on a 5 mil scale
-static const int THRESHOLD = 192;
+static const double BORDERINCHES = 0.04;
+
+inline int OFFSET(int x, int y, QImage * image) { return (y * image->width()) + x; }
 
 QString GroundPlaneGenerator::ConnectorName = "connector0pad";
 
@@ -54,24 +56,21 @@ QString GroundPlaneGenerator::ConnectorName = "connector0pad";
 
 GroundPlaneGenerator::GroundPlaneGenerator()
 {
+	m_blurBy = 0;
+	m_strokeWidthIncrement = 0;
+	m_minRiseSize = m_minRunSize = 1;
 }
 
 GroundPlaneGenerator::~GroundPlaneGenerator() {
 }
 
-bool GroundPlaneGenerator::getBoardRects(const QString & boardSvg, QGraphicsItem * board, qreal res, qreal keepoutSpace, QList<QRect> & rects)
+bool GroundPlaneGenerator::getBoardRects(const QByteArray & boardByteArray, QGraphicsItem * board, double res, double keepoutSpace, QList<QRect> & rects)
 {
-	QByteArray boardByteArray;
-    QString tempColor("#000000");
-	QStringList exceptions;
-    if (!SvgFileSplitter::changeColors(boardSvg, tempColor, exceptions, boardByteArray)) {
-		return false;
-	}
 
 	QRectF br = board->sceneBoundingRect();
-	qreal bWidth = res * br.width() / FSvgRenderer::printerScale();
-	qreal bHeight = res * br.height() / FSvgRenderer::printerScale();
-	QImage image(bWidth, bHeight, QImage::Format_RGB32);
+	double bWidth = res * br.width() / GraphicsUtils::SVGDPI;
+	double bHeight = res * br.height() / GraphicsUtils::SVGDPI;
+	QImage image(bWidth, bHeight, QImage::Format_Mono);  
 	image.setDotsPerMeterX(res * GraphicsUtils::InchesPerMeter);
 	image.setDotsPerMeterY(res * GraphicsUtils::InchesPerMeter);
 	image.fill(0xffffffff);
@@ -79,31 +78,29 @@ bool GroundPlaneGenerator::getBoardRects(const QString & boardSvg, QGraphicsItem
 	QSvgRenderer renderer(boardByteArray);
 	QPainter painter;
 	painter.begin(&image);
-	painter.setRenderHint(QPainter::Antialiasing);
+	painter.setRenderHint(QPainter::Antialiasing, false);
 	renderer.render(&painter);
 	painter.end();
 
 #ifndef QT_NO_DEBUG
-	//image.save("getBoardRects.png");
+	image.save(FolderUtils::getUserDataStorePath("") + "/getBoardRects.png");
 #endif
 
 	QColor keepaway(255,255,255);
-	int threshold = 1;
 
 	// now add keepout area to the border
 	QImage image2 = image.copy();
 	painter.begin(&image2);
+	painter.setRenderHint(QPainter::Antialiasing, false);
 	painter.fillRect(0, 0, image2.width(), keepoutSpace, keepaway);
 	painter.fillRect(0, image2.height() - keepoutSpace, image2.width(), keepoutSpace, keepaway);
 	painter.fillRect(0, 0, keepoutSpace, image2.height(), keepaway);
 	painter.fillRect(image2.width() - keepoutSpace, 0, keepoutSpace, image2.height(), keepaway);
 
 	for (int y = 0; y < image.height(); y++) {
-		QRgb* scanLine = (QRgb *) image.scanLine(y);
 		for (int x = 0; x < image.width(); x++) {
-			QRgb current = *(scanLine + x);
-			int gray = qGray(current);
-			if (gray <= threshold) {			
+			QRgb current = image.pixel(x, y);
+			if (current != 0xffffffff) {			
 				continue;
 			}
 
@@ -113,10 +110,10 @@ bool GroundPlaneGenerator::getBoardRects(const QString & boardSvg, QGraphicsItem
 	painter.end();
 
 #ifndef QT_NO_DEBUG
-	//image2.save("getBoardRects2.png");
+	image2.save(FolderUtils::getUserDataStorePath("") + "/getBoardRects2.png");
 #endif
 
-	scanLines(image2, bWidth, bHeight, rects, threshold, 1);
+	scanLines(image2, bWidth, bHeight, rects);
 
 	// combine parallel equal-sized rects
 	int ix = 0;
@@ -149,25 +146,29 @@ bool GroundPlaneGenerator::getBoardRects(const QString & boardSvg, QGraphicsItem
 }
 
 bool GroundPlaneGenerator::generateGroundPlaneUnit(const QString & boardSvg, QSizeF boardImageSize, const QString & svg, QSizeF copperImageSize, 
-												   QStringList & exceptions, QGraphicsItem * board, qreal res, const QString & color, const QString & layerName,
+												   QStringList & exceptions, QGraphicsItem * board, double res, const QString & color,
 												   QPointF whereToStart) 
 {
-	qreal bWidth, bHeight;
+	double bWidth, bHeight;
 	QImage * image = generateGroundPlaneAux(boardSvg, boardImageSize, svg, copperImageSize, exceptions, board, res, bWidth, bHeight);
 	if (image == NULL) return false;
 
-	QPoint s(qRound(res * (whereToStart.x() - board->pos().x()) / FSvgRenderer::printerScale()),
-			qRound(res * (whereToStart.y() - board->pos().y()) / FSvgRenderer::printerScale()));
+	QRectF bsbr = board->sceneBoundingRect();
+
+	QPoint s(qRound(res * (whereToStart.x() - bsbr.topLeft().x()) / GraphicsUtils::SVGDPI),
+			qRound(res * (whereToStart.y() - bsbr.topLeft().y()) / GraphicsUtils::SVGDPI));
+
+	QBitArray redMarker(image->height() * image->width(), false);
 
 	QRgb pixel = image->pixel(s);
-	int gray = qGray(pixel);
-	if (gray <= THRESHOLD) {
+	DebugDialog::debug(QString("unit %1").arg(pixel, 0, 16));
+	if (pixel != 0xffffffff) {
 		// starting off in bad territory
 		delete image;
 		return false;
 	}
 
-	// step 1 flood fill white to red  (keep max locations)
+	// step 1 flood fill white to "red" (keep max locations)
 
 	int minY = image->height();
 	int maxY = 0;
@@ -177,18 +178,17 @@ bool GroundPlaneGenerator::generateGroundPlaneUnit(const QString & boardSvg, QSi
 	stack << s;
 	while (stack.count() > 0) {
 		QPoint p = stack.takeFirst();
+
 		if (p.x() < 0) continue;
 		if (p.y() < 0) continue;
 		if (p.x() >= image->width()) continue;
 		if (p.y() >= image->height()) continue;
+		if (redMarker.testBit(OFFSET(p.x(), p.y(), image))) continue;			// already been here
 
 		QRgb pixel = image->pixel(p);
-		if (pixel == 0xffff0000) continue;			// already been here
+		if (pixel != 0xffffffff) continue;			// black; bail
 
-		int gray = qGray(pixel);
-		if (gray <= THRESHOLD) continue;			// black; bail
-
-		image->setPixel(p,  0xffff0000);
+		redMarker.setBit(OFFSET(p.x(), p.y(), image), true);
 		if (p.x() > maxX) maxX = p.x();
 		if (p.x() < minX) minX = p.x();
 		if (p.y() > maxY) maxY = p.y();
@@ -203,54 +203,42 @@ bool GroundPlaneGenerator::generateGroundPlaneUnit(const QString & boardSvg, QSi
 	//image->save("testPoly1.png");
 
 	// step 2 replace white with black
+	image->fill(0);
 
+	// step 3 replace "red" with white
 	for (int y = 0; y < image->height(); y++) {
-		QRgb* scanLine = (QRgb *) image->scanLine(y);
 		for (int x = 0; x < image->width(); x++) {
-			QRgb pixel = *(scanLine + x);
-			if (pixel == 0xffff0000) continue;
-
-			image->setPixel(x, y,  0xff000000);
+			if (redMarker.testBit(OFFSET(x, y, image))) {
+				image->setPixel(x, y, 1);
+			}
 		}
 	}
 
-	//image->save("testPoly2.png");
+#ifndef QT_NO_DEBUG
+	image->save(FolderUtils::getUserDataStorePath("") + "/testGroundPlaneUnit3.png");
+#endif
 
-	// step 3 replace red with white
-
-	for (int y = 0; y < image->height(); y++) {
-		QRgb* scanLine = (QRgb *) image->scanLine(y);
-		for (int x = 0; x < image->width(); x++) {
-			QRgb pixel = *(scanLine + x);
-			if (pixel != 0xffff0000) continue;
-
-			image->setPixel(x, y,  0xffffffff);
-		}
-	}
-
-	//image->save("testPoly3.png");
-
-	scanImage(*image, bWidth, bHeight, MILS, res, color, layerName, true, 8, true, QSizeF(.05, .05), 1 / FSvgRenderer::printerScale());
+	scanImage(*image, bWidth, bHeight, GraphicsUtils::StandardFritzingDPI / res, res, color, true, true, QSizeF(.05, .05), 1 / GraphicsUtils::SVGDPI, QPointF(0,0));
 	delete image;
 	return true;
 }
 
 
 bool GroundPlaneGenerator::generateGroundPlane(const QString & boardSvg, QSizeF boardImageSize, const QString & svg, QSizeF copperImageSize, 
-												QStringList & exceptions, QGraphicsItem * board, qreal res, const QString & color, const QString & layerName) 
+												QStringList & exceptions, QGraphicsItem * board, double res, const QString & color) 
 {
 
-	qreal bWidth, bHeight;
+	double bWidth, bHeight;
 	QImage * image = generateGroundPlaneAux(boardSvg, boardImageSize, svg, copperImageSize, exceptions, board, res, bWidth, bHeight);
 	if (image == NULL) return false;
 
-	scanImage(*image, bWidth, bHeight, MILS, res, color, layerName, true, 8, true, QSizeF(.05, .05), 1 / FSvgRenderer::printerScale());
+	scanImage(*image, bWidth, bHeight, GraphicsUtils::StandardFritzingDPI / res, res, color, true, true, QSizeF(.05, .05), 1 / GraphicsUtils::SVGDPI, QPointF(0,0));
 	delete image;
 	return true;
 }
 
 QImage * GroundPlaneGenerator::generateGroundPlaneAux(const QString & boardSvg, QSizeF boardImageSize, const QString & svg, QSizeF copperImageSize, 
-													QStringList & exceptions, QGraphicsItem * board, qreal res, qreal & bWidth, qreal & bHeight) 
+													QStringList & exceptions, QGraphicsItem * board, double res, double & bWidth, double & bHeight) 
 {
 	QByteArray boardByteArray;
     QString tempColor("#ffffff");
@@ -258,35 +246,32 @@ QImage * GroundPlaneGenerator::generateGroundPlaneAux(const QString & boardSvg,
 		return NULL;
 	}
 
-	/*
-	QFile file0("testGroundFillBoard.svg");
-	file0.open(QIODevice::WriteOnly);
-	QTextStream out0(&file0);
-	out0 << boardByteArray;
-	file0.close();
-	*/
-
-
+	
+	//QFile file0("testGroundFillBoard.svg");
+	//file0.open(QIODevice::WriteOnly);
+	//QTextStream out0(&file0);
+	//out0 << boardByteArray;
+	//file0.close();
+	
 	QByteArray copperByteArray;
-	if (!SvgFileSplitter::changeStrokeWidth(svg, 50, false, copperByteArray)) {
+	if (!SvgFileSplitter::changeStrokeWidth(svg, m_strokeWidthIncrement, false, true, copperByteArray)) {
 		return NULL;
 	}
+	
+	//QFile file1("testGroundFillCopper.svg");
+	//file1.open(QIODevice::WriteOnly);
+	//QTextStream out1(&file1);
+	//out1 << copperByteArray;
+	//file1.close();
+	
 
-	/*
-	QFile file1("testGroundFillCopper.svg");
-	file1.open(QIODevice::WriteOnly);
-	QTextStream out1(&file1);
-	out1 << copperByteArray;
-	file1.close();
-	*/
-
-	qreal svgWidth = res * qMax(boardImageSize.width(), copperImageSize.width()) / FSvgRenderer::printerScale();
-	qreal svgHeight = res * qMax(boardImageSize.height(), copperImageSize.height()) / FSvgRenderer::printerScale();
+	double svgWidth = res * qMax(boardImageSize.width(), copperImageSize.width()) / GraphicsUtils::SVGDPI;
+	double svgHeight = res * qMax(boardImageSize.height(), copperImageSize.height()) / GraphicsUtils::SVGDPI;
 
 	QRectF br =  board->sceneBoundingRect();
-	bWidth = res * br.width() / FSvgRenderer::printerScale();
-	bHeight = res * br.height() / FSvgRenderer::printerScale();
-	QImage * image = new QImage(qMax(svgWidth, bWidth), qMax(svgHeight, bHeight), QImage::Format_RGB32);
+	bWidth = res * br.width() / GraphicsUtils::SVGDPI;
+	bHeight = res * br.height() / GraphicsUtils::SVGDPI;
+	QImage * image = new QImage(qMax(svgWidth, bWidth), qMax(svgHeight, bHeight), QImage::Format_Mono); //
 	image->setDotsPerMeterX(res * GraphicsUtils::InchesPerMeter);
 	image->setDotsPerMeterY(res * GraphicsUtils::InchesPerMeter);
 	image->fill(0x0);
@@ -294,45 +279,87 @@ QImage * GroundPlaneGenerator::generateGroundPlaneAux(const QString & boardSvg,
 	QSvgRenderer renderer(boardByteArray);
 	QPainter painter;
 	painter.begin(image);
-	painter.setRenderHint(QPainter::Antialiasing);
-	renderer.render(&painter, QRectF(0, 0, res * boardImageSize.width() / FSvgRenderer::printerScale(), res * boardImageSize.height() / FSvgRenderer::printerScale()));
+	painter.setRenderHint(QPainter::Antialiasing, false);
+	QRectF boardBounds(0, 0, res * boardImageSize.width() / GraphicsUtils::SVGDPI, res * boardImageSize.height() / GraphicsUtils::SVGDPI); 
+	DebugDialog::debug("boardbounds", boardBounds);
+	renderer.render(&painter, boardBounds);
 	painter.end();
 
-	//image->save("testGroundFillBoard.png");
+#ifndef QT_NO_DEBUG
+	image->save(FolderUtils::getUserDataStorePath("") + "/testGroundFillBoard.png");
+#endif
 
+	for (double m = 0; m < BORDERINCHES; m += (1.0 / res)) {   // 1 mm
+		QList<QPoint> points;
+		collectBorderPoints(*image, points);
+
+#ifndef QT_NO_DEBUG
+		/*
+		// for debugging
+		double pixelFactor = GraphicsUtils::StandardFritzingDPI / res;
+		QPolygon polygon;
+		foreach(QPoint p, points) {
+			polygon.append(QPoint(p.x() * pixelFactor, p.y() * pixelFactor));
+		}
+
+		QList<QPolygon> polygons;
+		polygons.append(polygon);
+		QPointF offset;
+		this
+		QString pSvg = makePolySvg(polygons, res, bWidth, bHeight, pixelFactor, "#ffffff", false,  NULL, QSizeF(0,0), 0, QPointF(0, 0));
+		*/
+#endif
+
+		foreach (QPoint p, points) image->setPixel(p, 0);
+	}
+
+#ifndef QT_NO_DEBUG
+	image->save(FolderUtils::getUserDataStorePath("") + "/testGroundFillBoardBorder.png");
+#endif
+	
 	// "blur" the image a little
+
 	QSvgRenderer renderer2(copperByteArray);
 	painter.begin(image);
-	QRectF bounds(0, 0, res * copperImageSize.width() / FSvgRenderer::printerScale(), res * copperImageSize.height() / FSvgRenderer::printerScale());
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(1, 0);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(-1, 0);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(0, 1);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(0, -1);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(1, 1);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(-1, -1);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(-1, 1);
-	renderer2.render(&painter, bounds);
-	bounds.moveTo(1, -1);
+	painter.setRenderHint(QPainter::Antialiasing, false);
+	QRectF bounds(0, 0, res * copperImageSize.width() / GraphicsUtils::SVGDPI, res * copperImageSize.height() / GraphicsUtils::SVGDPI);
+	DebugDialog::debug("copperbounds", bounds);
 	renderer2.render(&painter, bounds);
+	if (m_blurBy != 0) {
+		bounds.moveTo(m_blurBy, 0);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(-m_blurBy, 0);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(0, m_blurBy);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(0, -m_blurBy);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(m_blurBy, m_blurBy);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(-m_blurBy, -m_blurBy);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(-m_blurBy, m_blurBy);
+		renderer2.render(&painter, bounds);
+		bounds.moveTo(m_blurBy, -m_blurBy);
+		renderer2.render(&painter, bounds);
+	}
 	painter.end();
 
-	//image->save("testGroundFillCopper.png");
+#ifndef QT_NO_DEBUG
+	image->save(FolderUtils::getUserDataStorePath("") + "/testGroundFillCopper.png");
+#endif
+
+	emit postImageSignal(this, image, board);	
+
 	return image;
 }
 
-void GroundPlaneGenerator::scanImage(QImage & image, qreal bWidth, qreal bHeight, qreal pixelFactor, qreal res, 
-									 const QString & colorString, const QString & layerName, bool makeConnector, 
-									 int minRunSize, bool makeOffset, QSizeF minAreaInches, qreal minDimensionInches)  
+void GroundPlaneGenerator::scanImage(QImage & image, double bWidth, double bHeight, double pixelFactor, double res, 
+									 const QString & colorString, bool makeConnector, 
+									 bool makeOffset, QSizeF minAreaInches, double minDimensionInches, QPointF polygonOffset)  
 {
 	QList<QRect> rects;
-	scanLines(image, bWidth, bHeight, rects, THRESHOLD, minRunSize);
+	scanLines(image, bWidth, bHeight, rects);
 	QList< QList<int> * > pieces;
 	splitScanLines(rects, pieces);
 	foreach (QList<int> * piece, pieces) {
@@ -346,12 +373,12 @@ void GroundPlaneGenerator::scanImage(QImage & image, qreal bWidth, qreal bHeight
 		// note: there is always one
 		joinScanLines(newRects, polygons);
 		QPointF offset;
-		QString pSvg = makePolySvg(polygons, res, bWidth, bHeight, pixelFactor, colorString, layerName, makeConnector, makeOffset ? &offset : NULL, minAreaInches, minDimensionInches);
+		QString pSvg = makePolySvg(polygons, res, bWidth, bHeight, pixelFactor, colorString, makeConnector, makeOffset ? &offset : NULL, minAreaInches, minDimensionInches, polygonOffset);
 		if (pSvg.isEmpty()) continue;
 
 		m_newSVGs.append(pSvg);
 		if (makeOffset) {
-			offset *= FSvgRenderer::printerScale();
+			offset *= GraphicsUtils::SVGDPI;
 			m_newOffsets.append(offset);			// offset now in pixels
 		}
 
@@ -390,28 +417,70 @@ void GroundPlaneGenerator::scanImage(QImage & image, qreal bWidth, qreal bHeight
 
 }
 
-void GroundPlaneGenerator::scanLines(QImage & image, int bWidth, int bHeight, QList<QRect> & rects, int threshold, int minRunSize)
+void GroundPlaneGenerator::scanLines(QImage & image, int bWidth, int bHeight, QList<QRect> & rects)
 {
+	if (m_minRiseSize > 1) {
+		for (int x = 0; x < bWidth; x++) {
+			bool inWhite = false;
+			int whiteStart = 0;
+			for (int y = 0; y < bHeight; y++) {
+				QRgb current = image.pixel(x, y);
+				if (inWhite) {
+					if (current == 0xffffffff) {			// qBlue(current) == 0xff    gray > 128
+						// another white pixel, keep moving
+						continue;
+					}
+
+					// got black: close up this segment;
+					inWhite = false;
+					if (y - whiteStart < m_minRiseSize) {
+						for (int j = whiteStart; j <= y; j++) {
+							image.setPixel(x, j, 0);
+						}
+						continue;
+					}
+
+				}
+				else {
+					if (current != 0xffffffff) {		// qBlue(current) != 0xff				
+						// another black pixel, keep moving
+						continue;
+					}
+
+					inWhite = true;
+					whiteStart = y;
+				}
+			}
+			if (inWhite) {
+				// close up the last segment
+				if (bHeight - whiteStart < m_minRiseSize) {
+					for (int j = whiteStart; j <= bHeight; j++) {
+						image.setPixel(x, j, 0);
+					}			
+				}
+			}
+		}
+	}
+
 	// threshold should be between 0 and 255 exclusive; smaller will include more of the svg
 	for (int y = 0; y < bHeight; y++) {
 		bool inWhite = false;
 		int whiteStart = 0;
-		QRgb* scanLine = (QRgb *) image.scanLine(y);
 		for (int x = 0; x < bWidth; x++) {
-			QRgb current = *(scanLine + x);
+			QRgb current = image.pixel(x, y);
 			//if (current != 0xff000000 && current != 0xffffffff) {
 				//DebugDialog::debug(QString("current %1").arg(current,0,16));
 			//}
-			int gray = qGray(current);
+			//DebugDialog::debug(QString("current %1 %2").arg(current,0,16).arg(gray, 0, 16));
 			if (inWhite) {
-				if (gray > threshold) {			// qBlue(current) == 0xff    gray > 128
+				if (current == 0xffffffff) {			// qBlue(current) == 0xff    gray > 128
 					// another white pixel, keep moving
 					continue;
 				}
 
 				// got black: close up this segment;
 				inWhite = false;
-				if (x - whiteStart < minRunSize) {
+				if (x - whiteStart < m_minRunSize) {
 					// not a big enough section
 					continue;
 				}
@@ -419,7 +488,7 @@ void GroundPlaneGenerator::scanLines(QImage & image, int bWidth, int bHeight, QL
 				rects.append(QRect(whiteStart, y, x - whiteStart, 1));
 			}
 			else {
-				if (gray <= threshold) {		// qBlue(current) != 0xff				
+				if (current != 0xffffffff) {		// qBlue(current) != 0xff				
 					// another black pixel, keep moving
 					continue;
 				}
@@ -430,7 +499,7 @@ void GroundPlaneGenerator::scanLines(QImage & image, int bWidth, int bHeight, QL
 		}
 		if (inWhite) {
 			// close up the last segment
-			if (bWidth - whiteStart >= minRunSize) {
+			if (bWidth - whiteStart >= m_minRunSize) {
 				rects.append(QRect(whiteStart, y, bWidth - whiteStart, 1));
 			}
 		}
@@ -663,9 +732,9 @@ void GroundPlaneGenerator::joinScanLines(QList<QRect> & rects, QList<QPolygon> &
 	}
 }
 
-QString GroundPlaneGenerator::makePolySvg(QList<QPolygon> & polygons, qreal res, qreal bWidth, qreal bHeight, qreal pixelFactor, 
-										const QString & colorString, const QString & layerName, bool makeConnectorFlag, QPointF * offset, 
-										QSizeF minAreaInches, qreal minDimensionInches) 
+QString GroundPlaneGenerator::makePolySvg(QList<QPolygon> & polygons, double res, double bWidth, double bHeight, double pixelFactor, 
+										const QString & colorString, bool makeConnectorFlag, QPointF * offset, 
+										QSizeF minAreaInches, double minDimensionInches, QPointF polygonOffset) 
 {
 	int minX = 0;
 	int minY = 0;
@@ -703,7 +772,11 @@ QString GroundPlaneGenerator::makePolySvg(QList<QPolygon> & polygons, qreal res,
 		.arg(bHeight / res)
 		.arg(bWidth * pixelFactor)
 		.arg(bHeight * pixelFactor);
-	pSvg += QString("<g id='%1'>\n").arg(layerName);
+	QString transform;
+	if (polygonOffset.x() != 0 || polygonOffset.y() != 0) {
+		transform = QString("transform='translate(%1, %2)'").arg(polygonOffset.x()).arg(polygonOffset.y());
+	}
+	pSvg += QString("<g id='%1' %2>\n").arg(m_layerName).arg(transform);
 	if (makeConnectorFlag) {
 		makeConnector(polygons, res, pixelFactor, colorString, minX, minY, pSvg);
 	}
@@ -718,7 +791,7 @@ QString GroundPlaneGenerator::makePolySvg(QList<QPolygon> & polygons, qreal res,
 	return pSvg;
 }
 
-void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res, qreal pixelFactor, const QString & colorString, int minX, int minY, QString & pSvg)
+void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, double res, double pixelFactor, const QString & colorString, int minX, int minY, QString & pSvg)
 {
 	//	see whether the standard circular connector will fit somewhere inside a polygon:
 	//	http://stackoverflow.com/questions/4279478/maximum-circle-inside-a-non-convex-polygon
@@ -727,12 +800,12 @@ void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res,
 
 	//	code presently uses a version of the Poles of Inaccessibility algorithm:
 
-	static const qreal standardConnectorWidth = .075;		 // inches
-	qreal targetDiameter = res * pixelFactor * standardConnectorWidth;
-	qreal targetDiameterAnd = targetDiameter * 1.25;
-	qreal targetRadius = targetDiameter / 2;
-	qreal targetRadiusAnd = targetDiameterAnd / 2;
-	qreal targetRadiusAndSquared = targetRadiusAnd * targetRadiusAnd;
+	static const double standardConnectorWidth = .075;		 // inches
+	double targetDiameter = res * pixelFactor * standardConnectorWidth;
+	double targetDiameterAnd = targetDiameter * 1.25;
+	double targetRadius = targetDiameter / 2;
+	double targetRadiusAnd = targetDiameterAnd / 2;
+	double targetRadiusAndSquared = targetRadiusAnd * targetRadiusAnd;
 	foreach (QPolygon poly, polygons) {
 		QRect boundingRect = poly.boundingRect(); 
 		if (boundingRect.width() < targetDiameterAnd) continue;
@@ -748,10 +821,10 @@ void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res,
 		int xDivisor = qRound(boundingRect.width() / targetRadius);
 		int yDivisor = qRound(boundingRect.height() / targetRadius);
 
-		qreal dx = (boundingRect.width() - targetDiameterAnd) / xDivisor;
-		qreal dy = (boundingRect.height() - targetDiameterAnd) / yDivisor;
-		qreal x;
-		qreal y = boundingRect.top() + targetRadiusAnd - dy;
+		double dx = (boundingRect.width() - targetDiameterAnd) / xDivisor;
+		double dy = (boundingRect.height() - targetDiameterAnd) / yDivisor;
+		double x;
+		double y = boundingRect.top() + targetRadiusAnd - dy;
 		for (int iy = 0; iy <= yDivisor; iy++) {
 			y += dy;
 			x = boundingRect.left() + targetRadiusAnd - dx;
@@ -761,7 +834,7 @@ void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res,
 
 				bool gotOne = true;
 				foreach (QLineF line, polyLines) {
-					qreal distance, dx, dy;
+					double distance, dx, dy;
 					bool atEndpoint;
 					GraphicsUtils::distanceFromLine(x, y, line.p1().x(), line.p1().y(), line.p2().x(), line.p2().y(), 
 													dx, dy, distance, atEndpoint);
@@ -793,8 +866,8 @@ void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res,
 	// couldn't find anything big enough above, so
 	// try to find a poly with an area that's big enough to click, but not so big as to get in the way
 	int useIndex = -1;
-	QList<qreal> areas;
-	qreal divisor = res * pixelFactor * res * pixelFactor;
+	QList<double> areas;
+	double divisor = res * pixelFactor * res * pixelFactor;
 	foreach (QPolygon poly, polygons) {
 		areas.append(calcArea(poly) / divisor);
 	}
@@ -836,8 +909,8 @@ void GroundPlaneGenerator::makeConnector(QList<QPolygon> & polygons, qreal res,
 	}
 }
 
-qreal GroundPlaneGenerator::calcArea(QPolygon & poly) {
-	qreal total = 0;
+double GroundPlaneGenerator::calcArea(QPolygon & poly) {
+	double total = 0;
 	for (int ix = 0; ix < poly.count(); ix++) {
 		QPoint p0 = poly.at(ix);
 		QPoint p1 = poly.at((ix + 1) % poly.count());
@@ -867,3 +940,258 @@ const QStringList & GroundPlaneGenerator::newSVGs() {
 const QList<QPointF> & GroundPlaneGenerator::newOffsets() {
 	return m_newOffsets;
 }
+
+void removeRedundant(QList<QPoint> & points)
+{
+	QPoint current = points.last();
+	int ix = points.count() - 2;
+	int soFar = 1;
+	while (ix > 0) {
+		if (points.at(ix).x() != current.x()) {
+			current = points.at(ix--);
+			soFar = 1;
+			continue;
+		}
+
+		if (++soFar > 2) {
+			points.removeAt(ix + 1);
+		}
+		ix--;
+	}
+}
+
+void GroundPlaneGenerator::collectBorderPoints(QImage & image, QList<QPoint> & points)
+{
+	// background is black
+
+    int currentX = 0, currentY = 0;
+	bool gotSomething = false;
+
+	for (int y = 0; y < image.height(); y++) {
+		for (int x = 0; x < image.width(); x++) {
+			QRgb current = image.pixel(x, y);
+			if (current != 0xffffffff) {				
+				// another black pixel, keep moving
+				continue;
+			}
+
+			currentX = x;
+			currentY = y;
+			//DebugDialog::debug(QString("first point %1 %2").arg(currentX).arg(currentY));
+			points.append(QPoint(currentX, currentY));
+			gotSomething = true;
+			break;
+		}
+		if (gotSomething) break;
+	}
+
+	if (!gotSomething) {
+		DebugDialog::debug("first border point not found");
+		return;
+	}
+
+    //int inc = 0;
+	while (true) {
+        if (try8(currentX, currentY, image, points)) ;
+		else {
+            
+            QPoint p = points.first();
+            if (qAbs(p.x() - currentX) < 4 && qAbs(p.y() - currentY) < 4) {
+                // we're near the beginning again
+                break;
+            }
+
+            bool keepGoing = false;
+            for (int ix = points.count() - 2; ix >= 0; ix--) {
+                QPoint p = points.at(ix);
+                if (try8(p.x(), p.y(), image, points)) {
+                    keepGoing = true; 
+                    break;
+                }
+            }
+
+            if (!keepGoing) break;
+        }
+
+		QPoint p = points.last();
+		currentX = p.x();
+		currentY = p.y();
+        //DebugDialog::debug(QString("next point %1 %2").arg(currentX).arg(currentY));
+        //if (++inc % 100 == 0) {
+        //    DebugDialog::debug("\n");
+        //}
+	}
+}
+
+void GroundPlaneGenerator::scanOutline(QImage & image, double bWidth, double bHeight, double pixelFactor, double res, 
+									 const QString & colorString, bool makeConnector, 
+									 bool makeOffset, QSizeF minAreaInches, double minDimensionInches)  
+{
+	QList<QPoint> points;
+
+	collectBorderPoints(image, points);
+	if (points.count() == 0) {
+		DebugDialog::debug("no border points");
+		return;
+	}
+
+	removeRedundant(points);
+
+	QPolygon polygon;
+	foreach(QPoint p, points) {
+		polygon.append(QPoint(p.x() * pixelFactor, p.y() * pixelFactor));
+	}
+
+	QList<QPolygon> polygons;
+	polygons.append(polygon);
+	QPointF offset;
+	QString pSvg = makePolySvg(polygons, res, bWidth, bHeight, pixelFactor, colorString, makeConnector, makeOffset ? &offset : NULL, minAreaInches, minDimensionInches, QPointF(0, 0));
+	if (!pSvg.isEmpty()) {
+		m_newSVGs.append(pSvg);
+		if (makeOffset) {
+			offset *= GraphicsUtils::SVGDPI;
+			m_newOffsets.append(offset);			// offset now in pixels
+		}
+	}
+
+	/*
+	QFile file4("testPoly.svg");
+	file4.open(QIODevice::WriteOnly);
+	QTextStream out4(&file4);
+	out4 << pSvg;
+	file4.close();
+	*/
+}
+
+
+bool GroundPlaneGenerator::try8(int x, int y, QImage & image, QList<QPoint> & points) {
+    if (tryNextPoint(x, y + 1, image, points)) return true;
+	else if (tryNextPoint(x + 1, y, image, points)) return true;
+	else if (tryNextPoint(x, y - 1, image, points)) return true;
+	else if (tryNextPoint(x - 1, y, image, points)) return true;
+	else if (tryNextPoint(x + 1, y + 1, image, points)) return true;
+	else if (tryNextPoint(x - 1, y + 1, image, points)) return true;
+	else if (tryNextPoint(x + 1, y - 1, image, points)) return true;
+	else if (tryNextPoint(x - 1, y - 1, image, points)) return true;
+    return false;
+}
+
+bool GroundPlaneGenerator::tryNextPoint(int x, int y, QImage & image, QList<QPoint> & points)
+{
+	if (x < 0) return false;
+	if (y < 0) return false;
+	if (x >= image.width()) return false;
+	if (y >= image.height()) return false;
+
+	foreach (QPoint p, points) {
+		if (p.x() == x && p.y() == y) {
+			// already visited
+			return false;
+		}
+
+		if (qAbs(p.x() - x) > 3 && qAbs(p.y() - y) > 3) {
+			// too far away from the start of the polygon
+			break;
+		}
+	}
+
+
+	for (int i = points.count() - 1; i >= 0; i--) {
+		QPoint p = points.at(i);
+		if (p.x() == x && p.y() == y) {
+			// already visited
+			return false;
+		}
+
+		if (qAbs(p.x() - x) > 3 && qAbs(p.y() - y) > 3) {
+			// too far away from from the current point
+			break;
+		}
+	}
+
+	QRgb pixel = image.pixel(x, y);
+    //DebugDialog::debug(QString("pixel %1,%2 %3").arg(x).arg(y).arg(pixel, 0, 16));
+	if (pixel != 0xffffffff) {						
+		// empty pixel, not on the border
+		return false;
+	}
+
+	if (x + 1 == image.width()) {
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	pixel = image.pixel(x + 1, y);
+	if (pixel != 0xffffffff) {						
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	if (y + 1 == image.height()) {
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	pixel = image.pixel(x, y + 1);
+	if (pixel != 0xffffffff) {						
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	if (x - 1  < 0) {
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	pixel = image.pixel(x - 1, y);
+	if (pixel != 0xffffffff) {						
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	if (y - 1  < 0) {
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	pixel = image.pixel(x, y - 1);
+	if (pixel != 0xffffffff) {						
+		points.append(QPoint(x, y));
+		return true;
+	}
+
+	return false;
+}
+
+void GroundPlaneGenerator::setBlurBy(double blurBy) {
+	m_blurBy = blurBy;
+}
+
+void GroundPlaneGenerator::setStrokeWidthIncrement(double swi) {
+	m_strokeWidthIncrement = swi;
+}
+
+void GroundPlaneGenerator::setLayerName(const QString & layerName) {
+	m_layerName = layerName;
+}
+
+const QString & GroundPlaneGenerator::layerName() {
+	return m_layerName;
+}
+
+void GroundPlaneGenerator::setMinRunSize(int mrus, int mris) {
+	m_minRunSize = mrus;
+	m_minRiseSize = mris;
+}
+
+QString GroundPlaneGenerator::mergeSVGs(const QString & initialSVG, const QString & layerName) {
+    QDomDocument doc;
+    if (!initialSVG.isEmpty()) {
+		TextUtils::mergeSvg(doc, initialSVG, layerName);        
+    }
+	foreach (QString newSvg, m_newSVGs) {
+		TextUtils::mergeSvg(doc, newSvg, layerName);
+	}
+	return TextUtils::mergeSvgFinish(doc);
+}
+
