@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6181 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-07-18 15:52:11 +0200 (Wed, 18 Jul 2012) $
 
 ********************************************************************/
 
@@ -28,7 +28,7 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 
 #include "../utils/misc.h"
 #include "../utils/textutils.h"
-#include "../debugdialog.h"
+//#include "../debugdialog.h"
 #include "svgpathparser.h"
 #include "svgpathlexer.h"
 #include "svgpathrunner.h"
@@ -41,14 +41,14 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 static QString findStyle("%1[\\s]*:[\\s]*([^;]*)[;]?");
 
 struct HVConvertData {
-	qreal x;
-	qreal y;
-	qreal subX;
-	qreal subY;
+	double x;
+	double y;
+	double subX;
+	double subY;
 	QString path;
 };
 
-void appendPair(QString & path, qreal a1, qreal a2) {
+void appendPair(QString & path, double a1, double a2) {
 	path.append(QString::number(a1));
 	path.append(',');
 	path.append(QString::number(a2));
@@ -81,9 +81,9 @@ bool SvgFileSplitter::splitString(QString & contents, const QString & elementID)
 	m_byteArray.clear();
 
 	// gets rid of some crap inserted by illustrator which can screw up polygons and paths
-	contents.remove(QChar('\t'));
-	contents.remove(QChar('\n'));
-	contents.remove(QChar('\r'));
+	contents.replace('\t', ' ');
+	contents.replace('\n', ' ');
+	contents.replace('\r', ' ');
 
 	// get rid of inkscape stuff too
 	contents.remove(TextUtils::SodipodiDetector);
@@ -93,6 +93,7 @@ bool SvgFileSplitter::splitString(QString & contents, const QString & elementID)
 	int errorColumn;
 
 	if (!m_domDocument.setContent(contents, true, &errorStr, &errorLine, &errorColumn)) {
+        //DebugDialog::debug(QString("parse error: %1 l:%2 c:%3\n\n%4").arg(errorStr).arg(errorLine).arg(errorColumn).arg(contents));
 		return false;
 	}
 
@@ -129,9 +130,10 @@ bool SvgFileSplitter::splitString(QString & contents, const QString & elementID)
 		element.removeAttribute("id");
 	}
 
-	QString elementText;
-	QTextStream textStream(&elementText);
-	element.save(textStream, 0);
+	QDomDocument document;
+	QDomNode node = document.importNode(element, true);
+	document.appendChild(node);
+	QString elementText = document.toString();
 
 	if (!superTransforms.isEmpty()) {
 		for (int i = 0; i < superTransforms.count() - 1; i++) {
@@ -182,14 +184,14 @@ QString SvgFileSplitter::elementString(const QString & elementID) {
 	QDomElement mainElement = TextUtils::findElementWithAttribute(root, "id", elementID);
 	if (mainElement.isNull()) return ___emptyString___;
 
-	QString text;
-	QTextStream textStream(&text);
-	mainElement.save(textStream, 0);
+	QDomDocument document;
+	QDomNode node = document.importNode(mainElement, true);
+	document.appendChild(node);
 
-	return text;
+	return document.toString();
 }
 
-bool SvgFileSplitter::normalize(qreal dpi, const QString & elementID, bool blackOnly)
+bool SvgFileSplitter::normalize(double dpi, const QString & elementID, bool blackOnly)
 {
 	// get the viewbox and the width and height
 	// then normalize them
@@ -199,7 +201,7 @@ bool SvgFileSplitter::normalize(qreal dpi, const QString & elementID, bool black
 	QDomElement root = m_domDocument.documentElement();
 	if (root.isNull()) return false;
 
-	qreal sWidth, sHeight, vbWidth, vbHeight;
+	double sWidth, sHeight, vbWidth, vbHeight;
 	if (!TextUtils::getSvgSizes(m_domDocument, sWidth, sHeight, vbWidth, vbHeight)) return false;
 
 	root.setAttribute("width", QString("%1in").arg(sWidth));
@@ -218,7 +220,7 @@ bool SvgFileSplitter::normalize(qreal dpi, const QString & elementID, bool black
 	return true;
 }
 
-QPainterPath SvgFileSplitter::painterPath(qreal dpi, const QString & elementID)
+QPainterPath SvgFileSplitter::painterPath(double dpi, const QString & elementID)
 {
 	// only partially implemented because so far we only use this for polygons generated for the groundplane
 
@@ -247,32 +249,32 @@ void SvgFileSplitter::painterPathChild(QDomElement & element, QPainterPath & ppa
 	// only partially implemented
 
 	if (element.nodeName().compare("circle") == 0) {
-		qreal cx = element.attribute("cx").toDouble();
-		qreal cy = element.attribute("cy").toDouble();
-		qreal r = element.attribute("r").toDouble();
-		qreal stroke = element.attribute("stroke-width").toDouble();
+		double cx = element.attribute("cx").toDouble();
+		double cy = element.attribute("cy").toDouble();
+		double r = element.attribute("r").toDouble();
+		double stroke = element.attribute("stroke-width").toDouble();
 		ppath.addEllipse(QRectF(cx - r - (stroke / 2), cy - r - (stroke / 2), (r * 2) + stroke, (r * 2) + stroke));
 	}
 	else if (element.nodeName().compare("line") == 0) {
 
 		/*
-		qreal x1 = element.attribute("x1").toDouble();
-		qreal y1 = element.attribute("y1").toDouble();
-		qreal x2 = element.attribute("x2").toDouble();
-		qreal y2 = element.attribute("y2").toDouble();
-		qreal stroke = element.attribute("stroke-width").toDouble();
+		double x1 = element.attribute("x1").toDouble();
+		double y1 = element.attribute("y1").toDouble();
+		double x2 = element.attribute("x2").toDouble();
+		double y2 = element.attribute("y2").toDouble();
+		double stroke = element.attribute("stroke-width").toDouble();
 
 		// treat as parallel lines stroke width apart?
 		*/
 	}
 	else if (element.nodeName().compare("rect") == 0) {
-		qreal width = element.attribute("width").toDouble();
-		qreal height = element.attribute("height").toDouble();
-		qreal x = element.attribute("x").toDouble();
-		qreal y = element.attribute("y").toDouble();
-		qreal stroke = element.attribute("stroke-width").toDouble();
-		qreal rx = element.attribute("rx").toDouble();
-		qreal ry = element.attribute("ry").toDouble();
+		double width = element.attribute("width").toDouble();
+		double height = element.attribute("height").toDouble();
+		double x = element.attribute("x").toDouble();
+		double y = element.attribute("y").toDouble();
+		double stroke = element.attribute("stroke-width").toDouble();
+		double rx = element.attribute("rx").toDouble();
+		double ry = element.attribute("ry").toDouble();
 		if (rx != 0 || ry != 0) { 
 			ppath.addRoundedRect(x - (stroke / 2), y - (stroke / 2), width + stroke, height + stroke, rx, ry);
 		}
@@ -281,11 +283,11 @@ void SvgFileSplitter::painterPathChild(QDomElement & element, QPainterPath & ppa
 		}
 	}
 	else if (element.nodeName().compare("ellipse") == 0) {
-		qreal cx = element.attribute("cx").toDouble();
-		qreal cy = element.attribute("cy").toDouble();
-		qreal rx = element.attribute("rx").toDouble();
-		qreal ry = element.attribute("ry").toDouble();
-		qreal stroke = element.attribute("stroke-width").toDouble();
+		double cx = element.attribute("cx").toDouble();
+		double cy = element.attribute("cy").toDouble();
+		double rx = element.attribute("rx").toDouble();
+		double ry = element.attribute("ry").toDouble();
+		double stroke = element.attribute("stroke-width").toDouble();
 		ppath.addEllipse(QRectF(cx - rx - (stroke / 2), cy - ry - (stroke / 2), (rx * 2) + stroke, (ry * 2) + stroke));
 	}
 	else if (element.nodeName().compare("polygon") == 0 || element.nodeName().compare("polyline") == 0) {
@@ -326,8 +328,8 @@ void SvgFileSplitter::painterPathChild(QDomElement & element, QPainterPath & ppa
 }
 
 void SvgFileSplitter::normalizeTranslation(QDomElement & element,
-											qreal sNewWidth, qreal sNewHeight,
-											qreal vbWidth, qreal vbHeight)
+											double sNewWidth, double sNewHeight,
+											double vbWidth, double vbHeight)
 {
 	QString attr = element.attribute("transform");
 	if (attr.isEmpty()) return;
@@ -335,8 +337,8 @@ void SvgFileSplitter::normalizeTranslation(QDomElement & element,
 	QMatrix matrix = TextUtils::elementToMatrix(element);
 	if (matrix.dx() == 0 && matrix.dy() == 0) return;
 
-	qreal dx = matrix.dx() * sNewWidth / vbWidth;
-	qreal dy = matrix.dy() * sNewHeight / vbHeight;
+	double dx = matrix.dx() * sNewWidth / vbWidth;
+	double dy = matrix.dy() * sNewHeight / vbHeight;
 	if (dx == 0 && dy == 0) return;
 
 	matrix.setMatrix(matrix.m11(), matrix.m12(), matrix.m21(), matrix.m22(), dx, dy);
@@ -346,8 +348,8 @@ void SvgFileSplitter::normalizeTranslation(QDomElement & element,
 
 
 void SvgFileSplitter::normalizeChild(QDomElement & element,
-									 qreal sNewWidth, qreal sNewHeight,
-									 qreal vbWidth, qreal vbHeight, bool blackOnly)
+									 double sNewWidth, double sNewHeight,
+									 double vbWidth, double vbHeight, bool blackOnly)
 {
 	normalizeTranslation(element, sNewWidth, sNewHeight, vbWidth, vbHeight);
 
@@ -356,7 +358,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 	if (nodeName.compare("g") == 0) {
 		fixStyleAttribute(element);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 		doChildren = true;
 	}
 	else if (nodeName.compare("circle") == 0) {
@@ -365,7 +367,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 		normalizeAttribute(element, "cy", sNewHeight, vbHeight);
 		normalizeAttribute(element, "r", sNewWidth, vbWidth);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("line") == 0) {
 		fixStyleAttribute(element);
@@ -374,7 +376,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 		normalizeAttribute(element, "x2", sNewWidth, vbWidth);
 		normalizeAttribute(element, "y2", sNewHeight, vbHeight);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("rect") == 0) {
 		fixStyleAttribute(element);
@@ -391,7 +393,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 		if (!element.attribute("ry").isEmpty()) {
 			normalizeAttribute(element, "ry", sNewHeight, vbHeight);
 		}
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("ellipse") == 0) {
 		fixStyleAttribute(element);
@@ -400,7 +402,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 		normalizeAttribute(element, "rx", sNewWidth, vbWidth);
 		normalizeAttribute(element, "ry", sNewHeight, vbHeight);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("polygon") == 0 || nodeName.compare("polyline") == 0) {
 		fixStyleAttribute(element);
@@ -419,12 +421,12 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 				element.setAttribute("points", pathUserData.string);
 			}
 		}
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("path") == 0) {
 		fixStyleAttribute(element);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 		QString data = element.attribute("d").trimmed();
 		if (!data.isEmpty()) {
 			const char * slot = SLOT(normalizeCommandSlot(QChar, bool, QList<double> &, void *));
@@ -445,7 +447,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 		normalizeAttribute(element, "y", sNewHeight, vbHeight);
 		normalizeAttribute(element, "stroke-width", sNewWidth, vbWidth);
 		normalizeAttribute(element, "font-size", sNewWidth, vbWidth);
-		setStrokeOrFill(element, blackOnly, "black");
+		setStrokeOrFill(element, blackOnly, "black", false);
 	}
 	else if (nodeName.compare("linearGradient") == 0) {
 		if (element.attribute("gradientUnits").compare("userSpaceOnUse") == 0) {
@@ -455,7 +457,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 			normalizeAttribute(element, "y2", sNewWidth, vbWidth);
 		}
 		else {
-			DebugDialog::debug(QString("unable to handle linearGradient with gradientUnits=%1").arg(element.attribute("gradientUnits")));
+			//DebugDialog::debug(QString("unable to handle linearGradient with gradientUnits=%1").arg(element.attribute("gradientUnits")));
 		}
 	}
 	else if (nodeName.compare("radialGradient") == 0) {
@@ -467,7 +469,7 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 			normalizeAttribute(element, "r", sNewWidth, vbWidth);
 		}
 		else {
-			DebugDialog::debug(QString("unable to handle radialGradient with gradientUnits=%1").arg(element.attribute("gradientUnits")));
+			//DebugDialog::debug(QString("unable to handle radialGradient with gradientUnits=%1").arg(element.attribute("gradientUnits")));
 		}
 	}
 	else {
@@ -483,19 +485,19 @@ void SvgFileSplitter::normalizeChild(QDomElement & element,
 	}
 }
 
-bool SvgFileSplitter::normalizeAttribute(QDomElement & element, const char * attributeName, qreal num, qreal denom)
+bool SvgFileSplitter::normalizeAttribute(QDomElement & element, const char * attributeName, double num, double denom)
 {
-	qreal n = element.attribute(attributeName).toDouble() * num / denom;
+	double n = element.attribute(attributeName).toDouble() * num / denom;
 	element.setAttribute(attributeName, QString::number(n));
 	return true;
 }
 
-QString SvgFileSplitter::shift(qreal x, qreal y, const QString & elementID, bool shiftTransforms)
+QString SvgFileSplitter::shift(double x, double y, const QString & elementID, bool shiftTransforms)
 {
 	QDomElement root = m_domDocument.documentElement();
 
 	QDomElement mainElement = TextUtils::findElementWithAttribute(root, "id", elementID);
-	if (mainElement.isNull()) return false;
+	if (mainElement.isNull()) return "";
 
 	QDomElement childElement = mainElement.firstChildElement();
 	while (!childElement.isNull()) {
@@ -511,7 +513,7 @@ QString SvgFileSplitter::shift(qreal x, qreal y, const QString & elementID, bool
 
 }
 
-bool SvgFileSplitter::shiftTranslation(QDomElement & element, qreal x, qreal y)
+bool SvgFileSplitter::shiftTranslation(QDomElement & element, double x, double y)
 {
 	QString attr = element.attribute("transform");
 	if (attr.isEmpty()) return false;
@@ -519,9 +521,9 @@ bool SvgFileSplitter::shiftTranslation(QDomElement & element, qreal x, qreal y)
 	QMatrix m0 = TextUtils::elementToMatrix(element);
 
 	bool ok1, ok2, ok3;
-	qreal _x = element.attribute("_x").toDouble(&ok1);
-	qreal _y = element.attribute("_y").toDouble(&ok2);
-	qreal _r = element.attribute("_r").toDouble(&ok3);
+	double _x = element.attribute("_x").toDouble(&ok1);
+	double _y = element.attribute("_y").toDouble(&ok2);
+	double _r = element.attribute("_r").toDouble(&ok3);
 	if (ok1 && ok2 && ok3) {
 		QMatrix mx = QMatrix().translate(-_x - x, -_y - y) * QMatrix().rotate(_r) * QMatrix().translate(_x + x, _y + y);
 		TextUtils::setSVGTransform(element, mx);
@@ -534,13 +536,13 @@ bool SvgFileSplitter::shiftTranslation(QDomElement & element, qreal x, qreal y)
 	// calculate the original cx and cy and retranslate (there must be an easier way)
 	// and this doesn't seem to work...
 
-	qreal sinTheta = m0.m21();
-	qreal cosTheta = m0.m11();
-	qreal cosThetaMinusOne = cosTheta - 1;
+	double sinTheta = m0.m21();
+	double cosTheta = m0.m11();
+	double cosThetaMinusOne = cosTheta - 1;
 
-	qreal cx = (m0.dx() + (sinTheta * m0.dy() / cosThetaMinusOne)) / 
+	double cx = (m0.dx() + (sinTheta * m0.dy() / cosThetaMinusOne)) / 
 			    (cosThetaMinusOne + (sinTheta * sinTheta / cosThetaMinusOne));
-	qreal cy = (m0.dy() - (sinTheta * cx)) / cosThetaMinusOne;
+	double cy = (m0.dy() - (sinTheta * cx)) / cosThetaMinusOne;
 
 	cx += x;
 	cy += y;
@@ -553,7 +555,7 @@ bool SvgFileSplitter::shiftTranslation(QDomElement & element, qreal x, qreal y)
 }
 
 
-void SvgFileSplitter::shiftChild(QDomElement & element, qreal x, qreal y, bool shiftTransforms)
+void SvgFileSplitter::shiftChild(QDomElement & element, double x, double y, bool shiftTransforms)
 {
 	if (shiftTransforms) {
 		shiftTranslation(element, x, y);
@@ -724,7 +726,7 @@ void SvgFileSplitter::shiftCommandSlot(QChar command, bool relative, QList<doubl
 	switch(command.toAscii()) {
 		case 'v':
 		case 'V':
-			DebugDialog::debug("'v' and 'V' are now removed by preprocessing; shouldn't be here");
+			//DebugDialog::debug("'v' and 'V' are now removed by preprocessing; shouldn't be here");
 			/*
 			d = args[0];
 			if (!relative) {
@@ -735,7 +737,7 @@ void SvgFileSplitter::shiftCommandSlot(QChar command, bool relative, QList<doubl
 			break;
 		case 'h':
 		case 'H':
-			DebugDialog::debug("'h' and 'H' are now removed by preprocessing; shouldn't be here");
+			//DebugDialog::debug("'h' and 'H' are now removed by preprocessing; shouldn't be here");
 			/*
 			d = args[0];
 			if (!relative) {
@@ -830,10 +832,11 @@ QVector<QVariant> SvgFileSplitter::simpleParsePath(const QString & data) {
 }
 
 
-bool SvgFileSplitter::parsePath(const QString & data, const char * slot, PathUserData & pathUserData, QObject * slotTarget, bool convertHV) {
-	QVector<QVariant> symStack = simpleParsePath(data);
+bool SvgFileSplitter::parsePath(const QString & dataString, const char * slot, PathUserData & pathUserData, QObject * slotTarget, bool convertHV) {
+	QVector<QVariant> symStack = simpleParsePath(dataString);
 
-	if (convertHV) {
+	if (convertHV && (dataString.contains("h", Qt::CaseInsensitive) || dataString.contains("v",  Qt::CaseInsensitive))) 
+	{  
 		SVGPathRunner svgPathRunner;
 		HVConvertData data;
 		data.x = data.y = data.subX = data.subY = 0;
@@ -854,7 +857,7 @@ void SvgFileSplitter::convertHVSlot(QChar command, bool relative, QList<double>
 	Q_UNUSED(relative);
 	HVConvertData * data = (HVConvertData *) userData;
 
-	qreal x, y;
+	double x, y;
 	switch(command.toAscii()) {
 		case 'M':
 			data->path.append(command);
@@ -1029,14 +1032,14 @@ void SvgFileSplitter::convertHVSlot(QChar command, bool relative, QList<double>
 	}
 }
 
-void SvgFileSplitter::setStrokeOrFill(QDomElement & element, bool blackOnly, const QString & color)
+void SvgFileSplitter::setStrokeOrFill(QDomElement & element, bool blackOnly, const QString & color, bool force)
 {
 	if (!blackOnly) return;
 
 	// if stroke attribute is not empty make it black
 	// if fill attribute is not empty and not "none" make it black
 	QString stroke = element.attribute("stroke");
-	if (!stroke.isEmpty()) {
+	if (!stroke.isEmpty() || force) {
 		if (stroke.compare("none") != 0) {
 			element.setAttribute("stroke", color);
 		}
@@ -1061,28 +1064,37 @@ void SvgFileSplitter::fixStyleAttributeRecurse(QDomElement & element) {
 void SvgFileSplitter::fixColorRecurse(QDomElement & element, const QString & newColor, const QStringList & exceptions) {
 	fixStyleAttribute(element);
 	bool gotException = false;
+	QString s = element.attribute("stroke");
+	QString f = element.attribute("fill");
+	QString id = element.attribute("id");
 	foreach (QString e, exceptions) {
-		QString s = element.attribute("stroke");
-		QString f = element.attribute("fill");
 		if (s.isEmpty()) {
 			if (f.isEmpty()) {
-				gotException = true;			// not really, but saves a little extra processing time
 			}
 			else {
-				gotException = exceptions.contains(f);
+				if (exceptions.contains(f)) {
+					gotException = true;
+					break;
+				}
 			}
 		}
 		else {		
 			if (f.isEmpty()) {
-				gotException = exceptions.contains(s);
+				if (exceptions.contains(s)) {
+					gotException = true;
+					break;
+				}
 			}
 			else {
-				gotException = exceptions.contains(s) && exceptions.contains(f);
+				if (exceptions.contains(s) && exceptions.contains(f)) {
+					gotException = true;
+					break;
+				}
 			}
 		}
 	}
 	if (!gotException) {
-		setStrokeOrFill(element, true, newColor);
+		setStrokeOrFill(element, true, newColor, !id.isEmpty()); 
 	}
 
 	QDomElement childElement = element.firstChildElement();
@@ -1150,7 +1162,7 @@ bool SvgFileSplitter::getSvgSizeAttributes(const QString & svg, QString & width,
 	return false;
 }
 
-bool SvgFileSplitter::changeStrokeWidth(const QString & svg, qreal delta, bool absolute, QByteArray & byteArray) {
+bool SvgFileSplitter::changeStrokeWidth(const QString & svg, double delta, bool absolute, bool changeOpacity, QByteArray & byteArray) {
 	QString errorStr;
 	int errorLine;
 	int errorColumn;
@@ -1170,20 +1182,33 @@ bool SvgFileSplitter::changeStrokeWidth(const QString & svg, qreal delta, bool a
 		return false;
 	}
 
-	changeStrokeWidth(root, delta, absolute);
+	changeStrokeWidth(root, delta, absolute, changeOpacity);
 	byteArray = domDocument.toByteArray();
 	return true;
 }
 
-void SvgFileSplitter::changeStrokeWidth(QDomElement & element, qreal delta, bool absolute) {
+void SvgFileSplitter::changeStrokeWidth(QDomElement & element, double delta, bool absolute, bool changeOpacity) {
 	bool ok;
-	qreal sw = element.attribute("stroke-width").toDouble(&ok);
+	double sw = element.attribute("stroke-width").toDouble(&ok);
 	if (ok) {
 		element.setAttribute("stroke-width", QString::number((absolute) ? delta : sw + delta));
 	}
+
+    if (changeOpacity) {
+	    QString fillo = element.attribute("fill-opacity");
+	    if (!fillo.isEmpty()) {
+		    element.setAttribute("fill-opacity", "1.0");
+	    }
+
+	    QString sillo = element.attribute("stroke-opacity");
+	    if (!sillo.isEmpty()) {
+		    element.setAttribute("stroke-opacity", "1.0");
+	    }
+    }
+
 	QDomElement child = element.firstChildElement();
 	while (!child.isNull()) {
-		changeStrokeWidth(child, delta, absolute);
+		changeStrokeWidth(child, delta, absolute, changeOpacity);
 		child = child.nextSiblingElement();
 	}
 }
@@ -1240,9 +1265,9 @@ void SvgFileSplitter::changeColors(QDomElement & element, QString & toColor, QSt
 	}
 }
 
-bool SvgFileSplitter::shiftAttribute(QDomElement & element, const char * attributeName, qreal d)
+bool SvgFileSplitter::shiftAttribute(QDomElement & element, const char * attributeName, double d)
 {
-	qreal n = element.attribute(attributeName).toDouble() + d;
+	double n = element.attribute(attributeName).toDouble() + d;
 	element.setAttribute(attributeName, QString::number(n));
 	return true;
 }
@@ -1280,3 +1305,11 @@ void SvgFileSplitter::gWrap(const QHash<QString, QString> & attributes)
 {
 	TextUtils::gWrap(m_domDocument, attributes);
 }
+
+void SvgFileSplitter::gReplace(const QString & id)
+{
+	QDomElement element = TextUtils::findElementWithAttribute(m_domDocument.documentElement(), "id", id);
+	if (element.isNull()) return;
+
+	element.setTagName("g");
+}
