@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6265 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-08-08 04:07:28 +0200 (Wed, 08 Aug 2012) $
 
 ********************************************************************/
 
 #include "folderutils.h"
+#include "lockmanager.h"
 #include <QDesktopServices>
 #include <QCoreApplication>
 #include <QSettings>
@@ -35,32 +36,27 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include "../debugdialog.h"
 #include "../lib/quazip/quazip.h"
 #include "../lib/quazip/quazipfile.h"
-#include "../lib/qtlockedfile/qtlockedfile.h"
 #include "../lib/qtsysteminfo/QtSystemInfo.h"
 
 
 FolderUtils* FolderUtils::singleton = NULL;
 QString FolderUtils::m_openSaveFolder = "";
-const QString FolderUtils::LockFileName = "___lockfile___.txt";
 
 FolderUtils::FolderUtils() {
 	m_openSaveFolder = ___emptyString___;
-	m_folders << "/bins" 
+	m_folders 
+		<< "/bins" 
 		<< "/partfactory"
 		<< "/parts/user" << "/parts/contrib"
 		<< "/parts/svg/user/icon" << "/parts/svg/user/breadboard" << "/parts/svg/user/schematic" << "/parts/svg/user/pcb"
 		<< "/parts/svg/contrib/icon" << "/parts/svg/contrib/breadboard" << "/parts/svg/contrib/schematic" << "/parts/svg/contrib/pcb"
-		<< "/backup";
+		<< "/backup" 
+		<< "/fzz";
 }
 
 FolderUtils::~FolderUtils() {
 }
 
-// finds the user parts folder based on local desktop (OS) defaults
-QString FolderUtils::getUserPartsFolder() {
-	return QDesktopServices::storageLocation(QDesktopServices::DataLocation) + "/UserCreatedParts";
-}
-
 // finds a subfolder of the application directory searching backward up the tree
 QDir * FolderUtils::getApplicationSubFolder(QString search) {
 	if (singleton == NULL) {
@@ -99,7 +95,7 @@ QString FolderUtils::getApplicationSubFolderPath(QString search) {
 QString FolderUtils::getUserDataStorePath(QString folder) {
 	QString settingsFile = QSettings(QSettings::IniFormat,QSettings::UserScope,"Fritzing","Fritzing").fileName();
 	return QFileInfo(settingsFile).dir()
-		.absolutePath()+(folder!=___emptyString___?"/"+folder:"");
+        .absolutePath()+(folder.isEmpty()?"":QString("/")+folder);
 }
 
 const QStringList & FolderUtils::getUserDataStoreFolders() {
@@ -155,7 +151,35 @@ const QString FolderUtils::libraryPath()
 
 const QString FolderUtils::applicationDirPath() {
 	if (m_appPath.isEmpty()) {
+#ifdef Q_WS_WIN
 		return QCoreApplication::applicationDirPath();
+#else
+		// look in standard Fritzing location (applicationDirPath) then in standard linux locations
+		QStringList candidates;
+		candidates.append(QCoreApplication::applicationDirPath());
+#ifdef PKGDATADIR
+		candidates.append(QLatin1String(PKGDATADIR));
+#else
+		candidates.append("/usr/share/fritzing");
+		candidates.append("/usr/local/share/fritzing");
+#endif
+		candidates.append(QDir::homePath() + "/.local/share/fritzing");
+		foreach (QString candidate, candidates) {
+                        //DebugDialog::debug(QString("candidate:%1").arg(candidate));
+			QDir dir(candidate);
+                        if (!dir.exists("parts")) continue;
+
+                        if (dir.exists("bins")) {
+				m_appPath = candidate;
+				return m_appPath;
+			}
+
+		}
+
+		m_appPath = QCoreApplication::applicationDirPath();
+                DebugDialog::debug("data folders not found");
+
+#endif
 	}
 
 	return m_appPath;
@@ -175,18 +199,24 @@ const QStringList & FolderUtils::userDataStoreFolders() {
 }
 
 void FolderUtils::setOpenSaveFolder(const QString& path) {
+	setOpenSaveFolderAux(path);
+	QSettings settings;
+	settings.setValue("openSaveFolder", m_openSaveFolder);
+}
+
+void FolderUtils::setOpenSaveFolderAux(const QString& path) 
+{
 	QFileInfo fileInfo(path);
 	if(fileInfo.isDir()) {
 		m_openSaveFolder = path;
 	} else {
 		m_openSaveFolder = fileInfo.path().remove(fileInfo.fileName());
 	}
-	QSettings settings;
-	settings.setValue("openSaveFolder", m_openSaveFolder);
 }
 
+
 const QString FolderUtils::openSaveFolder() {
-	if(m_openSaveFolder == ___emptyString___) {
+	if(m_openSaveFolder.isEmpty()) {
 		QSettings settings;
 		QString tempFolder = settings.value("openSaveFolder").toString();
 		if (!tempFolder.isEmpty()) {
@@ -296,7 +326,7 @@ void FolderUtils::rmdir(QDir & dir) {
 	dir.rmdir(dir.path());
 }
 
-bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &filepath) {
+bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &filepath, const QStringList & skipSuffixes) {
 	DebugDialog::debug("zipping "+dirToCompress.path()+" into "+filepath);
 
 	QString tempZipFile = QDir::temp().path()+"/"+getRandText()+".zip";
@@ -316,6 +346,19 @@ bool FolderUtils::createZipAndSaveTo(const QDir &dirToCompress, const QString &f
 	QDir::setCurrent(dirToCompress.path());
 	foreach(QFileInfo file, files) {
 		if(!file.isFile()||file.fileName()==filepath) continue;
+		if (file.fileName().contains(LockManager::LockedFileName)) continue;
+
+        bool skip = false;
+        foreach (QString suffix, skipSuffixes) {
+            if (file.fileName().endsWith(suffix)) {
+                skip = true;
+                break;
+            }
+        }
+        if (skip) continue;
+
+//#pragma message("remove fzz check")
+//if (file.fileName().endsWith(".fzz")) continue;
 
 		inFile.setFileName(file.fileName());
 
@@ -438,100 +481,64 @@ bool FolderUtils::unzipTo(const QString &filepath, const QString &dirToDecompres
 	return true;
 }
 
-void FolderUtils::initLockedFiles(const QString & prefix, QString & folder, QHash<QString, class QtLockedFile *> & lockedFiles) {
-	// first create our own unique folder and lock it
-	QDir backupDir(FolderUtils::getUserDataStorePath(prefix));
-	QString lockedSubfolder = FolderUtils::getRandText();
-	backupDir.mkdir(lockedSubfolder);
-	folder = backupDir.absoluteFilePath(lockedSubfolder);
-	QtLockedFile * lockFile = new QtLockedFile(folder + "/" + LockFileName);
-	lockFile->open(QFile::WriteOnly);
-	lockFile->lock(QtLockedFile::WriteLock, true);
-	lockedFiles.insert(lockedSubfolder, lockFile);
-}
-
-
-void FolderUtils::releaseLockedFiles(const QString & folder, QHash<QString, class QtLockedFile *> & lockedFiles) 
+void FolderUtils::collectFiles(const QDir & parent, QStringList & filters, QStringList & files)
 {
-	// remove backup files; this is a clean exit
-	QDir backupDir(folder);
-	backupDir.cdUp();
-	foreach (QString sub, lockedFiles.keys()) {
-        QtLockedFile * lockedFile = lockedFiles.value(sub);
-		lockedFile->unlock();
-		lockedFile->close();
-		delete lockedFile;
-        FolderUtils::rmdir(backupDir.absoluteFilePath(sub));
+	QFileInfoList fileInfoList = parent.entryInfoList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+	foreach (QFileInfo fileInfo, fileInfoList) {
+		files.append(fileInfo.absoluteFilePath());
 	}
-	lockedFiles.clear();
-}
 
-bool FolderUtils::checkLockedFilesAux(const QDir & parent, QStringList & filters)
-{
 	QFileInfoList dirList = parent.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::NoSymLinks);
 	foreach (QFileInfo dirInfo, dirList) {
 		QDir dir(dirInfo.filePath());
 		//DebugDialog::debug(QString("looking in backup dir %1").arg(dir.absolutePath()));
-		QFileInfoList fileInfoList = dir.entryInfoList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
-		if (!fileInfoList.isEmpty()) return true;
 
-		if (checkLockedFilesAux(dir, filters)) return true;
-	}
-
-	return false;
-}
-
-void FolderUtils::checkLockedFiles(const QString & prefix, QFileInfoList & backupList, QStringList & filters, QHash<QString, class QtLockedFile *> & lockedFiles, bool recurse)
-{
-	QDir backupDir(FolderUtils::getUserDataStorePath(prefix));
-	QFileInfoList dirList = backupDir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::NoSymLinks);
-	foreach (QFileInfo dirInfo, dirList) {
-		QDir dir(dirInfo.filePath());
-		//DebugDialog::debug(QString("looking in backup dir %1").arg(dir.absolutePath()));
-		QFileInfoList fileInfoList = dir.entryInfoList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
-		bool gotRecurse = false;
-		if (recurse && fileInfoList.isEmpty()) {
-			gotRecurse = checkLockedFilesAux(dir, filters);
-		}
-
-		if (fileInfoList.isEmpty() && !gotRecurse) {
-			// could mean this backup folder is just being created by another process
-			// could also mean it's leftover crap.
-			// check the date and only delete if it's old
-
-			// don't lock the file, in case this is a race condition:
-			// if another instance starting up is about to lock the file, 
-			// the date will be recent and we won't delete 
-			
-			QDateTime lastModified = dirInfo.lastModified();
-			if (lastModified < QDateTime::currentDateTime().addSecs(-5 * 60)) {
-				// At startup, creating the backup folder, the lock file, and locking it are successive operations
-				// so even giving it 5 minutes here is probably way more than we need
-				FolderUtils::rmdir(dirInfo.filePath());
-			}
-			
-			continue;
-		}
-		
-		QtLockedFile * otherLockFile = new QtLockedFile(dir.absoluteFilePath(FolderUtils::LockFileName));
-		bool isOpen = otherLockFile->open(QFile::WriteOnly);
-		if (!isOpen) {
-			// somebody else owns the file
-			delete otherLockFile;
-			continue;
-		}
-
-		if (!otherLockFile->lock(QtLockedFile::WriteLock, false)) {
-			// somebody else owns the file
-			otherLockFile->close();
-			delete otherLockFile;
-			continue;
-		}
-
-		// we own the file
-		lockedFiles.insert(dirInfo.fileName(), otherLockFile);		
-		foreach (QFileInfo fileInfo, fileInfoList) {
-			backupList << fileInfo;
-		}
+		collectFiles(dir, filters, files);
 	}
 }
+
+
+
+void FolderUtils::makePartFolderHierarchy(const QString & prefixFolder, const QString & destFolder) {
+	QDir dir(prefixFolder);
+
+	dir.mkdir(destFolder);
+	dir.mkdir("svg");
+	dir.cd("svg");
+	dir.mkdir(destFolder);
+	dir.cd(destFolder);
+	dir.mkdir("icon");
+	dir.mkdir("breadboard");
+	dir.mkdir("schematic");
+	dir.mkdir("pcb");
+}
+
+void FolderUtils::copyBin(const QString & dest, const QString & source) {
+    if(QFileInfo(dest).exists()) return;
+
+    // this copy action, is not working on windows, because is a resources file
+    if(!QFile(source).copy(dest)) {
+#ifdef Q_WS_WIN // may not be needed from qt 4.5.2 on
+        DebugDialog::debug("Failed to copy a file from the resources");
+        QDir binsFolder = QFileInfo(dest).dir().absolutePath();
+        QStringList binFiles = binsFolder.entryList(QDir::AllEntries | QDir::NoDotAndDotDot);
+        foreach(QString binName, binFiles) {
+            if(binName.startsWith("qt_temp.")) {
+                QString filePath = binsFolder.absoluteFilePath(binName);
+                bool success = QFile(filePath).rename(dest);
+                Q_UNUSED(success);
+                break;
+            }
+        }
+#endif
+    }
+    QFlags<QFile::Permission> ps = QFile::permissions(dest);
+    QFile::setPermissions(
+        dest,
+        QFile::WriteOwner | QFile::WriteUser | ps
+#ifdef Q_WS_WIN
+        | QFile::WriteOther | QFile::WriteGroup
+#endif
+
+    );
+}
\ No newline at end of file
