@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2011 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 6268 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-08-09 08:53:47 +0200 (Thu, 09 Aug 2012) $
 
 ********************************************************************/
 
+#include <QNetworkInterface>
 #include <QTextDocument>
+
 #include "textutils.h"
 #include "misc.h"
 //#include "../debugdialog.h"
@@ -33,6 +35,9 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include <QBuffer>
 #include <QFile>
 
+#include <qmath.h>
+#include <qnumeric.h>
+
 const QString TextUtils::CreatedWithFritzingString("Created with Fritzing (http://www.fritzing.org/)");
 const QString TextUtils::CreatedWithFritzingXmlComment("<!-- " + CreatedWithFritzingString + " -->\n");
 
@@ -51,7 +56,7 @@ const QString TextUtils::MicroSymbol = QString::fromUtf16(&MicroSymbolCode, 1);
 const QString TextUtils::AdobeIllustratorIdentifier = "Generator: Adobe Illustrator";
 
 QList<QString> PowerPrefixes;
-QList<qreal> PowerPrefixValues;
+QList<double> PowerPrefixValues;
 const QString TextUtils::PowerPrefixesString = QString("pnmkMGTu\\x%1").arg(MicroSymbolCode, 4, 16, QChar('0'));
 
 void TextUtils::findElementsWithAttribute(QDomElement & element, const QString & att, QList<QDomElement> & elements) {
@@ -94,9 +99,9 @@ QSet<QString> TextUtils::getRegexpCaptures(const QString &pattern, const QString
 	return captures;
 }
 
-qreal TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrator) {
+double TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrator) {
 	QString string = s;
-	qreal divisor = 1.0;
+	double divisor = 1.0;
 	if (string.endsWith("cm", Qt::CaseInsensitive)) {
 		divisor = 2.54;
 		string.chop(2);
@@ -122,7 +127,7 @@ qreal TextUtils::convertToInches(const QString & s, bool * ok, bool isIllustrato
 	}
 
 	bool fine;
-	qreal result = string.toDouble(&fine);
+	double result = string.toDouble(&fine);
 	if (!fine) {
 		if (ok) *ok = false;
 		return 0;
@@ -171,25 +176,30 @@ QString TextUtils::replaceTextElement(const QString & svg, const QString & id, c
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		node.normalize();
+		replaceChildText(doc, node, newValue);
+		return doc.toString();
+	}
+		
+	return svg;
+}
 
-		bool gotText = false;
-		QDomNodeList childList = node.childNodes();
-		for (int j = 0; j < childList.count(); j++) {
-			QDomNode child = childList.item(i);
-			if (child.isText()) {
-				gotText = true;
-				child.setNodeValue(newValue);
-				return doc.toString();
-			}
-		}
+QByteArray TextUtils::replaceTextElement(const QByteArray & svg, const QString & id, const QString & newValue) {
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	QDomDocument doc;
+	if (!doc.setContent(svg, &errorStr, &errorLine, &errorColumn)) return svg;
 
-		if (!gotText) {
-			QDomText t = doc.createTextNode(newValue);
-			node.appendChild(t);
-			return doc.toString();
-		}
+	QDomElement root = doc.documentElement();
+	QDomNodeList domNodeList = root.elementsByTagName("text");
+	for (int i = 0; i < domNodeList.count(); i++) {
+		QDomElement node = domNodeList.item(i).toElement();
+		if (node.isNull()) continue;
 
+		if (node.attribute("id").compare(id) != 0) continue;
+
+		replaceChildText(doc, node, newValue);
+		return doc.toByteArray();
 	}
 		
 	return svg;
@@ -211,25 +221,8 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 		foreach (QString id, hash.keys()) {
 			if (node.attribute("id").compare(id) != 0) continue;
 
-			node.normalize();
-
-			bool gotText = false;
-			QDomNodeList childList = node.childNodes();
-			for (int j = 0; j < childList.count(); j++) {
-				QDomNode child = childList.item(i);
-				if (child.isText()) {
-					gotText = true;
-					changed = true;
-					child.setNodeValue(hash.value(id));
-					break;
-				}
-			}
-
-			if (!gotText) {
-				changed = true;
-				QDomText t = doc.createTextNode(hash.value(id));
-				node.appendChild(t);
-			}
+			replaceChildText(doc, node, hash.value(id));
+			changed = true;
 			break;
 		}
 	}
@@ -239,6 +232,32 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 	return doc.toString();
 }
 
+
+void TextUtils::replaceElementChildText(QDomDocument & doc, QDomElement & root, const QString & elementName, const QString & text) {
+	QDomElement element = root.firstChildElement(elementName);
+	if (element.isNull()) {
+		doc.createElement(elementName);
+		root.appendChild(element);
+	}
+	replaceChildText(doc, element, text);
+}
+
+void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QString & text) {
+	node.normalize();
+
+	QDomNodeList childList = node.childNodes();
+	for (int j = 0; j < childList.count(); j++) {
+		QDomNode child = childList.item(j);
+		if (child.isText()) {
+			child.setNodeValue(text);
+			return;
+		}
+	}
+
+	QDomText t = doc.createTextNode(text);
+	node.appendChild(t);
+}
+
 bool TextUtils::mergeSvg(QDomDocument & doc1, const QString & svg, const QString & id) 
 {
 	QString errorStr;
@@ -286,7 +305,9 @@ QString TextUtils::mergeSvgFinish(QDomDocument & doc) {
 QString TextUtils::mergeSvg(const QString & svg1, const QString & svg2, const QString & id, bool flip) {
 
 	QDomDocument doc1;
-	if (!TextUtils::mergeSvg(doc1, svg1, id)) return ___emptyString___;
+    if (!svg1.isEmpty()) {
+	    if (!TextUtils::mergeSvg(doc1, svg1, id)) return ___emptyString___;
+    }
 
 	if (!TextUtils::mergeSvg(doc1, svg2, id)) return ___emptyString___;
 
@@ -294,7 +315,7 @@ QString TextUtils::mergeSvg(const QString & svg1, const QString & svg2, const QS
 		QDomElement svg = doc1.documentElement();
 		QString viewBox = svg.attribute("viewBox");
 		QStringList coords = viewBox.split(" ", QString::SkipEmptyParts);
-		qreal width = coords[2].toDouble();
+		double width = coords[2].toDouble();
 		QMatrix matrix;
 		matrix.translate(width / 2, 0);
 		matrix.scale(-1, 1);
@@ -307,17 +328,17 @@ QString TextUtils::mergeSvg(const QString & svg1, const QString & svg2, const QS
 	return mergeSvgFinish(doc1);
 }
 
-QString TextUtils::makeSVGHeader(qreal printerScale, qreal dpi, qreal width, qreal height) {
+QString TextUtils::makeSVGHeader(double printerScale, double dpi, double width, double height) {
 
-	qreal trueWidth = width / printerScale;
-	qreal trueHeight = height / printerScale;
+	double trueWidth = width / printerScale;
+	double trueHeight = height / printerScale;
 
 	return 
-		QString("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n%5"
-							 "<svg xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns=\"http://www.w3.org/2000/svg\" "
-							 "version=\"1.2\" baseProfile=\"tiny\" "
-							 "x=\"0in\" y=\"0in\" width=\"%1in\" height=\"%2in\" "
-							 "viewBox=\"0 0 %3 %4\" >\n"
+		QString("<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n%5"
+							 "<svg xmlns:svg='http://www.w3.org/2000/svg' xmlns='http://www.w3.org/2000/svg' "
+							 "version='1.2' baseProfile='tiny' "
+							 "x='0in' y='0in' width='%1in' height='%2in' "
+							 "viewBox='0 0 %3 %4' >\n"
 							 )
 						.arg(trueWidth)
 						.arg(trueHeight)
@@ -477,7 +498,7 @@ bool TextUtils::pxToInches(QDomElement &elem, const QString &attrName, bool isIl
 	QString attrValue = elem.attribute(attrName);
 	if(attrValue.endsWith("px")) {
 		bool ok;
-		qreal value = TextUtils::convertToInches(attrValue, &ok, isIllustrator);
+		double value = TextUtils::convertToInches(attrValue, &ok, isIllustrator);
 		if(ok) {
 			QString newValue = QString("%1in").arg(value);
 			elem.setAttribute(attrName,newValue);
@@ -500,7 +521,7 @@ void TextUtils::setSVGTransform(QDomElement & element, QMatrix & matrix)
 QString TextUtils::svgTransform(const QString & svg, QTransform & transform, bool translate, const QString extras) {
 	if (transform.isIdentity()) return svg;
 
-	return QString("<g transform=\"matrix(%1,%2,%3,%4,%5,%6)\" %8 >%7</g>")
+	return QString("<g transform='matrix(%1,%2,%3,%4,%5,%6)' %8 >%7</g>")
 			.arg(transform.m11())
 			.arg(transform.m12())
 			.arg(transform.m21())
@@ -511,7 +532,7 @@ QString TextUtils::svgTransform(const QString & svg, QTransform & transform, boo
 			.arg(extras);
 }
 
-bool TextUtils::getSvgSizes(QDomDocument & doc, qreal & sWidth, qreal & sHeight, qreal & vbWidth, qreal & vbHeight) 
+bool TextUtils::getSvgSizes(QDomDocument & doc, double & sWidth, double & sHeight, double & vbWidth, double & vbHeight) 
 {
 	bool isIllustrator = isIllustratorDoc(doc);
 
@@ -535,12 +556,12 @@ bool TextUtils::getSvgSizes(QDomDocument & doc, qreal & sWidth, qreal & sHeight,
 	if (!sviewboxStr.isEmpty()) {
 		QStringList strings = sviewboxStr.split(" ");
 		if (strings.size() == 4) {
-			qreal tempWidth = strings[2].toDouble(&vbWidthOK);
+			double tempWidth = strings[2].toDouble(&vbWidthOK);
 			if (vbWidthOK) {
 				vbWidth = tempWidth;
 			}
 
-			qreal tempHeight= strings[3].toDouble(&vbHeightOK);
+			double tempHeight= strings[3].toDouble(&vbHeightOK);
 			if (vbHeightOK) {
 				vbHeight = tempHeight;
 			}
@@ -561,7 +582,7 @@ bool TextUtils::getSvgSizes(QDomDocument & doc, qreal & sWidth, qreal & sHeight,
 	return true;
 }
 
-bool TextUtils::findText(QDomNode & node, QString & text) {
+bool TextUtils::findText(const QDomNode & node, QString & text) {
 	if (node.isText()) {
 		text = node.nodeValue();
 		return true;
@@ -577,9 +598,9 @@ bool TextUtils::findText(QDomNode & node, QString & text) {
 	return false;
 }
 
-qreal TextUtils::convertToInches(const QString & string) {
+double TextUtils::convertToInches(const QString & string) {
 	bool ok; 
-	qreal retval = TextUtils::convertToInches(string, &ok, false);
+	double retval = TextUtils::convertToInches(string, &ok, false);
 	if (!ok) return 0;
 
 	return retval;
@@ -591,6 +612,21 @@ QString TextUtils::escapeAnd(const QString & string) {
 	return s;
 }
 
+QString TextUtils::convertExtendedChars(const QString & str) 
+{
+	QString result;
+    foreach (QChar c, str) {
+		if (c < 128) {
+			result.append(c);
+		}
+		else {
+			result.append(QString("&#x%1;").arg(c.unicode(), 0, 16));
+		}
+	}
+
+	return result;
+}
+
 QString TextUtils::stripNonValidXMLCharacters(const QString & str) 
 {
 	QString result;
@@ -613,7 +649,12 @@ QString TextUtils::stripNonValidXMLCharacters(const QString & str)
 				result.append(hs);
 				in_hs = false;
 			}
-			result.append(c);
+			if (c > 255) {
+				result.append(QString("&#%1").arg(c, 0, 16));
+			}
+			else {
+				result.append(c);
+			}
 		}
 		else {
 			in_hs = false;
@@ -675,7 +716,7 @@ bool TextUtils::addCopper1(const QString & filename, QDomDocument & domDocument,
 
 }
 
-QString TextUtils::convertToPowerPrefix(qreal q) {
+QString TextUtils::convertToPowerPrefix(double q) {
 	initPowerPrefixes();
 
 	for (int i = 0; i < PowerPrefixes.count(); i++) {
@@ -688,17 +729,17 @@ QString TextUtils::convertToPowerPrefix(qreal q) {
 	return QString::number(q);
 }
 
-qreal TextUtils::convertFromPowerPrefixU(QString & val, const QString & symbol) 
+double TextUtils::convertFromPowerPrefixU(QString & val, const QString & symbol) 
 {
 	val.replace('u', MicroSymbol);
 	return convertFromPowerPrefix(val, symbol);
 }
 
-qreal TextUtils::convertFromPowerPrefix(const QString & val, const QString & symbol) 
+double TextUtils::convertFromPowerPrefix(const QString & val, const QString & symbol) 
 {
 	initPowerPrefixes();
 
-	qreal multiplier = 1;
+	double multiplier = 1;
 	QString temp = val;
 	if (temp.endsWith(symbol)) {
 		temp.chop(symbol.length());
@@ -739,6 +780,19 @@ void TextUtils::collectLeaves(QDomElement & element, int & index, QVector<QDomEl
 	}
 }
 
+void TextUtils::collectLeaves(QDomElement & element, QList<QDomElement> & leaves) {
+	if (element.hasChildNodes()) {
+		QDomElement c = element.firstChildElement();
+		while (!c.isNull()) {
+			collectLeaves(c, leaves);
+			c = c.nextSiblingElement();
+		}
+	}
+	else {
+		leaves.append(element);
+	}
+}
+
 QMatrix TextUtils::elementToMatrix(QDomElement & element) {
 	QString transform = element.attribute("transform");
 	if (transform.isEmpty()) return QMatrix();
@@ -749,7 +803,7 @@ QMatrix TextUtils::elementToMatrix(QDomElement & element) {
 QMatrix TextUtils::transformStringToMatrix(const QString & transform) {
 
 	// doesn't handle multiple transform attributes
-	QList<qreal> floats = getTransformFloats(transform);
+	QList<double> floats = getTransformFloats(transform);
 
 	if (transform.startsWith("translate")) {
 		return QMatrix().translate(floats[0], (floats.length() > 1) ? floats[1] : 0);
@@ -778,12 +832,12 @@ QMatrix TextUtils::transformStringToMatrix(const QString & transform) {
 	return QMatrix();
 }
 
-QList<qreal> TextUtils::getTransformFloats(QDomElement & element){
+QList<double> TextUtils::getTransformFloats(QDomElement & element){
 	return getTransformFloats(element.attribute("transform"));
 }
 
-QList<qreal> TextUtils::getTransformFloats(const QString & transform){
-    QList<qreal> list;
+QList<double> TextUtils::getTransformFloats(const QString & transform){
+    QList<double> list;
     int pos = 0;
 
 	while ((pos = TextUtils::floatingPointMatcher.indexIn(transform, pos)) != -1) {
@@ -822,6 +876,30 @@ void TextUtils::gWrap(QDomDocument & domDocument, const QHash<QString, QString>
 	}
 }
 
+
+bool TextUtils::noPattern(QString &svg) {
+	if (!svg.contains("<pattern")) return false;
+
+	QDomDocument svgDom;
+	QString errorMsg;
+	int errorLine;
+	int errorCol;
+	if(!svgDom.setContent(svg, true, &errorMsg, &errorLine, &errorCol)) {
+		return false;
+	}
+
+	QDomNodeList nodeList = svgDom.elementsByTagName("pattern");
+    for (int i = 0; i < nodeList.count(); i++) {
+        QDomNode pattern = nodeList.at(i);
+
+        pattern.parentNode().removeChild(pattern);
+	}
+
+    svg = removeXMLEntities(svgDom.toString());
+	return true;
+}
+
+
 bool TextUtils::tspanRemove(QString &svg) {
 	if (!svg.contains("<tspan")) return false;
 
@@ -872,6 +950,60 @@ bool TextUtils::tspanRemove(QString &svg) {
 	return true;
 }
 
+bool TextUtils::noUse(QString &svg) {
+	if (!svg.contains("<use")) return false;
+
+	QDomDocument svgDom;
+	QString errorMsg;
+	int errorLine;
+	int errorCol;
+	if (!svgDom.setContent(svg, true, &errorMsg, &errorLine, &errorCol)) {
+		return false;
+	}
+
+    QDomElement root = svgDom.documentElement();
+    if (root.isNull()) return false;
+
+	QList<QDomElement> uses;
+	QDomNodeList useNodeList = svgDom.elementsByTagName("use");
+    for (int i = 0; i < useNodeList.count(); i++) {
+        QDomElement use = useNodeList.item(i).toElement();
+        QString refid = use.attribute("href");
+        if (refid.isEmpty()) continue;
+
+        QString id = use.attribute("id");
+        if (id.isEmpty()) continue;
+
+		uses.append(use);
+	}
+
+	foreach (QDomElement use, uses) {
+		QString transform = use.attribute("transform");
+        QString refid = use.attribute("href");
+        QString id = use.attribute("id");
+
+        QDomElement g = svgDom.createElement("g");
+		use.parentNode().replaceChild(g, use);
+        g.setAttribute("transform", transform);
+
+        if (refid.startsWith("#")) {
+            refid.remove(0, 1);
+            if (refid.isEmpty()) continue;
+        }
+
+        QDomElement toCopy = findElementWithAttribute(root, "id", refid);
+        if (toCopy.isNull()) continue;
+
+        QDomElement copy = toCopy.cloneNode(true).toElement();
+        g.appendChild(copy);
+        copy.setAttribute("id", id);
+	}
+
+	svg = removeXMLEntities(svgDom.toString());
+	return true;
+}
+
+
 QDomElement TextUtils::copyText(QDomDocument & svgDom, QDomElement & parent, QDomElement & text, const QString & defaultX, const QString & defaultY, bool copyAttributes)
 {
 	QDomNode cnode = text.firstChild();
@@ -902,19 +1034,33 @@ QDomElement TextUtils::copyText(QDomDocument & svgDom, QDomElement & parent, QDo
 	return ___emptyElement___;
 }
 
-void TextUtils::slamStrokeAndFill(QDomElement & element, const QString & stroke, const QString & fill)
+QString TextUtils::slamStrokeAndFill(const QString & svg, const QString & stroke, const QString & strokeWidth, const QString & fill)
+{
+	QDomDocument doc;
+    if (doc.setContent(svg)) {
+        QDomElement root = doc.documentElement();
+        slamStrokeAndFill(root, stroke, strokeWidth, fill);
+        return doc.toString();
+    }
+
+    return svg;
+}
+
+void TextUtils::slamStrokeAndFill(QDomElement & element, const QString & stroke, const QString & strokeWidth, const QString & fill)
 {
 	// assumes style elements have been normalized already
 	QString strokeAtt = element.attribute("stroke");
 	QString fillAtt = element.attribute("fill");
-	if (!strokeAtt.isEmpty() || !fillAtt.isEmpty()) {
+	QString strokeWidthAtt = element.attribute("stroke-width");
+	if (!strokeAtt.isEmpty() || !fillAtt.isEmpty() || !strokeWidthAtt.isEmpty()) {
 		element.setAttribute("stroke", stroke);
 		element.setAttribute("fill", fill);
+		element.setAttribute("stroke-width", strokeWidth);
 	}
 
 	QDomElement child = element.firstChildElement();
 	while (!child.isNull()) {
-		slamStrokeAndFill(child, stroke, fill);
+		slamStrokeAndFill(child, stroke, strokeWidth, fill);
 		child = child.nextSiblingElement();
 	}
 }
@@ -924,20 +1070,20 @@ struct MatchThing
 {
 	int pos;
 	int len;
-	qreal val;
+	double val;
 };
 
-QString TextUtils::incrementTemplate(const QString & filename, int pins, qreal increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun) 
+QString TextUtils::incrementTemplate(const QString & filename, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun, void * userData) 
 {
 	QFile file(filename);
 	file.open(QFile::ReadOnly);
 	QString templateString = file.readAll();
 	file.close();
 
-	return incrementTemplateString(templateString, pins, increment, multiFun, copyFun);
+	return incrementTemplateString(templateString, pins, increment, multiFun, copyFun, userData);
 }
 
-QString TextUtils::incrementTemplateString(const QString & templateString, int pins, qreal increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun)
+QString TextUtils::incrementTemplateString(const QString & templateString, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun, void * userData)
 {
 	QString string;
 
@@ -960,35 +1106,46 @@ QString TextUtils::incrementTemplateString(const QString & templateString, int p
 			MatchThing * mt = &matchThings[j];
 			argCopy.replace(mt->pos, mt->len, (*multiFun)(i, increment, mt->val));   
 		}
-		string += (*copyFun)(i, argCopy);
+		string += (*copyFun)(i, argCopy, userData);
 	}
 
 	return string;
 }
 
-QString TextUtils::standardCopyPinFunction(int pin, const QString & argString)
+QString TextUtils::standardCopyPinFunction(int pin, const QString & argString, void *)
 {
 	return argString.arg(pin);
 }
 
-QString TextUtils::standardMultiplyPinFunction(int pin, qreal increment, qreal value)
+QString TextUtils::standardMultiplyPinFunction(int pin, double increment, double value)
 {
 	return QString::number(value + (pin * increment));
 }
 
 
-QString TextUtils::incMultiplyPinFunction(int pin, qreal increment, qreal value) 
+QString TextUtils::incMultiplyPinFunction(int pin, double increment, double value) 
 {
 	return QString::number(value + ((pin + 1) * increment));
 }
 
+QString TextUtils::incCopyPinFunction(int pin, const QString & argString, void *) 
+{ 
+	return argString.arg(pin + 1); 
+}
 
-QString TextUtils::noCopyPinFunction(int, const QString & argString) 
+QString TextUtils::noCopyPinFunction(int, const QString & argString, void *) 
 { 
 	return argString; 
 }
 
-qreal TextUtils::getViewBoxCoord(const QString & svg, int coord)
+QString TextUtils::negIncCopyPinFunction(int pin, const QString & argString, void * userData) 
+{ 
+	int pins = *((int *) userData);
+	int offset = *(((int *) userData) + 1);
+	return argString.arg(pins - (pin + offset)); 
+}
+
+double TextUtils::getViewBoxCoord(const QString & svg, int coord)
 {
 	QRegExp re("viewBox=['\\\"]([^'\\\"]+)['\\\"]");
 	int ix = re.indexIn(svg);
@@ -998,4 +1155,259 @@ qreal TextUtils::getViewBoxCoord(const QString & svg, int coord)
 	QStringList coords = vb.split(" ");
 	QString c = coords.at(coord);
 	return c.toDouble();
-}
\ No newline at end of file
+}
+
+QString TextUtils::makeLineSVG(QPointF p1, QPointF p2, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+{
+	p1.setX(p1.x() * dpi / printerScale);
+	p1.setY(p1.y() * dpi / printerScale);
+	p2.setX(p2.x() * dpi / printerScale);
+	p2.setY(p2.y() * dpi / printerScale);
+
+	QString stroke = (blackOnly) ? "black" : colorString;
+	return QString("<line stroke-linecap='round' stroke='%6' x1='%1' y1='%2' x2='%3' y2='%4' stroke-width='%5' />")
+					.arg(p1.x())
+					.arg(p1.y())
+					.arg(p2.x())
+					.arg(p2.y())
+					.arg(width * dpi / printerScale)
+					.arg(stroke);
+}
+
+QString TextUtils::makeCubicBezierSVG(const QPolygonF & poly, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+{
+	QString stroke = (blackOnly) ? "black" : colorString;
+	return QString("<path stroke-linecap='round' fill='none' stroke-width='%1' stroke='%2' d='M%3,%4C%5,%6, %7,%8 %9,%10' />")
+					.arg(width * dpi / printerScale)
+					.arg(stroke)
+					.arg(poly.at(0).x() * dpi / printerScale)
+					.arg(poly.at(0).y() * dpi / printerScale)
+					.arg(poly.at(1).x() * dpi / printerScale)
+					.arg(poly.at(1).y() * dpi / printerScale)
+					.arg(poly.at(2).x() * dpi / printerScale)
+					.arg(poly.at(2).y() * dpi / printerScale)
+					.arg(poly.at(3).x() * dpi / printerScale)
+					.arg(poly.at(3).y() * dpi / printerScale)
+					;
+}
+
+
+QString TextUtils::makeRectSVG(QRectF r, QPointF offset, double dpi, double printerScale)
+{
+	r.translate(-offset.x(), -offset.y());
+	double l = r.left() * dpi / printerScale;
+	double t = r.top() * dpi / printerScale;
+	double w = r.width() * dpi / printerScale;
+	double h = r.height() * dpi / printerScale;
+
+	QString stroke = "black";
+	return QString("<rect stroke-linecap='round' stroke='%6' x='%1' y='%2' width='%3' height='%4' stroke-width='%5' fill='none' />")
+			.arg(l)
+			.arg(t)
+			.arg(w)
+			.arg(h)
+			.arg(1 * dpi / printerScale)
+			.arg(stroke);
+}
+
+
+QString TextUtils::makePolySVG(const QPolygonF & poly, QPointF offset, double width, QString colorString, double dpi, double printerScale, bool blackOnly) 
+{
+	QString polyString = QString("<polyline stroke-linecap='round' stroke-linejoin='round' fill='none' stroke='%1' stroke-width='%2' points='\n").arg(blackOnly ? "black" : colorString).arg(width);
+	int space = 0;
+	foreach (QPointF p, poly) {
+		polyString += QString("%1,%2 %3")
+			.arg((p.x() - offset.x()) * dpi / printerScale)
+			.arg((p.y() - offset.y()) * dpi / printerScale)
+			.arg((++space % 8 == 0) ?  "\n" : "");
+	}
+	polyString += "'/>\n";
+	return polyString;
+}
+
+QPolygonF TextUtils::polygonFromElement(QDomElement & element) 
+{
+	QPolygonF poly;
+	QDomElement point = element.firstChildElement("point");
+	while (!point.isNull()) {
+		double x = point.attribute("x").toDouble();
+		double y = point.attribute("y").toDouble();
+		poly.append(QPointF(x, y));
+		point = point.nextSiblingElement("point");
+	}
+	return poly;
+}
+
+QString TextUtils::pointToSvgString(QPointF p, QPointF offset, double dpi, double printerScale)
+{
+	QString point;
+	point += QString::number((p.x() - offset.x()) * dpi / printerScale);
+	point += ",";
+	point += QString::number((p.y() - offset.y()) * dpi / printerScale);
+	return point;
+}
+
+QString TextUtils::removeSVGHeader(QString & string) {
+	int ix = string.indexOf("<svg");
+	if (ix < 0) return string;
+
+	ix = string.indexOf(">", ix);
+	if (ix < 0) return string;
+
+	int jx = string.indexOf("</svg>");
+	if (jx < 0) return string;
+
+	string.remove(jx, 6);
+	string.remove(0, ix + 1);
+	return string;
+}
+
+
+QString TextUtils::getMacAddress()
+{
+	// http://stackoverflow.com/questions/7609953/obtaining-mac-address-on-windows-in-qt
+    foreach (QNetworkInterface interface, QNetworkInterface::allInterfaces())
+    {
+        // Return only the first non-loopback MAC Address
+        if (!(interface.flags() & QNetworkInterface::IsLoopBack))
+            return interface.hardwareAddress();
+    }
+    return QString();
+}
+
+
+QString TextUtils::expandAndFill(const QString & svg, const QString & color, double expandBy)
+{
+	QDomDocument domDocument;
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	bool result = domDocument.setContent(svg, &errorStr, &errorLine, &errorColumn);
+	if (!result) {
+		return "";
+	}
+
+	QDomElement root = domDocument.documentElement();
+	expandAndFillAux(root, color, expandBy);
+
+	return domDocument.toString();
+}
+
+void TextUtils::expandAndFillAux(QDomElement & element, const QString & color, double expandBy)
+{
+	bool gotChildren = false;
+	QDomElement child = element.firstChildElement();
+	while (!child.isNull()) {
+		gotChildren = true;
+		expandAndFillAux(child, color, expandBy);
+		child = child.nextSiblingElement();
+	}	
+
+	if (gotChildren) return;
+
+	QString fill = element.attribute("fill");
+	QString stroke = element.attribute("stroke");
+	QString strokeWidth = element.attribute("stroke-width");
+	if (stroke.isEmpty() && fill.isEmpty()) {
+		return;
+	}
+		
+	element.setAttribute("fill", color);
+	element.setAttribute("stroke", color);
+
+	if (strokeWidth.isEmpty()) {
+		strokeWidth = "0";
+	}
+
+	double sw = strokeWidth.toDouble();
+	sw += expandBy;
+	element.setAttribute("stroke-width", QString::number(sw));
+
+}
+
+bool TextUtils::writeUtf8(const QString & fileName, const QString & text)
+{
+	QFile file(fileName);
+	if (file.open(QIODevice::WriteOnly)) {
+		QTextStream out(&file);
+		out.setCodec("UTF-8");
+		out << text;
+		file.close();
+        return true;
+    }
+
+    return false;
+}
+
+int TextUtils::getPinsAndSpacing(const QString & expectedFileName, QString & spacingString) 
+{
+    QStringList pieces = expectedFileName.split("_");
+    int pix = 0;
+    foreach (QString piece, pieces) {
+        bool ok;
+        piece.toInt(&ok);
+        if (ok) break;
+
+        pix++;
+    }
+    if (pix >= pieces.count()) return 0;
+
+	int pins = pieces.at(pix).toInt();
+
+	spacingString = "100mil";
+    for (++pix; pix < pieces.count(); pix++) {
+        if (QRegExp("\\d").indexIn(pieces.at(pix)) == 0) {
+            spacingString = pieces.at(pix);
+            return pins;
+        }
+    }
+
+    return pins;
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(const QString & svg)
+{
+	QXmlStreamReader streamReader(svg);
+    return parseForWidthAndHeight(streamReader);
+}
+
+QSizeF TextUtils::parseForWidthAndHeight(QXmlStreamReader & svg)
+{
+    svg.setNamespaceProcessing(false);
+
+	QSizeF size(0,0);
+
+	bool isIllustrator = false;
+	bool bad = false;
+
+	while (!svg.atEnd() && !bad) {
+        switch (svg.readNext()) {
+		case QXmlStreamReader::Comment:
+			if (!isIllustrator) {
+				isIllustrator = TextUtils::isIllustratorFile(svg.text().toString());
+			}
+			break;
+        case QXmlStreamReader::StartElement:
+			if (svg.name().toString().compare("svg") == 0) {
+				QString ws = svg.attributes().value("width").toString();
+				QString hs = svg.attributes().value("height").toString();
+				bool okw, okh;
+				double w = TextUtils::convertToInches(ws, &okw, isIllustrator);
+				double h = TextUtils::convertToInches(hs, &okh, isIllustrator);
+				if (!okw || qIsNaN(w) || qIsInf(w) || !okh || qIsNaN(h) || qIsInf(h)) {
+					bad = true;
+					break;
+				}
+
+				size.setWidth(w);
+				size.setHeight(h);
+				return size;
+			}
+			break;		
+		default:
+			break;		
+		}
+	}
+
+    return size;
+}
